;;; -*- Mode: Lisp; Package: ystok.sql.sys -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2005 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Access database - Specialization for SQL expressions and types

(in-package :ystok.sql.sys)

(eval-when (:compile-toplevel :load-toplevel :execute)
  (import 'ystok.odbc::access-3.5-database :ystok.sql.sys))
  ;(export '(access-3.5-database) :ystok.sql.sys))

;;; Type specifiers

(defmethod db-sql-type-specifier ((database desktop-mixin) (lisp-type (eql 'string))
                                  &rest args)
  (declare (ignore database lisp-type))
  (if args
      (let ((length (first args)))
        (if (<= length 255)
            (format nil "VARCHAR(~A)" length)
            "LONGTEXT"))
      "VARCHAR"))

(defmethod db-sql-type-specifier ((database desktop-mixin) (lisp-type (eql 'boolean))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "LOGICAL")	; Excel

(defmethod db-sql-type-specifier ((database access-mixin) (lisp-type (eql 'boolean))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "BIT")

(defmethod db-sql-type-specifier ((database access-mixin) (lisp-type (eql :clob))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "LONGTEXT")

;;; NUMBER for Excel
(defmethod db-sql-type-specifier ((database access-mixin) (lisp-type (eql 'double-float))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "FLOAT")					; Jet 4.0

(defmethod db-sql-type-specifier ((database access-3.5-database)
                                  (lisp-type (eql 'double-float)) &rest args)
  (declare (ignore database lisp-type args))
  "DOUBLE")

(defmethod db-sql-type-specifier ((database desktop-mixin) (lisp-type (eql :timestamp))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "DATETIME")

(defmethod db-sql-type-specifier ((database desktop-mixin)(lisp-type (eql 'universal-time))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "DATETIME")

#|
(defmethod db-sql-type-specifier (database (lisp-type (eql 'double-float)) &rest args)
  (declare (ignore database lisp-type args))
  "DOUBLE")|#

;;; Local-time specifics
#+local-time
(defmethod db-sql-type-specifier ((database desktop-mixin)
                                  (lisp-type (eql 'local-time:date))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "DATETIME")
#+local-time
(defmethod db-sql-type-specifier ((database desktop-mixin)
                                  (lisp-type (eql 'local-time:local-time))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "DATETIME")
#+local-time
(defmethod db-sql-type-specifier ((database desktop-mixin)
                                  (lisp-type (eql 'local-time:utc-time))
                                  &rest args)
  (declare (ignore database lisp-type args))
  "DATETIME")

;;; Functions

(defsql-function () Iif Fix DatePart)

(defconstant +access-function-replacements+
  '((truncate . Fix)			; truncate is undefined within Jet 3.5 driver
    (upper . UCase) (lower . LCase)
    (current-date . Date)
    (current-time . Time) (localtime . Time)
    (current-timestamp . Now) (localtimestamp . Now)))

(yl:keep-symbol-names (mapcar #'cdr +access-function-replacements+))

(defmethod output-sql ((expr sql-function-exp) (database desktop-mixin))
 ;;; Replace some standard SQL-99 or ODBC scalar functions by builtin Access's ones.
  (if-bind (fn (cdr-assoc (name expr) +access-function-replacements+ :test #'string=))
    (%output-function-exp fn (slot-value expr 'args) database)
    (call-next-method)))

(defmethod output-sql ((expr sql-case-exp) (database desktop-mixin))
 ;;; Replace by nested iif(IsNull(arg1),iif(IsNull(arg2), # ,arg2),arg1)
  ;; NB: We could exploit Switch() unless its incompliance: 
  ;;	 on Jet4 it returns a string instead of a value of declared type.
  (with-slots (name args) expr
    (case name
      (COALESCE
       (labels ((out-args (args &aux (first (first args)) (rest (rest args)))
                  (if rest
                      (progn (write-string "Iif(IsNull(" *sql-stream*)
                             (output-sql first database)
                             (write-string ")," *sql-stream*)
                             (out-args rest)
                             (write-char #\, *sql-stream*)
                             (output-sql first database)
                             (write-char #\) *sql-stream*))
                      (output-sql first database))))
         (out-args args)))
      (COND
       (labels ((out-args (args)
                  (if args
                      (destructuring-bind (arg result) (first args)
                        (if (eq arg t)			; ELSE clause
                            (output-sql result database)
                            (progn (write-string "Iif(" *sql-stream*)
                                   (output-sql arg database)
                                   (write-char #\, *sql-stream*)
                                   (output-sql result database)
                                   (write-char #\, *sql-stream*)
                                   (out-args (rest args))
                                   (write-char #\) *sql-stream*))))
                      (output-sql *null* database))))
         (out-args args)))
      (CASE
       (labels ((out-args (operand args)
                  (if args
                      (destructuring-bind (arg result) (first args)
                        (if (eq arg t)			; ELSE clause
                            (output-sql result database)
                            (progn (write-string "Iif(" *sql-stream*)
                                   (output-sql operand database)
                                   (write-char #\= *sql-stream*)
                                   (output-sql arg database)
                                   (write-char #\, *sql-stream*)
                                   (output-sql result database)
                                   (write-char #\, *sql-stream*)
                                   (out-args operand (rest args))
                                   (write-char #\) *sql-stream*))))
                      (output-sql *null* database))))
         (out-args (first args) (rest args))))
      (otherwise
       (call-next-method)))))

(defmethod output-sql ((expr sql-relational-exp) (database desktop-mixin))
 ;;; Due to a bug in the Desktop dirvers: (expr IS NULL) is converted to (expr ISNULL),
  ;; when passed as an argument of scalar (and other kind?) functions, 
  ;; output IsNull(expr) instead.
  (with-slots (operator components) expr
    (if (and (eq operator 'IS) (null (second components)))
        (progn
          (write-string "IsNull(" *sql-stream*)
          (output-sql (first components) database)
          (write-char #\) *sql-stream*))
        (call-next-method))))

(defmethod output-sql ((expr sql-assignment-exp) (database desktop-mixin))
 ;;; Output column = value
  ;; Access allows qualifier in column, but not a scalar subquery in value.
  ;; To update, specify a join query instead of a table or view.
  (with-slots (operator components) expr
    ;(output-sql (first components) database)
    (write-sql-identifier (first components) *sql-stream* database nil)
    (write-char #\Space *sql-stream*)
    (output-sql operator database)
    (write-char #\Space *sql-stream*)
    (output-sql (second components) database)))

(defmethod output-sql ((expr sql-concat-exp) (database desktop-mixin))
 ;;; Instead of scalar function escapes, write standard SQL syntax but replace || with &
  (with-slots (components) expr
    (%output-relational-exp "&" components database)))
    
(defmethod output-sql ((expr sql-join-exp) (database desktop-mixin))
 ;; Using is not supported
  (with-slots (operator components on) expr
    (%output-join-exp operator components on nil database)))

;;; Sentences

(defmethod output-sql ((expr sql-query-exp) (database desktop-mixin))
 ;;; - Prohibit both TOP and LIMIT in favor of SQL_ATTR_MAX_ROWS statement attribute.
  ;; - Detect TRANSFORM and PIVOT (replacing OFFSET)
  (with-slots (distinct selections from where group-by having set-operation order-by pivot)
      expr
   (let ((selections selections))
    (when *in-subselect*
      (write-char #\( *sql-stream*))
    (when (eq (first selections) :transform)
      (write-string "TRANSFORM " *sql-stream*)
      (output-sql (second selections) database)
      (write-char #\Space *sql-stream*)
      (setq selections (cddr selections)))
    (write-string "SELECT " *sql-stream*)
    (when distinct
      (write-string "DISTINCT " *sql-stream*)
      (unless (eql distinct t)
        (write-string "ON " *sql-stream*)
        (output-sql distinct database)
        (write-char #\Space *sql-stream*)))
    ;(when limit			  ; NB: DISTINCT and TOP should not coexist.
    ;  (write-string " TOP " *sql-stream*)
    ;  (output-sql limit database)
    ;  (write-char #\Space *sql-stream*))
    (let ((*in-subselect* t))				; enable aliases
      ;; A la (output-sql (apply #'vector selections)), but hanlde AS keyword.
      (loop ;with *in-subselect* = t
            for (arg . rest) on selections
            unless (eq arg :as)
            do (output-sql arg database)
            and when rest
                   if (eq (first rest) :as)
                   do (write-string " AS " *sql-stream*)
                   else do (write-char #\, *sql-stream*))       
      (write-string " FROM " *sql-stream*)
      (if (listp from)
          (output-sql (apply #'vector from) database)
          (output-sql from database))
      (when where
        (write-string " WHERE " *sql-stream*)
        (output-sql where database))
     )
    (when group-by
      (write-string " GROUP BY " *sql-stream*)
      (if (listp group-by)
          (output-sql (apply #'vector group-by) database)
          (output-sql group-by database)))
    (when having
      (write-string " HAVING " *sql-stream*)
      (output-sql having database))
    (when set-operation
      (output-sql set-operation database))
    (when order-by
      (write-string " ORDER BY " *sql-stream*)
      (if (listp order-by)
          ;; A la (output-sql (apply #'vector order-by) database), but handle ASC/DESC
          (loop with prev = nil
                for arg in order-by
                if (sql-keywordp arg) do (write-char #\Space *sql-stream*)
                else if prev do (write-char #\, *sql-stream*)
                do (output-sql arg database)
                   (setq prev t))
          ;; ASC or DESC can be embedded into sql-column as :asc or :desc
          (output-sql order-by database)))
    (when pivot
      (write-string " PIVOT " *sql-stream*)
      (if (and (typep pivot 'sql-relational-exp) (eq (slot-value pivot 'operator) 'in))
          ;; Without outer parentheses!
          (with-slots (components) pivot
            (output-sql (first components) database)
            (write-string " IN " *sql-stream*)
            (output-sql (second components) database))
          (output-sql pivot database)))
    (when *in-subselect*
      (write-char #\) *sql-stream*)))))

(defmethod output-table-constraint (constraint (database access-mixin) &optional separator)
 ;;; Ignore
  ;; - unsupported :check constraint,
  ;; - :on-update and :on-delete options in FOREIGN KEY as they are only supported
  ;;   through Microsoft OLE DB Provider with Jet version 4.X.
  (when (eq (first constraint) :constraint)
    (setq constraint (rest constraint)))
  (unless (keywordp (first constraint))
    (unless (eq (second constraint) :check)	; not implemented in Access at all
      (when separator
        (write-char #\, *sql-stream*)
        (setq separator nil))
      (write-string "CONSTRAINT " *sql-stream*)
      (output-sql-identifier (first constraint) database)	; provide constraint name
      (write-char #\Space *sql-stream*))
    (setq constraint (rest constraint)))
  (when constraint		; only name when used in ALTER TABLE DROP CONSTRAINT
    (case (first constraint)
      (:foreign-key
       (when separator
         (write-char #\, *sql-stream*))
       (destructuring-bind (cols ref-keyword parent-table referenced-cols
                            &key on-update on-delete)
           (rest constraint)
         (declare (ignore on-update on-delete))
         (unless (eq ref-keyword :references)
           (error 'sql-reader-error :format-control "Keyword :references expected."))
         (write-string "FOREIGN KEY" *sql-stream*)
         (output-sql-identifiers cols database)
         ;(output-sql (listify cols) database)
         (write-string " REFERENCES " *sql-stream*)
         (output-sql-identifier parent-table database)
         (output-sql-identifiers referenced-cols database)))
         ;(output-sql (listify referenced-cols) database)))
      ((:primary-key :unique :not-null)
       (when separator
         (write-char #\, *sql-stream*))
       (output-sql (first constraint) database)
       (output-sql-identifiers (second constraint) database))
       ;(output-sql (listify (second constraint)) database))
      ((:check))				; not implemented in Access at all
      (otherwise
       (error 'sql-reader-error :format-control "Unexpected table constraint ~S."
           	  	        :format-arguments (list constraint))))))

(defmethod output-sql ((expr sql-alter-table-exp) (database access-mixin))
 ;;; ALTER TABLE table 
 ;;;	{ ADD {COLUMN field type[(size)] [NOT NULL] [CONSTRAINT index] }...
  ;;	| ALTER {COLUMN field type[(size)] | CONSTRAINT multifieldindex}... 
  ;;	| DROP {COLUMN field | CONSTRAINT indexname}... }
  ;; SQL Standard NB:
  ;;	According to SQL-2003/99/92, you may only add one column per ALTER TABLE
  ;;	statement. The SQL-89 allowed a form of ALTER TABLE that specified several
  ;;	columns in parentheses.	Jet 3.5 also prohibit adding or deleting more than
  ;;	one field or index at the moment.
  ;; NB: ALTER COLUMN field CONSTRAINT NOT NULL is not supported but works in ADD COLUMN.
  (with-slots (name operator args) expr
    (write-string "ALTER TABLE " *sql-stream*)
    (output-sql-identifier name database)
    (write-char #\Space *sql-stream*)
    (write-string (symbol-name operator) *sql-stream*)
    (write-char #\Space *sql-stream*)
    (do (arg
         (count 0)
         (separator nil)			; true when previous element was output
         (rest (listify args) (rest rest)))
        ((null rest)
         (when (zerop count)			; if no item was output, 
           (throw 'write-sql-to-string nil)))	; do not output the statement at all
      (declare (fixnum count))
      (when (if (and (consp (setq arg (first rest))) (eq (first arg) :constraint))
                (output-table-constraint arg database separator)
                (progn (when separator
                         (write-char #\, *sql-stream*))
                  (write-string "COLUMN " *sql-stream*)
                  (output-column arg database (eq operator :drop))
                  t))
        (setq separator t)
        (incf count)))))
  #|(when modify
        (when (plusp count)
          (write-char #\, *sql-stream*))
        (write-string " ALTER " *sql-stream*)
        (loop with local fixnum = 0
              for (arg . rest) on (listify modify)
              when (plusp local) do (write-char #\, *sql-stream*)
              when (and (consp arg) (eq (first arg) :constraint))
              do (output-table-constraint arg database)
              else do (write-string "COLUMN " *sql-stream*)
                      (output-column arg database)
            when (or rest drop)
              do (write-char #\, *sql-stream*)))
    (when drop
      (write-string " DROP " *sql-stream*)
      (loop for (arg . rest) on (listify drop)
            when (and (consp arg) (eq (first arg) :constraint))
              do (output-table-constraint arg database)
              else do (write-string "COLUMN " *sql-stream*)
                      (output-column arg database t)
            when rest
            do (write-char #\, *sql-stream*))) ))|#

(defmethod output-sql ((expr sql-create-view-exp) (database access-mixin))
 ;;; Columns and other propertices neither works for Jet 3.5 nor 4.0
  (%output-create-view-no-columns expr database))

(defmethod output-sql ((expr sql-create-index-exp) (database access-mixin))
 ;;; Provide options [WITH { PRIMARY | DISALLOW NULL | IGNORE NULL }]
  (with-slots (name on columns plist) expr
    (write-string "CREATE " *sql-stream*)
    (when (getf plist :unique) (output-sql :unique database))
    (write-string " INDEX " *sql-stream*)
    (output-sql-identifier name database)
    (write-string " ON " *sql-stream*)
    (output-sql-identifier on database)
    (output-sql (listify columns) database)
    (when-bind (plist (remove-properties plist '(:unique :type :using :where)))
      (write-string " WITH " *sql-stream*)
      (%output-plist plist database))))

;;; DB-methods

(defmethod db-type-initialize ((db-type (eql :access)) &optional version)
 ;;; Simulate type-subtype relationship between :odbc and :access
  ;; and map them both to the odbc-environment.
  (or (find-environment :odbc :errorp nil)
      (db-type-initialize :odbc version)))

(defmethod db-name-from-spec ((db-type (eql :access)) (connection-spec pathname))
  (declare (ignore db-type))
  connection-spec)

(defmethod db-connect ((db-type (eql :access)) connection-spec 
                       &rest args &key &allow-other-keys)
  ;; Push the specific back-end type into args for possible retreival via driver-connect
  (apply #'db-connect :odbc connection-spec  :database-type :access  args))


(defmethod db-drop-index ((database access-mixin) index &rest args
                          &key on (if-does-not-exist :error))
  (when (ecase if-does-not-exist
          (:ignore (index-exists-p index :database database :on on))
          (:error  t))
    (execute-command
     (write-drop-exp-to-string :index index database
                               (nconc (when on (list :on on))	; push :on at the head
                                      (remove-properties args '(:on :if-does-not-exist))))
     :database database)))


;;; SQL-schema: roles, users, and privileges - are not supported via ODBC
;;;  CREATE GROUP group pid[, group pid, ]
;;;  CREATE USER user password pid [, user password pid, ]
;;;  ADD USER user[, user, ] TO group
;;;  ALTER USER user PASSWORD newpassword oldpassword
;;;  DROP GROUP group[, group, ]
;;;  DROP USER user[, user, ] [FROM group]
;;;  GRANT {privilege[, privilege, ]}
;;;	ON {TABLE table | OBJECT object | CONTAINER container }
;;;	TO {authorizationname[, authorizationname, ]}
;;;  REVOKE {privilege[, privilege, ]}
;;;	ON {TABLE table | OBJECT object | CONTAINTER container} 
;;;	FROM {authorizationname[, authorizationname, ]}

(defmethod db-create-role ((database access-mixin) name &rest args)
  (declare (ignore args)))
(defmethod db-drop-role ((database access-mixin) name)
  (declare (ignore args)))

(defmethod db-create-user ((database access-mixin) name &rest args)
  (declare (ignore args)))

(defmethod db-grant-role ((database access-mixin) group to &rest args)
  (declare (ignore args)))
(defmethod db-revoke-role ((database access-mixin) group from &rest args)
  (declare (ignore args)))

(defmethod db-grant ((database access-mixin) privilege on to &rest options)
  (declare (ignore options)))
(defmethod db-revoke ((database access-mixin) privilege on from &rest options)
  (declare (ignore options)))
