;;; -*- Mode: Lisp; Package: ystok.odbc -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2006 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; PostgreSQL database - Specialization for ODBC

(in-package :ystok.odbc)

(yl:export-toplevel ()
  #:postgresql-odbc-database)

(defmethod db-lisp-to-c-type ((database postgresql-odbc-database)
                              (lisp-type (eql 'boolean)))
  SQL_C_BIT)

;(defparameter *postgresql-driver* "PostgreSQL ANSI")	; or "PostgreSQL Unicode"

(defmethod db-name-from-spec ((db-type (eql :postgresql)) (connection-spec list))
  (declare (ignore db-type))
  (db-name-from-spec :odbc connection-spec))

(defmethod db-user-name-from-spec ((db-type (eql :postgresql)) (connection-spec list))
  (declare (ignore db-type))
  (db-user-name-from-spec :odbc connection-spec))

(defmethod db-password-from-spec ((db-type (eql :postgresql)) (connection-spec list))
  (declare (ignore db-type))
  (db-password-from-spec :odbc connection-spec))

(defmethod driver-connect ((db odbc-database) (be-type (eql :postgresql))
                           (driver-av-pairs list) &rest args &key dsn)
 ;;; Provide pre-system and -user driver attribute completion for PostgreSQL. Also
  ;; - set database-name, 
  ;; - replace the driver av-pair (:host value) with ("server" value).
  ;; Args: driver-av-pairs - List of (attribute value) pairs.
  (declare (ignore be-type))
  (unless (assoc #1="DRIVER" driver-av-pairs :test #'string-equal)
    (push (list #1#
                (if (getf args :unicode) "PostgreSQL Unicode" "PostgreSQL ANSI"))
          driver-av-pairs))
  (unless (assoc #2="SERVER" driver-av-pairs :test #'string-equal)
    (when-bind (host (assoc "host" driver-av-pairs :test #'string-equal))
      (setq driver-av-pairs (substitute (cons #2# (cdr host)) host
                                        driver-av-pairs))))
  (unless (assoc #3="DATABASE" driver-av-pairs :test #'string-equal)
    (push (list #3# (string dsn)) driver-av-pairs)
    (setq args (remove-properties args '(:dsn))))
  (when-bind (result (apply #'driver-connect db :odbc driver-av-pairs args))
    (unless (database-name db)
      (setf (database-name db)
            (or (cdr-assoc #3# driver-av-pairs :test #'string-equal) dsn)))
    result))

;;; Do we have to make :cascade t the default?

(defmethod db-drop-table ((database postgresql-odbc-database) table &rest args
                          &key (if-does-not-exist :error)
                               cascade) ; (not (getf args :restrict))))
 ;;; Returns [ODBC SQLSTATE 42P01 error 7] instead of 42S02
  ;; Arg: arg ::= CASCADE or RESTRICT (default)?
  (%drop-schema-object :table table database if-does-not-exist
                       (list* :cascade cascade
                              (remove-properties args '(:if-does-not-exist :cascade)))
                       "42P01"))

(defmethod db-drop-view ((database postgresql-odbc-database) view &rest args
                         &key (if-does-not-exist :error))
  (%drop-schema-object :view view database if-does-not-exist
                       (remove-properties args '(:if-does-not-exist))
                       "42P01"))
