;;; -*- Mode: Lisp; Package: ystok.odbc -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2006 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; SQLite3 database - Specialization for ODBC

(in-package :ystok.odbc)

(yl:export-toplevel ()
  #:*sqlite3-driver*
  #:sqlite3-odbc-database
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; As connection specification, pathname accompanied by explicit database-type is allowed:
;;;  (sql:connect pathname :database-type :sqlite3)
;;; In long form, driver specification string can be used in pattern like this:
;;;  (sql:connect nil :driver-av-pairs `((:database ,pathname) (:driver ,odbc:*sqlite3-driver*)))

(defparameter *sqlite3-driver* "SQLite3 ODBC Driver")

(defmethod db-name-from-spec ((db-type (eql :sqlite3)) (connection-spec list))
  (declare (ignore db-type))
  (db-name-from-spec :odbc connection-spec))

(defmethod db-user-name-from-spec ((db-type (eql :sqlite3)) (connection-spec list))
  (declare (ignore db-type))
  (db-user-name-from-spec :odbc connection-spec))

(defmethod driver-connect ((db odbc-database) (be-type (eql :sqlite3))
                           (driver-av-pairs list) &rest args &key dsn)
 ;;; Provide pre-system and -user driver attribute completion for SQLite3;
  ;; also set database-name.
  ;; Args: driver-av-pairs List of (attribute value) pairs
  (declare (ignore be-type))
  (unless (assoc :driver driver-av-pairs :test #'string-equal)
    (push (list :driver *sqlite3-driver*) driver-av-pairs))
  (when (and (pathnamep dsn) (not (assoc :database driver-av-pairs :test #'string-equal)))
    (push (list :database (namestring dsn)) driver-av-pairs)
    (setq args (remove-properties args '(:dsn))))
  (when-bind (result (apply #'driver-connect db :odbc driver-av-pairs args))
    (unless (database-name db)
      (setf (database-name db)
            (or (cdr-assoc :database driver-av-pairs :test #'string-equal)
                (and (pathname dsn) dsn))))
    result))

(defmethod db-create ((db-type (eql 'sqlite3-odbc-database)) pathname &rest args)
  (declare (ignore db-type))
  (apply #'db-create :sqlite3 pathname args))

#|
#+old
(defmethod db-describe-columns ((database sqlite3-mixin) table
                                &key column catalog schema)
 ;;; SQLite3 driver before 0.71 SQLColumns ignores the column argument and returns all.
  (declare (ignore schema))
  (let ((records (with-cursor-handle-results (handle database)
                   (%sql-columns handle (name catalog) nil
                                 (ordinary-argument table "%") nil)))
        (column (ordinary-argument column)))
    (if (and column (string/= column "%"))
        (delete (string column) records :key #'fourth :test-not #'string-equal)
        records)))

#+old
(defmethod db-column-type ((database sqlite3-odbc-database) column table schema
                           &key catalog)
 ;;; Values: 1>Native SQL type specifier, 2> column size, 3> decimal digits, 4> nullable
  ;; SQLite3 driver SQLColumns:
  ;; 1) Ignores the column argument and returns all,
  ;; 2) Adds the length in the parentheses, e.g. "VARCHAR(30)" to TYPE_NAME.
  (loop with name = (string (ordinary-argument column))
        for (column-name type-name column-size decimal-digits nullable)
        in	; 4 COLUMN_NAME, 6 TYPE_NAME, 7 COLUMN_SIZE, 9 DECIMAL_DIGITS, 11 NULLABLE
           (with-cursor-handle-results (handle database :column-numbers '(4 6 7 9 11))
                        (%sql-columns handle
                                      (name catalog)
                                      (if (eq schema :all)
                                          SQL_ALL_SCHEMAS
                                          (ordinary-argument schema))
                                      (ordinary-argument table)
                                      nil)) ;(ordinary-argument column)
        when (string-equal name column-name)
          do (when-bind (position (position #\( type-name :test #'char=))  ; strip length
               (setq type-name (subseq type-name 0 position)))
             (return (values (let ((keyword (intern type-name :keyword)))
                               (cond ((memq keyword '(:char :varchar :binary :varbinary))
                                      (list keyword column-size))
                                     ((memq keyword '(:decimal :numeric :number))
                                      (list* keyword column-size
                                             (and decimal-digits (list decimal-digits))))
                                     (t keyword)))
                             column-size decimal-digits nullable))))
|#
