;;; Some scrap for MS Desktop Text Driver

(in-package :odbc)

(defparameter *text-driver* "Microsoft Text Driver (*.txt; *.csv)")

(defclass text-odbc-database (access-odbc-database) ())

(defmethod db-connect :after ((db odbc-database) dsn &key &allow-other-keys)
  (when (string-equal (db-get-info db SQL_DBMS_NAME) "TEXT")
    (change-class db 'text-odbc-database)))

;;; Take into account the fact that no constraints allowed.
(defmethod output-sql ((expr sql:sql-create-table-exp) (database text-odbc-database))
  (with-slots (name attributes constraints) expr
    (write-string "CREATE TABLE " *sql-stream*)
    (output-sql-identifier name database)
    (write-string " (" *sql-stream*)
    (loop for (arg . rest) on attributes
          unless (and (consp arg) (keywordp (first arg)))
              do (sql-sys::output-column arg database)
          when rest
            do (write-string ", " *sql-stream*))
    (write-char #\) *sql-stream*)))

(defmethod sql-sys::output-column-constraints (constraints
                                               (database text-odbc-database)))


;;; MSDN: Understanding Schema.ini Files
;;; The following registry keys must be set to provide defaults for a new tables in
;;;	\HKLME\SOFTWARE\Microsoft\Jet\3.5\Engines\Text (Or Jet\4.0 correspondingly)
;;; before creating their definitions in Schema.ini.
;;;  CharacterSet="ANSI"		; is default but changed to 1251 in Schema.ini
;;;  Format="TabDelimited"		; default is "Delimited(;)"
;;;  FirstRowHasNames=1 (Boolean) 	; default is Binary 01 -    

;;; The following attributes must be set per table in Schema.ini
;;;  CharacterSet=ANSI		;  1251   Access
;;;  [ColNameHeader=True]
;;;  [Format=TabDelimited]
;;;  DateTimeFormat="yyyy-mm-dd hh:nn:ss"
;;;  DecimalSymbol=.
;;; The below settings are needed only if we write table by means of the text driver.
;;;   MSDN: Indicates the number of decimal digits in the fractional portion of number. 
;;;   Actually: Column precision, i.e. total number of digits in decimal representation
;;;  NumberDigits=15		;    
;;;  CurrencyDigits=4		;  Access
;;;  CurrencySymbol=" "		;  ! -  Access  .
;;;  , ..     
;;;  CurrencyThousandSymbol
;;;  CurrencyDecimalSymbol 

;;; The following registry key must be set on every workstation 
;;;	\HKLME\SOFTWARE\Microsoft\Jet\3.5\Engines\Text (Or Jet\4.0 correspondingly)
;;;  ExportCurrencySymbols=0 (Boolean)	; default is Binary 01 
;;; Though this doesn't help when exporting using Access - use per table CurrencySymbol!

#|
(defvar *sandbox-path* "E:/Projects/Test")
(odbc::%sql-config-data-source odbc::ODBC_CONFIG_SYS_DSN *text-driver*
      "DSN=TextTest"
   ;:CHARACTERSET "ANSI"		;  
   ;:COLNAMEHEADER "FALSE"	;  
   :DEFAULTDIR	(namestring *sandbox-path*)	;"E:\\Projects\\Test"
   :DESCRIPTION "Chivas replication sandbox"
   ;:DRIVERID	27
   :EXTENSIONS	"tab" ;"csv,tab"
   :FIL		"text"
   ;:FILETYPE	"tab"
   ;:FORMAT	"TABDELIMITED"	;  
   :MAXSCANROWS 1	; default=25, 0 means all the file
 )
(setq *sandbox-database* (sql:connect "TextTest"))
    ;:driver-attributes `((:format "TABDELIMITED"))))
    ; :driver-attributes `((:dbq ,(namestring *sandbox-path*)))))
(sql:disconnect :database *sandbox-database*)
(odbc::db-describe-tables *sandbox-database* "%")
(odbc::db-describe-type *sandbox-database* 0)

(sql:create-table [tab2.tab]
  '(([ID] integer)
    ([Text_] (string 64))
    ([Longchar_] (string 300))
    ([Boolean_] boolean)
    ([Double_] double-float)
    ([Single_] single-float)
    ([Currency_] :currency)
    ([DateTime_] timestamp))
  :database *sandbox-database*)

(sql:drop-table [tab2.tab] :database *sandbox-database*)

(setq str (random-string 300))
(sql:insert-records :into [tab.tab]
  :av-pairs `(([id] 1)
              ([Text_] " ")
              ([Longchar_] ,str)
              ([Boolean_] 1)
              ([Double_] 12.34506789)
              ([Single_] 12.345)
              ([Currency_] -54321)
              ([DateTime_] "2001-04-01"))
  :database *sandbox-database*)

(time (dotimes (i 1000)
        (sql:insert-records :into [tab.tab]
           :av-pairs `(([id] ,i)
                       ([Text_] "my-\"-another-'-site")
                       ([Longchar_] ,(cl-user::random-string i))
                       ([Boolean_] 2)
                       ([Double_] 12345.01234567890123)
                       ([Single_] 12.345)
                       ([Currency_] 98.54321)
                       ([DateTime_] "2001-04-01 10:15:23"))
           :database *sandbox-database*)))
;user time    =     15.062
;system time  =     31.609
;Elapsed time =   0:01:45
;Allocation   = 8929936 bytes standard / 6221325 bytes fixlen
;0 Page faults

(time (sql:with-prepared-statement (stmt 
         [insert :into [tab.tab]
                 :av-pairs '(([id] [? :type integer])
                             ([Text_] "my-\"-another-'-site")
                             ([Longchar_] [? :type (string 1000)])
                             ([Boolean_] 2)
                             ([Double_] 12345.01234567890123)
                             ([Single_] 12.345)
                             ([Currency_] 98.54321)
                             ([DateTime_] "2001-04-01 10:15:23"))])
        (dotimes (i 1000)
          (sql:execute-command stmt :values (list i (cl-user::random-string i))))))
; Big differency only in Allocation

(time (with-open-file (stream (concatenate 'string *sandbox-path* "/tab.txt")
                              :direction :output)
        (dotimes (i 1000)
          (prin1 i stream)  (write-char #\Tab stream)
          (prin1 "my-\"-another-'-site" stream)   (write-char #\Tab stream)
          (prin1 (cl-user::random-string i) stream)   (write-char #\Tab stream)
          (prin1 1 stream)  (write-char #\Tab stream)
          (prin1 12345.01234567890123 stream)  (write-char #\Tab stream)
          (prin1 12.345s0 stream)  (write-char #\Tab stream)
          (prin1 98.54321 stream)  (write-char #\Tab stream)
          (prin1 "2001-04-01 10:15:23" stream) (terpri stream))))
;user time    =      1.765
;system time  =      0.000
;Elapsed time =   0:00:02
;Allocation   = 5343256 bytes standard / 5766035 bytes fixlen
;0 Page faults

(time (sql:map-query nil (lambda (&rest args) (print args))
               [select [*] :from [tab.tab]]
               :database *sandbox-database*))
;user time    =      4.218
;system time  =      0.015
;Elapsed time =   0:00:04
;Allocation   = 17563120 bytes standard / 570537 bytes fixlen
;0 Page faults

(time (with-open-file (stream (concatenate 'string *sandbox-path* "/tab.txt")
                              :direction :input)
        (loop for id = (read stream nil nil)
              while id
              for Text_ = (read stream)
              for Longchar_ = (read stream)
              for Boolean_ = (read stream)
              for Double_  = (read stream)
              for Single_  = (read stream)
              for Currency_  = (read stream)
              for DateTime_  = (read stream)
              do (print (list id Text_ Longchar_ Boolean_ Double_ Single_ Currency_
                              DateTime_)))))
;user time    =      4.750
;system time  =      0.000
;Elapsed time =   0:00:05
;Allocation   = 18017880 bytes standard / 691350 bytes fixlen

(sql:select [*] :from [tab.tab])
 |#
