;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2007 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Lispworks system definition

(in-package :cl-user)

;;; In LW5, we have to remove off-the-shelf SQL packages at all.
#-lispworks4 (delete-package "SQL")
#-lispworks4 (delete-package "SQL-COMMON")

;(pushnew :debug *features*)
;(lw:removef *features* :debug)
;(yl:define-debug-kind :ffc :all "YSQL ffc stuff")
;(yl:define-debug-kind :sql :all "YSQL sql stuff")
;(yl:define-debug-kind :odbc :all "YSQL odbc stuff")
;(yl:debug-on :sql)
;(pushnew :Russian *features*)
;(pushnew :ys-product *features*)

;(load #P"PROJECTS:ylib;defsys")
;(lw:compile-system 'YSTOK-LIBRARY :load t)
;(load #P"PROJECTS:ylib;local-time;defsys")
;(lw:compile-system 'LOCAL-TIME :load t)

#+lispworks (toggle-source-debugging #+debug t #-debug nil)

;;; Back-ends types, for which the corresponding functionality is included into
;;; the resulting image or delivered application.
;;; During delivery, control in the delivery script!
#-deliver (pushnew :access *features*)
#-deliver (pushnew :oracle *features*)
#-deliver (pushnew :postgresql *features*)
#-deliver (pushnew :sqlite3 *features*)

(setf (logical-pathname-translations "YSQL")
      ;'(("**;*.*" "projects:;ysql;**;")))
      `(("**;*.*.*" ,(merge-pathnames 
                      (make-pathname :directory '(:relative :wild-inferiors)
                                     :name :wild  :type :wild  :version :newest
                                     ) ;:case :common
                      *load-truename*))))


;(eval-when (:load-toplevel :compile-toplevel :execute)
;  (defconstant +source-only+ nil))

(defsystem YSQL.SQL (:default-pathname "YSQL:sql;"
                     :object-pathname
	#+(and (not ys-product) debug (not lispworks4.3)) "YSQL:bin;debug;open;"
	#+(and ys-product debug (not lispworks4.3))	  "YSQL:bin;debug;"
	#-(or ys-product debug lispworks4.3)		  "YSQL:bin;open;"
	#+(and ys-product (not debug) (not lispworks4.3)) "YSQL:bin;"
	#+lispworks4.3					  "YSQL:bin;lw4.3;")
 :members
 ((YSTOK-LIBRARY	:type :system)
  #+local-time (LOCAL-TIME :type :system)	; as may not be loaded
  "packages"
  "conditions"
  "generics"
  "helpers"
  "expressions"
  "syntax"
  "functional"
  "role-privileges"
  "transactions"
  ("metaobjects"	:features :lispworks)
  ("object-oriented"	:features :lispworks)
  "loop"
  ;"streams"
  ;"recording"
 )
 :rules
 ((:in-order-to :compile "packages"
   (:caused-by (:compile YSTOK-LIBRARY #+local-time LOCAL-TIME))
   (:requires (:load YSTOK-LIBRARY #+local-time LOCAL-TIME)))
  (:in-order-to :load "packages"
   (:requires (:load YSTOK-LIBRARY #+local-time LOCAL-TIME)))
         
  (:in-order-to :compile ("conditions" "generics")
   (:caused-by (:compile "packages"))
   (:requires (:load "packages")))
  (:in-order-to :load ("conditions" "generics")
   (:requires (:load "packages")))
  
  (:in-order-to :compile "helpers"
   (:caused-by (:compile "conditions" "generics"))
   (:requires (:load "conditions" "generics")))
  (:in-order-to :load "helpers"
   (:requires (:load "conditions" "generics")))

  (:in-order-to :compile "expressions"
   (:caused-by (:compile "helpers"))
   (:requires (:load "helpers")))
  (:in-order-to :load "expressions"
   (:requires (:load "helpers")))

  (:in-order-to :compile "syntax"
   (:caused-by (:compile "expressions"))
   (:requires (:load "expressions")))
  (:in-order-to :load "syntax"
   (:requires (:load "expressions")))

  (:in-order-to :compile ("functional" "transactions" "metaobjects" "object-oriented")
   (:caused-by (:compile "syntax" "expressions"))
   (:requires (:load "syntax" "expressions")))
  (:in-order-to :load ("functional" "transactions" "metaobjects" "object-oriented")
   (:requires (:load "syntax" "expressions")))

  (:in-order-to :compile "object-oriented" (:caused-by (:compile "metaobjects"))
   (:requires (:load "functional" "transactions" "metaobjects")))
  (:in-order-to :load "object-oriented" (:caused-by (:load "packages"))
   (:requires (:load "transactions" "metaobjects")))

  (:in-order-to :compile ("loop")
   ;(:caused-by (:compile "functional" "object-oriented"))
   (:requires (:load "functional" "object-oriented")))
  (:in-order-to :load ("loop")
   (:requires (:load "functional" "object-oriented")))

  (:in-order-to :compile "role-privileges" (:requires (:load "functional")))
  (:in-order-to :load    "role-privileges" (:requires (:load "functional")))
))

(defsystem YSQL.FFC (:default-pathname "YSQL:ffc;lispworks;"
                     :object-pathname
	#+debug					"YSQL:bin;debug;"
	#-(or debug lispworks4.3)		"YSQL:bin;"
	#+(and (not debug) lispworks4.3)	"YSQL:bin;lw4.3;")
 :members
 ((YSTOK-LIBRARY :type :system)
  "ffc-package"
  "ffc-lispworks")
 :rules
 ((:in-order-to :compile :all (:caused-by (:compile :previous))
   (:requires (:load :previous)))
  (:in-order-to :load :all
   (:requires (:load :previous))) ))
 ;:finally-do (pushnew :ffc *features*))

(defsystem YSQL.ODBC (:default-pathname "YSQL:odbc;"
                      :object-pathname
	#+debug					"YSQL:bin;debug;"
	#-(or debug lispworks4.3)		"YSQL:bin;"
	#+(and (not debug) lispworks4.3)	"YSQL:bin;lw4.3;")
 :members
 ((YSQL.FFC	:type :system)
  (YSQL.SQL	:type :system)
  "odbc-package"
  "odbc-loader"
  "odbc-basics"
  "odbc-constants"
  "odbc-constants-ext"
  "odbc-ff-interface"
  ("odbc-unicode"	:features :Unicode)
  "odbc-functions"
  ("odbc-local-time"	:features :local-time)
  "odbc-descriptors"
  ;#-pcl ("odbc-streams" :source-only #.+source-only+)
  "odbc-cursor"
  "odbc-sql"
  "odbc-prepared"
  ("odbc-object-oriented" :features :lispworks)
  ("odbc-installer"	:features :win32) )
 :rules
 ((:in-order-to :compile "odbc-package"
   ;(:caused-by (:compile YSQL.FFC YSQL.SQL))
   (:requires (:load YSQL.FFC YSQL.SQL)))
  (:in-order-to :load "odbc-package"
   (:requires (:load YSQL.FFC YSQL.SQL)))

  (:in-order-to :compile ("odbc-package" "odbc-loader") (:caused-by (:compile YSQL.FFC))
   (:requires (:load YSQL.FFC)))
  (:in-order-to :load ("odbc-package" "odbc-loader")
   (:requires (:load YSQL.FFC)))

  (:in-order-to :compile "odbc-basics"
   (:caused-by (:compile "odbc-package"))
   (:requires (:load "odbc-package")))
  (:in-order-to :load "odbc-basics"
   (:requires (:load "odbc-package")))

  (:in-order-to :compile "odbc-constants" (:caused-by (:compile "odbc-basics"))
   (:requires (:load "odbc-basics")))
  (:in-order-to :load "odbc-constants"
   (:requires (:load "odbc-basics")))

  (:in-order-to :compile ("odbc-constants-ext" "odbc-installer")
   (:caused-by (:compile "odbc-constants"))
   (:requires (:load "odbc-constants")))
  (:in-order-to :load ("odbc-constants-ext" "odbc-installer")
   (:requires (:load "odbc-constants")))

  (:in-order-to :compile "odbc-ff-interface"
   (:caused-by (:compile "odbc-package" "odbc-loader"))
   (:requires (:load "odbc-package" "odbc-loader")))
  (:in-order-to :load "odbc-ff-interface"
   (:requires (:load "odbc-package" "odbc-loader")))

  (:in-order-to :compile "odbc-unicode" 
   (:caused-by (:compile "odbc-loader" "odbc-constants-ext"))
   (:requires (:load "odbc-loader" "odbc-constants-ext")))
  (:in-order-to :load "odbc-unicode"
   (:requires (:load "odbc-loader" "odbc-constants-ext")))

  (:in-order-to :compile ("odbc-functions" "odbc-local-time")
   (:caused-by (:compile  "odbc-constants-ext" "odbc-ff-interface"))
   (:requires (:load "odbc-constants-ext" "odbc-ff-interface")))
  (:in-order-to :load ("odbc-functions" "odbc-local-time")
   (:requires (:load "odbc-constants-ext" "odbc-ff-interface")))

  (:in-order-to :compile ("odbc-descriptors")
   (:caused-by (:compile "odbc-functions" "odbc-local-time"))
   (:requires (:load "odbc-functions" "odbc-local-time")))
  (:in-order-to :load ("odbc-descriptors")
   (:requires (:load "odbc-functions" "odbc-local-time")))

  (:in-order-to :compile "odbc-cursor"
   (:caused-by (:compile "odbc-descriptors"))
   (:requires (:load "odbc-descriptors")))
  (:in-order-to :load "odbc-cursor"
   (:requires (:load "odbc-descriptors")))

  (:in-order-to :compile ("odbc-sql" "odbc-prepared" "odbc-object-oriented")
          ;(:caused-by (:compile "odbc-cursor" "odbc-unicode"))
   (:requires (:load "odbc-cursor" "odbc-unicode")))
  (:in-order-to :load ("odbc-sql" "odbc-prepared" "odbc-object-oriented")
   (:requires (:load "odbc-cursor" "odbc-unicode")))
))

(defsystem YSQL.DBMS (:default-pathname "YSQL:dbms;"
                      :object-pathname
	#+debug					"YSQL:bin;debug;"
	#-(or debug lispworks4.3)		"YSQL:bin;open;"
	#+(and (not debug) lispworks4.3)	"YSQL:bin;lw4.3;")
 :members
 ((YSQL.ODBC	:type :system)
  "sequences"

  ("access-odbc"	:features :access)
  ("access-sql" 	:features :access)

  ("oracle-odbc"	:features :oracle)
  ("oracle-sql"		:features :oracle)

  ("sqlite3-odbc"	:features :sqlite3)
  ("sqlite3-sql"	:features :sqlite3)

  ("postgresql-odbc"	:features :postgresql)
  ("postgresql-sql"	:features :postgresql))
 :rules
 ((:in-order-to :compile "sequences"
   (:requires (:load YSQL.ODBC)))
  (:in-order-to :load "sequences"
   (:requires (:load YSQL.ODBC)))

  (:in-order-to :compile ("access-odbc" "oracle-odbc" "postgresql-odbc" "sqlite3-odbc")
   (:caused-by (:compile YSQL.ODBC "sequences"))
   (:requires (:load  "sequences")))
  (:in-order-to :load ("access-odbc" "oracle-odbc" "postgresql-odbc" "sqlite3-odbc")
   (:requires (:load "sequences")))
  
  (:in-order-to :compile "access-sql" (:caused-by (:compile "access-odbc"))
   (:requires (:load "access-odbc")))
  (:in-order-to :load "access-sql"
   (:requires (:load "access-odbc")))

  (:in-order-to :compile "oracle-sql" (:caused-by (:compile "oracle-odbc"))
   (:requires (:load "oracle-odbc")))
  (:in-order-to :load "oracle-sql"
   (:requires (:load "oracle-odbc")))

  (:in-order-to :compile "postgresql-sql" (:caused-by (:compile "postgresql-odbc"))
   (:requires (:load "postgresql-odbc")))
  (:in-order-to :load "postgresql-sql"
   (:requires (:load "postgresql-odbc")))

  (:in-order-to :compile "sqlite3-sql" (:caused-by (:compile "sqlite3-odbc"))
   (:requires (:load "sqlite3-odbc")))
  (:in-order-to :load "sqlite3-sql"
   (:requires (:load "sqlite3-odbc")))
))

(defsystem YSQL (:object-pathname
	#+debug						     "YSQL:bin;debug;"
	#+(and (not debug) (not lispworks4.3) (not Russian)) "YSQL:bin;"
	#+(and (not debug) (not lispworks4.3) Russian
               access postgresql)			     "YSQL:bin;ru;"
	#+(and (not debug) (not lispworks4.3) Russian
               access (not postgresql))			     "YSQL:bin;ru;access"
	#+(and (not debug) (not lispworks4.3) Russian
               (not access) postgresql)			     "YSQL:bin;ru;postgresql"
	#+(and (not debug) lispworks4.3 (not Russian))	     "YSQL:bin;lw4.3;"
	#+(and (not debug) lispworks4.3 Russian)	     "YSQL:bin;lw4.3;ru;")
 :members
 ((YSQL.SQL	:type :system)
  (YSQL.DBMS	:type :system)
  ("utils/localization"		:features :ys-product)
  ;("utils/schema-utils"	:features :ys-product)
  ("utils/sql-utils"		:features :ys-product)
  ("utils/sql-widgets"		:features (and :ys-product :ywidgets))
  ("utils/schema-systema"	:features :ys-product) )
 :rules
 ((:in-order-to :compile "localization" (:requires (:load YSQL.SQL)))
  (:in-order-to :load    "localization" (:requires (:load YSQL.SQL)))

  (:in-order-to :compile "sql-utils" (:requires (:load YSQL.DBMS)))
  (:in-order-to :compile ("sql-utils" "sql-widgets")
   (:caused-by (:compile "localization"))
   (:requires (:load "localization")))

  (:in-order-to :load ("sql-utils")
   (:caused-by (:load YSQL.DBMS "localization"))
   (:requires (:load YSQL.DBMS "localization")))

  (:in-order-to :compile ("sql-widgets")
   (:caused-by (:compile "sql-utils"))
   (:requires (:load "sql-utils")))
  (:in-order-to :load ("sql-widgets")
   (:caused-by (:load "sql-utils"))
   (:requires (:load "sql-utils")))

  (:in-order-to :compile ("schema-systema") (:requires (:load "sql-widgets")))
  (:in-order-to :load ("schema-systema") (:requires (:load "sql-widgets")))
))


#+ys-product (ys:defmodule :ffc (1 0) :depends-on :ylib)
#+ys-product (ys:defmodule :sql (1 0) :depends-on (:ylib :local-time))
#+ys-product (ys:defmodule :odbc (1 0 :abbrev "odb") :depends-on (:ffc :sql))
#+ys-product (ys:defmodule :sql-utils (1 0 :abbrev "squ") :depends-on (:odbc))

#||
(lw:compile-system 'YSQL :load t :force nil)
(lw:load-system 'YSQL :force t)
(compile-file-if-needed #P"YSQL:dbms;oracle-odbc" :load t
  :output-file #+(and (not lispworks4.3) (not Russian)) "YSQL:bin;dbms;oracle-odbc.fsl"
               #+(and (not lispworks4.3) Russian)	"YSQL:bin;ru;dbms;oracle-odbc.fsl")

(compile-file-if-needed #P"YSQL:dbms;oracle-sql" :load t
  :output-file #+(and (not lispworks4.3) (not Russian)) "YSQL:bin;dbms;oracle-sql.fsl"
               #+(and (not lispworks4.3) Russian)	"YSQL:bin;ru;dbms;oracle-sql.fsl")
(sql:enable-sql-reader-syntax)
(sql:disable-sql-reader-syntax)
||#
