;;; Collection of example commands tested in LWW 4.4 against MS Excel

(lw:load-system :Ystok-ODBC)

(defvar *excel-driver* "Microsoft Excel Driver (*.xls)")

;(use-package :sql)

(sql:enable-sql-reader-syntax)
(sql:disable-sql-reader-syntax)

(sql:initialize-database-type :database-type :odbc)

(sql:connect nil)
(setq e (sql:connect "excel-test")) ; SQL_TXN_CAPABLE = SQL_TC_NONE
(setq e (sql:connect "mems"))
(setq e (sql:connect "portcosts"))
(setq e (sql:connect nil :if-exists :warn-old ; via DriverConnect
         :driver-av-pairs `((:driver ,*excel-driver*)
                            (:dbq "E:\\Projects\\Maersk\\Chivas\\sites\\cph\\mems.xls")
                            ;("FIRSTROWHASNAMES" 1) ;- is not allowed
                            ;("DRIVERID" 790)	; Excel 97
                            ;("READONLY" :true)
                            ;("MAXSCANROWS" 8)	; in [0..16]; defaults to 8; 0 means all
                         )))

(sql:disconnect :database e)
(sql-sys::disconnect-all-databases)
(sql:connected-databases)

(sql:start-sql-recording)
(sql:delete-sql-stream *standard-output* :type :all)
(sql:start-sql-recording :type :result)
(setq stm (open #P"sql.log" :direction :output :if-exists :append))
(sql:add-sql-stream stm)
(sql:list-sql-streams)
(sql:delete-sql-stream stm)
(close stm)
(sql:stop-sql-recording :type :all)


(odbc::%sql-free-handle :hdbc (odbc::hdbc db))
(odbc::%sql-free-handle :henv (odbc::henv db))
(setf (odbc::henv db) nil)

;;; Environment attributes
(odbc::%sql-get-attribute :henv (odbc::henv db) odbc::SQL_ATTR_CONNECTION_POOLING)
(odbc::%sql-get-attribute :henv (odbc::henv db) odbc::SQL_ATTR_ODBC_VERSION)
;(odbc::%sql-get-attribute :henv (odbc::henv db) odbc::SQL_ATTR_OUTPUT_NTS)

;;; Connection attributes
(odbc::%sql-get-attribute :hdbc ffc:+sql-null-handle+ odbc::SQL_ATTR_TRACEFILE)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc sql:*default-database*) odbc::SQL_ATTR_CURRENT_CATALOG)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc db) odbc::SQL_ATTR_QUIET_MODE)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc db) odbc::SQL_ATTR_ACCESS_MODE) ;SQLUINTEGER
(odbc::%sql-get-attribute :hdbc (odbc::hdbc db) odbc::SQL_ATTR_TXN_ISOLATION)
(odbc::%sql-get-attribute :hdbc (odbc::handle e) odbc::SQL_ATTR_AUTOCOMMIT)
(odbc::%sql-get-attribute :hdbc (odbc::handle e) odbc::SQL_ATTR_CONNECTION_DEAD)
;=> [ODBC state HY092 error 86]
;   [Microsoft][ODBC Excel Driver]Invalid attribute/option identifier 

;;; Diagnostics
(multiple-value-list
 (odbc::%sql-get-diag-record :hdbc (odbc::handle sql:*default-database*) 1))

(odbc::%sql-get-diag-field :hdbc (odbc::handle sql:*default-database*)
                           odbc::SQL_DIAG_RETURNCODE)
(odbc::%sql-get-diag-field :hdbc (odbc::handle sql:*default-database*)
                           odbc::SQL_DIAG_NUMBER)
(odbc::%sql-get-diag-field :hdbc (odbc::handle sql:*default-database*)
                           odbc::SQL_DIAG_SQLSTATE)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc sql:*default-database*) 34)
(sql:open-cursor [select [abracadabra] :from [tab]])
(sql:execute-command "CREATE VIEW abracadabra ON Non_existing")
(sql:execute-command "DROP VIEW abracadabra")

;;; CATALOG Functions
;;; get-info
(odbc::%get-info-result-type odbc::SQL_DBMS_NAME)
(odbc::db-get-info sql:*default-database* odbc::SQL_SERVER_NAME) ;=> "EXCEL"
(odbc::db-get-info sql:*default-database* odbc::SQL_DBMS_NAME) ;=> "EXCEL"
(odbc::db-get-info sql:*default-database* odbc::SQL_DBMS_VER)
; => "08.00.0000"  - Excel version
(odbc::%sql-get-info (odbc::hdbc db) odbc::SQL_PROCEDURES)
(odbc::%sql-get-info (odbc::hdbc sql:*default-database*) odbc::SQL_ODBC_VER)
; Win2K => "03.52.0000" 

(odbc::db-get-info sql:*default-database* odbc::SQL_PROCEDURES)
; Access,Oracle => "Y" (Access,Oracle)
(odbc::db-get-info sql:*default-database* odbc::SQL_DRIVER_ODBC_VER) 
; Access => "03.51" (Jet 4) or "02.50" (Jet 3.5)
(odbc::db-get-info sql:*default-database* odbc::SQL_DRIVER_VER)
; Excel => "04.00.6200" (Jet 4) or ? (Jet 3.5)
(odbc::db-get-info sql:*default-database* odbc::SQL_TXN_CAPABLE)
; Access => 2 (SQL_TC_ALL) ; can contain DDL statements and DML statements in any order
(odbc::db-get-info sql:*default-database* odbc::SQL_TXN_ISOLATION_OPTION)
; Access => 2 (SQL_TXN_READ_COMMITTED)
(odbc::db-get-info sql:*default-database* odbc::SQL_DEFAULT_TXN_ISOLATION) ; => 2
(odbc::db-get-info sql:*default-database* odbc::SQL_MULTIPLE_ACTIVE_TXN)
; Access => "Y" supports more than one active transaction at the same time on more than one connection in the same environment

(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_NAME)
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_USAGE)
;=> 5 (SQL_CU_)= 1(DML_STATEMENTS) + 4(TABLE_DEFINITION)
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_TERM) ; => "Workbook"
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_LOCATION)
; Access,Excel => 1 SQL_CL_START
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_NAME_SEPARATOR)
; Access,Excel => "." 
; Text => "\\"
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_CATALOG_NAME_LEN)
; Access,Excel => 260

(odbc::db-get-info sql:*default-database* odbc::SQL_SCHEMA_USAGE)	;=> 0
(odbc::db-get-info sql:*default-database* odbc::SQL_SCHEMA_TERM)	;=> ""
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_SCHEMA_NAME_LEN);=> 0

(odbc::db-get-info sql:*default-database* odbc::SQL_IDENTIFIER_QUOTE_CHAR);=> "`"
(odbc::db-get-info sql:*default-database* odbc::SQL_SPECIAL_CHARACTERS)
; Access,Excel => "~@#$%^&*_-+=\\}{\"';:?/><,.![]|"
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_TABLE_NAME_LEN)
; Access,Excel => 64

(odbc::db-get-info sql:*default-database* odbc::SQL_GETDATA_EXTENSIONS)
; Access => 15 = SQL_GD_ANY_COLUMN | SQL_GD_ANY_ORDER | SQL_GD_BLOCK | SQL_GD_BOUND
(odbc::db-get-info sql:*default-database* odbc::SQL_PARAM_ARRAY_ROW_COUNTS)
; Access => 2 SQL_PARC_NO_BATCH - only one row count available
(odbc::db-get-info sql:*default-database* odbc::SQL_PARAM_ARRAY_SELECTS)
; Access => 2 SQL_PAS_NO_BATCH - There is only one result set available

(odbc::db-get-info sql:*default-database* odbc::SQL_OJ_CAPABILITIES)
; Access => 83 = SQL_OJ_LEFT(1)+SQL_OJ_RIGHT(2)+SQL_OJ_NOT_ORDERED(16)+
;              + SQL_OJ_ALL_COMPARISON_OPS(64)

(odbc::db-get-info sql:*default-database* odbc::SQL_CURSOR_COMMIT/ROLLBACK_BEHAVIOR)
; Access => 1 = (SQL_CB_CLOSE) 
; Closes cursors but preserve plans for prepared stmts on manual commit/rollback
; Nevertheless, in autocommit mode Access acts like SQL_CB_PRESERVE, i.e. does not
; close a cursor after SQLSetPos()
(odbc::db-get-info sql:*default-database* odbc::SQL_CURSOR_SENSITIVITY)
; Access => 0 SQL_UNSPECIFIED
; It is unspecified whether cursors on the statement handle make visible the changes
; made to a result set by another cursor within the same transaction.
(odbc::db-get-info sql:*default-database* odbc::SQL_ROW_UPDATES)
; Access => "N", cannot detect updates to data
; Oracle => "Y"
(odbc::db-get-info sql:*default-database* odbc::SQL_BOOKMARK_PERSISTENCE)
; Access => 80 = SQL_BP_UPDATE(#x00000010) + SQL_BP_SCROLL(#x00000040)
(odbc::db-get-info sql:*default-database* odbc::SQL_SCROLL_OPTIONS)
; Access => 19 = SQL_SO_FORWARD_ONLY | SQL_SO_STATIC | SQL_SO_KEYSET_DRIVEN
(odbc::db-get-info sql:*default-database* odbc::SQL_SCROLL_CONCURRENCY)
; Access => 11 = SQL_SCCO_READ_ONLY(1) + SQL_SCCO_LOCK(2) + SQL_SCCO_OPT_VALUES(8)

(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_SENSITIVITY)
; Access => 7 = SQL_SS_ADDITIONS(1) + SQL_SS_DELETIONS(2) + SQL_SS_UPDATES(4)
; 	Indicates whether a cursor is sensitive to changes made by another cursor.

(odbc::db-get-info sql:*default-database* odbc::SQL_DYNAMIC_CURSOR_ATTRIBUTES1) ; => 0
(odbc::db-get-info sql:*default-database* odbc::SQL_DYNAMIC_CURSOR_ATTRIBUTES2) ; => 0
(odbc::db-get-info sql:*default-database* odbc::SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1)
; Access => 1 SQL_CA1_NEXT
(odbc::db-get-info sql:*default-database* odbc::SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2)
; Access => 1 SQL_CA2_READ_ONLY_CONCURRENCY
;	Whether cursors can detect their own updates, deletes, and inserts. 
(odbc::db-get-info sql:*default-database* odbc::SQL_KEYSET_CURSOR_ATTRIBUTES1)
; Access => 990799 = #xF1E4F (SQL_CA1_)
;	= NEXT(1) + ABSOLUTE(2) + RELATIVE(4) + BOOKMARK(8)
;	+ LOCK_NO_CHANGE
;	+ POS_POSITION + POS_UPDATE + POS_DELETE + POS_REFRESH
;	+BULK_ADD+BULK_UPDATE_BY_BOOKMARK+BULK_DELETE_BY_BOOKMARK+BULK_FETCH_BY_BOOKMARK
(odbc::db-get-info sql:*default-database* odbc::SQL_KEYSET_CURSOR_ATTRIBUTES2)
; Access => 127 = #x7F (SQL_CA2_)
;	= READ_ONLY_CONCURRENCY(1) + LOCK_CONCURRENCY(2)
;	+ OPT_ROWVER_CONCURRENCY(4) + OPT_VALUES_CONCURRENCY(8)
;	+ SENSITIVITY_ADDITIONS(16)+SENSITIVITY_DELETIONS(32) + SENSITIVITY_UPDATES(64)
(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_CURSOR_ATTRIBUTES1)
; Access => 528911 = #x8120F (SQL_CA1_)
;	= NEXT(1) + ABSOLUTE(2) + RELATIVE(4) + BOOKMARK(8)
;	+ POS_POSITION + POS_REFRESH + BULK_FETCH_BY_BOOKMARK
(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_CURSOR_ATTRIBUTES2)
; Access => 1 SQL_CA2_READ_ONLY_CONCURRENCY

(odbc::db-get-info sql:*default-database* odbc::SQL_NUMERIC_FUNCTIONS)
(odbc::db-get-info sql:*default-database* odbc::SQL_SQL92_NUMERIC_VALUE_FUNCTIONS)
(odbc::db-get-info sql:*default-database* odbc::SQL_SQL_CONFORMANCE)
;=> 1 = SQL_SC_SQL92_ENTRY (CAST is not supported)

(format nil "#x~x" 528911)

(defun list-info (database stream)
  (let ((constants (remove-if (complement #'constantp) (apropos-list "SQL_" :odbc))))
    ;(loop for key being the hash-keys of odbc::*get-info-result-types*
    ;          using (hash-value value)
    (loop for c in (sort constants #'string-lessp)
          for value = (ignore-errors (odbc::db-get-info database (symbol-value c)))
          when value
          do (format stream "~&~A -> ~S" c value))))

;(list-info db t)

(sql:list-tables) ;=> nil
(sql:list-tables :database e :schema nil) ; Optional feature
;; VERIFY: Calling SQLTables with parameter TableType = null results in tables with two types: TABLE and SYSTEM TABLE.
(odbc:db-describe-tables e "%" :table-type "SYSTEM TABLE") ;=>nil
(odbc:db-describe-tables e "%" :table-type "SYNONYM") ;=>nil
(odbc:db-describe-tables e "%" :table-type "TABLE") ;=> tables defined within worksheets
;(("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\tico_portcosts" NIL "mytable" "TABLE" NIL))
;("TABLE_CAT" "TABLE_SCHEM" "TABLE_NAME" "TABLE_TYPE" "REMARKS") 
(sql:table-exists-p "Sheet1$" :schema nil)

;;;    -  !
(odbc:db-describe-tables e "%" :catalog "E:\\Dima\\TENNIS\\Organs\\Rtt\\ms.xls")

(sql:list-attributes "Sheet1$" :database e)
(odbc::db-describe-columns e "Sheet1$")
(("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "keyfig" 12 "VARCHAR" 255 510 NIL NIL 1 NIL NIL 12 NIL 510 1 "YES" 1)
 ("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "Keyfig_name" 12 "VARCHAR" 255 510 NIL NIL 1 NIL NIL 12 NIL 510 2 "YES" 2)
 ("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "VESSEL" 12 "VARCHAR" 255 510 NIL NIL 1 NIL NIL 12 NIL 510 3 "YES" 3)
 ("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "Vessel_name" 12 "VARCHAR" 255 510 NIL NIL 1 NIL NIL 12 NIL 510 4 "YES" 4)
 ("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "Month" 12 "VARCHAR" 255 510 NIL NIL 1 NIL NIL 12 NIL 510 5 "YES" 5)
 ("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "month_name" 12 "VARCHAR" 255 510 NIL NIL 1 NIL NIL 12 NIL 510 6 "YES" 6)
 ("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "this" 8 "NUMBER" 53 8 NIL 2 1 NIL NIL 8 NIL NIL 7 "YES" 7)
 ("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "last" 8 "NUMBER" 53 8 NIL 2 1 NIL NIL 8 NIL NIL 8 "YES" 8)
 ("E:\\Projects\\MAERSK\\Chivas\\Sites\\Cph\\mems" NIL "Sheet1$" "diff" 8 "NUMBER" 53 8 NIL 2 1 NIL NIL 8 NIL NIL 9 "YES" 9)) 

(sql:attribute-type "Datetime_" "tab" :database a)
(sql:attribute-type "Currency_" "tab" :database a)
(sql:attribute-type [EmployeeID] [tblEmployee])

(odbc::%sql-func-exists a odbc::SQL_API_SQLGETTYPEINFO)
(odbc::%initialize-type-info a)
(odbc::db-describe-type e 0) ; =>
(("LOGICAL"  -7   1 NIL NIL NIL 0 0 2 NIL 0 NIL NIL 0 0 -7 NIL NIL NIL)	; a la BIT
 ("CURRENCY"  2  19 NIL NIL NIL 1 0 2   0 1 0 NIL 4 4 2 NIL 10 NIL)
 ("NUMBER"    8  53 NIL NIL NIL 1 0 2   0 0 0 NIL NIL NIL 8 NIL 2 NIL)	; a la DOUBLE
 ;; case-sensitive!
 ("VARCHAR"  12 255 "'" "'" NIL 1 1 3 NIL 0 NIL NIL NIL NIL 12 NIL NIL NIL)
 ("DATETIME" 93  19 "#" "#" NIL 1 0 2 NIL 0 NIL NIL 0 0 9 3 NIL NIL))

(odbc::%get-type-descriptor a odbc::SQL_TYPE_DATE)	; => nil
(odbc::%get-type-descriptor a odbc::SQL_TYPE_TIMESTAMP)

(ODBC::%sql-get-functions (odbc::handle a) odbc::SQL_API_SQLDESCRIBEPARAM)
(odbc::%sql-func-exists a odbc::SQL_API_SQLDESCRIBEPARAM) ; Access => nil
(odbc::%sql-func-exists a odbc::SQL_API_SQLPARAMDATA) 	  ; Access => t
(odbc::%sql-func-exists a odbc::SQL_API_SQLTABLES)
(odbc::%sql-func-exists a odbc::SQL_API_SQLBULKOPERATIONS) ; Access => t

(odbc::%sql-func-exists a odbc::SQL_API_SQLSPECIALCOLUMNS) ; Access => t
(odbc::db-describe-special-columns a odbc::SQL_BEST_ROWID nil nil "Bills" odbc::SQL_SCOPE_CURROW)
; => ((0 "s_GUID" -11 "GUID" 36 16 NIL 1)
;;    (0 "BillID" 4 "COUNTER" 10 4 0 1))  ; SQL_SCOPE_CURROW ... SQL_PC_NOT_PSEUDO
(odbc::db-describe-special-columns a "tab" :type odbc::SQL_ROWVER
                                           :scope odbc::SQL_SCOPE_CURROW) ;=>nil
(odbc::db-describe-special-columns a "Table2" :type odbc::SQL_BEST_ROWID
                                              :scope odbc::SQL_SCOPE_TRANSACTION)
; => ((0 "a2" 4 "INTEGER" 10 4 0 1) (0 "a1" 12 "VARCHAR" 50 50 NIL 1)) 

(odbc::%sql-func-exists a odbc::SQL_API_SQLPRIMARYKEYS) ; Access => nil
(odbc::db-describe-primary-keys a nil nil "tab") ;   MS Access

(odbc::%sql-func-exists a odbc::SQL_API_SQLFOREIGNKEYS) ; Access => nil
(odbc::db-describe-foreign-keys a "degrees" "physicians")

(odbc::%sql-func-exists a odbc::SQL_API_SQLSTATISTICS)  ; Access => t
(odbc::db-describe-statistics a nil nil "tab")
; => (("E:\\Lisp\\Test\\sample" NIL "tab" NIL "tab" NIL 0 NIL NIL NIL 58 2 NIL)
      ("E:\\Lisp\\Test\\sample" NIL "tab" 0 "tab" "PrimaryKey" 3 1 "id" "A" 21 1 NIL)
      ("E:\\Lisp\\Test\\sample" NIL "tab" 1 "tab" "id" 3 1 "id" "A" 1 1 NIL))


;;; DDL

(setq e [create-table [Table1]
  `(([id] integer :primary-key)
    ([tab-id] integer         ; :default 2 - Jet 3.5 not supported
      :constraint [table1-id2]
      :references [tab] [id]) ; :on-update :cascade - Jet 4 only
    ;([text-moo] (string 32) :default "Moo"
    ;                                 :check ,[and [> [id] 0] [>= [text-moo] "m%"]]))
    ([bin] bit-vector)
    ([l] integer))
   :constraints
   '(([fk_table1_l_tab_id] :foreign-key ([l]) :references [tab] [id])
    ; (:check ,[> [tab-id] 5]) ; MSA not supported
    )])
(sql:drop-table [Table1])
(sql:create-table [Table1]
  `(([id] integer)
    ([str] string))
)
(sql:insert-records :into [Table1] :attributes '([id] [str])
                :values (list 2 (random-string 30))))

(setq b (make-array 3 :element-type '(unsigned-byte 8)
            :initial-contents `(,(char-code #\2) ,(char-code #\1) ,(char-code #\0))))
(sql:insert-records :into [Table1] :attributes '([id] [bin])
                    :values (list 210 b))
(sql:select [id] [bin] :from [table1])
(sql:select [id] [the '(vector unsigned-byte) [bin]] :from [table1])
(sql:select [id] [bin :type string] :from [table1])

(sql:create-table 'ThirdTable2 (FirstName CHAR(10), LastName LONGTEXT);")
(execute-command "CREATE TABLE ThirdTable4 (s short REFERENCES ThirdTable3(s) ON UPDATE CASCADE );")

(sql-sys::alter-table [Table1] :add '(([id4] integer) ([id5] integer)))
(sql-sys::alter-table [Table1] :drop '([id3] [id4] [id5]))
;; Jet 3.5: add and drop don't work together
(sql-sys::alter-table [Table1] :add '(([id3] integer)) :drop '([id2])) 
(sql-sys::alter-table [Table1] :drop '((:constraint [ct_tab_long_table1_l])))

(sql-sys::alter-table [Table1]
  :add '(([id3] integer)
         (:constraint [fk_id3] :foreign-key ([id3]) :references [tab] [id])))
(sql-sys:alter-table [Table1] :drop '((:constraint [fk_id3]) [id3]))
(sql-sys:alter-table [Table1] :modify '(([l] integer))); (string 25))))


(sql:create-index [id2d2_ix] :on [Table1] :attributes '([id2] [d2 :desc]) :unique t
                 :with :ignore-null))
(sql:drop-index [id2d2_ix] :on [Table1])

(sql:create-view [v3] :as [select [id] [text_] :from [tab]])
(odbc::db-describe-tables a nil nil "v3" nil)
(sql:drop-view [v3]) 	; Jet 3.5 is not supported
(sql:commit)


;;; DML
(defun random-string (size)
  (let ((str (make-string size :element-type 'base-character)))
    (dotimes (i size)
      (setf (char str i) (code-char (+ 97 (random 26)))))
    str))

(sql:insert-records :into [sheet1$]
                    :values "abcd")

(sql:insert-records :into [sheet1$]
                    :values '("abcd" ""))

(dotimes (i 10)
  (sql:insert-records :into [Table1] :attributes '([id] [str])
                      :values (list i (random-string 30))))
(sql:update-records [Table1] :attributes [str] :values "My string"
                             :where [= [id] 2])

(sql:delete-records :from [Table1] :where [= [id] 8])

(sql:commit)
(sql:insert-records :into [tab2]
                    :attributes '([id] [text2_])
                    :values (list 10 "  "))
(sql:insert-records :into [tab2]
                    :av-pairs '(([id] 11) ([text2_] "  ")))
(sql:insert-records :into [tab2]
                    :attributes '([id] [text2_])
                    :query [SELECT ["tt" ID] ["tt" TEXT_]
                                   :FROM [tab "tt"]
                                   :where [< ["tt" ID] 15]])
(sql:insert-records :into [tab2] :attributes [id] :values 40)
(setq n 10)
(sql:delete-records :from [Tab2] :where [>= [ID] n]])
(sql:update-records [tab2] :attributes '([text2_]) :values " "
                           :where [>= [ID] 5])
(sql:update-records [tab2] :attributes [text2_] :values " "
                           :where [= [ID] 5])

;;; Queries
;;  case-sensitive!
(sql:select [*] :from [sheet1$] :where [like [vessel-name] "%agnete%"]) ;=> nil!
(sql:select [*] :from [sheet1$] :where [like [vessel-name] "%Agnete%"])
(sql:select [*] :from [sheet1$] :where [= [month] "01"])
(sql:select [*] :from [sheet1$] :where [= [vessel-name] "Agnete Maersk"])
(sql:select [*] :from [sheet1$] :limit 1)

(sql:select [*] :from (sql:sql-expression :table "$"))

(sql:query [select [*] :from [regions] :limit 3])
(sql:query [select [*] :from [tab10]])
(sql:query [select [id] :as [i] [first [Text_]] :from [tab] :group-by [id]
                   :order-by `([id] :desc ,[first [Text_]])])
(sql:query [select [id :as i] [Text_] :from [tab]
                   :order-by '([id :desc] [Text_])])
(sql:query "SELECT ID FROM tab" :flatp t)

(sql:select [text_] :from [tab] :flatp t :distinct t)
(sql:query  [select [id] [text_] :from [tab] :order-by [id :desc] :flatp nil])
(sql:map-query 'nil (lambda (id text) (format t "~&id=~S txt=~S" id text)
                                       id)
               [select [id] [text_] :from [tab] :order-by [id :desc] :flatp nil])

(time (sql:query "SELECT Text_,Long_,Single_,Memo_ FROM tab" :database a))
(sql:query "SELECT TOP 5 ID,Boolean_,DateTime_,Byte_,Short_,Double_ FROM tab" :database a)
(sql:do-query ((txt b c d m  &rest rest)
               "SELECT TOP 2 Text_,Byte_,Currency_,DateTime_,Memo_ FROM tab"
               :database a)
  (format t "~&txt=~S:~S b=~S:~S c=~S:~S d=~S m=~S rest=~S" 
    txt (type-of txt) b (type-of b) c (type-of c) (and d (format-time d)) m rest))
;;; LWW:   double-float!
;;;a=640000:FIXNUM b=12.34:SINGLE-FLOAT c=5555.66666d0:DOUBLE-FLOAT rest=NIL
;;;a=536870913:BIGNUM b=43.21:SINGLE-FLOAT c=6666.5555d0:DOUBLE-FLOAT rest=NIL
(sql:do-query ((f1) "SELECT Ole_ FROM tab;" :database a)
  (format t "~&1=~S:~S" 
    f1 (type-of f1)))

(sql:do-query ((a b &rest rest)
           [select [*] :from [tab2]])
          (print (list a b)))


(do-query ((a1 a2) "SELECT a1,a2 FROM Table2")
  (format t "~&1=~S 2=~S" a1 a2))

(do-query ((a b) "SELECT Text_,Text2_ FROM outer_join;")
  (format t "~&a=~S:~S b=~S:~S" 
    a (type-of a) b (type-of b)))


(select [count [id]] :from [tab2])
(setq e [cast [a] :as :integer])

;; Local-time
(setq rs (sql:query "SELECT TOP 5 ID,DateTime_ FROM tab" :database a))

(with-transaction
  (insert-records :into [foo bar] :values '(17 "jazz")))
(with-transaction
  (insert-records :into [foo baz] :values '(17 "Pacito d'Riveira")))

(select [id] :from [foo bar-baz])

(select [count [id]] :from [foo bar-baz])

(do-query ((a b)
           [select [*] :from [foo bar]])
          (print (list a b)))

(execute-query "select id from foo.bar union select id from foo.bar order by id desc")

(select-union [select id :from [foo bar]]
              [select id :from [foo baz]]
              :order-by '((id desc)))

(select-union [select [conc [key] ", Kategorie: " [category]]
                      :from [lexicon keys]
                      :where [< [rownum] 3]]
              [select [conc [category] ", Lemma: " [key]]
                      :from [lexicon keys]
                      :where [< [rownum] 3]]
              :flatp t)


(select [conc [key] ", Kategorie: " [category]]
        :from [lexicon keys]
        :where [<= 3 [id] 9]
        :flatp t)

(select [ceil 3.3] :from [sys dual])
(select [sysdate] :from [sys dual])

(let ((example 'example) (id 'id) (ex-id 'ex-id))
  (declare (ignorable example id ex-id))
  (select [keys key] [example]
          :from `([lexicon keys]
                  ,[select `((,[lpad [?example] 50 "+"] [example])
                             (,[?id] [ex-id]))
                           ;(list [?example] (list [?id] [?ex-id]))
                           :from [lexicon examples]
                           :where [> [id] 30000]])
          :where [and [= [keys id] [ex-id]]
                      [like [keys key] "hautesk%"]]))

(update-records [foo bar]
                :av-pairs '(([id] [+ [id] +13]))
                :attributes '([text])
                :values '([select [text] :from [foo bar]
                                  :where [= [id] 7]])
                :where [> [id] 14])

(update-records [foo bar]
                :av-pairs '(([id] [+ [id] +13])
                            ([text] [select [text] 
                                            :from (([foo bar] [zup]))
                                            :where [= [zup id] [bar id]]
                                            ]))
                :where [> [id] 14])

;;; Cursors
(odbc::%sql-get-attribute :hdbc (odbc::handle ystok.sql.sys:*default-database*) odbc::SQL_ATTR_AUTOCOMMIT)
(odbc::%sql-set-attribute :hdbc (odbc::handle ystok.sql.sys:*default-database*) odbc::SQL_ATTR_AUTOCOMMIT odbc::SQL_AUTOCOMMIT_ON)
(odbc::%sql-get-attribute :hstmt (odbc::handle aq) odbc::SQL_ATTR_CONCURRENCY)

(setq eq (sql:open-cursor [select [Port] [Vessel] :from [sheet1$]]))
(setq eq (sql:open-cursor [select [Port] [Vessel :type (string 3)] :from [sheet1$]]))
(setq eq (sql:open-cursor [select [id] [text_] [datetime_] [double_] ;[memo_] [ole_]
                                  :from [tab] :distinct nil]
                          :cursor-type :keyset-driven
                          :concurrency :optimistic ;:lock
                          :rowset-size 5
                          :start-column 0
                          :database a)) 
(sql:fetch-rows eq :count 4 :flatp nil)
(sql:fetch-item aq 1 :current t)
(sql:fetch-item aq :bookmark :current t)
(setq l (make-list 4))
(sql:fetch-into eq l)
(sql:fetch-into aq l :columns '([text_] "memo_") :start 2 :current t)
(sql:fetch-into aq l :start-column 2 :start 0 :current t)
(sql:fetch-rows  aq :count 4 :flatp nil :ignore-columns '(:bookmark))
(setq vo (make-array 4))
(sql:fetch-into aq vo :columns '([text_]) :start 2 :current t)

(odbc::current-row-status aq)
(odbc::set-cursor-position aq :prior)
(odbc::set-cursor-position aq :first)
(odbc::set-cursor-position aq :absolute :offset 0)
(odbc::set-cursor-position aq :relative :offset 1)
(odbc::set-cursor-position aq :bookmark :offset 515)
(odbc::set-cursor-position aq :bookmark :offset #(4 2 0 0))
(sql:fetch-rows aq :count 1)
(print-row-status-array aq)
(sql:current-row-status aq)
(sql:current-row-number aq)

(odbc::%sql-fetch (odbc::handle aq))
(odbc::%sql-fetch-scroll (odbc::handle aq) :relative :offset 1)

(odbc::%sql-refresh-record (odbc::handle aq) 0)
(odbc::%sql-lock-record (odbc::handle aq) 1 :exclusive)
(odbc::%sql-get-attribute :hstmt (odbc::handle aq) odbc::SQL_ATTR_ROW_NUMBER)
;; row number in entire result set, not in rowset =>  0,  
(odbc::%sql-get-attribute :hstmt (odbc::handle aq) odbc::SQL_ATTR_CONCURRENCY)

(sql:update-records aq :attributes '(text_ [datetime_] [double_]) ;[memo_] [ole_])
                       :values `(,(format nil "Updated by cursor ~D" (random 100))
                                 ,(get-universal-time)
                                 666.777
                                 ;" -   "
                                 );nil)
                       );:where 525)
(sql:update-records aq :attributes '(text_ [datetime_] [double_]) ;[memo_] [ole_])
                       :values `(,(format nil "Updated by cursor ~D" (random 100))
                                 ,(local-time:current-date)
                                 777.88
                                 ;" -   "
                                 );nil)
                       :where 513)
(sql:update-records aq :av-pairs '((text_ "113-701111")) :where 520)
(sql:update-records aq :attributes 'text_ :values "113-70" :where 520)

(sql:insert-records :into aq :attributes '(text_ [datetime_] [double_])
                       :values `(,(format nil "Added by cursor ~D" (random 100))
                                 ,(local-time:current-timestamp 'local-time:utc-time)
                                 888.999))
                                 
(sql:insert-records :into aq :attributes '(text_ [datetime_] [memo_] [ole_])
                       :values `(,(format nil "Added by cursor ~D" (random 100))
                                 ,(local-time:current-timestamp 'local-time:utc-time)
                                 " -   bulk"
                                 nil))

(sql:delete-records :from aq :where 515)
(odbc::%write-row aq #'(lambda (hstmt) (odbc::%sql-bulk-operations hstmt :add)) 0
                  '(text_ [datetime_] [memo_] [ole_])
                  `(,(format nil "Added by bulk ~D" (random 100))
                    ,(get-universal-time)
                    " -   bulk"
                    nil))
(sql:close-statement eq)
(sql:commit)

(defun set-all-records (cursor attribute value)
  (sql:set-cursor-position cursor :first)
  (loop 
        count (sql:update-records cursor :attributes attribute :values value)
        while (sql:set-cursor-position cursor :next :retrieve-data nil)))
(set-all-records aq [text_] "113")

(setq cur1 (sql-sys:db-open-cursor a [select [long1] [text1] [datetime1] :from [tab1]]
                                 :cursor-type :keyset-driven
                                 :concurrency :optimistic
                                 :rowset-size 10
                                 ))
(time (dotimes (i 100000)
        (sql:insert-records :into cur1 ;:attributes '([long1] [text1] [datetime1])
             :values `(,i
                       ,(concatenate 'string (princ-to-string (random 20)) " - ")
                       ,(+ (get-universal-time) (random 20))))))

(time (loop while (sql:fetch-rows cur1 :count 1)))
(sql:close-statement cur1)

;; SQL_BINARY

(create-table [foo raw-test] '((id integer)
                               (bar (raw 10))
                               (a (varchar 8))))

(insert-records :into [foo raw-test]
                :values (list 1 #*101110010101000011100000000 "raw!"))

;;; Parameters, prepared statements
(sql-sys:collect-parameters [select [Text_] :from [tab]
                                    :where [< [id] [? :name a]]
                                    :order-by [? :name b]])
(odbc::%sql-func-exists a odbc::SQL_API_SQLEXECUTE)
(sql:execute-command [insert :into [tab] :attributes [text_] :values "abc"]
                     :parameters '([?]) :values '(10))
 :values `(,(apply #'concatenate 'string (make-list 30 :initial-element "123456789 "))))


(setq pa (sql:prepare-statement
    [insert :into [tab] :attributes '([Text_] [Boolean_] [Currency_] [DateTime_]
                                      [Byte_] [Short_] [Long_] [Single_] [Double_])
            :values '([? :name text_ :type string]
                      [? :name boolean_  :type :boolean]
                      [? :name Currency_ :type :numeric]
                      [? :name DateTime_ :type :timestamp]
                      [? :name Byte_	 :type unsigned-byte]
                      [? :name Short_	 :type (integer -32767 32767)]
                      [? :name Long_	 :type integer]
                      [? :name Single_	 :type single-float]
                      [? :name Double_	 :type double-float]) ]
    :database a))
(sql:execute-command pa 
    :values `("From prepared" 0 111.11 ,(get-universal-time) 255 222 3333 4.44 55.5555))

(odbc::%execute-parameterized pa nil
    :values '("  prepared" 1))
(sql:commit :database a)
(sql:rollback :database a)
(sql:close-statement pa)
(sql:cancel-statement pa)

(setq pa (odbc::db-prepare-statement a
   [insert :into [tab] :attributes [Memo_] :values [? :name memo_ :type (string 300)]]))
(sql:execute-command pa 
 :values `(,(apply #'concatenate 'string (make-list 30 :initial-element "123456789 "))))

(odbc::db-execute-parameterized pa 
   [insert :into [tab] :attributes [Memo_] :values [? :name memo_]]
 :values `(,(apply #'concatenate 'string (make-list 30 :initial-element "123456789 "))))

(setq pa (odbc::db-prepare-statement a
   [insert :into [tab] :attributes [Ole_]
           :values [? :name ole_ :type (vector unsigned-byte 300)]]))
(setq v (make-array 300 :element-type '(unsigned-byte 8)))
(dotimes (i (length v)) (setf (aref v i) (mod i 256)))
(odbc::db-execute-parameterized pa nil :values `(,v))

(sql:with-prepared-statement (pa "INSERT INTO tab (Ole_) VALUES (?)"
                        :parameters '([? :name ole_ :type (vector unsigned-byte 300)]))
  (sql:execute-command pa :values `(,v)))

(sql:map-query 'nil
               (lambda (id text) (format t "~&id=~S txt=~S" id text) id)
               [select [id] [text_] :from [tab] :order-by [id :desc] :flatp nil
                       :where [between [id] [? :integer] [? :integer]]]
               :values (list (capi:prompt-for-integer "Enter lower id")
                             (capi:prompt-for-integer "Enter upper id")))

(sql:do-query ((id text &rest rest)
               [select [id] [text_] [long_] :from [tab]
                       :where [<= [id] [? :integer]]]
               :values (list (capi:prompt-for-integer "Enter max id")))
  (format t "~&id=~S text_=~S rest=~S" id text rest))

(sql:with-transaction ()
 (sql:with-prepared-statement (stmt [insert :into [tab]
                                            :attributes '([Text_] [DateTime_] [Long_])
                                            :values '([?] 
                                                      [? :type :universal-time]
                                                      [? :type integer])])
  (dotimes (i 4)
    (sql:execute-command stmt :values (list "" (get-universal-time)
                                            (+ i 200))))))
;; Try paramset-size
(setq pa (sql:prepare-statement
       [insert :into [tab] :av-pairs '(([Text_] [? :name text_ :type string]))]
       :paramset-size 2))

(odbc::db-map-parameters a 'list 
       [insert :into [tab] :av-pairs '(([Text_] [? :name text_ :type string]))]
       '("map-parameters1" "map-parameters2"))

(sql-sys::map-parameters 'list 
       [select [Text_] [Memo_] :from [tab] :where [= [id] [? :type integer]]]
       '((5 15 17))
       :database a)


;; Cronometrics
(sql:with-transaction ()
  (sql:delete-records :from [tab2]))

(time
 (dotimes (i 1000)
   (sql:with-transaction ()
     (sql:insert-records :into [tab2]
                     :attributes '([id] [text2_])
                     :values (list i (format nil "~A" (random 1000)))))))
; LW:
; user time    =      5.015
; system time  =      0.703
; Elapsed time =   0:00:06
; Allocation   = 3887584 bytes standard / 5264677 bytes fixlen
; 0 Page faults
; Allegro:
; cpu time (non-gc) 22,534 msec user, 17,315 msec system
; cpu time (gc)     520 msec user, 0 msec system
; cpu time (total)  23,054 msec user, 17,315 msec system
; real time  416,408 msec (00:06:56.408)
; space allocation:
;  169,716 cons cells, 674,656 other bytes, 0 static bytes

;; Much faster:
(time
  (sql:with-prepared-statement (stmt [insert :into [tab2]
                                             :attributes '([id] [text2_])
                                             :values '([? :type integer]
                                                       [? :type string])])
    (sql:with-transaction ()
      (dotimes (i 1000)
        (sql:execute-command stmt 
                             :values (list i (format nil "~A" (random 1000))))))))
; LW
; user time    =      0.890
; system time  =      0.093
; Elapsed time =   0:00:01
; Allocation   = 647984 bytes standard / 1573990 bytes fixlen
; 0 Page faults

; Allegro: 8.4 seconds used.
; cpu time (non-gc) 16,665 msec user, 14,191 msec system
; cpu time (gc)     490 msec user, 0 msec system
; cpu time (total)  17,155 msec user, 14,191 msec system
; real time  327,922 msec (00:05:27.922)
; space allocation:
;  152,160 cons cells, 371,568 other bytes, 1602 static bytes

;; Even faster:
(time
 (sql:with-transaction ()
   (sql:with-prepared-statement (stmt [insert :into [tab2]
                                              :attributes '([id] [text2_])
                                              :values '([? :type integer]
                                                        [? :type string])])
      (dotimes (i 1000)
        (sql:execute-command stmt 
                             :values (list i (format nil "~A" (random 1000))))))))
;; LW
; user time    =      0.828
; system time  =      0.078
; Elapsed time =   0:00:01
; Allocation   = 651408 bytes standard / 1578852 bytes fixlen
; 0 Page faults

;;; Escape sequences: Timestamp literals
(sql:execute-command "INSERT INTO tab(DateTime_) VALUES({ts'2003-02-13 13:30:31'})")

;;; Escape sequences: Joins
;;; Escaped syntax leads to
;;;	[ODBC state 42000 error -3100] [Microsoft][ODBC Excel Driver]
;;;	Reserved error (-3100); there is no message for this error.
;;; Native t1 INNER JOIN t2 ON t1.a1=t2.a2 works.
(sql:query [select [tab id] [text_] [table1 id]
                   :from [inner-join [tab] [table1]
                                    :on [= [tab id] [table1 id]]]]
           :database a)
(sql:query [select [tab id] [text_] [table1 id]
                   :from '([tab] [table1])
                   :where [= [tab id] [table1 id]]]
           :database a)

;;; Escape sequences: Procedures
(sql:query "{call outer_join()}")		; ordinary select query without params
(sql:query [funcall [outer-join]])

;; Standard VB functions are available
;; User-defined VB-functions
(sql:execute-command "{call testfun}")		; neither supported as top calls
(sql:query "{call view_calling_testfun}")	; nor from inside views

(setq s [=call [?] [testfun] [id]])
;(sql:defsql-function (:procedure t) "TESTFUN")
(setq s [testfun [id]])
(setq s [funcall [testfun] [id]])

(do-query ((a b) "SELECT {fn CONCAT(tab.Text_,tab.Memo_)}, {fn ABS(tab2.Text2_)} 
 FROM {oj tab LEFT OUTER JOIN tab2 ON tab.id = tab2.ID};")
  (format t "~&a=~S:~S b=~S:~S" 
    a (type-of a) b (type-of b)))

(setq s [funcall 'tab-update-text (+ 6 5)])	; action query with parameters
(sql:execute-command [funcall [tab-update-text] (+ 6 5)])
(sql:do-query ((a b) "[funcall [qTabText_ByID(2,2+3)}" :database a) 
  (format t "~&a=~S b=~S" a b))

;;; NZ(v1,v2) = COALESCE(v1,v2)

;;; STREAMS are not implemented in YstokSQL

(do-query ((id (:stream text-stream))
           [select [id] [text] :from [foo bar] :where [< [id] 30]])
          (print id)
          (do ((char (read-char text-stream nil)
                     (read-char text-stream nil))
               (i 0 (1+ i)))
              ((null char))
            (print char)))

(do-query ((id (:stream stream))
           [select [id] [text] :from [foo bar] :where [< [id] 19]])
          (print id)
          (let ((str (make-string 3 :initial-element #\?)))
            (print (peek-char nil stream))
            (print (read-sequence str stream))
            (print str)
            (print (peek-char nil stream))
            (print (peek-char nil stream))
            (print (read-char stream nil))
            (print (read-sequence str stream))
            (print str)
            (print (read-char stream nil))
            ))

(create-table [foo long-test]
              '((id integer :primary-key)
                (text long)))

