;;; Test in LWW 4.2-4.4 against Oracle on Windows NT/2000/XP
;;; CAUTION: Don't load this file as a whole - evaluate form step-by-step instead!
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Driver 8.1 against Server 8.1.7
;;;  ,      , 
;;; ..      .
;;;   Listenter,  - !
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Oracle 10.1.0.2
;;; InstantClient 10.1.0.3 driver vs. 10.1.0.2 driver (come with Server).
;;; 
;;; 1) Aplication > SQLGetData Extensions - absent in IC!
;;;    Check this option to turn on support of the SQLGetData extensions
;;;    SQL_GD_ANY_COLUMN, SQL_GD_ANY_ORDER and SQL_GD_BOUND.
;;;    (SQL_GD_BLOCK is always supported.) By default is off.
;;;
;;; 2) Workarounds > Force Retrieval of Longs  - absent in IC!
;;;    Check this box to read the entire Oracle long data type into memory during
;;;    SQLFetch. Oracle restricts any read/write operations to be performed once
;;;    the streaming of data from a long column has started. Therefore, the contents
;;;    of an LOB column cannot be accessed until the data for all long columns has been
;;;    retrieved. The Oracle ODBC Driver will force the entire read on the long column
;;;    and place it into memory until the application is ready to receive it.
;;;    This frees up the application to do other read/write operations on the database.
;;;    The size of the data in the long column determines the memory consumption on
;;;    the client. By default is off.
;;;    Note:
;;;    This feature could exhaust a PC of memory if the data in the long column is too
;;;    big. This feature degrades performance because the data for all long columns must
;;;    be copied twice, once to internal buffers and once to the applications buffer.
;;;
;;; 3) Translation Options tab - absent in IC! (depends on national language?
;;;    removed due to a non-amendable bug mentioned in Release Notes?)
;;;
;;; 4) Workarounds > Bind TIMESTAMP as DATE - only in IC!

;; Don't forget about TNS_ADMIN environment variable when the above are both installed!
(setf (environment-variable "TNS_ADMIN") "D:\\oracle\\ora10.1\\NETWORK\\ADMIN")

(sql:enable-sql-reader-syntax)
(sql:disable-sql-reader-syntax)

(sql:initialize-database-type :database-type :odbc)
(sql:list-databases nil :database-type :odbc)

(connect nil)
;(setq a (sql:connect '("Sample" :user-id "admin") :autocommit nil))
;(setq a (sql:connect '("Jet4" :user-id "admin") :autocommit nil))
(setq o (sql:connect nil :autocommit t :if-exists :warn-old)) ; via DriverConnect
(setq o (sql:connect "OraTEST1ST/scott/tiger" :if-exists :warn-old :autocommit t))
(setq o (sql:connect "Ora10TEST1ST/scott/tiger" :if-exists :warn-old :autocommit t))
(setq o (sql:connect "OraIC101/system/manager" :if-exists :warn-old :autocommit t))
(setq o (sql:connect "OraIC102/scott/tiger10" :if-exists :warn-old :autocommit t))
(setq o (sql:connect '(nil :user "scott" :password "tiger10")
                     :driver-av-pairs '((:driver "Oracle in instantclient10_2")
                                        (:dbq "ORCL10")
                                        (:num :us))	; Use US settings for numbers
                     :autocommit t))

(setq o sql:*default-database*)

(sql:connected-databases)
(sql:disconnect :database o)

#|(sql:start-sql-recording)
(sql:delete-sql-stream *standard-output* :type :all)
(sql:start-sql-recording :type :result)
(setq stm (open #P"sql.log" :direction :output :if-exists :append))
(sql:add-sql-stream stm)
(sql:list-sql-streams)
(sql:delete-sql-stream stm)
(close stm)
(sql:stop-sql-recording :type :all)|#


(odbc::%sql-free-handle :hdbc (odbc::hdbc db))
(odbc::%sql-free-handle :henv (odbc::henv db))
(setf (odbc::henv db) nil)

;;; Environment attributes
(odbc::%sql-get-attribute :henv (odbc::henv db) odbc::SQL_ATTR_CONNECTION_POOLING)
(odbc::%sql-get-attribute :henv (odbc::handle o) odbc::SQL_ATTR_ODBC_VERSION)
;(odbc::%sql-get-attribute :henv (odbc::henv db) odbc::SQL_ATTR_OUTPUT_NTS)

;;; Connection attributes
(odbc::%sql-get-attribute :hdbc ffc:+sql-null-handle+ odbc::SQL_ATTR_TRACEFILE)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc db) odbc::SQL_ATTR_CURRENT_CATALOG)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc db) odbc::SQL_ATTR_QUIET_MODE)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc db) odbc::SQL_ATTR_ACCESS_MODE) ;SQLUINTEGER
(odbc::%sql-get-attribute :hdbc (odbc::handle o) odbc::SQL_ATTR_AUTOCOMMIT)
(odbc::%sql-set-attribute :hdbc (odbc::handle o) odbc::SQL_ATTR_AUTOCOMMIT odbc::SQL_AUTOCOMMIT_ON)
(odbc::%sql-get-attribute :hdbc (odbc::handle o) odbc::SQL_ATTR_CONNECTION_TIMEOUT)
;=> Not impelmenented
(odbc::%sql-get-attribute :hdbc (odbc::handle o) odbc::SQL_ATTR_LOGIN_TIMEOUT)
;=> Not impelmenented

;;; CATALOG Functions 

(odbc::%get-info-result-type odbc::SQL_DBMS_NAME)
(odbc::db-get-info sql:*default-database* odbc::SQL_SERVER_NAME);=> "ORCL10 "? "TEST1ST"
(odbc::db-get-info sql:*default-database* odbc::SQL_DBMS_NAME)	;=> "Oracle"
(odbc::db-get-info sql:*default-database* odbc::SQL_DBMS_VER)	;=> "10.01.0020", "08.01.0741"

(odbc::%sql-get-info (odbc::hdbc sql:*default-database*) odbc::SQL_ODBC_VER)
; Win2K => "03.52.0000" 

(odbc::db-get-info sql:*default-database* odbc::SQL_DRIVER_ODBC_VER) 
; Ora8 => "03.51", Ora10 => "03.52"
(odbc::db-get-info sql:*default-database* odbc::SQL_IDENTIFIER_CASE)	;=> 1=SQL_IC_UPPER
;(odbc::db-get-info sql:*default-database* odbc::SQL_QUOTED_IDENTIFIER_CASE) ;unsupported

(odbc::db-get-info sql:*default-database* odbc::SQL_TXN_CAPABLE)
; Ora => 3 (SQL_TC_DDL_COMMIT) ??
(odbc::db-get-info sql:*default-database* odbc::SQL_TXN_ISOLATION_OPTION)
; MsOra => 2 (SQL_TXN_READ_COMMITTED)
(odbc::db-get-info sql:*default-database* odbc::SQL_GETDATA_EXTENSIONS)
; Ora8 => 4 = SQL_GD_BLOCK
; i.e. long- should be place after bound (that would require SQL_GD_ANY_ORDER!)
; Nevertheless, that works in SQL*Plus.
; Ora10 => 15 = SQL_GD_ANY_COLUMN + SQL_GD_ANY_ORDER + SQL_GD_BLOCK  + odbc::SQL_GD_BOUND
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_NAME)	;=> nil
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_USAGE)
; Oracle => 3 (SQL_CU_)= 1(DML_STATEMENTS) + 2(PROCEDURE_INVOCATION)
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_TERM) ; => "Database Link"
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_LOCATION)	;=> 2 SQL_CL_END
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_NAME_SEPARATOR) ; => "@"
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_CATALOG_NAME_LEN) ; => 0
(odbc::db-get-info sql:*default-database* odbc::SQL_SCHEMA_USAGE)	; => 31
(odbc::db-get-info sql:*default-database* odbc::SQL_SCHEMA_TERM)	; => "Owner"
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_SCHEMA_NAME_LEN); => 30

(odbc::db-get-info sql:*default-database* odbc::SQL_IDENTIFIER_QUOTE_CHAR)	; => "\""
(odbc::db-get-info sql:*default-database* odbc::SQL_SPECIAL_CHARACTERS)
;=> "$#" - why so few?	(Access => "~@#$%^&*_-+=\\}{\"';:?/><,.![]|")
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_TABLE_NAME_LEN)	; => 30

(odbc::db-get-info sql:*default-database* odbc::SQL_PARAM_ARRAY_ROW_COUNTS)
; Oracle => 2 SQL_PARC_NO_BATCH - only one row count available
(odbc::db-get-info sql:*default-database* odbc::SQL_PARAM_ARRAY_SELECTS)
; Oracle => 3 SQL_PAS_NO_SELECT - does not allow generating result set with an array of parameters
(odbc::db-get-info sql:*default-database* odbc::SQL_PROCEDURES)	;=> "Y" 
(odbc::db-get-info sql:*default-database* odbc::SQL_OJ_CAPABILITIES)
; Oracle => 123 = SQL_OJ_LEFT(1)+SQL_OJ_RIGHT(2)+SQL_OJ_NESTED(8)
;               + SQL_OJ_NOT_ORDERED(16)+ SQL_OJ_INNER(32)
;               + SQL_OJ_ALL_COMPARISON_OPS(64), ..   SQL_OJ_FULL (4)

(odbc::%sql-func-exists o odbc::SQL_API_SQLBULKOPERATIONS)
; Ora8 => nil, Ora10 => T, MsOra=> T
(odbc::db-get-info sql:*default-database* odbc::SQL_ROW_UPDATES)	; => "Y"
(odbc::db-get-info sql:*default-database* odbc::SQL_BOOKMARK_PERSISTENCE)
; Ora8 => 0
; Ora10 => 88 = SQL_BP_TRANSACTION + SQL_BP_UPDATE + SQL_BP_SCROLL
; MsOra => 90 = SQL_BP_DELETE(#x00000002) + SQL_BP_TRANSACTION(#x00000008)
;	      + SQL_BP_UPDATE(#x00000010) + SQL_BP_SCROLL(#x00000040)
(odbc::db-get-info sql:*default-database* odbc::SQL_SCROLL_OPTIONS)
; Oracle,MsOra => 19 = SQL_SO_FORWARD_ONLY | SQL_SO_STATIC | SQL_SO_KEYSET_DRIVEN 
(odbc::db-get-info sql:*default-database* odbc::SQL_CURSOR_COMMIT_BEHAVIOR)
; Oracle => 2 = SQL_CB_PRESERVE
; MsOra => 1 =  SQL_CB_CLOSE
(odbc::db-get-info sql:*default-database* odbc::SQL_CURSOR_ROLLBACK_BEHAVIOR)
; Oracle => 2 = SQL_CB_PRESERVE
; MsOra => 1 =  SQL_CB_CLOSE
(odbc::db-get-info sql:*default-database* odbc::SQL_SCROLL_CONCURRENCY)
; Oracle => 7 = SQL_SCCO_READ_ONLY(1) + SQL_SCCO_LOCK(2) + SQL_SCCO_OPT_ROWVER(4)
; MsOra => 11 = SQL_SCCO_READ_ONLY(1) + SQL_SCCO_LOCK(2) + SQL_SCCO_OPT_VALUES(8)
(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_SENSITIVITY)
; Oracle => 0 Indicates whether a cursor is sensitive to changes made by another cursor.
; MsOra => 5
(odbc::db-get-info sql:*default-database* odbc::SQL_DYNAMIC_CURSOR_ATTRIBUTES1) ; => Err
(odbc::db-get-info sql:*default-database* odbc::SQL_DYNAMIC_CURSOR_ATTRIBUTES2) ; => Err
(odbc::db-get-info sql:*default-database* odbc::SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1)
; Oracle, MsOra => 57345 = #xE001 = SQL_CA1_NEXT (1)
;	+ POSITIONED_UPDATE + POSITIONED_DELETE + SELECT_FOR_UPDATE
(odbc::db-get-info sql:*default-database* odbc::SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2)
; Oracle => 2183 = #x887 (SQL_CA2_...)
;	= READ_ONLY_CONCURRENCY + LOCK_CONCURRENCY + OPT_ROWVER_CONCURRENCY
;	+ MAX_ROWS_SELECT + MAX_ROWS_CATALOG
(odbc::db-get-info sql:*default-database* odbc::SQL_KEYSET_CURSOR_ATTRIBUTES1)
; Oracle => 65095 = #xFE47 (SQL_CA1_...)
;	= NEXT + ABSOLUTE + RELATIVE
;	+ LOCK_NO_CHANGE + LOCK_UNLOCK
;	+ POS_POSITION + POS_UPDATE + POS_DELETE + POS_REFRESH
;	+ <SQL_ADD is permited in SQLSetPos>
;	+ POSITIONED_UPDATE + POSITIONED_DELETE + SELECT_FOR_UPDATE
; MsOra => 130639 = #x1FE4F
;	= NEXT + ABSOLUTE + RELATIVE + BOOKMARK
;	+ LOCK_NO_CHANGE
;	+ POS_POSITION + POS_UPDATE + POS_DELETE + POS_REFRESH
;	+ POSITIONED_UPDATE + POSITIONED_DELETE + SELECT_FOR_UPDATE
;	+ BULK_ADD
(odbc::db-get-info sql:*default-database* odbc::SQL_KEYSET_CURSOR_ATTRIBUTES2)
; Oracle => 2183 = #x887
(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_CURSOR_ATTRIBUTES1)
; Ora8 => 65095 = #xFE47, Ora10 => 65103
; MsOra => 130639
(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_CURSOR_ATTRIBUTES2)
; Oracle => 4231 = #x1087 (SQL_CA2_)
; 	= READ_ONLY_CONCURRENCY + LOCK_CONCURRENCY + OPT_ROWVER_CONCURRENCY
;	+ MAX_ROWS_SELECT + CRC_EXACT
(odbc::db-get-info sql:*default-database* odbc::SQL_POS_OPERATIONS) ; deprecated in 3.0
;=> 31 = SQL_POS_POSITION+SQL_POS_REFRESH+SQL_POS_UPDATE+SQL_POS_DELETE+SQL_POS_ADD

(format nil "#x~x" 130639)

(odbc::db-get-info sql:*default-database* odbc::SQL_SQL_CONFORMANCE)
;=> 1=SQL_SC_SQL92_ENTRY
(odbc::db-get-info sql:*default-database* odbc::SQL_SQL92_VALUE_EXPRESSIONS)
;=> 1=SQL_SVE_CASE
(odbc::db-get-info sql:*default-database* odbc::SQL_CONVERT_FUNCTIONS)
;=> #x00000001=;_CONVERT

;(list-info db t)

(odbc::db-describe-type o 0)
; Ora10 =>
(("BLOB" -4 2147483647 "'" "'" NIL 1 1 0 NIL 0 NIL "BLOB" NIL NIL -4 NIL NIL NIL)
 ("LONG RAW" -4 2147483647 "'" "'" NIL 1 0 0 NIL 0 NIL "LONG RAW" NIL NIL -4 NIL NIL NIL)
 ("BFILE" -4 2147483647 "'" "'" NIL 1 1 0 NIL 0 NIL "BFILE" NIL NIL -4 NIL NIL NIL)
 ("RAW" -3 2000 "'" "'" "max length" 1 0 3 NIL 0 NIL "RAW" NIL NIL -3 NIL NIL NIL)
 ("CLOB" -1 2147483647 "'" "'" NIL 1 1 0 NIL 0 NIL "CLOB" NIL NIL -1 NIL NIL NIL)
 ("LONG" -1 2147483647 "'" "'" NIL 1 1 0 NIL 0 NIL "LONG" NIL NIL -1 NIL NIL NIL)
 ("CHAR" 1 2000 "'" "'" "max length" 1 1 3 NIL 0 0 "CHAR" NIL NIL 1 NIL NIL NIL)
 ("DECIMAL" 3 38 NIL NIL "precision,scale" 1 0 3 0 0 0 "DECIMAL" 0 38 3 NIL 10 NIL)
 ("BINARY_FLOAT" 7 7 NIL NIL NIL 1 0 3 0 0 0 "BINARY_FLOAT" NIL NIL 7 NIL 10 NIL);SQL_REAL
 ("BINARY_DOUBLE" 6 15 NIL NIL NIL 1 0 3 0 0 0 "BINARY_DOUBLE" NIL NIL 6 NIL 10 NIL)
 ("DOUBLE PRECISION" 8 15 NIL NIL NIL 1 0 3 0 0 0 "DOUBLE PRECISION" NIL NIL 8 NIL 10 NIL)
 ("DATE" 93 19 "'" "'" NIL 1 0 3 NIL 0 0 "DATE" 0 0 9 3 NIL NIL)   ; scale must be [0..9]
 ("VARCHAR2" 12 4000 "'" "'" "max length" 1 1 3 NIL 0 0 "VARCHAR2" NIL NIL 12 NIL NIL NIL)
 ("NCHAR" -8 1000 "N'" "'" "max length" 1 1 3 NIL 0 0 "NCHAR" NIL NIL -8 NIL NIL NIL)
 ("NVARCHAR2" -9 2000 "N'" "'" "max length" 1 1 3 NIL 0 0 "NVARCHAR2" NIL NIL -9 NIL NIL NIL)
 ("NCLOB" -10 2147483647 "N'" "'" NIL 1 1 0 NIL 0 NIL "NCLOB" NIL NIL -10 NIL NIL NIL))
)
("TYPE_NAME" "DATA_TYPE" "COLUMN_SIZE" "LITERAL_PREFIX" "LITERAL_SUFFIX" "CREATE_PARAMS" "NULLABLE" "CASE_SENSITIVE" "SEARCHABLE" "UNSIGNED_ATTRIBUTE" "FIXED_PREC_SCALE" "AUTO_UNIQUE_VALUE" "LOCAL_TYPE_NAME" "MINIMUM_SCALE" "MAXIMUM_SCALE" "SQL_DATA_TYPE" "SQL_DATETIME_SUB" "NUM_PREC_RADIX" "INTERVAL_PRECISION") 

; Ora8 =>
(("BLOB" -4 2147483647 "'" "'" NIL 1 1 0 NIL 0 NIL "BLOB" NIL NIL -4 NIL NIL NIL)
 ("LONG RAW" -4 2147483647 "'" "'" NIL 1 0 0 NIL 0 NIL "LONG RAW" NIL NIL -4 NIL NIL NIL)
 ("BFILE" -4 2147483647 "'" "'" NIL 1 1 0 NIL 0 NIL "BFILE" NIL NIL -4 NIL NIL NIL)
 ("RAW" -3 2000 "'" "'" "max length" 1 0 3 NIL 0 NIL "RAW" NIL NIL -3 NIL NIL NIL)
 ("CLOB" -1 2147483647 "'" "'" NIL 1 1 0 NIL 0 NIL "CLOB" NIL NIL -1 NIL NIL NIL)
 ("LONG" -1 2147483647 "'" "'" NIL 1 1 0 NIL 0 NIL "LONG" NIL NIL -1 NIL NIL NIL)
 ("CHAR" 1 2000 "'" "'" "max length" 1 1 3 NIL 0 0 "CHAR" NIL NIL 1 NIL NIL NIL)
 ("DECIMAL" 3 38 NIL NIL "precision,scale" 1 0 3 0 0 0 "DECIMAL" 0 38 3 NIL 10 NIL)
 ("DOUBLE PRECISION" 8 15 NIL NIL NIL 1 0 3 0 0 0 "DOUBLE PRECISION" NIL NIL 8 NIL 10 NIL)
 ("DATE" 93 19 "'" "'" NIL 1 0 3 NIL 0 0 "DATE" 0 0 9 3 NIL NIL)
 ("VARCHAR2" 12 4000 "'" "'" "max length" 1 1 3 NIL 0 0 "VARCHAR2" NIL NIL 12 NIL NIL NIL)
)

(odbc::%get-type-descriptor o odbc::SQL_BIGINT) ;=> nil

(sql:list-tables)

(sql:list-tables :database o :schema :all)
(odbc::db-describe-tables o "%")
(sql:list-tables :database o :schema "SCOTT")   ; Schema SCOTT is uppercased in the server!
(sql:list-tables :database o :schema "HR")
(odbc::db-describe-tables o "USER_TAB_COLUMNS" :table-type "")

(sql:list-attributes "EMP" :database o)
(odbc::db-describe-columns o "EMPLOYEE")
(odbc::db-describe-columns o "EMP" :schema "SCOTT")
(odbc::db-describe-columns o "EMP" :schema :all)
; Ora10 =>
(("" "SCOTT" "EMP" "EMPNO" 3 "DECIMAL" 4 6 0 10 0 "" "" 3 4352 0 1 "NO")
 ("" "SCOTT" "EMP" "ENAME" 12 "VARCHAR2" 10 10 NIL NIL 1 "Employee first name" "" 12 4352 10 2 "YES")
 ("" "SCOTT" "EMP" "JOB" 12 "VARCHAR2" 9 9 NIL NIL 1 "" "" 12 4352 9 3 "YES")
 ("" "SCOTT" "EMP" "MGR" 3 "DECIMAL" 4 6 0 10 1 "" "" 3 4352 0 4 "YES")
 ("" "SCOTT" "EMP" "HIREDATE" 93 "DATE" 19 16 NIL NIL 1 "" "" 9 3 0 5 "YES")
 ("" "SCOTT" "EMP" "SAL" 3 "DECIMAL" 7 9 2 10 1 "" "" 3 0 0 6 "YES")
 ("" "SCOTT" "EMP" "COMM" 3 "DECIMAL" 7 9 2 10 1 "" "" 3 0 0 7 "YES")
 ("" "SCOTT" "EMP" "DEPTNO" 3 "DECIMAL" 2 4 0 10 1 "" "" 3 0 0 8 "YES")) 

(odbc::db-describe-columns o nil)
(odbc::db-describe-columns o "RUEMP")
;Ora10 => 
(("" "SCOTT" "RUEMP" "EMPNO" 3 "DECIMAL" 4 6 0 10 0 "" "" 3 4352 0 1 "NO")
 ("" "SCOTT" "RUEMP" "ENAME" 12 "NVARCHAR2" 10 10 NIL NIL 1 "Employee first name" "" 12 4352 10 2 "YES")
 ("" "SCOTT" "RUEMP" "JOB" 12 "NVARCHAR2" 9 9 NIL NIL 1 "" "" 12 4352 9 3 "YES")
 ("" "SCOTT" "RUEMP" "HIREDATE" 93 "DATE" 19 16 NIL NIL 1 "" "" 9 3 0 4 "YES")) 
;Ora8 =>
(("" "SCOTT" "RUEMP" "EMPNO" 3 "DECIMAL" 4 6 0 10 0 "" "" 3 6912 8496896 1 "NO")
 ("" "SCOTT" "RUEMP" "ENAME" 12 "NVARCHAR2" 10 10 NIL NIL 1 "Employee first name" "" 12 6912 8496896 2 "YES")
 ("" "SCOTT" "RUEMP" "JOB" 12 "NVARCHAR2" 9 9 NIL NIL 1 "" "" 12 6912 8496896 3 "YES")
 ("" "SCOTT" "RUEMP" "HIREDATE" 93 "DATE" 19 16 NIL NIL 1 "" "" 9 3 8496896 4 "YES"))

; Oracle    REMARKS,  COLUMN_DEF ( "")
(sql:query "SELECT * FROM USER_TAB_COMMENTS;") ;=>
(sql:query "SELECT * FROM ALL_TAB_COMMENTS WHERE NOT COMMENTS Is Null;")
;=> ("OWNER" "TABLE_NAME" "TABLE_TYPE" "COMMENTS") 
(multiple-value-list
 (sql:query "SELECT * FROM USER_COL_COMMENTS WHERE NOT COMMENTS Is Null;"))
(sql:query "SELECT * FROM USER_COL_COMMENTS WHERE TABLE_NAME='RUEMP' AND NOT COMMENTS Is Null")
; => ("TABLE_NAME" "COLUMN_NAME" "COMMENTS")
(sql:query "SELECT * FROM ALL_COL_COMMENTS WHERE TABLE_NAME='RUEMP' AND NOT COMMENTS Is Null")
;=> ("OWNER" "TABLE_NAME" "COLUMN_NAME" "COMMENTS") 
(sql:query "SELECT * FROM ALL_COL_COMMENTS WHERE  NOT COMMENTS Is Null")

(odbc::db-describe-columns o "USER_TAB_COLUMNS" :schema :all)
(sql:query "SELECT column_name, default_length, data_default FROM USER_TAB_COLUMNS WHERE TABLE_NAME='MYTAB'")

(odbc::db-describe-columns o "COL" :schema :all)
(sql:query "SELECT cname, coltype, defaultval FROM COL WHERE TNAME='MYTAB'")

(odbc::db-describe-columns o "TYPE_TABLE")
(sql:attribute-type "T_INT" "TYPE_TABLE" :database o)
(sql:attribute-type "BIRTH_TS" "EMPLOYEE" :database o)
(sql:attribute-type "HIREDATE" "EMP" :database o)

(odbc::db-describe-special-columns o "mytab"
                          :type odbc::SQL_BEST_ROWID :scope odbc::SQL_SCOPE_CURROW)
;=> ((2 "ROWID" 1 "ROWID" NIL 18 NIL 2)) SQL_SCOPE_SESSION ... SQL_PC_PSEUDO
; MsOra => ((1 "ROWID" 12 "VARCHAR2" 18 18 NIL 2)) SQL_SCOPE_TRANSACTION ...
(odbc::db-describe-special-columns o "mytab" :type odbc::SQL_ROWVER ) ;=>nil
 
(odbc::db-describe-primary-keys o "mytab")
;=> ((NIL "SCOTT" "MYTAB" "ID" 1 "SYS_C001287"))

(sql:query "select * from nls_session_parameters;") ;=>
(("NLS_LANGUAGE" "AMERICAN")
 ("NLS_TERRITORY" "AMERICA")
 ("NLS_CURRENCY" "$")
 ("NLS_ISO_CURRENCY" "AMERICA")
 ("NLS_NUMERIC_CHARACTERS" ".,")
 ("NLS_CALENDAR" "GREGORIAN")
 ("NLS_DATE_FORMAT" "DD-MON-RR")
 ("NLS_DATE_LANGUAGE" "AMERICAN")
 ("NLS_SORT" "BINARY")
 ("NLS_TIME_FORMAT" "HH.MI.SSXFF AM")
 ("NLS_TIMESTAMP_FORMAT" "DD-MON-RR HH.MI.SSXFF AM")
 ("NLS_TIME_TZ_FORMAT" "HH.MI.SSXFF AM TZR")
 ("NLS_TIMESTAMP_TZ_FORMAT" "DD-MON-RR HH.MI.SSXFF AM TZR")
 ("NLS_DUAL_CURRENCY" "$")
 ("NLS_COMP" "BINARY")
 ("NLS_LENGTH_SEMANTICS" "BYTE")
 ("NLS_NCHAR_CONV_EXCP" "FALSE")) 

(("NLS_LANGUAGE" "RUSSIAN")
 ("NLS_TERRITORY" "CIS")
 ("NLS_CURRENCY" ".")
 ("NLS_ISO_CURRENCY" "CIS")
 ("NLS_NUMERIC_CHARACTERS" ", ")
 ("NLS_CALENDAR" "GREGORIAN")
 ("NLS_DATE_FORMAT" "DD.MM.RR")
 ("NLS_DATE_LANGUAGE" "RUSSIAN")
 ("NLS_SORT" "RUSSIAN")
 ("NLS_TIME_FORMAT" "HH24:MI:SSXFF")
 ("NLS_TIMESTAMP_FORMAT" "DD.MM.RR HH24:MI:SSXFF")
 ("NLS_TIME_TZ_FORMAT" "HH24:MI:SSXFF TZH:TZM")
 ("NLS_TIMESTAMP_TZ_FORMAT" "DD.MM.RR HH24:MI:SSXFF TZH:TZM")
 ("NLS_DUAL_CURRENCY" ".")
 ("NLS_COMP" "BINARY")) 
(sql:execute-command "alter session set NLS_COMP=ANSI;")
(sql:execute-command "alter session set NLS_SORT='RUSSIAN_CI';")
(sql:execute-command "alter session set NLS_NUMERIC_CHARACTERS=', '") ;'.,'
(sql:execute-command "alter session set NLS_TIMESTAMP_FORMAT='YYYY-MM-DD HH24:MI:SS.FF';")


;;; DDL
(defun random-string (size)
  (let ((str (make-string size :element-type 'base-character)))
    (dotimes (i size)
      (setf (char str i) (code-char (+ 97 (random 26)))))
    str))
(defun increasing-vector (size)
  (let ((v (make-array size :element-type '(unsigned-byte 8))))
    (loop for i upfrom 0 below size
          and j = 0 then (if (= j 199) 0 (1+ j))
          do (setf (aref v i) (if (zerop (mod i 100)) (truncate i 100) (mod i 100))))
    v))

(setq v (make-array 300 :element-type '(unsigned-byte 8)))
(dotimes (i (length v)) (setf (aref v i) (mod i 256)))

(sql:drop-table [mytab] :database o)
(sql:create-table [mytab]
  `(([id] integer :primary-key)
    ([empno] integer :constraint [emp-empno]
             :references [emp] [empno] :on-delete :set-null) ; on-update doesn't exists
    ([char_] (string 32) :default "Mmyyytable default")
                     ; :check ,[and [> [id] 0] [>= [text-moo] "m%"]]))
    ([smallint_] (integer -1000 1000))
    ([date_] date)
    ([real_] single-float)
    ([double_] double-float)
    ([raw_] (bit-vector 64))
    ([longtext_] (string 20000))
    ;([longraw_] (vector unsigned-byte 30000)) ;      LONG!
    ([clob_] :clob)
    ([blob_] :blob))
   )
  :constraints
   '(;([table1-id] :foreign-key ([tab-id]) :references [tab] [id])
    ; (:check ,[> [tab-id] 5]) ; MSA not supported
   )
   :database o 
)
(odbc::db-describe-columns o "mytab") ;=>
(("" "SCOTT" "MYTAB" "ID" 3 "DECIMAL" 38 40 0 10 0 "" "" 3 512 8126720 1 "NO")
 ("" "SCOTT" "MYTAB" "EMPNO" 3 "DECIMAL" 38 40 0 10 1 "" "" 3 512 8126720 2 "YES")
 ("" "SCOTT" "MYTAB" "CHAR_" 12 "VARCHAR2" 32 32 NIL NIL 1 "" "" 12 512 8126720 3 "YES")
 ("" "SCOTT" "MYTAB" "SMALLINT_" 3 "DECIMAL" 4 6 0 10 1 "" "" 3 512 8126720 4 "YES")
 ("" "SCOTT" "MYTAB" "DATE_" 93 "DATE" 19 16 NIL NIL 1 "" "" 9 3 8126720 5 "YES")
 ("" "SCOTT" "MYTAB" "REAL_" 8 "DOUBLE PRECISION" 19 8 NIL 10 1 "" "" 8 0 8126720 6 "YES")
 ("" "SCOTT" "MYTAB" "DOUBLE_" 8 "DOUBLE PRECISION" 38 8 NIL 10 1 "" "" 8 0 8126720 7 "YES")
 ("" "SCOTT" "MYTAB" "RAW_" -3 "RAW" 8 8 NIL NIL 1 "" "" -3 0 8126720 8 "YES")
 ("" "SCOTT" "MYTAB" "LONGTEXT_" -1 "LONG" 2147483647 2147483647 NIL NIL 1 "" "" -1 0 8126720 9 "YES")
 ("" "SCOTT" "MYTAB" "CLOB_" -1 "CLOB" 2147483647 2147483647 NIL NIL 1 "" "" -1 0 8126720 10 "YES")
 ("" "SCOTT" "MYTAB" "BLOB_" -4 "BLOB" 2147483647 2147483647 NIL NIL 1 "" "" -4 0 8126720 11 "YES")) 

(sql:drop-table "EMPLOYEE")

(sql:execute-command "CREATE TABLE EMPLOYEE (HEIGHT FLOAT, MARRIED CHAR(1), BIRTHDAY DATE, BD_UTIME TIMESTAMP WITH LOCAL TIME ZONE, EMPLID INTEGER NOT NULL, GROUPID INTEGER NOT NULL, FIRST_NAME VARCHAR(30), LAST_NAME VARCHAR(30), EMAIL VARCHAR(100), ECOMPANYID INTEGER, MANAGERID INTEGER, CONSTRAINT \"PK_EMPLOYEE\" PRIMARY KEY(EMPLID))")

;; Sequences
(sql:execute-command "CREATE SEQUENCE MY_SEQ;")
(sql:commit)
(sql:select [my-seq nextval] [my-seq currval] :from 'dual)
(sql:execute-command "DROP SEQUENCE MY_SEQ;")

;;; DML
(sql:insert-records :into [mytab]
  :attributes '([id] [empno] [char_] [smallint_] [date_] [real_] [double_]
                                  [raw_] [longtext_] [clob_] [blob_])
 :values `(1 105 "  " 9876 "10.03.2003" 43.21 123.456789
           #*0011001000110011 "23 Longtext   by insert"  "CLOB   by insert" ,v))

(sql:insert-records :into [mytab]
  :attributes '([id] [empno] [char_] [smallint_])
  :values `(130 104 "  " [my-seq nextval]))

(sql:commit :database o)

(sql:query "SELECT * FROM mytab where id>120")

(sql:insert-records :into [tab2]
                    :attributes '([id] [text2_])
                    :values (list 10 "  "))
(sql:insert-records :into [tab2]
                    :av-pairs '(([id] 11) ([text2_] "  ")))
(sql:insert-records :into [tab2]
                    :attributes '([id] [text2_])
                    :query [SELECT ["tt" ID] ["tt" TEXT_]
                                   :FROM [tab "tt"]
                                   :where [< ["tt" ID] 15]])
(sql:insert-records :into [tab2] :attributes [id] :values 40)
(setq n 14)
(sql:delete-records :from [mytab] :where [< [ID] n]])
(sql:update-records [tab2] :attributes '([text2_]) :values " "
                           :where [>= [ID] 5])
(sql:update-records [tab2] :attributes [text2_] :values " "
                           :where [= [ID] 5])

;;; The Oracle driver does not pass fractional seconds to the native SQL
(sql:execute-command "UPDATE EMPLOYEE SET BIRTH_TS={ts '2006-10-03 10:00:00.123'} WHERE EMPLID=1") ; does not work
(sql:execute-command
 "UPDATE EMPLOYEE SET BIRTH_TS=TO_TIMESTAMP('2006-10-03 10:00:00.123','YYYY-MM-DD HH24:MI:SS.FF3') WHERE EMPLID=1") ; works!
(setq ts (lt:localtimestamp))
(sql:update-records [employee] :av-pairs `((birth-ts ,ts)) :where [= [emplid] 1])


;; SQL_BINARY

(create-table
 [foo raw-test]
 '((id integer)
   (bar (raw 10))
   (a (varchar 8))))

(insert-records :into [foo raw-test]
                :values (list 1 #*101110010101000011100000000 "raw!"))

;;; Queries

(setq s (sql:select [ROWID] [mytab *] :from [mytab])) ; :where [= [ENAME] ""])

(sql:query "SELECT ROWID FROM scott.emp")
(sql:select [empno] [ename] [sal] :from [emp :schema scott])
(sql:select [*] :from [ruemp])

(sql:query [select [id] :as [i] :from [tab] :group-by [id]
                   :order-by '([id] :desc [Text_])])
(sql:query "SELECT ID FROM tab" :flatp t)

(sql:select [char_] :from [mytab] :flatp t :distinct t)
(sql:map-query 'nil (lambda (id text) (format t "~&id=~S txt=~S" id text)
                                       id)
               [select [id] [text_] :from [tab] :order-by [id :desc] :flatp nil])

(time (sql:query "SELECT Text_,Long_,Single_,Memo_ FROM tab" :database a))
(sql:query "SELECT TOP 5 ID,Boolean_,DateTime_,Byte_,Short_,Double_ FROM tab" :database a)

(sql:do-query ((a b &rest rest)
           [select [*] :from [tab2]])
          (print (list a b)))


(sql:select [*] :from 'TYPE_TABLE) :result-types '(integer))
(sql:select [addressid] [street-number] [street-name] :from 'addr)
(sql:select [*] :from 'employee :where [= [emplid] 1])
;(sql:query "SELECT CURRENT_TIMESTAMP FROM DUAL")	;=>crashes
(sql:select [localtimestamp 3] :from [DUAL])

;;; Parameters, prepared statements and procedures

(ystok.sql.sys:collect-parameters [select [Text_] :from [tab]
                                    :where [< [id] [? :name a]]
                                    :order-by [? :name b]])
(sql:query "SELECT * FROM mytab WHERE id >= 30 ORDER BY id")
(defvar id 35)
(setq po (sql:prepare-statement 
    [insert :into [mytab]
            :attributes '([id] [empno] [char_] [smallint_] [date_] [double_] ;[real_] 
                          [raw_] [longtext_] [clob_] [blob_])
            :values  '([? :name id	:type integer]
                       [? :name empno	:type integer]
                       [? :name char_]
                       [? :name smallint_ :type (integer -1000 1000)]
                       [? :name date_	:type :universal-time]
                       [? :name double_ :type double-float]
                       ;[? :name real_	:type single-float] ; works incorrectly in LW
                       [? :name raw_	:type bit-vector]
                       [? :name longtext_ :type (string 5000)]
                       [? :name clob_]
                       [? :name blob_ :type (vector unsigned-byte 300)])]
    :database o))
(sql:execute-command po
 :values `(,(incf id) 105 "  prepared" 987 ,(get-universal-time) 123.456789 ;3.5
           #*0011001000110011  "23 Longtext  prepared"  nil ,v))
           ;#*0011001000110011 "23 Longtext  prepared"  "CLOB  " ,v))

(odbc::db-execute-parameterized po 
    [insert :into [mytab]
            :attributes '([id] [empno] [char_] [smallint_] [date_] [real_] [double_]
                          [raw_] [longtext_] [clob_] [blob_])
            :values  '([? :name id	:type integer]
                       [? :name empno	:type integer]
                       [? :name char_]
                       [? :name smallint_ :type (integer -1000 1000)]
                       [? :name date_	:type :universal-time]
                       [? :name real_	:type single-float] ; works incorrectly in LW
                       [? :name double_ :type double-float]
                       [? :name raw_	:type bit-vector]
                       [? :name longtext_ :type (string 5000)]
                       [? :name clob_]
                       [? :name blob_])]
 :values `(,(incf id) 105 "  prepared" 987 ,(get-universal-time) 33.21 123.456789
           #*0011001000110011  "23 Longtext  prepared" nil ,v))

(sql:commit :database o)
(sql:close-statement po :drop-p nil)
(sql:cancel-statement po)
(sql:rollback :database o)

(sql:do-query ((id raw_ blob_)
               [select [id] [raw_] [blob_] :from [mytab] :where [>= [id] (1- id)]])
    (format t "~&id=~S raw_(~S)=~S~%blob_(~S)=~S"
            id (type-of raw_) raw_ (type-of blob_) blob_))

(setq pqo (odbc::db-prepare-statement o
    [select [id] [char_] :from [mytab] :where [= [id] [? :name id :type integer]]]))
(odbc::%execute-parameterized pqo nil nil '(24))
(sql:fetch-rows pqo)
(sql:close-statement pqo :drop-p nil)

(setq po (sql:prepare-statement 
          [insert :into [mytab] :attributes '([id] [char_])
                  :values  '([? :name id :type integer] [? :name char_])]
          :database o :paramset-size 2))
(sql:execute-command po :values '((44 45) ("text 44" "text 45")))

(setq po (sql:prepare-statement 
          [insert :into [mytab] :attributes '([id] [longtext_] [blob_])
                  :values  '([? :name id :type integer]
                             [? :type (string 5000)]
                             [? :name blob_ :type (vector unsigned-byte 6000)])]
          :database o :paramset-size 2))
(setq s1 (random-string 200) s2 (random-string 300)
      v1 (increasing-vector 300) v2 (increasing-vector 400))
(sql:execute-command po :values (list '(54 55) (list s1 s2) (list v1 v2)))

(sql:select [id] [longtext_] [blob_] [char_]  :from [mytab] :where [>= [id] 54]
            :database o)



(with-prepared-statement (query [insert :into [tab] :attributes '([id] [Text_]:values '($ $ $)])
  (bind-execute query "" 1001 (get-universal-time)))

(time
 (dotimes (i 1000)
   (with-transaction
     (insert-records :into [Table1]
                     :values (list (format nil "~A" (random 20)) i)))))
; cpu time (non-gc) 22,534 msec user, 17,315 msec system
; cpu time (gc)     520 msec user, 0 msec system
; cpu time (total)  23,054 msec user, 17,315 msec system
; real time  416,408 msec (00:06:56.408)
; space allocation:
;  169,716 cons cells, 674,656 other bytes, 0 static bytes

;; much faster:
(time
 (progn
  (with-prepared-statement (query [insert :into [Table1] :values '($ $ $)])
    (dotimes (i 1000)
      (with-transaction ()
        (bind-execute query (format nil "~A - " (random 20)) 
                      i
                      (+ (get-universal-time) (random 10000))))))))
;8.4 seconds used.
;Standard Allocation 348048 bytes.
;Fixlen Allocation 89595 bytes.
; cpu time (non-gc) 16,665 msec user, 14,191 msec system
; cpu time (gc)     490 msec user, 0 msec system
; cpu time (total)  17,155 msec user, 14,191 msec system
; real time  327,922 msec (00:05:27.922)
; space allocation:
;  152,160 cons cells, 371,568 other bytes, 1602 static bytes

(time (do-query ((txt l d) "SELECT TOP 1000 text1,long1,datetime1 FROM table1")
  (let ((s (format nil "~&txt=~S l=~S d=~S" 
                   txt l (and d (format-time d))))))))
;76.7 seconds used.
;Standard Allocation 702080 bytes.
;Fixlen Allocation 861531 bytes.

;;; Escape sequences, outer joins
(sql:query [select [emp empno] [ename] [mytab empno] [char_]
                   :from [right-join [emp] [mytab]
                                    :on [= [emp empno] [mytab empno]]]]
           :database o)

(sql:query [select [emp empno] [ename] [mytab empno] [char_]
                   :from [inner-join [emp] [mytab]
                                    :on [= [emp empno] [mytab empno]]]]
           :database o)
(sql:query [select [emp empno] [ename] [mytab empno] [char_]
                   :from '([emp] [mytab])
                   :where [= [emp empno] [mytab empno]]]
           :database o)

(sql:execute-command "INSERT INTO tab(DateTime_) VALUES({ts'2003-02-13 13:30:31'})")
(my-execute-command "{CALL testfun}")	;; VB-functions not supported

(do-query ((a b) "SELECT Text_,Text2_ FROM outer_join;")
  (format t "~&a=~S:~S b=~S:~S" 
    a (type-of a) b (type-of b)))
(do-query ((a b) "SELECT {fn CONCAT(tab.Text_,tab.Memo_)}, {fn ABS(tab2.Text2_)} 
 FROM {oj tab LEFT OUTER JOIN tab2 ON tab.id = tab2.ID};")
  (format t "~&a=~S:~S b=~S:~S" 
    a (type-of a) b (type-of b)))

(my-execute-command "{CALL tab_update_text(6+5)}")
(do-query ((a b) "{CALL qTabText_ByID(2,2+3)}") 
  (format t "~&a=~S b=~S" a b))

;;; Cursors
(defun print-row-status-array (cursor)
  (with-slots (odbc::row-descriptor) cursor
    (with-slots (odbc::array-size odbc::current-row-status) odbc::row-descriptor
      (if (> odbc::array-size 1)
          (dotimes (i odbc::array-size)
            (format t "~&row-status[~A]=~S" i
                    (case (odbc::desc-row-status odbc::row-descriptor i)
                      (#.odbc::SQL_ROW_SUCCESS 'odbc::SQL_ROW_SUCCESS)
                      (#.odbc::SQL_ROW_SUCCESS_WITH_INFO 'odbc::SQL_ROW_SUCCESS_WITH_INFO)
                      (#.odbc::SQL_ROW_UPDATED 'odbc::SQL_ROW_UPDATED)
                      (#.odbc::SQL_ROW_ADDED   'odbc::SQL_ROW_ADDED)
                      (#.odbc::SQL_ROW_ERROR   'odbc::SQL_ROW_ERROR)
                      (#.odbc::SQL_ROW_DELETED 'odbc::SQL_ROW_DELETED) 
                      (#.odbc::SQL_ROW_NOROW   'odbc::SQL_ROW_NOROW)
                      (otherwise (odbc::desc-row-status odbc::row-descriptor i)))))
          (format t "~&(array-size=1) : current-row-status=~S"
                  odbc::current-row-status)))))

(odbc::%sql-get-attribute :hdbc (odbc::handle o) odbc::SQL_ATTR_AUTOCOMMIT)
(setq oq (sql:open-cursor [select [*] :from [EMP]]
                           :database o
                           :cursor-type :keyset-driven
                           :concurrency :optimistic
                           :rowset-size 1
                          ))
(setq oq (sql:open-cursor [select [*] :from [RUEMP]]
                           :database o
                           :cursor-type :keyset-driven
                           :concurrency :optimistic
                           :rowset-size 1))
(setq oq (sql:open-cursor "SELECT * FROM TYPE_TABLE"
                          :result-types '(string string string)))

(sql:fetch-rows  oq :count 1 :flatp nil)
(odbc::set-cursor-position oq :next)
(odbc::set-cursor-position oq :prior)
(odbc::set-cursor-position oq :last)
(odbc::set-cursor-position oq :relative :offset -4)
(odbc::set-cursor-position oq :absolute :offset 2)
(odbc::set-cursor-position oq :bookmark :offset 4)
(sql:fetch-rows oq :count 1)
(odbc::%sql-fetch (odbc::handle aq))
(odbc::%sql-fetch-scroll (odbc::handle aq) :relative :offset 0)
(odbc::%sql-delete-record (odbc::handle oq) 1)
(odbc::%sql-update-record (odbc::handle oq) 1)

(odbc::%sql-set-attribute :hstmt (odbc::handle oq) odbc::SQL_ATTR_ROW_STATUS_PTR
                            (odbc::desc-array-status-ptr (odbc::row-descriptor oq)))
(print-row-status-array oq)
(odbc::%sql-refresh-record (odbc::handle oq) 1)
(sql:current-row-status oq)
(sql:current-row-number oq)
(odbc::%sql-get-attribute :hstmt (odbc::handle oq) odbc::SQL_ATTR_ROW_NUMBER)
;; row number in entire result set, not in rowset => works in Oracle!
(odbc::%sql-get-attribute :hstmt (odbc::handle oq) odbc::SQL_ATTR_CONCURRENCY)
 ;=>3 (SQL_CONCUR_ROWVER)
(odbc::%sql-get-attribute :hstmt (odbc::handle oq) odbc::SQL_ATTR_KEYSET_SIZE) ;=>0

;; With SQL_C_NUMERIC, updating SAL does not work, inserting signals.
;; Seems fine SQL_C_DOUBLE
(sql:update-records oq :attributes '([job] [hiredate] [sal])
                       :values `(,(format nil "Upd. ~D" (random 100))
                                 ,(get-universal-time)
                                 56.789)
                       ); :where 104)
(sql:insert-records :into oq
                    :attributes '([empno] [ename] [hiredate] [sal])
                    :values `(123
                              ,(format nil "Added ~D" (random 100))
                              ,(lt:current-timestamp)
                              99.9))
;Message: [Microsoft][ODBC driver for Oracle][Oracle]ORA-00001:    (SCOTT.PK_EMP). ???

(sql:delete-records :from oq) ; :where 515)
(sql:close-statement oq)
(sql:commit)

;;; The driver seems to fail to deliver long column data when rowset-size > 1
;;; SQLSetPos followed by SQLGetData crashes.
(setq oc (ystok.sql.sys:db-open-cursor o [select [ROWID] 
                                  [id] [empno] [char_] [smallint_] [date_] [double_]
                                  [raw_] [longtext_] [clob_] [blob_] 
                                                 :from [mytab]]
                                 :cursor-type :keyset-driven
                                 :concurrency :optimistic
                                 :rowset-size 1
                                 :start-column 1 ;0 bookmarks are not supported!
                                 ))
(odbc::%sql-get-attribute :hstmt (odbc::handle oc) odbc::SQL_ATTR_QUERY_TIMEOUT)
(odbc::%sql-set-attribute :hstmt (odbc::handle oc) odbc::SQL_ATTR_QUERY_TIMEOUT 1)


(sql:fetch-rows oc :count 4 :flatp nil)
(sql:fetch-rows oc :count nil)
(odbc::set-cursor-position oc :absolute :offset 1)
(odbc::set-cursor-position oc :relative :offset 0)
(odbc::set-cursor-position oc :prior)
(sql:update-records oc :attributes '([date_] [char_] [clob_])
                       :values `(,(get-universal-time)
                                 ,(format nil "Updated via cursor ~D" (random 100))
                                 "long updated via cursor"))
(odbc::%sql-refresh-record (odbc::handle oc) 0)
(print-row-status-array oc)

(sql:insert-records :into oc
      :attributes '([id] [date_] [char_]) ; [clob_])
      :values `(144
                ,(get-universal-time)
                ,(format nil "Added via cursor ~D" (random 100))
                )) ;"long inserted via cursor"))
;; New record ROWID is not available.
(sql:close-statement oc)
(sql:cancel-statement oc)
(sql:commit)

;;; STREAMS - not implemented

(do-query ((id (:stream text-stream))
           [select [id] [text] :from [foo bar] :where [< [id] 30]])
          (print id)
          (do ((char (read-char text-stream nil)
                     (read-char text-stream nil))
               (i 0 (1+ i)))
              ((null char))
            (print char)))

(do-query ((id (:stream stream))
           [select [id] [text] :from [foo bar] :where [< [id] 19]])
          (print id)
          (let ((str (make-string 3 :initial-element #\?)))
            (print (peek-char nil stream))
            (print (read-sequence str stream))
            (print str)
            (print (peek-char nil stream))
            (print (peek-char nil stream))
            (print (read-char stream nil))
            (print (read-sequence str stream))
            (print str)
            (print (read-char stream nil))
            ))

(create-table [foo long-test]
              '((id integer :primary-key)
                (text long)))

;; this one might crash in MCL with the DataDirect driver.

(with-prepared-statement (query [insert :into [foo long-test] :values '($ $)])
  (dotimes (i 10)
    (bind-execute-with-stream (query i (:stream stream))
      (dotimes (i 10)
        (write-sequence (random-string 2000) stream)))))

(defun my-execute-command (sql-string)
  (let ((query (odbc::get-free-query sql:*default-database*)))
    (when *trace-sql*
      (format (if (streamp *trace-sql*) *trace-sql* *trace-output*)
              "~&~A;~%" sql-string))
    (with-slots (odbc::hstmt odbc::database) query
      (unless (eq *active-transactions* t)
        (pushnew database *active-transactions*))
      (with-slots (odbc::henv odbc::hdbc) odbc::database
        (unless odbc::hstmt (setf odbc::hstmt (odbc::%new-statement-handle odbc::hdbc))) 
        (unwind-protect 
            (multiple-value-prog1 
                (odbc::%sql-exec-direct sql-string odbc::hstmt odbc::henv odbc::hdbc)
              (format t "~&columns-count=~A rows-count=~A" 
                (odbc::result-columns-count odbc::hstmt)
                (odbc::result-rows-count odbc::hstmt)))
          (db-close-query query))))))

#| Unused stuff.
(create-user [foo] :password "bar")
(execute :grant '(connect resource) :to [foo])

(create-table [foo bar]
              '((id integer :primary-key)
                (text (varchar 32))))

(create-table [foo baz]
              '((id integer :primary-key)
                (text (varchar 32))
                (:foreign-key [id] :references [foo bar] [id])))

(create-view [foo bar-baz]
             :replace-p t
             :as [select '([baz id] [bar text] ([baz text] [text1]))
                         :from '([foo bar] [foo baz])
                         :where [= [bar id] [baz id]]])

(with-transaction
  (insert-records :into [foo bar] :values '(17 "jazz")))
(with-transaction
  (insert-records :into [foo baz] :values '(17 "Pacito d'Riveira")))

(select [id] :from [foo bar-baz])

(select [count [id]] :from [foo bar-baz])

(do-query ((a b)
           [select [*] :from [foo bar]])
          (print (list a b)))

(execute-query "select id from foo.bar union select id from foo.bar order by id desc")

(select-union [select id :from [foo bar]]
              [select id :from [foo baz]]
              :order-by '((id desc)))

(let ((schema 'foo))
  (select [id] [text] :from [?schema baz] :order-by '((id desc))))

;; lexicon is a rather huge dictionary database with two tables:

(create-user [lexicon] :password "bar")
(execute :grant '(connect resource) :to [lexicon])

(create-table [lexicon keys]
              '((id integer :primary-key)
                (key (varchar 256) :not-null)
                (category (varchar 32) :not-null)
                (translation (varchar 256))))

(create-table [lexicon examples]
              '((id integer :not-null)
                (example_number integer :not-null)
                (example (varchar 256) :not-null)
                (translation (varchar 256) :not-null)
                (:foreign-key id :references [lexicon keys] (id))))

(select [key] :from [lexicon keys] :where [< rownum 2])

(select [*] :from [lexicon keys] :where [< rownum 2])

(do-query ((a)
           [select [key] :from [lexicon keys] 
                   :where [and [> [length [key]] 20]
                               [< rownum 20]]])
          (print a))

(select '([category] 
          ([count [length [key]] :distinct] [sum]))
        :from [lexicon keys]
        :group-by [category]
        :order-by '(([sum] :desc)))

(select [count [key] :distinct]
        :from [lexicon keys]
        :where [< [rownum] 3])

(select [substr [key] 1 2]
        :from [lexicon keys]
        :where [< [rownum] 8])

(select [key]
        :from [lexicon keys] 
        :where [and [< rownum 200] [< [length [key]] 4]])

(select [category]
        :distinct t
        :from [elhuyar keys] 
        :where [< rownum 200])

(let ((cat 'category)
      (ref-exp [id])
      (operator 'avg))
  (select `(,[?cat] (,[/ [?operator ?ref-exp] 2] ,[?operator]))
          :from [lexicon keys] 
          :where [> [length [key]] 8]
          :group-by [category] 
          :having [> [length [category]] 4]))

(let ((fun 'lpad))
  (select (list [?fun [category] 20 "-+"]
                (list [* [sum [id]] 2] [total]))
          :from [lexicon keys] 
          :where [> [length [key]] 8]
          :group-by [category] 
          :having [> [length [category]] 4]))

(select '([conc [lpad [category] 20 "-+"] "asdf"]
          ([* [sum [id]] 2] [total]))
        :from [lexicon keys] 
        :where [> [length [key]] 8]
        :group-by [category] 
        :having [> [length [category]] 4])

(select-union [select [conc [key] ", Kategorie: " [category]]
                      :from [lexicon keys]
                      :where [< [rownum] 3]]
              [select [conc [category] ", Lemma: " [key]]
                      :from [lexicon keys]
                      :where [< [rownum] 3]]
              :flatp t)


(select [conc [key] ", Kategorie: " [category]]
        :from [lexicon keys]
        :where [<= 3 [id] 9]
        :flatp t)

(select [ceil 3.3] :from [sys dual])
(select [sysdate] :from [sys dual])

(select [keys key] [keys2 key]
        :from '([lexicon keys] ([lexicon keys] [keys2]))
        :where [and [like [keys key]
                          [conc "%" [keys2 key] "%"]]
                    [like [keys2 key] "ehun%"]])

(select [keys key] [example]
        :from '([lexicon keys]
                [select ([example] ([id] [ex-id]))
                        :from [lexicon examples]
                        :where [> [id] 30000]])
        :where [and [= [keys id] [ex-id]]
                    [like [keys key] "hautesk%"]])

(let ((example 'example) (id 'id) (ex-id 'ex-id))
  (declare (ignorable example id ex-id))
  (select [keys key] [example]
          :from `([lexicon keys]
                  ,[select `((,[lpad [?example] 50 "+"] [example])
                             (,[?id] [ex-id]))
                           ;(list [?example] (list [?id] [?ex-id]))
                           :from [lexicon examples]
                           :where [> [id] 30000]])
          :where [and [= [keys id] [ex-id]]
                      [like [keys key] "hautesk%"]]))

(update-records [foo bar]
                :av-pairs '(([id] [+ [id] +13]))
                :attributes '([text])
                :values '([select [text] :from [foo bar]
                                  :where [= [id] 7]])
                :where [> [id] 14])

(update-records [foo bar]
                :av-pairs '(([id] [+ [id] +13])
                            ([text] [select [text] 
                                            :from (([foo bar] [zup]))
                                            :where [= [zup id] [bar id]]
                                            ]))
                :where [> [id] 14])
 |#

