;;; Test in LWW 4.4 against PostgreSQL 8.1 with psqlODBC 08.02.0105 driver on Windows 2000
;;; Recommended DSN/driver settings:
;;; - Recognize Unique Indexes: True
;;; - KSQO (Keyset Query Optimization): try False
;;; - Use Declare/Fetch: try True
;;; - Show System Tables: False
;;; - Updateable Cursors: True

(sql:enable-sql-reader-syntax)
(sql:disable-sql-reader-syntax)
#+ys-product (setq ys::*compile-for-lazyload* t)

(sql:initialize-database-type :database-type :odbc)
(sql:data-sources :odbc)

(connect nil)
(setq pg (sql:connect '("pgtest_ANSI" :user "postgres" :password "pg10") :autocommit t))
(setq pg (sql:connect '("test81" :user "postgres" :password "pg10") :autocommit t
           :database-type :postgresql
           :driver-av-pairs `((:host "localhost") (:port 5432))))
(setq pg (sql:connect () :autocommit t
           :driver-av-pairs `((:driver "PostgreSQL ANSI")
                              ("server" "localhost") (:port 5432) ; server as untranslated
                              (:database "test81")
                              ;(:uid "postgres") (:pwd "pg10")
                              ("ShowSystemTables" 1)
                             )))
(setq pg sql:*default-database*)

(sql:connected-databases)
(sql:disconnect :database pg)

(sql:start-sql-recording)
(sql:delete-sql-stream *standard-output* :type :all)
(sql:start-sql-recording :type :result)
(setq stm (open #P"sql.log" :direction :output :if-exists :append))
(sql:add-sql-stream stm)
(sql:list-sql-streams)
(sql:delete-sql-stream stm)
(close stm)
(sql:stop-sql-recording :type :all)

(odbc::%sql-free-handle :hdbc (odbc::hdbc db))
(odbc::%sql-free-handle :henv (odbc::henv db))
(setf (odbc::henv db) nil)

;;; Environment attributes
(odbc::%sql-get-attribute :henv (odbc::henv pg) odbc::SQL_ATTR_CONNECTION_POOLING)
(odbc::%sql-get-attribute :henv (odbc::henv pg) odbc::SQL_ATTR_ODBC_VERSION)
;(odbc::%sql-get-attribute :henv (odbc::henv db) odbc::SQL_ATTR_OUTPUT_NTS)

;;; Connection attributes
(odbc::%sql-get-attribute :hdbc ffc:+sql-null-handle+ odbc::SQL_ATTR_TRACEFILE)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc pg) odbc::SQL_ATTR_CURRENT_CATALOG)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc pg) odbc::SQL_ATTR_QUIET_MODE)
(odbc::%sql-get-attribute :hdbc (odbc::hdbc pg) odbc::SQL_ATTR_ACCESS_MODE) ;SQLUINTEGER
(odbc::%sql-get-attribute :hdbc (odbc::hdbc pg) odbc::SQL_ATTR_TXN_ISOLATION)
(odbc::%sql-get-attribute :hdbc (odbc::handle pg) odbc::SQL_ATTR_AUTOCOMMIT)
(odbc::%sql-set-attribute :hdbc (odbc::handle pg) odbc::SQL_ATTR_AUTOCOMMIT odbc::SQL_AUTOCOMMIT_ON)
(odbc::%sql-set-attribute :hdbc (odbc::handle pg) odbc::SQL_ATTR_CONNECTION_TIMEOUT 5) ;err
(odbc::%sql-get-attribute :hdbc (odbc::handle pg) odbc::SQL_ATTR_CONNECTION_TIMEOUT)
;=> 0 (Not impelmenented)
(odbc::%sql-set-attribute :hdbc (odbc::handle pg) odbc::SQL_ATTR_LOGIN_TIMEOUT 5)
(odbc::%sql-get-attribute :hdbc (odbc::handle pg) odbc::SQL_ATTR_LOGIN_TIMEOUT)	;=> 5
(odbc::%sql-get-attribute :hdbc (odbc::handle pg) odbc::SQL_ATTR_CONNECTION_DEAD) ;=> 0

;;; CATALOG Functions 

(odbc::%get-info-result-type odbc::SQL_DBMS_NAME)
(odbc::db-get-info sql:*default-database* odbc::SQL_SERVER_NAME);=> "localhost"
(odbc::db-get-info sql:*default-database* odbc::SQL_DBMS_NAME)	;=> "PostgreSQL"
(odbc::db-get-info sql:*default-database* odbc::SQL_DBMS_VER)	;=> "8.1.1"

(odbc::%sql-get-info (odbc::hdbc sql:*default-database*) odbc::SQL_ODBC_VER)
; Win2K => "03.52.0000" 

(odbc:db-get-info sql:*default-database* odbc::SQL_DRIVER_ODBC_VER)
;=> "03.00" (ANSI)
(odbc:db-get-info sql:*default-database* odbc::SQL_TXN_CAPABLE)
;=> 3 (SQL_TC_ALL)
(odbc:db-get-info sql:*default-database* odbc::SQL_TXN_ISOLATION_OPTION)
;=> 10 (SQL_TXN_READ_COMMITTED + SQL_TXN_SERIALIZABLE)
(odbc:db-get-info sql:*default-database* odbc::SQL_DEFAULT_TXN_ISOLATION)
; => 2 SQL_TXN_READ_COMMITTED
(odbc:db-get-info sql:*default-database* odbc::SQL_MULTIPLE_ACTIVE_TXN) ;=> T

(odbc:db-get-info sql:*default-database* odbc::SQL_MAX_DRIVER_CONNECTIONS)	;=> 128
(odbc:db-get-info sql:*default-database* odbc::SQL_MAX_CONCURRENT_ACTIVITIES)
;=> 0 - no specified limit or the limit is unknown
(odbc:db-get-info sql:*default-database* odbc::SQL_MAX_ASYNC_CONCURRENT_STATEMENTS)
;=> Not supported

(odbc::db-get-info sql:*default-database* odbc::SQL_GETDATA_EXTENSIONS)
;=> 15 = SQL_GD_ANY_COLUMN + SQL_GD_ANY_ORDER + SQL_GD_BLOCK  + SQL_GD_BOUND

(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_NAME)	;=> nil
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_USAGE)	;=> 0
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_TERM) 	;=> ""
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_LOCATION)	;=> 1 SQL_CL_START
(odbc::db-get-info sql:*default-database* odbc::SQL_CATALOG_NAME_SEPARATOR) ; => ""
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_CATALOG_NAME_LEN) ; => 0

(odbc::db-get-info sql:*default-database* odbc::SQL_SCHEMA_USAGE)
;=> 31 (SQL_SU_DML_STATEMENTS + _PROCEDURE_INVOCATION + _TABLE_DEFINITION
;	 + _INDEX_DEFINITION + _PRIVILEGE_DEFINITION)
(odbc::db-get-info sql:*default-database* odbc::SQL_SCHEMA_TERM)	; => "Schema"
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_SCHEMA_NAME_LEN); => 64

(odbc::db-get-info sql:*default-database* odbc::SQL_IDENTIFIER_CASE)	;=> 2=SQL_IC_LOWER
(sql.sys:db-identifier-case sql:*default-database*)			;=> :downcase
(odbc::db-get-info sql:*default-database* odbc::SQL_IDENTIFIER_QUOTE_CHAR)	; => "\""
(odbc::db-get-info sql:*default-database* odbc::SQL_SPECIAL_CHARACTERS)
;=> "_" -   ?	(Access => "~@#$%^&*_-+=\\}{\"';:?/><,.![]|")
(odbc::db-get-info sql:*default-database* odbc::SQL_MAX_TABLE_NAME_LEN)	;=> 64
(odbc::db-get-info sql:*default-database* odbc::SQL_NULL_COLLATION)	;=> 0=SQL_NC_HIGH

(odbc::%sql-func-exists sql:*default-database* odbc::SQL_API_SQLDESCRIBEPARAM)	;=> nil
(odbc::db-get-info sql:*default-database* odbc::SQL_PARAM_ARRAY_ROW_COUNTS)
;=> 1 SQL_PARC_BATCH - Individual row counts are available for each set of parameters
(odbc::db-get-info sql:*default-database* odbc::SQL_PARAM_ARRAY_SELECTS)
;=> 1 SQL_PAS_BATCH - There is one result set available per set of parameters
(odbc::db-get-info sql:*default-database* odbc::SQL_PROCEDURES)		;=> "Y" 
(odbc::db-get-info sql:*default-database* odbc::SQL_OJ_CAPABILITIES)
;=> 127 = SQL_OJ_LEFT(1) + SQL_OJ_RIGHT(2) + SQL_OJ_FULL(4) + SQL_OJ_NESTED(8)
;	+ SQL_OJ_NOT_ORDERED(16) + SQL_OJ_INNER(32) + SQL_OJ_ALL_COMPARISON_OPS(64)

(odbc::db-get-info sql:*default-database* odbc::SQL_ROW_UPDATES) ;=> nil or T (Updateable
(odbc::db-get-info sql:*default-database* odbc::SQL_BOOKMARK_PERSISTENCE)
;=> 90 = SQL_BP_DELETE(#x00000002) + SQL_BP_TRANSACTION(#x00000008)
;	+ SQL_BP_UPDATE(#x00000010) + SQL_BP_SCROLL(#x00000040)
(odbc::db-get-info sql:*default-database* odbc::SQL_SCROLL_OPTIONS)
;=> 17 = SQL_SO_FORWARD_ONLY + SQL_SO_STATIC
(odbc::db-get-info sql:*default-database* odbc::SQL_CURSOR_COMMIT_BEHAVIOR)
;=> 2 SQL_CB_PRESERVE
(odbc::db-get-info sql:*default-database* odbc::SQL_CURSOR_ROLLBACK_BEHAVIOR)
;=> 2 SQL_CB_PRESERVE
(odbc::db-get-info sql:*default-database* odbc::SQL_SCROLL_CONCURRENCY)
;=> 1 SQL_SCCO_READ_ONLY
;=> 5 (Updateable cursors)
(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_SENSITIVITY)
;=> 7 Indicates whether a cursor is sensitive to changes made by another cursor.

(odbc::db-get-info sql:*default-database* odbc::SQL_DYNAMIC_CURSOR_ATTRIBUTES1) ;=> 0
(odbc::db-get-info sql:*default-database* odbc::SQL_DYNAMIC_CURSOR_ATTRIBUTES2) ;=> 0

(odbc::db-get-info sql:*default-database* odbc::SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1)
;=> 1 SQL_CA1_NEXT
(odbc::db-get-info sql:*default-database* odbc::SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2)
;=> 4097= SQL_CA2_READ_ONLY_CONCURRENCY(1)+SQL_CA2_CRC_EXACT(#x1000)

(odbc::db-get-info sql:*default-database* odbc::SQL_KEYSET_CURSOR_ATTRIBUTES1)
;=> 990799 = #xF1E4F (SQL_CA1_...) -   updateable cursors!
;	= NEXT + ABSOLUTE + RELATIVE + BOOKMARK
;	+ LOCK_NO_CHANGE
;	+ POS_POSITION + POS_UPDATE + POS_DELETE + POS_REFRESH
;	+ BULK_ADD + UPDATE_BY_BOOKMARK + BULK_DELETE_BY_BOOKMARK + BULK_FETCH_BY_BOOKMARK
;?=> 4687 = #x124F (SQL_CA1_...)
;	= NEXT + ABSOLUTE + RELATIVE
;	+ LOCK_NO_CHANGE + LOCK_UNLOCK
;	+ POS_POSITION + POS_UPDATE + POS_DELETE + POS_REFRESH
;	+ <SQL_ADD is permited in SQLSetPos>
;	+ POSITIONED_UPDATE + POSITIONED_DELETE + SELECT_FOR_UPDATE
(odbc::db-get-info sql:*default-database* odbc::SQL_KEYSET_CURSOR_ATTRIBUTES2)
;=> 4213 = #x1075 (SQL_CA2_...)
;	= _READ_ONLY_CONCURRENCY(1) + _OPT_ROWVER_CONCURRENCY(4)
;	+ _SENSITIVITY_ADDITIONS + _SENSITIVITY_DELETIONS + _SENSITIVITY_UPDATES
;	+ _CRC_EXACT

(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_CURSOR_ATTRIBUTES1)
;=> 4687 = #x124F
;=> 990799 (Updatable cursors)
(odbc::db-get-info sql:*default-database* odbc::SQL_STATIC_CURSOR_ATTRIBUTES2)
;=> 4213
(odbc::db-get-info sql:*default-database* odbc::SQL_POS_OPERATIONS) ; deprecated in 3.0
;=> 31 = SQL_POS_POSITION + SQL_POS_REFRESH + ??

(format nil "#x~x" 4213)

(odbc::db-get-info sql:*default-database* odbc::SQL_NEED_LONG_DATA_LEN)	;=> nil

(odbc::%sql-func-exists sql:*default-database* odbc::SQL_API_SQLBULKOPERATIONS)
;=> nil or T (Updateable cursors)
(odbc:db-get-info sql:*default-database* odbc::SQL_NUMERIC_FUNCTIONS)	;=> 0
(odbc:db-get-info sql:*default-database* odbc::SQL_STRING_FUNCTIONS)	;=> 7289 = #x1C79

(sql:query "SELECT current_setting('LC_COLLATE');" :flatp t)	;=> "Russian_Russia.1251"


;(list-info db t)

(odbc::db-describe-type pg 0) ;=>
(NIL	;-11 								; SQL_GUID
 ("text" -10 8190 "'" "'" NIL 1 1 3 NIL 0 NIL NIL NIL NIL -10 NIL NIL 0) ;SQL_WLONGVARCHAR
 ("varchar" -9 254 "'" "'" "max. length" 1 1 3 NIL 0 NIL NIL NIL NIL -9 NIL NIL 0)
 ("char"   -8 254 "'" "'" "max. length" 1 1 3 NIL 0 NIL NIL NIL NIL -8 NIL NIL 0)
 ("bool"   -7 1 "'" "'" NIL 1 0 2 NIL 0 0 NIL 0 0 -7 NIL NIL 0)	   ; SQL_BIT (NULLABLE!)
 ("char"   -7 1 "'" "'" NIL 1 1 3 NIL 0 NIL NIL NIL NIL 1 NIL NIL 0)    ; Bools as Char
 ("int2"   -6 5 NIL NIL NIL 1 0 2 0 0 0 NIL 0 0 5 NIL 10 0)		; SQL_TINYINT
 ("int8"   -5 19 NIL NIL NIL 1 0 2 0 0 0 NIL 0 0 -5 NIL 10 0)		; SQL_BIGINT
 ("lo"     -4 -4 "'" "'" NIL 1 0 2 NIL 0 NIL NIL NIL NIL -4 NIL NIL 0)  ; SQL_LONGVARBINARY
 ("bytea"  -3 254 "'" "'" NIL 1 0 2 NIL 0 NIL NIL NIL NIL -3 NIL NIL 0)	; SQL_VARBINARY
 NIL	  ;-2								; SQL_BINARY
 ("text"   -1 8190 "'" "'" NIL 1 1 3 NIL 0 NIL NIL NIL NIL -10 NIL NIL 0) ;SQL_LONGVARCHAR
 NIL
 ("char"    1 254 "'" "'" "max. length" 1 1 3 NIL 0 NIL NIL NIL NIL -8 NIL NIL 0)
 ("numeric" 2  28 NIL NIL "precision, scale" 1 0 2 0 0 0 NIL 0 38 2 NIL 10 0) ;SQL_NUMERIC
 ("numeric" 3  28 NIL NIL "precision, scale" 1 0 2 0 0 0 NIL 0 38 2 NIL 10 0) ;SQL_DECIMAL
 ("int4"    4  10 NIL NIL NIL 1 0 2 0 0 0 NIL 0 0 4 NIL 10 0)
 ("serial"  4  10 NIL NIL NIL 0 0 2 1 0 1 NIL 0 0 4 NIL 10 0) ; UNSIGNED || AUTO_INCREMENT
 ("int2"    5   5 NIL NIL NIL 1 0 2 0 0 0 NIL 0 0 5 NIL 10 0)
 ("float8"  6  15 NIL NIL NIL 1 0 2 0 0 0 NIL 0 0 6 NIL 10 0)
 ("float4"  7   7 NIL NIL NIL 1 0 2 0 0 0 NIL 0 0 7 NIL 10 0)
 ("float8"  8  15 NIL NIL NIL 1 0 2 0 0 0 NIL 0 0 6 NIL 10 0)		;   radix=10!
 ("unknown"  9 254 "'" "'" NIL 1 0 2 NIL 0 NIL NIL NIL NIL 12 NIL NIL 0) ; SQL_DATE(TIME)
 ("time"    10   8 "'" "'" NIL 1 0 2 NIL 0 0 NIL NIL NIL 9 2 NIL 0)      ; SQL_INTERVAL?
 ("timestamptz" 11 19 "'" "'" NIL 1 0 2 NIL 0 0 NIL 0 6 9 3 NIL 0)	; SQL_TIMESTAMP
 ("varchar" 12 254 "'" "'" "max. length" 1 1 3 NIL 0 NIL NIL NIL NIL -9 NIL NIL 0)
 ("date"    91  10 "'" "'" NIL 1 0 2 NIL 0 0 NIL NIL NIL 9 1 NIL 0)	; SQL_TYPE_DATE
 ("time"    92   8 "'" "'" NIL 1 0 2 NIL 0 0 NIL NIL NIL 9 2 NIL 0)
 ("timestamptz" 93 19 "'" "'" NIL 1 0 2 NIL 0 0 NIL 0 6 9 3 NIL 0))
("TYPE_NAME" "DATA_TYPE" "PRECISION" "LITERAL_PREFIX" "LITERAL_SUFFIX" "CREATE_PARAMS" "NULLABLE" "CASE_SENSITIVE" "SEARCHABLE" "UNSIGNED_ATTRIBUTE" "MONEY" "AUTO_INCREMENT" "LOCAL_TYPE_NAME" "MINIMUM_SCALE" "MAXIMUM_SCALE" "SQL_DATA_TYPE" "SQL_DATATIME_SUB" "NUM_PREC_RADIX" "INTERVAL_PRECISION") 


(sql:list-tables) = (sql:list-tables :database pg :schema [public])

;;; Ensure the Show System Tables settings is checked on the Datasource > Page2
(sql:list-tables :database pg :schema "information_schema") ; [pg_catalog]
;=> ("sql_features" "sql_implementation_info" "sql_languages" "sql_packages"
;    "sql_sizing" "sql_sizing_profiles"   "cities" "weather" ...) ;    public
(sql:list-tables :database pg :schema :all)


(odbc::db-describe-tables pg "%")
(odbc::db-describe-tables o "USER_TAB_COLUMNS" :table-type "")

(sql:list-attributes [weather])
(odbc::db-describe-columns pg "weather")
(("" "public" "weather" "city" 12 "varchar" 80 80 NIL NIL 1 "" NIL 12 NIL 254 1 NIL 80 1043)
 ("" "public" "weather" "temp_lo" 4 "int4" 10 4 0 10 1 "" NIL 4 NIL NIL 2 NIL 11 23)
 ("" "public" "weather" "temp_hi" 4 "int4" 10 4 0 10 1 "" NIL 4 NIL NIL 3 NIL 11 23)
 ("" "public" "weather" "prcp" 7 "float4" 7 4 0 10 1 "" NIL 7 NIL NIL 4 NIL 13 700)
 ("" "public" "weather" "date" 91 "date" 10 6 NIL NIL 1 "" NIL 9 1 NIL 5 NIL 10 1082)) 
(sql:attribute-type "temp_hi" "weather" :database pg) ;=> int4

(odbc::db-describe-columns pg "employee")

(odbc::db-describe-special-columns o "mytab"
                          :type odbc::SQL_BEST_ROWID :scope odbc::SQL_SCOPE_CURROW)
;=> ((2 "ROWID" 1 "ROWID" NIL 18 NIL 2)) SQL_SCOPE_SESSION ... SQL_PC_PSEUDO
; MsOra => ((1 "ROWID" 12 "VARCHAR2" 18 18 NIL 2)) SQL_SCOPE_TRANSACTION ...
(odbc::db-describe-special-columns o "mytab" :type odbc::SQL_ROWVER ) ;=>nil
 
(odbc::db-describe-primary-keys o "mytab")
;=> ((NIL "SCOTT" "MYTAB" "ID" 1 "SYS_C001287"))

;;; DDL
(defun random-string (size)
  (let ((str (make-string size :element-type 'base-character)))
    (dotimes (i size)
      (setf (char str i) (code-char (+ 97 (random 26)))))
    str))
(defun increasing-vector (size)
  (let ((v (make-array size :element-type '(unsigned-byte 8))))
    (loop for i upfrom 0 below size
          and j = 0 then (if (= j 199) 0 (1+ j))
          do (setf (aref v i) (if (zerop (mod i 100)) (truncate i 100) (mod i 100))))
    v))

(setq v (make-array 300 :element-type '(unsigned-byte 8)))
(dotimes (i (length v)) (setf (aref v i) (mod i 256)))

(sql:table-exists-p [mytab])
(sql:drop-table [mytab] :database pg :if-does-not-exist :ignore)
(sql:create-table [mytab]
  `(([id] :serial :primary-key)
    ([int_] integer) ; :constraint [emp-empno]
    ;         :references [emp] [empno] :on-delete :set-null) ; on-update doesn't exists
    ([char_] (string 32) :default "Mmyyytable default")
                     ; :check ,[and [> [id] 0] [>= [text-moo] "m%"]]))
    ([smallint_] (integer -1000 1000))
    ([bool_] boolean)
    ([date_] date)
    ([ts_] lt:local-time)
    ([real_] single-float)
    ([double_] double-float)
    ([raw_] (:bit 64)) ;([raw_] (bit-vector 64))
    ([longtext_] (string 20000))
    ;([longraw_] (vector unsigned-byte 30000)) ;      LONG!
    ([clob_] :clob) ;([clob_] :text)
    ([blob_] :blob) ;([blob_] :bytea)
   )
  :constraints
   '(;([table1-id] :foreign-key ([tab-id]) :references [tab] [id])
    ; (:check ,[> [tab-id] 5]) ; MSA not supported
   )
  :with 'oids
  :database pg
)
(odbc::db-describe-columns pg "mytab") ;=>
(;("" "public" "mytab" "id" 4 "int4" 10 4 0 10 0 "" NIL 4 NIL NIL 1 NIL 11 23)
 (NIL "public" "mytab" "id" 4 "int4" 10 4 0 10 0 "" "nextval('mytab_id_seq'::regclass)" 4 NIL NIL 1 NIL 11 23 1 1 20290)
 ("" "public" "mytab" "char_" 12 "varchar" 32 32 NIL NIL 1 "" NIL 12 NIL 254 2 NIL 32 1043)
 ("" "public" "mytab" "smallint_" 5 "int2" 5 2 0 10 1 "" NIL 5 NIL NIL 3 NIL 6 21)
 ("" "public" "mytab" "bool_" 1 "bool" 1 1 0 NIL 1 "" NIL 1 NIL NIL 5 NIL 1 16)
 ("" "public" "mytab" "date_" 91 "date" 10 6 NIL NIL 1 "" NIL 9 1 NIL 4 NIL 10 1082)
 (NIL "public" "mytab" "ts_" 93 "timestamp" 19 16 NIL NIL 1 "" NIL 9 3 NIL 7 NIL 19 1114 0 7 20319)
 ("" "public" "mytab" "real_" 7 "float4" 7 4 0 10 1 "" NIL 7 NIL NIL 5 NIL 13 700)
 ("" "public" "mytab" "double_" 6 "float8" 15 8 0 10 1 "" NIL 6 NIL NIL 6 NIL 22 701)
 ("" "public" "mytab" "raw_" 12 "bit" 254 254 NIL NIL 1 "" NIL 12 NIL NIL 7 NIL 254 1560)
 ("" "public" "mytab" "longtext_" -1 "varchar" 20000 20000 NIL NIL 1 "" NIL -1 NIL 254 8 NIL 20000 1043)
 ("" "public" "mytab" "clob_" -1 "text" 8190 8190 NIL NIL 1 "" NIL -1 NIL NIL 9 NIL 8190 25)
 ("" "public" "mytab" "blob_" -3 "bytea" 254 254 NIL NIL 1 "" NIL -3 NIL NIL 10 NIL 254 17)) 
("TABLE_QUALIFIER" "TABLE_OWNER" "TABLE_NAME" "COLUMN_NAME" "DATA_TYPE" "TYPE_NAME" "PRECISION" "LENGTH" "SCALE" "RADIX" "NULLABLE" "REMARKS" "COLUMN_DEF" "SQL_DATA_TYPE" "SQL_DATETIME_SUB" "CHAR_OCTET_LENGTH" "ORDINAL_POSITION" "IS_NULLABLE" "DISPLAY_SIZE" "FIELD_TYPE" "AUTO_INCREMENT" "PHYSICAL NUMBER" "TABLE OID") 

(sql:attribute-type "bool_" "mytab") ;=> :|bool|, 1, 0, 1

(sql:create-index [idx-employee-last-name] :on [employee] :columns [last-name])
(sql:drop-index "idx_employee_last_name")
(sql:sql (sql:create-index [idx-employee-name] :on [employee]
                  :columns [upper [|| [last-name] #\Space [first-name]]]
                  :unique t))
(sql:execute-command "CREATE UNIQUE INDEX idx_employee_name ON employee (upper(last_name || ' ' || first_name))")
(sql:drop-index "idx_employee_name")
(sql:list-indexes :on [employee])
(odbc::db-describe-statistics sql:*default-database* "employee" :schema "public")

(odbc::db-describe-statistics sql:*default-database* "foo") ;=>
((NIL "public" "foo" 1 NIL "foo_pkey" 3 1 "bar" "A" NIL NIL NIL)
 (NIL "public" "foo" 1 NIL "foo_baz_key" 3 1 "baz" "A" NIL NIL NIL)) 
("TABLE_QUALIFIER" "TABLE_OWNER" "TABLE_NAME" "NON_UNIQUE" "INDEX_QUALIFIER" "INDEX_NAME" "TYPE" "SEQ_IN_INDEX" "COLUMN_NAME" "COLLATION" "CARDINALITY" "PAGES" "FILTER_CONDITION") 

(odbc::db-describe-primary-keys pg [mytab])
(odbc::db-describe-primary-keys pg [employee])

;; Sequences
(sql:execute-command "CREATE SEQUENCE MY_SEQ;")
(sql:commit)
(sql:select [*] :from 'my_seq)			; to examine the generator
(sql:query "SELECT nextval('MY_SEQ'), currval('MY_SEQ')")
(sql:execute-command "DROP SEQUENCE MY_SEQ;")

;;; DML
(sql:insert-records :into [mytab]
  :attributes '([int_] [char_] [smallint_] [bool_] [date_] [ts_] [real_] [double_]
                [longtext_] [clob_])
               ;[raw_] [blob_])
 :values `(105 "  " 9876 :false "10.03.2003" "2006-04-15 13:25"
           43.21 123.456789
           "23 Longtext   by insert"  "CLOB   by insert"))
           ;"\\001" ; #*0011001000110011  ; ,v))

(sql:insert-records :into [mytab]
  :attributes '([id] [int_] [char_] [smallint_])
  :values `(3 104 "  " ,[function 'nextval "my_seq"]))

(sql:commit :database pg)

(sql:query "SELECT * FROM mytab") ;" where id>120")

(sql:insert-records :into [tab2]
                    :attributes '([id] [text2_])
                    :values (list 10 "  "))
(sql:insert-records :into [tab2]
                    :av-pairs '(([id] 11) ([text2_] "  ")))
(sql:insert-records :into [tab2]
                    :attributes '([id] [text2_])
                    :query [SELECT ["tt" ID] ["tt" TEXT_]
                                   :FROM [tab "tt"]
                                   :where [< ["tt" ID] 15]])
(sql:insert-records :into [tab2] :attributes [id] :values 40)
(setq n 14)
(sql:delete-records :from [mytab] :where [< [ID] n]])

(sql:update-records [tab2] :attributes [text2_] :values " "
                           :where [= [ID] 5])

;;; Deleting and updating joins is not supported!
(sql:delete-records :from [left-join [mytab] [employee]
                                     :on [= [mytab id] [employee emplid]]]
                    :where [null [employee emplid]])

(sql:update-records [inner-join [mytab] [employee] :on [= [mytab id] [employee emplid]]]
                    :attributes `(,[char_]) :values " ")
                           :where [>= [ID] 5])


(sql:execute-command "UPDATE EMPLOYEE SET BIRTH_TS = {ts '2006-09-27 12:00:05.515'} WHERE (EMPLID = 1)")

(sql:create-view 'mytab_oid :as [select [oid] [*] :from [mytab]])


;; SQL_BINARY

(create-table
 [foo raw-test]
 '((id integer)
   (bar (raw 10))
   (a (varchar 8))))

(insert-records :into [foo raw-test]
                :values (list 1 #*101110010101000011100000000 "raw!"))

;;; Queries

(setq s (sql:select [oid] [xmin] [cmin] [mytab *] :from [mytab] :order-by [id]))
(sql:select [*] :from [mytab-oid])

(sql:query "SELECT * FROM weather;")
(sql:select [city] [temp-lo] :from [weather])

(sql:select [*] :from [mytab] :order-by [date_ :desc])
(sql:query [select [id] [bool_ :type boolean] :as [i]
                   :from [mytab]
                   :order-by '([id] :desc [char_])])
(sql:select [char_] :from [mytab] :flatp t :distinct t)

(sql:query "SELECT ID FROM tab" :flatp t)
(sql:map-query 'nil (lambda (id text) (format t "~&id=~S txt=~S" id text)
                                       id)
               [select [id] [text_] :from [tab] :order-by [id :desc] :flatp nil])

(sql:select [emplid] [xmin] :from [employee] :timeout 3)
(sql:select [avg [emplid]] :from [employee] ;:result-types '(double-float)
            :flatp t)
(sql:select [*] :from [big])

(time (sql:query "SELECT char_,real_ FROM mytab" :database pg :timeout 5))
(sql:query "SELECT TOP 5 ID,Boolean_,DateTime_,Byte_,Short_,Double_ FROM tab" :database a)


;;; Parameters, prepared statements and procedures

(ystok.sql.sys:collect-parameters [select [Text_] :from [tab]
                                    :where [< [id] [? :name a]]
                                    :order-by [? :name b]])
(sql:query "SELECT * FROM mytab WHERE id >= 1 ORDER BY id")
(defvar id 35)
(setq pp (sql:prepare-statement 
    [insert :into [mytab]
            :columns '([id] [char_] [int_] [smallint_] [bool_] [date_] [real_]
                       [double_] 
                       [raw_] [longtext_] [clob_] [blob_])
            :values  '([? :name id	:type :integer]
                       [? :name char_]
                       [? :name int_	:type :integer]
                       [? :name smallint_ :type (integer -1000 1000)]
                       [? :name boolean_ :type boolean]
                       [? :name date_	:type sql:universal-time]
                       [? :name double_ :type double-float]
                       [? :name real_	:type single-float] ; works incorrectly in LW
                       [? :name raw_	:type bit-vector]
                       [? :name longtext_ :type (string 5000)]
                       [? :name clob_]
                       [? :name blob_ :type (vector unsigned-byte 300)])]
    :database pg))
(sql:execute-command pp
 :values `(,(incf id) "  prepared" 105 987 :true
           ,(lt:current-date) 123.456789 3.5
           #*0011001000110011  "23 Longtext  prepared"  nil nil))
           ;#*0011001000110011 "23 Longtext  prepared"  "CLOB  " ,v))

(odbc::%execute-parameterized pp 
    [insert :into [mytab]
            :columns '([id] [char_] [int_] [smallint_] [date_] [real_] [double_]
                       [raw_] [longtext_] [clob_] [blob_])
            :values  '([? :name id	:type integer]
                       [? :name empno	:type integer]
                       [? :name char_]
                       [? :name smallint_ :type (integer -1000 1000)]
                       [? :name date_	:type :universal-time]
                       [? :name real_	:type single-float] ; works incorrectly in LW
                       [? :name double_ :type double-float]
                       [? :name raw_	:type bit-vector]
                       [? :name longtext_ :type (string 5000)]
                       [? :name clob_]
                       [? :name blob_])]
 :values `(,(incf id) 105 "  prepared" 987 ,(get-universal-time) 33.21 123.456789
           #*0011001000110011  "23 Longtext  prepared" nil ,v))

(sql:commit :database pg)
(sql:close-statement pp :drop-p t)
(sql:cancel-statement pp)
(sql:rollback :database pg)

(sql:do-query ((id raw_ blob_)
               [select [id] [raw_] [blob_] :from [mytab] :where [>= [id] (1- id)]])
    (format t "~&id=~S raw_(~S)=~S~%blob_(~S)=~S"
            id (type-of raw_) raw_ (type-of blob_) blob_))

(setq pqp (odbc::db-prepare-statement pg
    [select [id] [char_] :from [mytab] :where [= [id] [? :name id :type :integer]]]))
(odbc::%execute-parameterized pqp nil nil '(2))
(sql:fetch-rows pqp)
(sql:close-statement pqp :drop-p nil)

(setq po (sql:prepare-statement 
          [insert :into [mytab] :attributes '([id] [char_])
                  :values  '([? :name id :type integer] [? :name char_])]
          :database o :paramset-size 2))
(sql:execute-command po :values '((44 45) ("text 44" "text 45")))

(setq po (sql:prepare-statement 
          [insert :into [mytab] :attributes '([id] [longtext_] [blob_])
                  :values  '([? :name id :type integer]
                             [? :type (string 5000)]
                             [? :name blob_ :type (vector unsigned-byte 6000)])]
          :database o :paramset-size 2))
(setq s1 (random-string 200) s2 (random-string 300)
      v1 (increasing-vector 300) v2 (increasing-vector 400))
(sql:execute-command po :values (list '(54 55) (list s1 s2) (list v1 v2)))

(sql:select [id] [longtext_] [blob_] [char_]  :from [mytab] :where [>= [id] 54]
            :database o)



(with-prepared-statement (query [insert :into [tab] :attributes '([id] [Text_]:values '($ $ $)])
  (bind-execute query "" 1001 (get-universal-time)))

(time
 (dotimes (i 1000)
   (with-transaction
     (insert-records :into [Table1]
                     :values (list (format nil "~A" (random 20)) i)))))
; cpu time (non-gc) 22,534 msec user, 17,315 msec system
; cpu time (gc)     520 msec user, 0 msec system
; cpu time (total)  23,054 msec user, 17,315 msec system
; real time  416,408 msec (00:06:56.408)
; space allocation:
;  169,716 cons cells, 674,656 other bytes, 0 static bytes

;; much faster:
(time
 (progn
  (with-prepared-statement (query [insert :into [Table1] :values '($ $ $)])
    (dotimes (i 1000)
      (with-transaction ()
        (bind-execute query (format nil "~A - " (random 20)) 
                      i
                      (+ (get-universal-time) (random 10000))))))))
;8.4 seconds used.
;Standard Allocation 348048 bytes.
;Fixlen Allocation 89595 bytes.
; cpu time (non-gc) 16,665 msec user, 14,191 msec system
; cpu time (gc)     490 msec user, 0 msec system
; cpu time (total)  17,155 msec user, 14,191 msec system
; real time  327,922 msec (00:05:27.922)
; space allocation:
;  152,160 cons cells, 371,568 other bytes, 1602 static bytes

(time (do-query ((txt l d) "SELECT TOP 1000 text1,long1,datetime1 FROM table1")
  (let ((s (format nil "~&txt=~S l=~S d=~S" 
                   txt l (and d (format-time d))))))))
;76.7 seconds used.
;Standard Allocation 702080 bytes.
;Fixlen Allocation 861531 bytes.

;;; Escape sequences, outer joins
(sql:query [select [emp empno] [ename] [mytab empno] [char_]
                   :from [right-join [emp] [mytab]
                                    :on [= [emp empno] [mytab empno]]]]
           :database o)

(sql:query [select [emp empno] [ename] [mytab empno] [char_]
                   :from [inner-join [emp] [mytab]
                                    :on [= [emp empno] [mytab empno]]]]
           :database o)
(sql:query [select [emp empno] [ename] [mytab empno] [char_]
                   :from '([emp] [mytab])
                   :where [= [emp empno] [mytab empno]]]
           :database o)

(sql:execute-command "INSERT INTO tab(DateTime_) VALUES({ts'2003-02-13 13:30:31'})")
(my-execute-command "{CALL testfun}")	;; VB-functions not supported

(do-query ((a b) "SELECT Text_,Text2_ FROM outer_join;")
  (format t "~&a=~S:~S b=~S:~S" 
    a (type-of a) b (type-of b)))
(do-query ((a b) "SELECT {fn CONCAT(tab.Text_,tab.Memo_)}, {fn ABS(tab2.Text2_)} 
 FROM {oj tab LEFT OUTER JOIN tab2 ON tab.id = tab2.ID};")
  (format t "~&a=~S:~S b=~S:~S" 
    a (type-of a) b (type-of b)))

(my-execute-command "{CALL tab_update_text(6+5)}")
(do-query ((a b) "{CALL qTabText_ByID(2,2+3)}") 
  (format t "~&a=~S b=~S" a b))

;;; Cursors
(defun print-row-status-array (cursor)
  (with-slots (odbc::row-descriptor) cursor
    (with-slots (odbc::array-size odbc::current-row-status) odbc::row-descriptor
      (if (> odbc::array-size 1)
          (dotimes (i odbc::array-size)
            (format t "~&row-status[~A]=~S" i
                    (case (odbc::desc-row-status odbc::row-descriptor i)
                      (#.odbc::SQL_ROW_SUCCESS 'odbc::SQL_ROW_SUCCESS)
                      (#.odbc::SQL_ROW_SUCCESS_WITH_INFO 'odbc::SQL_ROW_SUCCESS_WITH_INFO)
                      (#.odbc::SQL_ROW_UPDATED 'odbc::SQL_ROW_UPDATED)
                      (#.odbc::SQL_ROW_ADDED   'odbc::SQL_ROW_ADDED)
                      (#.odbc::SQL_ROW_ERROR   'odbc::SQL_ROW_ERROR)
                      (#.odbc::SQL_ROW_DELETED 'odbc::SQL_ROW_DELETED) 
                      (#.odbc::SQL_ROW_NOROW   'odbc::SQL_ROW_NOROW)
                      (otherwise (odbc::desc-row-status odbc::row-descriptor i)))))
          (format t "~&(array-size=1) : current-row-status=~S"
                  odbc::current-row-status)))))

(odbc::%sql-get-attribute :hdbc (odbc::handle o) odbc::SQL_ATTR_AUTOCOMMIT)
(setq pq (sql:open-cursor ; [select [*] :from [mytab-oid]] - ctid does not exists!
                           [select [oid] [id] :from [mytab]]
                           :database pg
                           :cursor-type :keyset-driven
                           ;:concurrency :optimistic
                           :rowset-size 1
                          ))
(setq pq (sql:open-cursor [select [*] :from [mytab]]
                           :database pg
                           :cursor-type :keyset-driven
                           :concurrency :optimistic
                           :start-column 0   ;=> Processor fault sometimes
                           :rowset-size 1))

(sql:execute-command "CREATE TABLE t_table (t_float float8)")
(sql:execute-command "INSERT INTO t_table (t_float) VALUES (-8.0E-8)")
(setq pq3 (sql:open-cursor "SELECT t_float FROM t_table" :result-types '((string 30))))
(setq pq3 (sql:open-cursor "SELECT * FROM TYPE_BIGINT" :database pg))

(sql:fetch-rows pq :count 2 :flatp nil)
(setq row (sql:fetch-rows  pq3 :count 1 :flatp nil))
(odbc::set-cursor-position pq :next)
(odbc::set-cursor-position pq :prior)
(odbc::set-cursor-position pq3 :first)
(odbc::set-cursor-position pq :relative :offset -4)
(odbc::set-cursor-position pq :absolute :offset 2)
(odbc::set-cursor-position pq :bookmark :offset 6)
(sql:fetch-rows pq :count 1)
(odbc::%sql-fetch (odbc::handle aq))
(odbc::%sql-fetch-scroll (odbc::handle aq) :relative :offset 0)
(odbc::%sql-delete-record (odbc::handle oq) 1)
(odbc::%sql-update-record (odbc::handle oq) 1)

(odbc::%sql-set-attribute :hstmt (odbc::handle oq) odbc::SQL_ATTR_ROW_STATUS_PTR
                            (odbc::desc-array-status-ptr (odbc::row-descriptor oq)))
(print-row-status-array pq)
(odbc::%sql-refresh-record (odbc::handle pq) 1)
(sql:current-row-status pq)
(sql:current-row-number pq)
(odbc::%sql-get-attribute :hstmt (odbc::handle pq) odbc::SQL_ATTR_ROW_NUMBER)
;=> row number in entire result set, not in rowset - works!
(odbc::%sql-get-attribute :hstmt (odbc::handle pq) odbc::SQL_ATTR_CONCURRENCY)
;=>3 (SQL_CONCUR_ROWVER)
(odbc::%sql-get-attribute :hstmt (odbc::handle pq) odbc::SQL_ATTR_KEYSET_SIZE) ;=>0

(sql:update-records pq :columns '([char_] [date_] [real_])
                       :values `(,(format nil " ~D  " (random 100))
                                 ,(get-universal-time)
                                 56.2))
                       :where 3)
(sql:insert-records :into pq
                    :columns '( [char_] [date_] [real_])
                    :values `(;123
                              ,(format nil " ~D  " (random 100))
                              ,(lt:localtimestamp)
                              99.9))
;Message: [Microsoft][ODBC driver for Oracle][Oracle]ORA-00001:    (SCOTT.PK_EMP). ???

(sql:delete-records :from pq) ; :where 515)
(sql:close-statement pq)
(sql:commit)

;;; The driver seems to fail to deliver long column data when rowset-size > 1
;;; SQLSetPos followed by SQLGetData crashes.
(setq oc (ystok.sql.sys:db-open-cursor o [select [ROWID] 
                                  [id] [empno] [char_] [smallint_] [date_] [double_]
                                  [raw_] [longtext_] [clob_] [blob_] 
                                                 :from [mytab]]
                                 :cursor-type :keyset-driven
                                 :concurrency :optimistic
                                 :rowset-size 1
                                 :start-column 1 ;0 bookmarks are not supported!
                                 ))
(odbc::%sql-get-attribute :hstmt (odbc::handle pq) odbc::SQL_ATTR_QUERY_TIMEOUT)
(odbc::%sql-set-attribute :hstmt (odbc::handle oc) odbc::SQL_ATTR_QUERY_TIMEOUT 1)


(sql:fetch-rows oc :count 4 :flatp nil)
(sql:fetch-rows oc :count nil)
(odbc::set-cursor-position oc :absolute :offset 1)
(odbc::set-cursor-position oc :relative :offset 0)
(odbc::set-cursor-position oc :prior)
(sql:update-records oc :attributes '([date_] [char_] [clob_])
                       :values `(,(get-universal-time)
                                 ,(format nil "   ~D" (random 100))
                                 "long updated via cursor"))
(odbc::%sql-refresh-record (odbc::handle oc) 0)
(print-row-status-array oc)

(sql:insert-records :into oc
      :attributes '([id] [date_] [char_]) ; [clob_])
      :values `(144
                ,(get-universal-time)
                ,(format nil "   ~D" (random 100))
                )) ;"long inserted via cursor"))
;;    ROWID .
(sql:close-statement oc)
(sql:cancel-statement oc)
(sql:commit)

;;; Join leads to non-updateable cursor
(setq pq (sql:open-cursor [select [mytab *] [employee last-name]
                                  :from [inner-join [mytab] [employee]
                                                    :on [= [mytab id] [employee emplid]]]]
                           :database pg
                           :cursor-type :keyset-driven
                           :concurrency :optimistic
                           :start-column 0
                           :rowset-size 1))
(odbc::%sql-get-attribute :hstmt (odbc::handle pq) odbc::SQL_ATTR_CONCURRENCY)
;=> 1=SQL_CONCUR_READ_ONLY

;;; System information and administration

(get-working-directory)
(change-directory #P"D:/PostgreSQL/8.1/bin/")
(setf (environment-variable "PGPASSWORD") "pg10")
(sys:call-system-showing-output "dropdb --host localhost --username postgres test811")

(sql:create-database '("test811" :user "postgres" :password "pg10")
                     :database-type :postgresql)
(sql:destroy-database "test811/postgres/pg10" :database-type :postgresql)

(sql:query "select CURRENT_USER,current_database(),current_schema,current_schemas(TRUE)")
(sql:query "select inet_client_addr(),inet_client_port(),inet_server_addr(),inet_server_port()")
;=>(("127.0.0.1" 1049 "127.0.0.1" 5432))
(sql:query "SELECT current_setting('datestyle')")

#| Unused stuff
(create-user [foo] :password "bar")
(execute :grant '(connect resource) :to [foo])

 |#

