;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2005 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Constants from 
;;;  SQLEXT.H	This is the include for applications using the Microsoft SQL Extensions

(in-package :ystok.odbc)

;; generally useful constants
(define-symbol-macro SQL_SPEC_MAJOR (truncate $ODBCVER 100))	; Major version
(define-symbol-macro SQL_SPEC_MINOR (mod $ODBCVER 100))		; Minor version
(define-symbol-macro SQL_SPEC_STRING (format nil "~2,'0D.~2,'0D"
                                             SQL_SPEC_MAJOR SQL_SPEC_MINOR))

(defconstant SQL_SQLSTATE_SIZE 5)		; size of SQLSTATE
(defconstant SQL_MAX_DSN_LENGTH 32)		; maximum data source name size  

(defconstant SQL_MAX_OPTION_STRING_LENGTH 256)

;; sqlext.h
(defconstant SQL_LEN_DATA_AT_EXEC_OFFSET -100)
(defmacro %sql-len-data-at-exec (length) `(- SQL_LEN_DATA_AT_EXEC_OFFSET ,length))

;; binary length for driver specific attributes
;(defconstant SQL_LEN_BINARY_ATTR_OFFSET	 (-100)
;(defmacro %sql_len_binary_attr(length)	`(- SQL_LEN_BINARY_ATTR_OFFSET ,length))


;; /* return code SQL_NO_DATA_FOUND is the same as SQL_NO_DATA */
;; #if (ODBCVER < 0x0300)
;; (defconstant SQL_NO_DATA_FOUND 100)
;; #endif
;(defconstant SQL_NO_DATA_FOUND	SQL_NO_DATA)

;; /* an end handle type */
;; #if (ODBCVER >= 0x0300)
;#define SQL_HANDLE_SENV 5
;; #endif  /* ODBCVER >= 0x0300 */

;; /* env attribute */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_ATTR_ODBC_VERSION		200)
(defconstant SQL_ATTR_CONNECTION_POOLING	201)
(defconstant SQL_ATTR_CP_MATCH			202)
;; #endif  /* ODBCVER >= 0x0300 */

;; #if (ODBCVER >= 0x0300)
;; /* values for SQL_ATTR_CONNECTION_POOLING */
(defconstant SQL_CP_OFF			0) ;UL
(defconstant SQL_CP_ONE_PER_DRIVER	1) ;UL
(defconstant SQL_CP_ONE_PER_HENV	2) ;UL
(defconstant SQL_CP_DEFAULT		SQL_CP_OFF)

;; /* values for SQL_ATTR_CP_MATCH */
(defconstant SQL_CP_STRICT_MATCH	0) ;UL
(defconstant SQL_CP_RELAXED_MATCH	1) ;UL
(defconstant SQL_CP_MATCH_DEFAULT	SQL_CP_STRICT_MATCH)

;; /* values for SQL_ATTR_ODBC_VERSION */
(defconstant SQL_OV_ODBC2		2) ;UL
(defconstant SQL_OV_ODBC3		3) ;UL
;; #endif  /* ODBCVER >= 0x0300 */
 
;; options for SQLSetConnectOption/SQLGetConnectOption)
;(defconstant SQL_ACCESS_MODE	101)
;(defconstant SQL_AUTOCOMMIT	102)
;(defconstant SQL_LOGIN_TIMEOUT	103)
;(defconstant SQL_OPT_TRACE	104)
;(defconstant SQL_OPT_TRACEFILE	105)
;(defconstant SQL_TRANSLATE_DLL	106)
;(defconstant SQL_TRANSLATE_OPTION 107)
;(defconstant SQL_TXN_ISOLATION	108) 
;(defconstant SQL_CURRENT_QUALIFIER 109)
;(defconstant SQL_ODBC_CURSORS	110)
;(defconstant SQL_QUIET_MODE	111)
;(defconstant SQL_PACKET_SIZE	112)
;(defconstant SQL_CONN_OPT_MAX	SQL_PACKET_SIZE)

;; /* connection attributes with new names */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_ATTR_ACCESS_MODE	 101) ;SQL_ACCESS_MODE
(defconstant SQL_ATTR_AUTOCOMMIT	 102) ;SQL_AUTOCOMMIT
(defconstant SQL_ATTR_CONNECTION_TIMEOUT 113)
(defconstant SQL_ATTR_CURRENT_CATALOG	 109) ;SQL_CURRENT_QUALIFIER)
(defconstant SQL_ATTR_DISCONNECT_BEHAVIOR 114)
(defconstant SQL_ATTR_ENLIST_IN_DTC	1207)
(defconstant SQL_ATTR_ENLIST_IN_XA	1208)
(defconstant SQL_ATTR_LOGIN_TIMEOUT	 103) ;SQL_LOGIN_TIMEOUT
(defconstant SQL_ATTR_ODBC_CURSORS	 110) ;SQL_ODBC_CURSORS
(defconstant SQL_ATTR_PACKET_SIZE	 112) ;SQL_PACKET_SIZE)
(defconstant SQL_ATTR_QUIET_MODE	 111) ;SQL_QUIET_MODE
(defconstant SQL_ATTR_TRACE		 104) ;SQL_OPT_TRACE
(defconstant SQL_ATTR_TRACEFILE		 104) ;SQL_OPT_TRACEFILE
(defconstant SQL_ATTR_TRANSLATE_LIB	 106) ;SQL_TRANSLATE_DLL
(defconstant SQL_ATTR_TRANSLATE_OPTION	 107) ;SQL_TRANSLATE_OPTION
(defconstant SQL_ATTR_TXN_ISOLATION	 108) ;SQL_TXN_ISOLATION
;; #endif  /* ODBCVER >= 0x0300 */

(defconstant SQL_ATTR_CONNECTION_DEAD	1209)	; GetConnectAttr only 

;; #if (ODBCVER >= 0x0351)
#|	ODBC Driver Manager sets this connection attribute to a unicode driver 
	(which supports SQLConnectW) when the application is an ANSI application
	(which calls SQLConnect, SQLDriverConnect, or SQLBrowseConnect). 
	This is SetConnectAttr only and application does not set this attribute
	This attribute was introduced because some unicode driver's some APIs may 
	need to behave differently on ANSI or Unicode applications. A unicode 
	driver, which  has same behavior for both ANSI or Unicode applications,
	should return SQL_ERROR when the driver manager sets this connection 
	attribute. When a unicode driver returns SQL_SUCCESS on this attribute,
	the driver manager treates ANSI and Unicode connections differently in
	connection pooling.
 |#
(defconstant SQL_ATTR_ANSI_APP	115)
;; #endif

;; #if (ODBCVER < 0x0300)
(defconstant SQL_CONNECT_OPT_DRVR_START 1000)
;; #endif  /* ODBCVER < 0x0300 */

;; #if (ODBCVER < 0x0300)
;(defconstant SQL_CONN_OPT_MAX SQL_PACKET_SIZE)
;(defconstant SQL_CONN_OPT_MIN SQL_ACCESS_MODE)
;; #endif /* ODBCVER < 0x0300 */

;; SQL_ACCESS_MODE options
(defconstant SQL_MODE_READ_WRITE 0) ;UL
(defconstant SQL_MODE_READ_ONLY  1) ;UL
(defconstant SQL_MODE_DEFAULT	 SQL_MODE_READ_WRITE)

;; SQL_AUTOCOMMIT options
(defconstant SQL_AUTOCOMMIT_OFF	 0) ;UL
(defconstant SQL_AUTOCOMMIT_ON	 1) ;UL
(defconstant SQL_AUTOCOMMIT_DEFAULT SQL_AUTOCOMMIT_ON)

;; SQL_LOGIN_TIMEOUT options
(defconstant SQL_LOGIN_TIMEOUT_DEFAULT 15) ;15UL

;; SQL_OPT_TRACE options
(defconstant SQL_OPT_TRACE_OFF	0) ; 0UL
(defconstant SQL_OPT_TRACE_ON	1) ; 1UL
(defconstant SQL_OPT_TRACE_DEFAULT SQL_OPT_TRACE_OFF)
; (defconstant SQL_OPT_TRACE_FILE_DEFAULT	"\\SQL.LOG")

;; SQL_ODBC_CURSORS options)
(defconstant SQL_CUR_USE_IF_NEEDED	0) ;0UL
(defconstant SQL_CUR_USE_ODBC		1) ;1UL
(defconstant SQL_CUR_USE_DRIVER		2) ;2UL
(defconstant SQL_CUR_DEFAULT		SQL_CUR_USE_DRIVER)

;; #if (ODBCVER >= 0x0300)
;; /* values for SQL_ATTR_DISCONNECT_BEHAVIOR */
(defconstant SQL_DB_RETURN_TO_POOL	0) ;UL
(defconstant SQL_DB_DISCONNECT		1) ;UL
(defconstant SQL_DB_DEFAULT		SQL_DB_RETURN_TO_POOL)

;; /* values for SQL_ATTR_ENLIST_IN_DTC */
(defconstant SQL_DTC_DONE		0)
;; #endif  /* ODBCVER >= 0x0300 */

;; /* values for SQL_ATTR_CONNECTION_DEAD */
(defconstant SQL_CD_TRUE	1) ;1L	/* Connection is closed/dead */
(defconstant SQL_CD_FALSE	0) ;0L	/* Connection is open/available */

;; /* values for SQL_ATTR_ANSI_APP */
;; #if (ODBCVER >= 0x0351)
(defconstant SQL_AA_TRUE	1) ;1L	/* the application is an ANSI app */
(defconstant SQL_AA_FALSE	0) ;0L	/* the application is a Unicode app */
;; #endif

;;/* statement attributes */
(defconstant SQL_QUERY_TIMEOUT	0)
;(defconstant SQL_MAX_ROWS	1)
;(defconstant SQL_NOSCAN	2)
;(defconstant SQL_MAX_LENGTH	3)
;(defconstant SQL_ASYNC_ENABLE	4)
;(defconstant SQL_BIND_TYPE	5)
;(defconstant SQL_CURSOR_TYPE 	6)
;(defconstant SQL_CONCURRENCY 	7)
;(defconstant SQL_KEYSET_SIZE 	8)
(defconstant SQL_ROWSET_SIZE 	9)
;(defconstant SQL_SIMULATE_CURSOR 10)
;(defconstant SQL_RETRIEVE_DATA	11)
;(defconstant SQL_USE_BOOKMARKS	12)
(defconstant SQL_GET_BOOKMARK	13)	; GetStmtOption Only
(defconstant SQL_ROW_NUMBER	14)	; GetStmtOption Only

;; /* statement attributes for ODBC 3.0 */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_ATTR_ASYNC_ENABLE		 4) ;SQL_ASYNC_ENABLE
(defconstant SQL_ATTR_CONCURRENCY		 7) ;SQL_CONCURRENCY
(defconstant SQL_ATTR_CURSOR_TYPE		 6) ;SQL_CURSOR_TYPE
(defconstant SQL_ATTR_ENABLE_AUTO_IPD		15)
(defconstant SQL_ATTR_FETCH_BOOKMARK_PTR	16)
(defconstant SQL_ATTR_KEYSET_SIZE		 8) ;SQL_KEYSET_SIZE
(defconstant SQL_ATTR_MAX_LENGTH		 3) ;SQL_MAX_LENGTH
(defconstant SQL_ATTR_MAX_ROWS			 1) ;SQL_MAX_ROWS
(defconstant SQL_ATTR_NOSCAN			 2) ;SQL_NOSCAN
(defconstant SQL_ATTR_PARAM_BIND_OFFSET_PTR	17)
(defconstant SQL_ATTR_PARAM_BIND_TYPE		18)
(defconstant SQL_ATTR_PARAM_OPERATION_PTR	19)
(defconstant SQL_ATTR_PARAM_STATUS_PTR		20)
(defconstant SQL_ATTR_PARAMS_PROCESSED_PTR	21)
(defconstant SQL_ATTR_PARAMSET_SIZE		22)
(defconstant SQL_ATTR_QUERY_TIMEOUT		 0) ;SQL_QUERY_TIMEOUT
(defconstant SQL_ATTR_RETRIEVE_DATA		11) ;SQL_RETRIEVE_DATA
(defconstant SQL_ATTR_ROW_BIND_OFFSET_PTR	23)
(defconstant SQL_ATTR_ROW_BIND_TYPE		 5) ;SQL_BIND_TYPE
(defconstant SQL_ATTR_ROW_NUMBER		14) ;SQL_ROW_NUMBER
(defconstant SQL_ATTR_ROW_OPERATION_PTR		24)
(defconstant SQL_ATTR_ROW_STATUS_PTR		25)
(defconstant SQL_ATTR_ROWS_FETCHED_PTR		26)
(defconstant SQL_ATTR_ROW_ARRAY_SIZE		27)	
(defconstant SQL_ATTR_SIMULATE_CURSOR		10) ;SQL_SIMULATE_CURSOR
(defconstant SQL_ATTR_USE_BOOKMARKS		12) ;SQL_USE_BOOKMARKS
;; #endif /* ODBCVER >= 0x0300 */

;; #if (ODBCVER < 0x0300)
;(defconstant SQL_STMT_OPT_MAX	SQL_ROW_NUMBER)
;(defconstant SQL_STMT_OPT_MIN	SQL_QUERY_TIMEOUT)
;;#endif /* ODBCVER < 0x0300 */

;; /* New defines for SEARCHABLE column in SQLGetTypeInfo */
;; #if (ODBCVER >= 0x0300)
;(defconstant SQL_COL_PRED_CHAR		SQL_LIKE_ONLY)
;(defconstant SQL_COL_PRED_BASIC	SQL_ALL_EXCEPT_LIKE)
;; #endif /* ODBCVER >= 0x0300 */

;; /* the value of SQL_ATTR_PARAM_BIND_TYPE */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_PARAM_BIND_BY_COLUMN	 0) ;UL
(defconstant SQL_PARAM_BIND_TYPE_DEFAULT SQL_PARAM_BIND_BY_COLUMN)
;; #endif  /* ODBCVER >= 0x0300 */

;; SQL_QUERY_TIMEOUT options
;(defconstant SQL_QUERY_TIMEOUT_DEFAULT	0)

;; SQL_MAX_ROWS options
;(defconstant SQL_MAX_ROWS_DEFAULT	0)

;; SQL_NOSCAN options
(defconstant SQL_NOSCAN_OFF		0)	; 1.0 FALSE
(defconstant SQL_NOSCAN_ON		1)	; 1.0 TRUE
;(defconstant SQL_NOSCAN_DEFAULT	SQL_NOSCAN_OFF)

;; SQL_MAX_LENGTH options
;(defconstant SQL_MAX_LENGTH_DEFAULT	0) ;UL

;; SQL_ASYNC_ENABLE options
(defconstant SQL_ASYNC_ENABLE_OFF	0) ;UL
(defconstant SQL_ASYNC_ENABLE_ON	1) ;UL
;(defconstant SQL_ASYNC_ENABLE_DEFAULT	SQL_ASYNC_ENABLE_OFF)

;; SQL_BIND_TYPE options
;(defconstant SQL_BIND_BY_COLUMN	0UL)
;(defconstant SQL_BIND_TYPE_DEFAULT	SQL_BIND_BY_COLUMN)		; Default value

;; SQL_CONCURRENCY options
(defconstant SQL_CONCUR_READ_ONLY	1)
(defconstant SQL_CONCUR_LOCK 		2)
(defconstant SQL_CONCUR_ROWVER		3)
(defconstant SQL_CONCUR_VALUES		4)
;(defconstant SQL_CONCUR_DEFAULT	SQL_CONCUR_READ_ONLY)		; Default value

;; SQL_CURSOR_TYPE options
(defconstant SQL_CURSOR_FORWARD_ONLY 	0)
(defconstant SQL_CURSOR_KEYSET_DRIVEN	1)
(defconstant SQL_CURSOR_DYNAMIC		2)
(defconstant SQL_CURSOR_STATIC		3)
;(defconstant SQL_CURSOR_TYPE_DEFAULT	SQL_CURSOR_FORWARD_ONLY)	; Default value
 
;; SQL_ROWSET_SIZE options
(defconstant SQL_ROWSET_SIZE_DEFAULT 1) ;UL

;; SQL_KEYSET_SIZE options
;(defconstant SQL_KEYSET_SIZE_DEFAULT 0) ;UL

;; SQL_SIMULATE_CURSOR options
;(defconstant SQL_SC_NON_UNIQUE	0)
;(defconstant SQL_SC_TRY_UNIQUE	1)
;(defconstant SQL_SC_UNIQUE	2)

;; SQL_RETRIEVE_DATA options
(defconstant SQL_RD_OFF		0)
(defconstant SQL_RD_ON		1)
;(defconstant SQL_RD_DEFAULT	SQL_RD_ON)

;; SQL_USE_BOOKMARKS options
(defconstant SQL_UB_OFF	0)
;(defconstant SQL_UB_ON		1)
;(defconstant SQL_UB_DEFAULT	SQL_UB_OFF)

;; New values for SQL_USE_BOOKMARKS attribute 
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_UB_FIXED	1) ;SQL_UB_ON
(defconstant SQL_UB_VARIABLE	2)
;; #endif /* ODBCVER >= 0x0300 */
 
;; /* extended descriptor field */
;; #if (ODBCVER >= 0x0300)
(def-desc-attribute-type SQL_DESC_ARRAY_SIZE		20 SQL_IS_UINTEGER)
(def-desc-attribute-type SQL_DESC_ARRAY_STATUS_PTR	21 SQL_IS_POINTER)
(def-desc-attribute-type SQL_DESC_AUTO_UNIQUE_VALUE	11 SQL_IS_INTEGER) ;SQL_COLUMN_AUTO_INCREMENT
(def-desc-attribute-type SQL_DESC_BASE_COLUMN_NAME	22 :string)
(def-desc-attribute-type SQL_DESC_BASE_TABLE_NAME	23 :string)
(def-desc-attribute-type SQL_DESC_BIND_OFFSET_PTR	24 SQL_IS_POINTER)
(def-desc-attribute-type SQL_DESC_BIND_TYPE		25 SQL_IS_UINTEGER)
(def-desc-attribute-type SQL_DESC_CASE_SENSITIVE	12 SQL_IS_INTEGER) ;SQL_COLUMN_CASE_SENSITIVE
(def-desc-attribute-type SQL_DESC_CATALOG_NAME		17 :string) ;SQL_COLUMN_QUALIFIER_NAME
(def-desc-attribute-type SQL_DESC_CONCISE_TYPE		 2 SQL_IS_SMALLINT) ;SQL_COLUMN_TYPE
;(def-desc-attribute-type SQL_DESC_DATETIME_INTERVAL_PRECISION 26 SQL_IS_SMALLINT)
(def-desc-attribute-type SQL_DESC_DISPLAY_SIZE		 6 SQL_IS_SMALLINT) ;SQL_COLUMN_DISPLAY_SIZE
(def-desc-attribute-type SQL_DESC_FIXED_PREC_SCALE	 9 SQL_IS_SMALLINT) ;SQL_COLUMN_MONEY
(def-desc-attribute-type SQL_DESC_LABEL			18 :string) ;SQL_COLUMN_LABEL
(def-desc-attribute-type SQL_DESC_LITERAL_PREFIX	27 :string)
(def-desc-attribute-type SQL_DESC_LITERAL_SUFFIX	28 :string)
(def-desc-attribute-type SQL_DESC_LOCAL_TYPE_NAME	29 :string)
;(def-desc-attribute-type SQL_DESC_MAXIMUM_SCALE	30)
;(def-desc-attribute-type SQL_DESC_MINIMUM_SCALE	31)
(def-desc-attribute-type SQL_DESC_NUM_PREC_RADIX	32 SQL_IS_INTEGER)
(def-desc-attribute-type SQL_DESC_PARAMETER_TYPE	33 SQL_IS_SMALLINT)
(def-desc-attribute-type SQL_DESC_ROWS_PROCESSED_PTR	34 SQL_IS_POINTER)
;; #if (ODBCVER >= 0x0350)
(def-desc-attribute-type SQL_DESC_ROWVER	35 SQL_IS_SMALLINT)
;; #endif
(def-desc-attribute-type SQL_DESC_SCHEMA_NAME	16 :string) ;SQL_COLUMN_OWNER_NAME
(def-desc-attribute-type SQL_DESC_SEARCHABLE	13 SQL_IS_SMALLINT) ;SQL_COLUMN_SEARCHABLE
(def-desc-attribute-type SQL_DESC_TYPE_NAME	14 :string) ;SQL_COLUMN_TYPE_NAME
(def-desc-attribute-type SQL_DESC_TABLE_NAME	15 :string) ;SQL_COLUMN_TABLE_NAME
(def-desc-attribute-type SQL_DESC_UNSIGNED	 8 SQL_IS_SMALLINT) ;SQL_COLUMN_UNSIGNED
(def-desc-attribute-type SQL_DESC_UPDATABLE	10 SQL_IS_SMALLINT) ;SQL_COLUMN_UPDATABLE
;; #endif /* ODBCVER >= 0x0300 */

;; SQLColAttributes defines - used to remap to 2.0 driver.
(defconstant SQL_COLUMN_COUNT 0)
(defconstant SQL_COLUMN_NAME 1)
;(defconstant SQL_COLUMN_TYPE 2)
(defconstant SQL_COLUMN_LENGTH 3)
(defconstant SQL_COLUMN_PRECISION 4)
(defconstant SQL_COLUMN_SCALE 5)
;(defconstant SQL_COLUMN_DISPLAY_SIZE 6)
(defconstant SQL_COLUMN_NULLABLE 7)
#|
(defconstant SQL_COLUMN_UNSIGNED 8)
(defconstant SQL_COLUMN_MONEY 9)
(defconstant SQL_COLUMN_UPDATABLE 10)
(defconstant SQL_COLUMN_AUTO_INCREMENT	11)
(defconstant SQL_COLUMN_CASE_SENSITIVE	12)
(defconstant SQL_COLUMN_SEARCHABLE 13)
(defconstant SQL_COLUMN_TYPE_NAME 14)
(defconstant SQL_COLUMN_TABLE_NAME 15)
(defconstant SQL_COLUMN_OWNER_NAME 16)
(defconstant SQL_COLUMN_QUALIFIER_NAME	17)
(defconstant SQL_COLUMN_LABEL 18)
(defconstant SQL_COLATT_OPT_MAX SQL_COLUMN_LABEL)
;; #if (ODBCVER < 0x0300)
(defconstant SQL_COLUMN_DRIVER_START	1000)
;; #endif /* ODBCVER < 0x0300 */
(defconstant SQL_COLATT_OPT_MIN SQL_COLUMN_COUNT)

;; SQLColAttributes subdefines for SQL_COLUMN_UPDATABLE
(defconstant SQL_ATTR_READONLY 0)
(defconstant SQL_ATTR_WRITE 1)
(defconstant SQL_ATTR_READWRITE_UNKNOWN 2)
 |#

;; /* defines for diagnostics fields */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_DIAG_CURSOR_ROW_COUNT	-1249)
(defconstant SQL_DIAG_ROW_NUMBER	-1248)
(defconstant SQL_DIAG_COLUMN_NUMBER	-1247)
;; #endif /* ODBCVER >= 0x0300 */

;; SQL extended datatypes)
(defconstant SQL_DATE 		 9)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_INTERVAL	10)
;; #endif
(defconstant SQL_TIME		10)
(defconstant SQL_TIMESTAMP	11)
(defconstant SQL_LONGVARCHAR	-1)	; ISO/IEC 9075-3: SQL_CLOB 40
(defconstant SQL_BINARY		-2)
(defconstant SQL_VARBINARY	-3)
(defconstant SQL_LONGVARBINARY	-4)	; ISO/IEC 9075-3: SQL_BLOB 30
(defconstant SQL_BIGINT		-5)	; ISO/IEC 9075-3: 25
(defconstant SQL_TINYINT	-6)
(defconstant SQL_BIT		-7)	; ISO/IEC 9075-3: SQL_BOOLEAN 16
;; #if (ODBCVER >= 0x0350)
(defconstant SQL_GUID	       -11)
;; #endif  /* ODBCVER >= 0x0350 */

#|;; #if (ODBCVER >= 0x0300)
;; Interval code == typedef enum { SQL_IS_YEAR=1,... } SQLINTERVAL
(defconstant SQL_CODE_YEAR		1)
(defconstant SQL_CODE_MONTH		2)
(defconstant SQL_CODE_DAY		3)
(defconstant SQL_CODE_HOUR		4)
(defconstant SQL_CODE_MINUTE		5)
(defconstant SQL_CODE_SECOND		6)
(defconstant SQL_CODE_YEAR_TO_MONTH	7)
(defconstant SQL_CODE_DAY_TO_HOUR	8)
(defconstant SQL_CODE_DAY_TO_MINUTE	9)
(defconstant SQL_CODE_DAY_TO_SECOND	10)
(defconstant SQL_CODE_HOUR_TO_MINUTE	11)
(defconstant SQL_CODE_HOUR_TO_SECOND	12)
(defconstant SQL_CODE_MINUTE_TO_SECOND	13)
(defconstant SQL_INTERVAL_YEAR		   (+ 100 SQL_CODE_YEAR))
(defconstant SQL_INTERVAL_MONTH		   (+ 100 SQL_CODE_MONTH))
(defconstant SQL_INTERVAL_DAY		   (+ 100 SQL_CODE_DAY))
(defconstant SQL_INTERVAL_HOUR		   (+ 100 SQL_CODE_HOUR))
(defconstant SQL_INTERVAL_MINUTE	   (+ 100 SQL_CODE_MINUTE))
(defconstant SQL_INTERVAL_SECOND           (+ 100 SQL_CODE_SECOND))
(defconstant SQL_INTERVAL_YEAR_TO_MONTH	   (+ 100 SQL_CODE_YEAR_TO_MONTH))
(defconstant SQL_INTERVAL_DAY_TO_HOUR	   (+ 100 SQL_CODE_DAY_TO_HOUR))
(defconstant SQL_INTERVAL_DAY_TO_MINUTE	   (+ 100 SQL_CODE_DAY_TO_MINUTE))
(defconstant SQL_INTERVAL_DAY_TO_SECOND	   (+ 100 SQL_CODE_DAY_TO_SECOND))
(defconstant SQL_INTERVAL_HOUR_TO_MINUTE   (+ 100 SQL_CODE_HOUR_TO_MINUTE))
(defconstant SQL_INTERVAL_HOUR_TO_SECOND   (+ 100 SQL_CODE_HOUR_TO_SECOND))
(defconstant SQL_INTERVAL_MINUTE_TO_SECOND (+ 100 SQL_CODE_MINUTE_TO_SECOND))

;; #else
(defconstant SQL_INTERVAL_YEAR -80)
(defconstant SQL_INTERVAL_MONTH -81)
(defconstant SQL_INTERVAL_YEAR_TO_MONTH -82)
(defconstant SQL_INTERVAL_DAY -83)
(defconstant SQL_INTERVAL_HOUR -84)
(defconstant SQL_INTERVAL_MINUTE -85)
(defconstant SQL_INTERVAL_SECOND -86)
(defconstant SQL_INTERVAL_DAY_TO_HOUR -87)
(defconstant SQL_INTERVAL_DAY_TO_MINUTE -88)
(defconstant SQL_INTERVAL_DAY_TO_SECOND -89)
(defconstant SQL_INTERVAL_HOUR_TO_MINUTE -90)
(defconstant SQL_INTERVAL_HOUR_TO_SECOND -91)
(defconstant SQL_INTERVAL_MINUTE_TO_SECOND -92)
;; #endif (ODBCVER >= 0x0300)

;;#if (ODBCVER <= 0x0300)
;...
;; #else
;; /* The previous definitions for SQL_UNICODE_ are historical and obsolete */
(defconstant SQL_UNICODE		SQL_WCHAR)
(defconstant SQL_UNICODE_VARCHAR	SQL_WVARCHAR)
(defconstant SQL_UNICODE_LONGVARCHAR	SQL_WLONGVARCHAR)
(defconstant SQL_UNICODE_CHAR		SQL_WCHAR)
;; #endif

;;#if (ODBCVER < 0x0300)
;(defconstant SQL_TYPE_DRIVER_START SQL_INTERVAL_YEAR)
;(defconstant SQL_TYPE_DRIVER_END SQL_UNICODE)
;; #endif  /* ODBCVER < 0x0300 */
 |#

;; C datatype to SQL datatype mapping	SQL types

(defconstant SQL_C_CHAR		SQL_CHAR)	;; CHAR, VARCHAR, DECIMAL, NUMERIC
(defconstant SQL_C_LONG		SQL_INTEGER)	;; INTEGER 
(defconstant SQL_C_SHORT	SQL_SMALLINT)	;; SMALLINT 
(defconstant SQL_C_FLOAT	SQL_REAL)	;; REAL 
(defconstant SQL_C_DOUBLE	SQL_DOUBLE)	;; FLOAT, DOUBLE
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_C_NUMERIC	SQL_NUMERIC)	;; SQL_NUMERIC_STRUCT
;; #endif
(defconstant SQL_C_DEFAULT 	  99)

(defconstant SQL_SIGNED_OFFSET	 -20)
(defconstant SQL_UNSIGNED_OFFSET -22)

;; C datatype to SQL datatype mapping
(defconstant SQL_C_DATE 	SQL_DATE)
(defconstant SQL_C_TIME 	SQL_TIME)
(defconstant SQL_C_TIMESTAMP 	SQL_TIMESTAMP)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_C_TYPE_DATE	SQL_TYPE_DATE)
(defconstant SQL_C_TYPE_TIME	SQL_TYPE_TIME)
(defconstant SQL_C_TYPE_TIMESTAMP	SQL_TYPE_TIMESTAMP)
#|
(defconstant SQL_C_INTERVAL_YEAR	SQL_INTERVAL_YEAR)
(defconstant SQL_C_INTERVAL_MONTH	SQL_INTERVAL_MONTH)
(defconstant SQL_C_INTERVAL_DAY		SQL_INTERVAL_DAY)
(defconstant SQL_C_INTERVAL_HOUR	SQL_INTERVAL_HOUR)
(defconstant SQL_C_INTERVAL_MINUTE	SQL_INTERVAL_MINUTE)
(defconstant SQL_C_INTERVAL_SECOND	SQL_INTERVAL_SECOND)
(defconstant SQL_C_INTERVAL_YEAR_TO_MONTH	SQL_INTERVAL_YEAR_TO_MONTH)
(defconstant SQL_C_INTERVAL_DAY_TO_HOUR		SQL_INTERVAL_DAY_TO_HOUR)
(defconstant SQL_C_INTERVAL_DAY_TO_MINUTE	SQL_INTERVAL_DAY_TO_MINUTE)
(defconstant SQL_C_INTERVAL_DAY_TO_SECOND	SQL_INTERVAL_DAY_TO_SECOND)
(defconstant SQL_C_INTERVAL_HOUR_TO_MINUTE	SQL_INTERVAL_HOUR_TO_MINUTE)
(defconstant SQL_C_INTERVAL_HOUR_TO_SECOND	SQL_INTERVAL_HOUR_TO_SECOND)
(defconstant SQL_C_INTERVAL_MINUTE_TO_SECOND	SQL_INTERVAL_MINUTE_TO_SECOND) |#
;; #endif  /* ODBCVER >= 0x0300 */
(defconstant SQL_C_BINARY   SQL_BINARY)
(defconstant SQL_C_BIT	    SQL_BIT)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_C_SBIGINT  (+ SQL_BIGINT SQL_SIGNED_OFFSET))	; SIGNED BIGINT 
(defconstant SQL_C_UBIGINT  (+ SQL_BIGINT SQL_UNSIGNED_OFFSET))	; UNSIGNED BIGINT
;; #endif  /* ODBCVER >= 0x0300 */
(defconstant SQL_C_TINYINT  SQL_TINYINT)
(defconstant SQL_C_SLONG    (+ SQL_C_LONG SQL_SIGNED_OFFSET))	; SIGNED INTEGER
(defconstant SQL_C_SSHORT   (+ SQL_C_SHORT SQL_SIGNED_OFFSET))	; SIGNED SMALLINT
(defconstant SQL_C_STINYINT (+ SQL_TINYINT SQL_SIGNED_OFFSET))	; SIGNED TINYINT
(defconstant SQL_C_ULONG    (+ SQL_C_LONG SQL_UNSIGNED_OFFSET))	; UNSIGNED INTEGER
(defconstant SQL_C_USHORT   (+ SQL_C_SHORT SQL_UNSIGNED_OFFSET)); UNSIGNED SMALLINT
(defconstant SQL_C_UTINYINT (+ SQL_TINYINT SQL_UNSIGNED_OFFSET)); UNSIGNED TINYINT
;(defconstant SQL_C_BOOKMARK SQL_C_ULONG) ;Deprecated in 3.0	; BOOKMARK

;; #if (ODBCVER >= 0x0350)
(defconstant SQL_C_GUID	SQL_GUID)
;; #endif  /* ODBCVER >= 0x0350 */

(defconstant SQL_TYPE_NULL	0)
;; #if (ODBCVER < 0x0300)
;(defconstant SQL_TYPE_MIN	SQL_BIT)
;(defconstant SQL_TYPE_MAX	SQL_VARCHAR)
;; #endif

;; #if (ODBCVER >= 0x0300)
(defconstant SQL_C_VARBOOKMARK SQL_C_BINARY)
;; #endif  /* ODBCVER >= 0x0300 */

;; /* define for SQL_DIAG_ROW_NUMBER and SQL_DIAG_COLUMN_NUMBER */
;; #if (ODBCVER >= 0x0300)
;(defconstant SQL_NO_ROW_NUMBER		-1)
;(defconstant SQL_NO_COLUMN_NUMBER	-1)
;(defconstant SQL_ROW_NUMBER_UNKNOWN	-2)
;(defconstant SQL_COLUMN_NUMBER_UNKNOWN	-2)
;; #endif

;; SQLBindParameter extensions
(defconstant SQL_DEFAULT_PARAM	-5)
(defconstant SQL_IGNORE		-6)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_COLUMN_IGNORE	SQL_IGNORE)
;; #endif  /* ODBCVER >= 0x0300 */

;; Defines for SQLBindParameter and SQLProcedureColumns (returned in the result set) 
;; /* moved here from below */
(defconstant SQL_PARAM_TYPE_UNKNOWN 0)
(defconstant SQL_PARAM_INPUT 1)
(defconstant SQL_PARAM_INPUT_OUTPUT 2)
(defconstant SQL_RESULT_COL 3)
(defconstant SQL_PARAM_OUTPUT 4)
(defconstant SQL_RETURN_VALUE 5)
;; Defines used by Driver Manager when mapping SQLSetParam to SQLBindParameter
(defconstant SQL_PARAM_TYPE_DEFAULT	SQL_PARAM_INPUT_OUTPUT)
(defconstant SQL_SETPARAM_VALUE_MAX     -1)

;; SQLColAttributes subdefines for SQL_COLUMN_SEARCHABLE
;; These are also used by SQLGetInfo                    
(defconstant SQL_UNSEARCHABLE 0)
(defconstant SQL_LIKE_ONLY 1)
(defconstant SQL_ALL_EXCEPT_LIKE 2)
(defconstant SQL_SEARCHABLE 3)
(defconstant SQL_PRED_SEARCHABLE SQL_SEARCHABLE)

;; Special return values for SQLGetData
(defconstant SQL_NO_TOTAL -4)		; If data length cannot be determined

#|/****************************************/
/* SQLGetFunctions: additional values for */
/* fFunction to represent functions that  */
/* are not in the X/Open spec.		  */
/******************************************/|#

;; #if (ODBCVER >= 0x0300)
;(defconstant SQL_API_SQLALLOCHANDLESTD	 73)
(defconstant SQL_API_SQLBULKOPERATIONS	 24)
;; #endif /* ODBCVER >= 0x0300 */
;(defconstant SQL_API_SQLBROWSECONNECT	 55)
(defconstant SQL_API_SQLCOLUMNPRIVILEGES 56)
(defconstant SQL_API_SQLDESCRIBEPARAM    58)
;(defconstant SQL_API_SQLDRIVERCONNECT	 41)
;(defconstant SQL_API_SQLDRIVERS	 71)
;(defconstant SQL_API_SQLEXTENDEDFETCH	 59)
(defconstant SQL_API_SQLFOREIGNKEYS	 60)
;(defconstant SQL_API_SQLMORERESULTS	 61)
;(defconstant SQL_API_SQLNATIVESQL	 62)
(defconstant SQL_API_SQLNUMPARAMS        63)
;(defconstant SQL_API_SQLPARAMOPTIONS     64)
(defconstant SQL_API_SQLPRIMARYKEYS	 65)
(defconstant SQL_API_SQLPROCEDURECOLUMNS 66)
(defconstant SQL_API_SQLPROCEDURES       67)
;(defconstant SQL_API_SQLSETPOS		 68)
;(defconstant SQL_API_SQLSETSCROLLOPTIONS 69)
;(defconstant SQL_API_SQLTABLEPRIVILEGES	 70)

#|/*-------------------------------------------*/
/* SQL_EXT_API_LAST is not useful with ODBC  */
/* version 3.0 because some of the values    */
/* from X/Open are in the 10000 range.       */
/*-------------------------------------------*/
;; #if (ODBCVER < 0x0300)
(defconstant SQL_EXT_API_LAST SQL_API_SQLBINDPARAMETER)
(defconstant SQL_NUM_FUNCTIONS           23)
(defconstant SQL_EXT_API_START           40)
(defconstant SQL_NUM_EXTENSIONS (- SQL_EXT_API_LAST SQL_EXT_API_START -1))
;; #endif
 |#
#|/*--------------------------------------------*/
/* SQL_API_ALL_FUNCTIONS returns an array     */
/* of 'booleans' representing whether a       */
/* function is implemented by the driver.     */
/*                                            */
/* CAUTION: Only functions defined in ODBC    */
/* version 2.0 and earlier are returned, the  */
/* new high-range function numbers defined by */
/* X/Open break this scheme.   See the new    */
/* method -- SQL_API_ODBC3_ALL_FUNCTIONS      */
/*--------------------------------------------*/|#
(defconstant SQL_API_ALL_FUNCTIONS 0)

#|/*----------------------------------------------*/
/* 2.X drivers export a dummy function with  	*/
/* ordinal number SQL_API_LOADBYORDINAL to speed*/
/* loading under the windows operating system.  */
/* 						*/
/* CAUTION: Loading by ordinal is not supported */
/* for 3.0 and above drivers.			*/
/*----------------------------------------------*/
(defconstant SQL_API_LOADBYORDINAL 199) |#

#|/*----------------------------------------------*/
/* SQL_API_ODBC3_ALL_FUNCTIONS                  */
/* This returns a bitmap, which allows us to    */
/* handle the higher-valued function numbers.   */
/* Use  SQL_FUNC_EXISTS(bitmap,function_number) */
/* to determine if the function exists.         */
/*----------------------------------------------*/|#
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_ODBC3_ALL_FUNCTIONS	999)
(defconstant SQL_API_ODBC3_ALL_FUNCTIONS_SIZE	250)	; array of 250 words
;(defconstant SQL_FUNC_EXISTS(pfExists, uwAPI) \
;	 ((*(((UWORD*) (pfExists)) + ((uwAPI) >> 4)) & (1 << ((uwAPI) & 0x000F))) \
;	 ? SQL_TRUE : SQL_FALSE )
;; #endif  /* ODBCVER >= 0x0300 */ 


;; /* Values for SQLGetInfo in ODBC 2.0 that are not in the X/Open spec */
;(defconstant SQL_INFO_FIRST		 0)
;(defconstant SQL_ACTIVE_CONNECTIONS	 0) ;SQL_MAX_DRIVER_CONNECTIONS
;(defconstant SQL_ACTIVE_STATEMENTS	 1) ;SQL_MAX_CONCURRENT_ACTIVITIES
;(defconstant SQL_DRIVER_HDBC		 3)
;(defconstant SQL_DRIVER_HENV		 4)
;(defconstant SQL_DRIVER_HSTMT		 5)
(def-get-info-type SQL_DRIVER_NAME                      6 :string)
(def-get-info-type SQL_DRIVER_VER                       7 :string)
;(defconstant SQL_ODBC_API_CONFORMANCE             9)
(def-get-info-type SQL_ODBC_VER                        10 :string)
(def-get-info-type SQL_ROW_UPDATES                     11 :boolean)
;(defconstant SQL_ODBC_SAG_CLI_CONFORMANCE        12)
;(defconstant SQL_ODBC_SQL_CONFORMANCE            15)
(def-get-info-type SQL_PROCEDURES                      21 :boolean)
(def-get-info-type SQL_CONCAT_NULL_BEHAVIOR            22 :short)
(def-get-info-type SQL_CURSOR_ROLLBACK_BEHAVIOR        24 :short)
;(defconstant SQL_EXPRESSIONS_IN_ORDERBY          27)
;(defconstant SQL_MAX_OWNER_NAME_LEN              32) ;SQL_MAX_SCHEMA_NAME_LEN
(def-get-info-type SQL_MAX_PROCEDURE_NAME_LEN          33 :short)
;(defconstant SQL_MAX_QUALIFIER_NAME_LEN          34) ;SQL_MAX_CATALOG_NAME_LEN
(def-get-info-type SQL_MULT_RESULT_SETS                36 :boolean)
(def-get-info-type SQL_MULTIPLE_ACTIVE_TXN             37 :boolean)
;(defconstant SQL_OUTER_JOINS                     38)
;(defconstant SQL_OWNER_TERM                      39)
;(defconstant SQL_PROCEDURE_TERM                  40)
;(defconstant SQL_QUALIFIER_NAME_SEPARATOR        41)
;(defconstant SQL_QUALIFIER_TERM                  42)
(def-get-info-type SQL_SCROLL_OPTIONS                  44 :long)
;(defconstant SQL_TABLE_TERM                      45)

(def-get-info-type SQL_CONVERT_FUNCTIONS  48 :long) ; using SQL_FN_CVT_CAST or  _CONVERT masks
(def-get-info-type SQL_NUMERIC_FUNCTIONS  49 :long) ; using SQL_FN_NUM_<fn> masks 
(def-get-info-type SQL_STRING_FUNCTIONS   50 :long) ; using SQL_FN_STR_<fn> masks
(def-get-info-type SQL_SYSTEM_FUNCTIONS   51 :long) ; SQL_FN_SYS_DBNAME, _IFNULL, _USERNAME
(def-get-info-type SQL_TIMEDATE_FUNCTIONS 52 :long) ; using SQL_FN_TD_<fn> masks

#|;; Whether a from-type can be converted to SQL_CONVERT_<type> using SQL_CVT_<from-type> masks
(defconstant SQL_CONVERT_BIGINT      53)
(defconstant SQL_CONVERT_BINARY      54)
(defconstant SQL_CONVERT_BIT         55)
(defconstant SQL_CONVERT_CHAR        56)
(defconstant SQL_CONVERT_DATE        57)
(defconstant SQL_CONVERT_DECIMAL     58)
(defconstant SQL_CONVERT_DOUBLE      59)
(defconstant SQL_CONVERT_FLOAT       60)
(defconstant SQL_CONVERT_INTEGER     61)
(defconstant SQL_CONVERT_LONGVARCHAR 62)
(defconstant SQL_CONVERT_NUMERIC     63)
(defconstant SQL_CONVERT_REAL        64)
(defconstant SQL_CONVERT_SMALLINT    65)
(defconstant SQL_CONVERT_TIME        66)
(defconstant SQL_CONVERT_TIMESTAMP   67)
(defconstant SQL_CONVERT_TINYINT     68)
(defconstant SQL_CONVERT_VARBINARY   69)
(defconstant SQL_CONVERT_VARCHAR     70)
(defconstant SQL_CONVERT_LONGVARBINARY 71)
 |#
;(defconstant SQL_ODBC_SQL_OPT_IEF                73) ;SQL_INTEGRITY
;(defconstant SQL_CORRELATION_NAME	74 :short)
;(defconstant SQL_NON_NULLABLE_COLUMNS	75 :short)
;(defconstant SQL_DRIVER_HLIB		76)
(def-get-info-type SQL_DRIVER_ODBC_VER	77 :string)
(def-get-info-type SQL_LOCK_TYPES		78 :long :deprecated 3.0)
(def-get-info-type SQL_POS_OPERATIONS		79 :long :deprecated 3.0)
(def-get-info-type SQL_POSITIONED_STATEMENTS	80 :long :deprecated 3.0)
(def-get-info-type SQL_BOOKMARK_PERSISTENCE	82 :long)
(def-get-info-type SQL_STATIC_SENSITIVITY	83 :long :deprecated 3.0)
;(defconstant SQL_FILE_USAGE			84 :long)
;(defconstant SQL_COLUMN_ALIAS			87 :boolean)
;(defconstant SQL_GROUP_BY			88 :short)
(def-get-info-type SQL_KEYWORDS			89 :string) ; => comma-separated list
;(defconstant SQL_OWNER_USAGE 			91)
;(defconstant SQL_QUALIFIER_USAGE		92) ;SQL_CATALOG_USAGE 
;(defconstant SQL_QUOTED_IDENTIFIER_CASE	93 :short)
;(defconstant SQL_SUBQUERIES			95)
;(defconstant SQL_UNION				96 :long)
(def-get-info-type SQL_MAX_ROW_SIZE_INCLUDES_LONG 103 :boolean)
(def-get-info-type SQL_MAX_CHAR_LITERAL_LEN	108 :long)
;(defconstant SQL_TIMEDATE_ADD_INTERVALS	109)
;(defconstant SQL_TIMEDATE_DIFF_INTERVALS 	110)
;; Data source needs the length of a long data value before sending it to the data source
(def-get-info-type SQL_NEED_LONG_DATA_LEN	111 :boolean)
;(defconstant SQL_MAX_BINARY_LITERAL_LEN	112)
(def-get-info-type SQL_LIKE_ESCAPE_CLAUSE	113 :boolean)
;(defconstant SQL_QUALIFIER_LOCATION		114) ;SQL_CATALOG_LOCATION

;; SQL_INFO_LAST and SQL_INFO_DRIVER_START are not useful anymore, 
;; because  X/Open has values in the 10000 range.
;; #if (ODBCVER < 0x0300)
;(defconstant SQL_INFO_LAST		SQL_QUALIFIER_LOCATION
;(defconstant SQL_INFO_DRIVER_START	1000
;; #endif /* ODBCVER < 0x0300 */

;;/* ODBC 3.0 SQLGetInfo values that are not part of the X/Open standard at this time.*/

;; #if (ODBCVER >= 0x0300)
;(defconstant SQL_ACTIVE_ENVIRONMENTS		116)
;(defconstant SQL_ALTER_DOMAIN			117)
(def-get-info-type SQL_SQL_CONFORMANCE		118 :long)
;(defconstant SQL_DATETIME_LITERALS		119)
;(def-get-info-type SQL_ASYNC_MODE		10021 :long) ; new X/Open spec 
;(defconstant SQL_BATCH_ROW_COUNT		120)
;(defconstant SQL_BATCH_SUPPORT			121)
(def-get-info-type SQL_CATALOG_LOCATION		114 :short) ;SQL_QUALIFIER_LOCATION
(def-get-info-type SQL_CATALOG_NAME_SEPARATOR	 41 :string) ;SQL_QUALIFIER_NAME_SEPARATOR
(def-get-info-type SQL_CATALOG_TERM		 42 :string) ;SQL_QUALIFIER_TERM
(def-get-info-type SQL_CATALOG_USAGE		 92 :long) ;SQL_QUALIFIER_USAGE
;(defconstant SQL_CONVERT_WCHAR			122)
;(defconstant SQL_CONVERT_INTERVAL_DAY_TIME	123
;(defconstant SQL_CONVERT_INTERVAL_YEAR_MONTH	124
;(defconstant SQL_CONVERT_WLONGVARCHAR	125
;(defconstant SQL_CONVERT_WVARCHAR		126
;(defconstant SQL_CREATE_ASSERTION		127
;(defconstant SQL_CREATE_CHARACTER_SET	128
;(defconstant SQL_CREATE_COLLATION		129
;(defconstant SQL_CREATE_DOMAIN		130
;(defconstant SQL_CREATE_SCHEMA		131
;(defconstant SQL_CREATE_TABLE		132
;(defconstant SQL_CREATE_TRANSLATION		133
;(defconstant SQL_CREATE_VIEW			134
;(defconstant SQL_DRIVER_HDESC			135
;(defconstant SQL_DROP_ASSERTION		136
;(defconstant SQL_DROP_CHARACTER_SET		137
;(defconstant SQL_DROP_COLLATION		138
;(defconstant SQL_DROP_DOMAIN			139
;(defconstant SQL_DROP_SCHEMA			140
;(defconstant SQL_DROP_TABLE			141
;(defconstant SQL_DROP_TRANSLATION		142
;(defconstant SQL_DROP_VIEW			143
(def-get-info-type SQL_DYNAMIC_CURSOR_ATTRIBUTES1 144 :long)
(def-get-info-type SQL_DYNAMIC_CURSOR_ATTRIBUTES2 145 :long)
(def-get-info-type SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1 146 :long)
(def-get-info-type SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2 147 :long)
;(defconstant SQL_INDEX_KEYWORDS		148
;(defconstant SQL_INFO_SCHEMA_VIEWS		149
(def-get-info-type SQL_KEYSET_CURSOR_ATTRIBUTES1 150 :long)
(def-get-info-type SQL_KEYSET_CURSOR_ATTRIBUTES2 151 :long)
(def-get-info-type SQL_MAX_ASYNC_CONCURRENT_STATEMENTS 10022 :long) ;/* new X/Open spec */
;(defconstant SQL_ODBC_INTERFACE_CONFORMANCE	152
(def-get-info-type SQL_PARAM_ARRAY_ROW_COUNTS   153 :long)
(def-get-info-type SQL_PARAM_ARRAY_SELECTS     	154 :long)
(def-get-info-type SQL_SCHEMA_TERM		 39 :string) ;SQL_OWNER_TERM
(def-get-info-type SQL_SCHEMA_USAGE		 91 :long) ;SQL_OWNER_USAGE
;(defconstant SQL_SQL92_DATETIME_FUNCTIONS	155
;(defconstant SQL_SQL92_FOREIGN_KEY_DELETE_RULE	156		
;(defconstant SQL_SQL92_FOREIGN_KEY_UPDATE_RULE	157		
;(defconstant SQL_SQL92_GRANT			158
;(defconstant SQL_SQL92_NUMERIC_VALUE_FUNCTIONS	159
;(defconstant SQL_SQL92_PREDICATES		160
;(defconstant SQL_SQL92_RELATIONAL_JOIN_OPERATORS 161
;(defconstant SQL_SQL92_REVOKE			162
;(defconstant SQL_SQL92_ROW_VALUE_CONSTRUCTOR	163
;(defconstant SQL_SQL92_STRING_FUNCTIONS	164
;(def-get-info-type SQL_SQL92_VALUE_EXPRESSIONS	 165 :long)
;(defconstant SQL_STANDARD_CLI_CONFORMANCE	166
(def-get-info-type SQL_STATIC_CURSOR_ATTRIBUTES1 167 :long)
(def-get-info-type SQL_STATIC_CURSOR_ATTRIBUTES2 168 :long)

;(defconstant SQL_AGGREGATE_FUNCTIONS		169
;(defconstant SQL_DDL_INDEX			170
;(defconstant SQL_DM_VER			171
;(defconstant SQL_INSERT_STATEMENT		172
;(defconstant SQL_UNION_STATEMENT	SQL_UNION 
;; #endif  /* ODBCVER >= 0x0300 */

;(def-get-info-type SQL_THREAD_MODEL	1028 :short)	;SQL_USMALLINT
;; DataDirect Technologies THREAD-SAFE driver sessions thread model.  Value:
;;  0 :	The session is fully thread enabled and all requests will fully utilize threads.
;;  1 :	The session is restricted at the connection level, i.e. fully thread-enabled
;;	when simultaneous requests are made with statement handles that do not
;;	share the same connection handle. If multiple requests are made from
;;	the same connection, then the first request received by the driver
;;	is processed immediately and all subsequent requests are serialized.
;;  2 :	The session is thread-impaired () and all requests
;;	are serialized by the driver. Application should only use threads
;;	if it reduces program complexity.
;; To Do: Return value -1 if the driver is not thread-safe at all.

;(defconstant	SQL_DTC_TRANSITION_COST	1750)

#|;; /* The following SQL_ALTER_TABLE bitmasks are ODBC extensions */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_AT_ADD_COLUMN_SINGLE		0x00000020L	
(defconstant SQL_AT_ADD_COLUMN_DEFAULT		0x00000040L
(defconstant SQL_AT_ADD_COLUMN_COLLATION		0x00000080L
(defconstant SQL_AT_SET_COLUMN_DEFAULT		0x00000100L
(defconstant SQL_AT_DROP_COLUMN_DEFAULT		0x00000200L
(defconstant SQL_AT_DROP_COLUMN_CASCADE		0x00000400L
(defconstant SQL_AT_DROP_COLUMN_RESTRICT		0x00000800L
(defconstant SQL_AT_ADD_TABLE_CONSTRAINT		0x00001000L		
(defconstant SQL_AT_DROP_TABLE_CONSTRAINT_CASCADE	0x00002000L		
(defconstant SQL_AT_DROP_TABLE_CONSTRAINT_RESTRICT	0x00004000L		
(defconstant SQL_AT_CONSTRAINT_NAME_DEFINITION	0x00008000L
(defconstant SQL_AT_CONSTRAINT_INITIALLY_DEFERRED	0x00010000L
(defconstant SQL_AT_CONSTRAINT_INITIALLY_IMMEDIATE	0x00020000L
(defconstant SQL_AT_CONSTRAINT_DEFERRABLE		0x00040000L
(defconstant SQL_AT_CONSTRAINT_NON_DEFERRABLE	0x00080000L
;; #endif /* ODBCVER >= 0x0300 */

;; SQL_CONVERT_*  return value bitmasks
(defconstant SQL_CVT_CHAR		#x00000001L)
(defconstant SQL_CVT_NUMERIC 		#x00000002L)
(defconstant SQL_CVT_DECIMAL 		#x00000004L)
(defconstant SQL_CVT_INTEGER 		#x00000008L)
(defconstant SQL_CVT_SMALLINT		#x00000010L)
(defconstant SQL_CVT_FLOAT		#x00000020L)
(defconstant SQL_CVT_REAL		#x00000040L)
(defconstant SQL_CVT_DOUBLE		#x00000080L)
(defconstant SQL_CVT_VARCHAR 		#x00000100L)
(defconstant SQL_CVT_LONGVARCHAR 	#x00000200L)
(defconstant SQL_CVT_BINARY		#x00000400L)
(defconstant SQL_CVT_VARBINARY		#x00000800L)
(defconstant SQL_CVT_BIT 		#x00001000L)
(defconstant SQL_CVT_TINYINT 		#x00002000L)
(defconstant SQL_CVT_BIGINT		#x00004000L)
(defconstant SQL_CVT_DATE		#x00008000L)
(defconstant SQL_CVT_TIME		#x00010000L)
(defconstant SQL_CVT_TIMESTAMP		#x00020000L)
(defconstant SQL_CVT_LONGVARBINARY	#x00040000L)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_CVT_INTERVAL_YEAR_MONTH #x00080000L)
(defconstant SQL_CVT_INTERVAL_DAY_TIME	#x00100000L)
(defconstant SQL_CVT_WCHAR		#x00200000L)
(defconstant SQL_CVT_WLONGVARCHAR	#x00400000L)
(defconstant SQL_CVT_WVARCHAR		#x00800000L)
;; #endif  /* ODBCVER >= 0x0300 */

;; /* SQL_CONVERT_FUNCTIONS functions */
(defconstant SQL_FN_CVT_CONVERT		#x00000001)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_FN_CVT_CAST		#x00000002)
;; #endif  /* ODBCVER >= 0x0300 */

;; SQL_STRING_FUNCTIONS functions
(defconstant SQL_FN_STR_CONCAT			#x00000001)
(defconstant SQL_FN_STR_INSERT			#x00000002)
(defconstant SQL_FN_STR_LEFT 			#x00000004)
(defconstant SQL_FN_STR_LTRIM			#x00000008)
(defconstant SQL_FN_STR_LENGTH			#x00000010)
(defconstant SQL_FN_STR_LOCATE			#x00000020)
(defconstant SQL_FN_STR_LCASE			#x00000040)
(defconstant SQL_FN_STR_REPEAT			#x00000080)
(defconstant SQL_FN_STR_REPLACE			#x00000100)
(defconstant SQL_FN_STR_RIGHT			#x00000200)
(defconstant SQL_FN_STR_RTRIM			#x00000400)
(defconstant SQL_FN_STR_SUBSTRING		#x00000800)
(defconstant SQL_FN_STR_UCASE			#x00001000)
(defconstant SQL_FN_STR_ASCII			#x00002000)
(defconstant SQL_FN_STR_CHAR 			#x00004000)
(defconstant SQL_FN_STR_DIFFERENCE		#x00008000)
(defconstant SQL_FN_STR_LOCATE_2 		#x00010000)
(defconstant SQL_FN_STR_SOUNDEX			#x00020000)
(defconstant SQL_FN_STR_SPACE			#x00040000)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_FN_STR_BIT_LENGTH		#x00080000)
(defconstant SQL_FN_STR_CHAR_LENGTH		#x00100000)
(defconstant SQL_FN_STR_CHARACTER_LENGTH	#x00200000)
(defconstant SQL_FN_STR_OCTET_LENGTH		#x00400000)
(defconstant SQL_FN_STR_POSITION		#x00800000)
;; #endif  /* ODBCVER >= 0x0300 */

/* SQL_SQL92_STRING_FUNCTIONS */
#if (ODBCVER >= 0x0300)
(defconstant SQL_SSF_CONVERT			#x00000001)	
(defconstant SQL_SSF_LOWER			#x00000002)
(defconstant SQL_SSF_UPPER			#x00000004)
(defconstant SQL_SSF_SUBSTRING			#x00000008)
(defconstant SQL_SSF_TRANSLATE			#x00000010)
(defconstant SQL_SSF_TRIM_BOTH			#x00000020)
(defconstant SQL_SSF_TRIM_LEADING		#x00000040)
(defconstant SQL_SSF_TRIM_TRAILING		#x00000080)
#endif /* ODBCVER >= 0x0300 */

;; SQL_NUMERIC_FUNCTIONS functions
(defconstant SQL_FN_NUM_ABS	#x00000001)
(defconstant SQL_FN_NUM_ACOS 	#x00000002)
(defconstant SQL_FN_NUM_ASIN 	#x00000004)
(defconstant SQL_FN_NUM_ATAN 	#x00000008)
(defconstant SQL_FN_NUM_ATAN2	#x00000010)
(defconstant SQL_FN_NUM_CEILING	#x00000020)
(defconstant SQL_FN_NUM_COS	#x00000040)
(defconstant SQL_FN_NUM_COT	#x00000080)
(defconstant SQL_FN_NUM_EXP	#x00000100)
(defconstant SQL_FN_NUM_FLOOR	#x00000200)
(defconstant SQL_FN_NUM_LOG	#x00000400)
(defconstant SQL_FN_NUM_MOD	#x00000800)
(defconstant SQL_FN_NUM_SIGN 	#x00001000)
(defconstant SQL_FN_NUM_SIN	#x00002000)
(defconstant SQL_FN_NUM_SQRT 	#x00004000)
(defconstant SQL_FN_NUM_TAN	#x00008000)
(defconstant SQL_FN_NUM_PI	#x00010000)
(defconstant SQL_FN_NUM_RAND 	#x00020000)
(defconstant SQL_FN_NUM_DEGREES	#x00040000)
(defconstant SQL_FN_NUM_LOG10	#x00080000)
(defconstant SQL_FN_NUM_POWER	#x00100000)
(defconstant SQL_FN_NUM_RADIANS	#x00200000)
(defconstant SQL_FN_NUM_ROUND	#x00400000)
(defconstant SQL_FN_NUM_TRUNCATE #x00800000)

/* SQL_SQL92_NUMERIC_VALUE_FUNCTIONS */
#if (ODBCVER >= 0x0300)
(defconstant SQL_SNVF_BIT_LENGTH				0x00000001L
(defconstant SQL_SNVF_CHAR_LENGTH				0x00000002L
(defconstant SQL_SNVF_CHARACTER_LENGTH			0x00000004L
(defconstant SQL_SNVF_EXTRACT				0x00000008L
(defconstant SQL_SNVF_OCTET_LENGTH				0x00000010L
(defconstant SQL_SNVF_POSITION				0x00000020L
#endif  /* ODBCVER >= 0x0300 */

;; SQL_TIMEDATE_FUNCTIONS functions
(defconstant SQL_FN_TD_NOW		#x00000001L)
(defconstant SQL_FN_TD_CURDATE		#x00000002L)
(defconstant SQL_FN_TD_DAYOFMONTH	#x00000004L)
(defconstant SQL_FN_TD_DAYOFWEEK 	#x00000008L)
(defconstant SQL_FN_TD_DAYOFYEAR 	#x00000010L)
(defconstant SQL_FN_TD_MONTH 		#x00000020L)
(defconstant SQL_FN_TD_QUARTER		#x00000040L)
(defconstant SQL_FN_TD_WEEK		#x00000080L)
(defconstant SQL_FN_TD_YEAR		#x00000100L)
(defconstant SQL_FN_TD_CURTIME		#x00000200L)
(defconstant SQL_FN_TD_HOUR		#x00000400L)
(defconstant SQL_FN_TD_MINUTE		#x00000800L)
(defconstant SQL_FN_TD_SECOND		#x00001000L)
(defconstant SQL_FN_TD_TIMESTAMPADD	#x00002000L)
(defconstant SQL_FN_TD_TIMESTAMPDIFF 	#x00004000L)
(defconstant SQL_FN_TD_DAYNAME		#x00008000L)
(defconstant SQL_FN_TD_MONTHNAME	#x00010000L)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_FN_TD_CURRENT_DATE	#x00020000)
(defconstant SQL_FN_TD_CURRENT_TIME	#x00040000)
(defconstant SQL_FN_TD_CURRENT_TIMESTAMP #x00080000)
(defconstant SQL_FN_TD_EXTRACT		#x00100000)
;; #endif  /* ODBCVER >= 0x0300 */

/* SQL_SQL92_DATETIME_FUNCTIONS */
#if (ODBCVER >= 0x0300)
(defconstant SQL_SDF_CURRENT_DATE	0x00000001L
(defconstant SQL_SDF_CURRENT_TIME	0x00000002L
(defconstant SQL_SDF_CURRENT_TIMESTAMP	0x00000004L
#endif /* ODBCVER >= 0x0300 */

;; SQL_SYSTEM_FUNCTIONS functions
(defconstant SQL_FN_SYS_USERNAME	#x00000001)
(defconstant SQL_FN_SYS_DBNAME		#x00000002)
(defconstant SQL_FN_SYS_IFNULL		#x00000004)

;; SQL_TIMEDATE_ADD_INTERVALS and SQL_TIMEDATE_DIFF_INTERVALS functions
(defconstant SQL_FN_TSI_FRAC_SECOND	#x00000001L)
(defconstant SQL_FN_TSI_SECOND		#x00000002L)
(defconstant SQL_FN_TSI_MINUTE		#x00000004L)
(defconstant SQL_FN_TSI_HOUR 		#x00000008L)
(defconstant SQL_FN_TSI_DAY		#x00000010L)
(defconstant SQL_FN_TSI_WEEK 		#x00000020L)
(defconstant SQL_FN_TSI_MONTH		#x00000040L)
(defconstant SQL_FN_TSI_QUARTER		#x00000080L)
(defconstant SQL_FN_TSI_YEAR 		#x00000100L)
|#
;;; bitmasks for SQL_DYNAMIC_CURSOR_ATTRIBUTES1,
;;; SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1, 
;;; SQL_KEYSET_CURSOR_ATTRIBUTES1, and SQL_STATIC_CURSOR_ATTRIBUTES1 
;;#if (ODBCVER >= 0x0300)
;; supported SQLFetchScroll FetchOrientation's
(defconstant SQL_CA1_NEXT		#x00000001)
(defconstant SQL_CA1_ABSOLUTE		#x00000002)
(defconstant SQL_CA1_RELATIVE		#x00000004)
(defconstant SQL_CA1_BOOKMARK		#x00000008)
#|
/* supported SQLSetPos LockType's */
(defconstant SQL_CA1_LOCK_NO_CHANGE	#x00000040)
(defconstant SQL_CA1_LOCK_EXCLUSIVE	#x00000080)
(defconstant SQL_CA1_LOCK_UNLOCK	#x00000100)

/* supported SQLSetPos Operations */
(defconstant SQL_CA1_POS_POSITION	#x00000200)
(defconstant SQL_CA1_POS_UPDATE		#x00000400)
(defconstant SQL_CA1_POS_DELETE		#x00000800)
(defconstant SQL_CA1_POS_REFRESH	#x00001000)

/* positioned updates and deletes */
(defconstant SQL_CA1_POSITIONED_UPDATE	#x00002000)
(defconstant SQL_CA1_POSITIONED_DELETE	#x00004000)
(defconstant SQL_CA1_SELECT_FOR_UPDATE	#x00008000)

/* supported SQLBulkOperations operations */
(defconstant SQL_CA1_BULK_ADD			#x00010000)
(defconstant SQL_CA1_BULK_UPDATE_BY_BOOKMARK	#x00020000)
(defconstant SQL_CA1_BULK_DELETE_BY_BOOKMARK	#x00040000)
(defconstant SQL_CA1_BULK_FETCH_BY_BOOKMARK	#x00080000)
#endif  /* ODBCVER >= 0x0300 */

/* bitmasks for SQL_DYNAMIC_CURSOR_ATTRIBUTES2,
 * SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2, 
 * SQL_KEYSET_CURSOR_ATTRIBUTES2, and SQL_STATIC_CURSOR_ATTRIBUTES2 
 */
#if (ODBCVER >= 0x0300)
/* supported values for SQL_ATTR_SCROLL_CONCURRENCY */
(defconstant SQL_CA2_READ_ONLY_CONCURRENCY	#x00000001)
(defconstant SQL_CA2_LOCK_CONCURRENCY		#x00000002)
(defconstant SQL_CA2_OPT_ROWVER_CONCURRENCY	#x00000004)
(defconstant SQL_CA2_OPT_VALUES_CONCURRENCY	#x00000008)

/* sensitivity of the cursor to its own inserts, deletes, and updates */
(defconstant SQL_CA2_SENSITIVITY_ADDITIONS	#x00000010)
(defconstant SQL_CA2_SENSITIVITY_DELETIONS	#x00000020)
(defconstant SQL_CA2_SENSITIVITY_UPDATES	#x00000040)

/* semantics of SQL_ATTR_MAX_ROWS */
(defconstant SQL_CA2_MAX_ROWS_SELECT		#x00000080)
(defconstant SQL_CA2_MAX_ROWS_INSERT		#x00000100)
(defconstant SQL_CA2_MAX_ROWS_DELETE		#x00000200)
(defconstant SQL_CA2_MAX_ROWS_UPDATE		#x00000400)
(defconstant SQL_CA2_MAX_ROWS_CATALOG		#x00000800)
(defconstant SQL_CA2_MAX_ROWS_AFFECTS_ALL	(SQL_CA2_MAX_ROWS_SELECT | \
				SQL_CA2_MAX_ROWS_INSERT | SQL_CA2_MAX_ROWS_DELETE | \
				SQL_CA2_MAX_ROWS_UPDATE | SQL_CA2_MAX_ROWS_CATALOG)

/* semantics of SQL_DIAG_CURSOR_ROW_COUNT */
(defconstant SQL_CA2_CRC_EXACT			#x00001000)
(defconstant SQL_CA2_CRC_APPROXIMATE		#x00002000)

/* the kinds of positioned statements that can be simulated */
(defconstant SQL_CA2_SIMULATE_NON_UNIQUE		#x00004000)
(defconstant SQL_CA2_SIMULATE_TRY_UNIQUE		#x00008000)
(defconstant SQL_CA2_SIMULATE_UNIQUE			#x00010000)
#endif  /* ODBCVER >= 0x0300 */

;; SQL_ODBC_API_CONFORMANCE values
(defconstant SQL_OAC_NONE		#x0000)
(defconstant SQL_OAC_LEVEL1		#x0001)
(defconstant SQL_OAC_LEVEL2		#x0002)

;; SQL_ODBC_SAG_CLI_CONFORMANCE values
(defconstant SQL_OSCC_NOT_COMPLIANT	#x0000)
(defconstant SQL_OSCC_COMPLIANT		#x0001)

;; SQL_ODBC_SQL_CONFORMANCE values
(defconstant SQL_OSC_MINIMUM 		#x0000)
(defconstant SQL_OSC_CORE		#x0001)
(defconstant SQL_OSC_EXTENDED		#x0002)

;; SQL_CONCAT_NULL_BEHAVIOR values
(defconstant SQL_CB_NULL		#x0000)
(defconstant SQL_CB_NON_NULL		#x0001)

;; SQL_SCROLL_OPTIONS masks
(defconstant SQL_SO_FORWARD_ONLY  #x00000001)
(defconstant SQL_SO_KEYSET_DRIVEN #x00000002)
(defconstant SQL_SO_DYNAMIC	  #x00000004)
(defconstant SQL_SO_MIXED	  #x00000008)
(defconstant SQL_SO_STATIC	  #x00000010)

;; /* SQL_FETCH_DIRECTION masks */
; /* SQL_FETCH_RESUME is no longer supported */
;(defconstant SQL_FD_FETCH_RESUME #x00000040)
(defconstant SQL_FD_FETCH_BOOKMARK #x00000080)

;; SQL_CORRELATION_NAME values
(defconstant SQL_CN_NONE 				#x0000)
(defconstant SQL_CN_DIFFERENT			#x0001)
(defconstant SQL_CN_ANY					#x0002

;; SQL_NON_NULLABLE_COLUMNS values

(defconstant SQL_NNC_NULL			   	#x0000)
(defconstant SQL_NNC_NON_NULL			#x0001

;; /* SQL_NULL_COLLATION values */
(defconstant SQL_NC_START				#x0002)
(defconstant SQL_NC_END					#x0004

;; SQL_FILE_USAGE values
(defconstant SQL_FILE_NOT_SUPPORTED	#x0000)
(defconstant SQL_FILE_TABLE		#x0001)
(defconstant SQL_FILE_CATALOG		#x0002)
;(defconstant SQL_FILE_QUALIFIER	#x0002)
 |#
;(defconstant SQL_GD_BLOCK	#x00000004L)
(defconstant SQL_GD_BOUND	#x8)

#|
;; SQL_POSITIONED_STATEMENTS masks
(defconstant SQL_PS_POSITIONED_DELETE	#x00000001L)
(defconstant SQL_PS_POSITIONED_UPDATE	#x00000002L)
(defconstant SQL_PS_SELECT_FOR_UPDATE	#x00000004L

;; SQL_GROUP_BY values
(defconstant SQL_GB_NOT_SUPPORTED		#x0000)
(defconstant SQL_GB_GROUP_BY_EQUALS_SELECT	#x0001)
(defconstant SQL_GB_GROUP_BY_CONTAINS_SELECT	#x0002)
(defconstant SQL_GB_NO_RELATION			#x0003)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_GB_COLLATE			#x0004)
;; #endif
											
;; SQL_OWNER_USAGE masks
(defconstant SQL_SU_DML_STATEMENTS		#x00000001L)
(defconstant SQL_SU_PROCEDURE_INVOCATION #x00000002L)
(defconstant SQL_SU_TABLE_DEFINITION 	#x00000004L)
(defconstant SQL_SU_INDEX_DEFINITION 	#x00000008L)
(defconstant SQL_SU_PRIVILEGE_DEFINITION #x00000010L)

;; SQL_QUALIFIER_USAGE masks
(defconstant SQL_QU_DML_STATEMENTS		#x00000001L)
(defconstant SQL_QU_PROCEDURE_INVOCATION #x00000002L)
(defconstant SQL_QU_TABLE_DEFINITION 	#x00000004L)
(defconstant SQL_QU_INDEX_DEFINITION 	#x00000008L)
(defconstant SQL_QU_PRIVILEGE_DEFINITION #x00000010L

#if (ODBCVER >= 0x0300)
/* SQL_CATALOG_USAGE masks */
#define SQL_CU_DML_STATEMENTS			SQL_QU_DML_STATEMENTS
#define SQL_CU_PROCEDURE_INVOCATION		SQL_QU_PROCEDURE_INVOCATION 
#define SQL_CU_TABLE_DEFINITION			SQL_QU_TABLE_DEFINITION
#define SQL_CU_INDEX_DEFINITION			SQL_QU_INDEX_DEFINITION 
#define SQL_CU_PRIVILEGE_DEFINITION		SQL_QU_PRIVILEGE_DEFINITION 
#endif  /* ODBCVER >= 0x0300 */

;; SQL_SUBQUERIES masks
(defconstant SQL_SQ_COMPARISON			#x00000001L)
(defconstant SQL_SQ_EXISTS			#x00000002L)
(defconstant SQL_SQ_IN				#x00000004L)
(defconstant SQL_SQ_QUANTIFIED			#x00000008L)
(defconstant SQL_SQ_CORRELATED_SUBQUERIES	#x00000010L)

;; SQL_UNION masks
(defconstant SQL_U_UNION		#x00000001L)
(defconstant SQL_U_UNION_ALL		#x00000002L

;; SQL_BOOKMARK_PERSISTENCE values
(defconstant SQL_BP_CLOSE			#x00000001L)
(defconstant SQL_BP_DELETE			#x00000002L)
(defconstant SQL_BP_DROP 			#x00000004L)
(defconstant SQL_BP_TRANSACTION			#x00000008L)
(defconstant SQL_BP_UPDATE			#x00000010L)
(defconstant SQL_BP_OTHER_HSTMT			#x00000020L)
(defconstant SQL_BP_SCROLL			#x00000040L)

;; SQL_STATIC_SENSITIVITY values
(defconstant SQL_SS_ADDITIONS			#x00000001L)
(defconstant SQL_SS_DELETIONS			#x00000002L)
(defconstant SQL_SS_UPDATES			#x00000004L)

/* SQL_VIEW values */
#define	SQL_CV_CREATE_VIEW					#x00000001L
#define	SQL_CV_CHECK_OPTION					#x00000002L
#define	SQL_CV_CASCADED						#x00000004L
#define	SQL_CV_LOCAL						#x00000008L

;; SQL_LOCK_TYPES masks
(defconstant SQL_LCK_NO_CHANGE			#x00000001L)
(defconstant SQL_LCK_EXCLUSIVE			#x00000002L)
(defconstant SQL_LCK_UNLOCK			#x00000004L)
|#
;; SQL_POS_OPERATIONS masks: deprecated in 3.0 in favor of cursor attributes info types
;(defconstant SQL_POS_POSITION 1) ;; #x00000001
;(defconstant SQL_POS_REFRESH 2)  ;; #x00000002
;(defconstant SQL_POS_UPDATE 4)   ;; #x00000004
;(defconstant SQL_POS_DELETE 8)   ;; #x00000008
;(defconstant SQL_POS_ADD 16)     ;; #x00000010

;; SQL_CATALOG_LOCATION values 
(defconstant SQL_CL_START	#x0001)	;SQL_QL_START
(defconstant SQL_CL_END		#x0002)	;SQL_QL_END
;; SQL_QUALIFIER_LOCATION values
;(defconstant SQL_QL_START #x0001)
;(defconstant SQL_QL_END  #x0002)

#|

/* Here start return values for ODBC 3.0 SQLGetInfo */

#if (ODBCVER >= 0x0300)
/* SQL_AGGREGATE_FUNCTIONS bitmasks */
#define SQL_AF_AVG						#x00000001L
#define SQL_AF_COUNT					#x00000002L
#define SQL_AF_MAX						#x00000004L
#define SQL_AF_MIN						#x00000008L
#define SQL_AF_SUM						#x00000010L
#define SQL_AF_DISTINCT					#x00000020L
#define SQL_AF_ALL						#x00000040L	

/* SQL_SQL_CONFORMANCE bit masks */
#define	SQL_SC_SQL92_ENTRY				#x00000001L
#define	SQL_SC_FIPS127_2_TRANSITIONAL	#x00000002L
#define	SQL_SC_SQL92_INTERMEDIATE		#x00000004L
#define	SQL_SC_SQL92_FULL				#x00000008L

/* SQL_DATETIME_LITERALS masks */
#define SQL_DL_SQL92_DATE						#x00000001L
#define SQL_DL_SQL92_TIME						#x00000002L
#define SQL_DL_SQL92_TIMESTAMP					#x00000004L
#define SQL_DL_SQL92_INTERVAL_YEAR				#x00000008L
#define SQL_DL_SQL92_INTERVAL_MONTH				#x00000010L
#define SQL_DL_SQL92_INTERVAL_DAY				#x00000020L
#define SQL_DL_SQL92_INTERVAL_HOUR				#x00000040L
#define	SQL_DL_SQL92_INTERVAL_MINUTE			#x00000080L
#define SQL_DL_SQL92_INTERVAL_SECOND			#x00000100L
#define SQL_DL_SQL92_INTERVAL_YEAR_TO_MONTH		#x00000200L
#define SQL_DL_SQL92_INTERVAL_DAY_TO_HOUR		#x00000400L
#define SQL_DL_SQL92_INTERVAL_DAY_TO_MINUTE		#x00000800L
#define SQL_DL_SQL92_INTERVAL_DAY_TO_SECOND		#x00001000L
#define SQL_DL_SQL92_INTERVAL_HOUR_TO_MINUTE	#x00002000L
#define SQL_DL_SQL92_INTERVAL_HOUR_TO_SECOND	#x00004000L
#define SQL_DL_SQL92_INTERVAL_MINUTE_TO_SECOND	#x00008000L

/* values for SQL_BATCH_ROW_COUNT */
#define SQL_BRC_PROCEDURES			#x0000001
#define	SQL_BRC_EXPLICIT			#x0000002
#define	SQL_BRC_ROLLED_UP			#x0000004

/* bitmasks for SQL_BATCH_SUPPORT */
#define SQL_BS_SELECT_EXPLICIT				#x00000001L
#define SQL_BS_ROW_COUNT_EXPLICIT			#x00000002L
#define SQL_BS_SELECT_PROC					#x00000004L
#define SQL_BS_ROW_COUNT_PROC				#x00000008L

/* Values for SQL_PARAM_ARRAY_ROW_COUNTS getinfo */
#define SQL_PARC_BATCH		1
#define SQL_PARC_NO_BATCH	2

/* values for SQL_PARAM_ARRAY_SELECTS */
#define SQL_PAS_BATCH				1
#define SQL_PAS_NO_BATCH			2		
#define SQL_PAS_NO_SELECT			3

/* Bitmasks for SQL_INDEX_KEYWORDS */
#define SQL_IK_NONE							#x00000000L
#define SQL_IK_ASC							#x00000001L
#define SQL_IK_DESC							#x00000002L
#define SQL_IK_ALL							(SQL_IK_ASC | SQL_IK_DESC)

/* Bitmasks for SQL_INFO_SCHEMA_VIEWS */

#define SQL_ISV_ASSERTIONS					#x00000001L
#define SQL_ISV_CHARACTER_SETS				#x00000002L
#define SQL_ISV_CHECK_CONSTRAINTS			#x00000004L
#define SQL_ISV_COLLATIONS					#x00000008L
#define SQL_ISV_COLUMN_DOMAIN_USAGE			#x00000010L
#define SQL_ISV_COLUMN_PRIVILEGES			#x00000020L
#define SQL_ISV_COLUMNS						#x00000040L
#define SQL_ISV_CONSTRAINT_COLUMN_USAGE		#x00000080L
#define SQL_ISV_CONSTRAINT_TABLE_USAGE		#x00000100L
#define SQL_ISV_DOMAIN_CONSTRAINTS			#x00000200L
#define SQL_ISV_DOMAINS						#x00000400L
#define SQL_ISV_KEY_COLUMN_USAGE			#x00000800L
#define SQL_ISV_REFERENTIAL_CONSTRAINTS		#x00001000L
#define SQL_ISV_SCHEMATA					#x00002000L
#define SQL_ISV_SQL_LANGUAGES				#x00004000L
#define	SQL_ISV_TABLE_CONSTRAINTS			#x00008000L
#define SQL_ISV_TABLE_PRIVILEGES			#x00010000L
#define SQL_ISV_TABLES						#x00020000L
#define SQL_ISV_TRANSLATIONS				#x00040000L
#define SQL_ISV_USAGE_PRIVILEGES			#x00080000L
#define SQL_ISV_VIEW_COLUMN_USAGE			#x00100000L
#define SQL_ISV_VIEW_TABLE_USAGE			#x00200000L
#define SQL_ISV_VIEWS						#x00400000L

/* Bitmasks for SQL_ASYNC_MODE */

#define	SQL_AM_NONE		0
#define	SQL_AM_CONNECTION	1
#define	SQL_AM_STATEMENT	2

/* Bitmasks for SQL_ALTER_DOMAIN */
#define SQL_AD_CONSTRAINT_NAME_DEFINITION			#x00000001L	
#define	SQL_AD_ADD_DOMAIN_CONSTRAINT	 			#x00000002L
#define	SQL_AD_DROP_DOMAIN_CONSTRAINT	 			#x00000004L
#define	SQL_AD_ADD_DOMAIN_DEFAULT   	 			#x00000008L
#define	SQL_AD_DROP_DOMAIN_DEFAULT   	 			#x00000010L
#define SQL_AD_ADD_CONSTRAINT_INITIALLY_DEFERRED	#x00000020L
#define SQL_AD_ADD_CONSTRAINT_INITIALLY_IMMEDIATE	#x00000040L
#define SQL_AD_ADD_CONSTRAINT_DEFERRABLE			#x00000080L
#define SQL_AD_ADD_CONSTRAINT_NON_DEFERRABLE		#x00000100L


/* SQL_CREATE_SCHEMA bitmasks */
#define	SQL_CS_CREATE_SCHEMA				#x00000001L
#define	SQL_CS_AUTHORIZATION				#x00000002L
#define	SQL_CS_DEFAULT_CHARACTER_SET		#x00000004L

/* SQL_CREATE_TRANSLATION bitmasks */
#define	SQL_CTR_CREATE_TRANSLATION			#x00000001L

/* SQL_CREATE_ASSERTION bitmasks */
#define	SQL_CA_CREATE_ASSERTION					#x00000001L
#define	SQL_CA_CONSTRAINT_INITIALLY_DEFERRED	#x00000010L
#define	SQL_CA_CONSTRAINT_INITIALLY_IMMEDIATE	#x00000020L
#define	SQL_CA_CONSTRAINT_DEFERRABLE			#x00000040L
#define	SQL_CA_CONSTRAINT_NON_DEFERRABLE		#x00000080L

/* SQL_CREATE_CHARACTER_SET bitmasks */
#define	SQL_CCS_CREATE_CHARACTER_SET		#x00000001L
#define	SQL_CCS_COLLATE_CLAUSE				#x00000002L
#define	SQL_CCS_LIMITED_COLLATION			#x00000004L

/* SQL_CREATE_COLLATION bitmasks */
#define	SQL_CCOL_CREATE_COLLATION			#x00000001L

/* SQL_CREATE_DOMAIN bitmasks */
#define	SQL_CDO_CREATE_DOMAIN					#x00000001L
#define	SQL_CDO_DEFAULT							#x00000002L
#define	SQL_CDO_CONSTRAINT						#x00000004L
#define	SQL_CDO_COLLATION						#x00000008L
#define SQL_CDO_CONSTRAINT_NAME_DEFINITION		#x00000010L
#define SQL_CDO_CONSTRAINT_INITIALLY_DEFERRED	#x00000020L
#define SQL_CDO_CONSTRAINT_INITIALLY_IMMEDIATE	#x00000040L
#define SQL_CDO_CONSTRAINT_DEFERRABLE			#x00000080L
#define SQL_CDO_CONSTRAINT_NON_DEFERRABLE		#x00000100L

/* SQL_CREATE_TABLE bitmasks */
#define	SQL_CT_CREATE_TABLE						#x00000001L
#define	SQL_CT_COMMIT_PRESERVE					#x00000002L
#define	SQL_CT_COMMIT_DELETE					#x00000004L
#define	SQL_CT_GLOBAL_TEMPORARY					#x00000008L
#define	SQL_CT_LOCAL_TEMPORARY					#x00000010L
#define	SQL_CT_CONSTRAINT_INITIALLY_DEFERRED	#x00000020L
#define	SQL_CT_CONSTRAINT_INITIALLY_IMMEDIATE	#x00000040L
#define	SQL_CT_CONSTRAINT_DEFERRABLE			#x00000080L
#define	SQL_CT_CONSTRAINT_NON_DEFERRABLE		#x00000100L
#define SQL_CT_COLUMN_CONSTRAINT				#x00000200L
#define SQL_CT_COLUMN_DEFAULT					#x00000400L
#define SQL_CT_COLUMN_COLLATION					#x00000800L
#define SQL_CT_TABLE_CONSTRAINT					#x00001000L
#define SQL_CT_CONSTRAINT_NAME_DEFINITION		#x00002000L

/* SQL_DDL_INDEX bitmasks */
#define SQL_DI_CREATE_INDEX						#x00000001L
#define SQL_DI_DROP_INDEX						#x00000002L

/* SQL_DROP_COLLATION bitmasks */
#define	SQL_DC_DROP_COLLATION					#x00000001L

/* SQL_DROP_DOMAIN bitmasks */
#define	SQL_DD_DROP_DOMAIN						#x00000001L
#define	SQL_DD_RESTRICT							#x00000002L
#define	SQL_DD_CASCADE							#x00000004L

/* SQL_DROP_SCHEMA bitmasks */
#define	SQL_DS_DROP_SCHEMA						#x00000001L
#define SQL_DS_RESTRICT							#x00000002L
#define	SQL_DS_CASCADE							#x00000004L

/* SQL_DROP_CHARACTER_SET bitmasks */
#define	SQL_DCS_DROP_CHARACTER_SET				#x00000001L

/* SQL_DROP_ASSERTION bitmasks */
#define	SQL_DA_DROP_ASSERTION					#x00000001L

/* SQL_DROP_TABLE bitmasks */
#define	SQL_DT_DROP_TABLE						#x00000001L
#define	SQL_DT_RESTRICT							#x00000002L
#define	SQL_DT_CASCADE							#x00000004L

/* SQL_DROP_TRANSLATION bitmasks */
#define	SQL_DTR_DROP_TRANSLATION				#x00000001L

/* SQL_DROP_VIEW bitmasks */
#define	SQL_DV_DROP_VIEW						#x00000001L
#define	SQL_DV_RESTRICT							#x00000002L
#define	SQL_DV_CASCADE							#x00000004L

/* SQL_INSERT_STATEMENT bitmasks */
#define	SQL_IS_INSERT_LITERALS					#x00000001L
#define SQL_IS_INSERT_SEARCHED					#x00000002L
#define SQL_IS_SELECT_INTO						#x00000004L

/* SQL_ODBC_INTERFACE_CONFORMANCE values */
#define SQL_OIC_CORE							1UL
#define SQL_OIC_LEVEL1							2UL
#define SQL_OIC_LEVEL2							3UL

/* SQL_SQL92_FOREIGN_KEY_DELETE_RULE bitmasks */
#define SQL_SFKD_CASCADE						#x00000001L
#define SQL_SFKD_NO_ACTION						#x00000002L
#define SQL_SFKD_SET_DEFAULT					#x00000004L
#define SQL_SFKD_SET_NULL						#x00000008L

/* SQL_SQL92_FOREIGN_KEY_UPDATE_RULE bitmasks */
#define SQL_SFKU_CASCADE						#x00000001L
#define SQL_SFKU_NO_ACTION						#x00000002L
#define SQL_SFKU_SET_DEFAULT					#x00000004L
#define SQL_SFKU_SET_NULL						#x00000008L

/* SQL_SQL92_GRANT	bitmasks */
#define SQL_SG_USAGE_ON_DOMAIN					#x00000001L
#define SQL_SG_USAGE_ON_CHARACTER_SET			#x00000002L
#define SQL_SG_USAGE_ON_COLLATION				#x00000004L
#define SQL_SG_USAGE_ON_TRANSLATION				#x00000008L
#define SQL_SG_WITH_GRANT_OPTION				#x00000010L
#define SQL_SG_DELETE_TABLE						#x00000020L
#define SQL_SG_INSERT_TABLE						#x00000040L
#define SQL_SG_INSERT_COLUMN					#x00000080L
#define SQL_SG_REFERENCES_TABLE					#x00000100L
#define SQL_SG_REFERENCES_COLUMN				#x00000200L
#define SQL_SG_SELECT_TABLE						#x00000400L
#define SQL_SG_UPDATE_TABLE						#x00000800L
#define SQL_SG_UPDATE_COLUMN					#x00001000L	

/* SQL_SQL92_PREDICATES bitmasks */
#define SQL_SP_EXISTS							#x00000001L
#define SQL_SP_ISNOTNULL						#x00000002L
#define SQL_SP_ISNULL							#x00000004L
#define SQL_SP_MATCH_FULL						#x00000008L
#define SQL_SP_MATCH_PARTIAL					#x00000010L
#define SQL_SP_MATCH_UNIQUE_FULL				#x00000020L
#define SQL_SP_MATCH_UNIQUE_PARTIAL				#x00000040L
#define SQL_SP_OVERLAPS							#x00000080L
#define SQL_SP_UNIQUE							#x00000100L
#define SQL_SP_LIKE							#x00000200L
#define SQL_SP_IN							#x00000400L
#define SQL_SP_BETWEEN							#x00000800L
#define SQL_SP_COMPARISON						#x00001000L
#define SQL_SP_QUANTIFIED_COMPARISON			#x00002000L

/* SQL_SQL92_RELATIONAL_JOIN_OPERATORS bitmasks */
#define SQL_SRJO_CORRESPONDING_CLAUSE			0x00000001L
#define SQL_SRJO_CROSS_JOIN						0x00000002L
#define SQL_SRJO_EXCEPT_JOIN					0x00000004L
#define SQL_SRJO_FULL_OUTER_JOIN				0x00000008L
#define SQL_SRJO_INNER_JOIN						0x00000010L
#define SQL_SRJO_INTERSECT_JOIN					0x00000020L
#define SQL_SRJO_LEFT_OUTER_JOIN				0x00000040L
#define SQL_SRJO_NATURAL_JOIN					0x00000080L
#define SQL_SRJO_RIGHT_OUTER_JOIN				0x00000100L
#define SQL_SRJO_UNION_JOIN						0x00000200L

/* SQL_SQL92_REVOKE bitmasks */
#define SQL_SR_USAGE_ON_DOMAIN					0x00000001L
#define SQL_SR_USAGE_ON_CHARACTER_SET			0x00000002L
#define SQL_SR_USAGE_ON_COLLATION				0x00000004L
#define SQL_SR_USAGE_ON_TRANSLATION				0x00000008L
#define SQL_SR_GRANT_OPTION_FOR					0x00000010L
#define SQL_SR_CASCADE							0x00000020L
#define SQL_SR_RESTRICT							0x00000040L
#define SQL_SR_DELETE_TABLE						0x00000080L
#define SQL_SR_INSERT_TABLE						0x00000100L
#define SQL_SR_INSERT_COLUMN					0x00000200L
#define SQL_SR_REFERENCES_TABLE					0x00000400L
#define SQL_SR_REFERENCES_COLUMN				0x00000800L
#define SQL_SR_SELECT_TABLE						0x00001000L
#define SQL_SR_UPDATE_TABLE						0x00002000L
#define SQL_SR_UPDATE_COLUMN					0x00004000L

/* SQL_SQL92_ROW_VALUE_CONSTRUCTOR bitmasks */
#define SQL_SRVC_VALUE_EXPRESSION				0x00000001L
#define SQL_SRVC_NULL							0x00000002L
#define SQL_SRVC_DEFAULT						0x00000004L
#define SQL_SRVC_ROW_SUBQUERY					0x00000008L

/* SQL_SQL92_VALUE_EXPRESSIONS bitmasks */
#define SQL_SVE_CASE							0x00000001L
#define SQL_SVE_CAST							0x00000002L
#define SQL_SVE_COALESCE						0x00000004L
#define SQL_SVE_NULLIF							0x00000008L

/* SQL_STANDARD_CLI_CONFORMANCE bitmasks */
#define SQL_SCC_XOPEN_CLI_VERSION1				0x00000001L
#define SQL_SCC_ISO92_CLI						0x00000002L

/* SQL_UNION_STATEMENT bitmasks */
#define SQL_US_UNION							SQL_U_UNION
#define SQL_US_UNION_ALL						SQL_U_UNION_ALL

#endif  /* ODBCVER >= 0x0300 */

/* SQL_DTC_TRANSITION_COST bitmasks */
#define SQL_DTC_ENLIST_EXPENSIVE				0x00000001L
#define SQL_DTC_UNENLIST_EXPENSIVE				0x00000002L
|#
;; /* additional SQLDataSources fetch directions */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_FETCH_FIRST_USER   31)
(defconstant SQL_FETCH_FIRST_SYSTEM 32)
;; #endif

;; Defines for SQLSetPos
(defconstant SQL_ENTIRE_ROWSET	0)
;; Operations in SQLSetPos
(defconstant SQL_POSITION	0)	; 1.0 FALSE
(defconstant SQL_REFRESH	1)	; 1.0 TRUE
(defconstant SQL_UPDATE		2)
(defconstant SQL_DELETE		3)

(defconstant SQL_ADD 4)
;#define SQL_SETPOS_MAX_OPTION_VALUE SQL_ADD
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_UPDATE_BY_BOOKMARK 5)
(defconstant SQL_DELETE_BY_BOOKMARK 6)
(defconstant SQL_FETCH_BY_BOOKMARK  7)
;; #endif

;; Lock options in SQLSetPos
(defconstant SQL_LOCK_NO_CHANGE 0)	; 1.0 FALSE
(defconstant SQL_LOCK_EXCLUSIVE 1)	; 1.0 TRUE
(defconstant SQL_LOCK_UNLOCK    2)
;#define SQL_SETPOS_MAX_LOCK_VALUE SQL_LOCK_UNLOCK

#|
;; Macros for SQLSetPos
(defconstant SQL_POSITION_TO(hstmt,irow) SQLSetPos(hstmt,irow,SQL_POSITION,SQL_LOCK_NO_CHANGE))
(defconstant SQL_LOCK_RECORD(hstmt,irow,fLock) SQLSetPos(hstmt,irow,SQL_POSITION,fLock))
(defconstant SQL_REFRESH_RECORD(hstmt,irow,fLock) SQLSetPos(hstmt,irow,SQL_REFRESH,fLock))
(defconstant SQL_UPDATE_RECORD(hstmt,irow) SQLSetPos(hstmt,irow,SQL_UPDATE,SQL_LOCK_NO_CHANGE))
(defconstant SQL_DELETE_RECORD(hstmt,irow) SQLSetPos(hstmt,irow,SQL_DELETE,SQL_LOCK_NO_CHANGE))
(defconstant SQL_ADD_RECORD(hstmt,irow) SQLSetPos(hstmt,irow,SQL_ADD,SQL_LOCK_NO_CHANGE)
 |#
;; Column types and scopes in SQLSpecialColumns. 
(defconstant SQL_BEST_ROWID	1)
(defconstant SQL_ROWVER		2)

#|/* Defines for SQLStatistics */
(defconstant SQL_QUICK		0)
(defconstant SQL_ENSURE		1)

/* Defines for SQLStatistics (returned in the result set) 
   SQL_INDEX_CLUSTERED, SQL_INDEX_HASHED, and SQL_INDEX_OTHER are
   defined in sql.h */
#define SQL_TABLE_STAT                  0
 |#

;; Defines for SQLTables
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_ALL_CATALOGS	 "%")
(defconstant SQL_ALL_SCHEMAS	 "%")
(defconstant SQL_ALL_TABLE_TYPES "%")
;; #endif

;; Options for SQLDriverConnect
(defconstant SQL_DRIVER_NOPROMPT 0)
(defconstant SQL_DRIVER_COMPLETE 1)
(defconstant SQL_DRIVER_PROMPT 2)
(defconstant SQL_DRIVER_COMPLETE_REQUIRED 3)

;;  SQLExtendedFetch "fFetchType" values 
(defconstant SQL_FETCH_BOOKMARK 8)

;; SQLExtendedFetch "rgfRowStatus" element values
(defconstant SQL_ROW_SUCCESS 		0)
(defconstant SQL_ROW_DELETED 		1)
(defconstant SQL_ROW_UPDATED 		2)
(defconstant SQL_ROW_NOROW		3)
(defconstant SQL_ROW_ADDED		4)
(defconstant SQL_ROW_ERROR		5)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_ROW_SUCCESS_WITH_INFO	6)
(defconstant SQL_ROW_PROCEED		0)
(defconstant SQL_ROW_IGNORE		1)
;; #endif 

;; value for SQL_DESC_ARRAY_STATUS_PTR
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_PARAM_SUCCESS		0)
(defconstant SQL_PARAM_SUCCESS_WITH_INFO 6)
(defconstant SQL_PARAM_ERROR		5)
(defconstant SQL_PARAM_UNUSED		7)
(defconstant SQL_PARAM_DIAG_UNAVAILABLE	1)

(defconstant SQL_PARAM_PROCEED		0)
(defconstant SQL_PARAM_IGNORE		1)
;; #endif

;; Defines for SQLForeignKeys (UPDATE_RULE and DELETE_RULE)
(defconstant SQL_CASCADE	0)
(defconstant SQL_RESTRICT	1)
(defconstant SQL_SET_NULL	2)
(defconstant SQL_NO_ACTION	3)
(defconstant SQL_SET_DEFAULT	4)

;; #if (ODBCVER >= 0x0300)
;; /* Note that the following are in a different column of SQLForeignKeys than */
;; /* the previous #defines.   These are for DEFERRABILITY.                    */
(defconstant SQL_INITIALLY_DEFERRED  5)
(defconstant SQL_INITIALLY_IMMEDIATE 6)
(defconstant SQL_NOT_DEFERRABLE	     7)
;; #endif

#|
;;/* Defines for SQLBindParameter and SQLProcedureColumns ->MOVED to START of the FILE*/

;; Defines for SQLProcedures (returned in the result set)
(defconstant SQL_PT_UNKNOWN	0)
(defconstant SQL_PT_PROCEDURE	1)
(defconstant SQL_PT_FUNCTION 	2)

;; #ifndef RC_INVOKED
;; This define is too large for RC 

#define SQL_ODBC_KEYWORDS \
"ABSOLUTE,ACTION,ADA,ADD,ALL,ALLOCATE,ALTER,AND,ANY,ARE,AS,"\
"ASC,ASSERTION,AT,AUTHORIZATION,AVG,"\
"BEGIN,BETWEEN,BIT,BIT_LENGTH,BOTH,BY,CASCADE,CASCADED,CASE,CAST,CATALOG,"\
"CHAR,CHAR_LENGTH,CHARACTER,CHARACTER_LENGTH,CHECK,CLOSE,COALESCE,"\
"COLLATE,COLLATION,COLUMN,COMMIT,CONNECT,CONNECTION,CONSTRAINT,"\
"CONSTRAINTS,CONTINUE,CONVERT,CORRESPONDING,COUNT,CREATE,CROSS,CURRENT,"\
"CURRENT_DATE,CURRENT_TIME,CURRENT_TIMESTAMP,CURRENT_USER,CURSOR,"\
"DATE,DAY,DEALLOCATE,DEC,DECIMAL,DECLARE,DEFAULT,DEFERRABLE,"\
"DEFERRED,DELETE,DESC,DESCRIBE,DESCRIPTOR,DIAGNOSTICS,DISCONNECT,"\
"DISTINCT,DOMAIN,DOUBLE,DROP,"\
"ELSE,END,END-EXEC,ESCAPE,EXCEPT,EXCEPTION,EXEC,EXECUTE,"\
"EXISTS,EXTERNAL,EXTRACT,"\
"FALSE,FETCH,FIRST,FLOAT,FOR,FOREIGN,FORTRAN,FOUND,FROM,FULL,"\
"GET,GLOBAL,GO,GOTO,GRANT,GROUP,HAVING,HOUR,"\
"IDENTITY,IMMEDIATE,IN,INCLUDE,INDEX,INDICATOR,INITIALLY,INNER,"\
"INPUT,INSENSITIVE,INSERT,INT,INTEGER,INTERSECT,INTERVAL,INTO,IS,ISOLATION,"\
"JOIN,KEY,LANGUAGE,LAST,LEADING,LEFT,LEVEL,LIKE,LOCAL,LOWER,"\
"MATCH,MAX,MIN,MINUTE,MODULE,MONTH,"\
"NAMES,NATIONAL,NATURAL,NCHAR,NEXT,NO,NONE,NOT,NULL,NULLIF,NUMERIC,"\
"OCTET_LENGTH,OF,ON,ONLY,OPEN,OPTION,OR,ORDER,OUTER,OUTPUT,OVERLAPS,"\
"PAD,PARTIAL,PASCAL,PLI,POSITION,PRECISION,PREPARE,PRESERVE,"\
"PRIMARY,PRIOR,PRIVILEGES,PROCEDURE,PUBLIC,"\
"READ,REAL,REFERENCES,RELATIVE,RESTRICT,REVOKE,RIGHT,ROLLBACK,ROWS"\
"SCHEMA,SCROLL,SECOND,SECTION,SELECT,SESSION,SESSION_USER,SET,SIZE,"\
"SMALLINT,SOME,SPACE,SQL,SQLCA,SQLCODE,SQLERROR,SQLSTATE,SQLWARNING,"\
"SUBSTRING,SUM,SYSTEM_USER,"\
"TABLE,TEMPORARY,THEN,TIME,TIMESTAMP,TIMEZONE_HOUR,TIMEZONE_MINUTE,"\
"TO,TRAILING,TRANSACTION,TRANSLATE,TRANSLATION,TRIM,TRUE,"\
"UNION,UNIQUE,UNKNOWN,UPDATE,UPPER,USAGE,USER,USING,"\
"VALUE,VALUES,VARCHAR,VARYING,VIEW,WHEN,WHENEVER,WHERE,WITH,WORK,WRITE,"\
"YEAR,ZONE"
 |#

