;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2005 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Constants from
;;;  SQL.H	This is the the main include for ODBC Core functions.

(in-package :ystok.odbc)

;; /* special length/indicator values */
(defconstant SQL_NULL_DATA -1) 		; If the data value for a column is NULL.
(defconstant SQL_DATA_AT_EXEC -2)

;; /* return values from functions */ 
(defconstant SQL_SUCCESS 0)
(defconstant SQL_SUCCESS_WITH_INFO 1)
(defconstant SQL_NO_DATA 100)

(defconstant SQL_ERROR -1)
(defconstant SQL_INVALID_HANDLE -2)
(defconstant SQL_STILL_EXECUTING 2)
(defconstant SQL_NEED_DATA 99)

;; sql.h:  Test for SQL_SUCCESS or SQL_SUCCESS_WITH_INFO
;#define SQL_SUCCEEDED(rc) (((rc)&(~1))==0)
(defmacro %sql-succeeded (res) `(zerop (logandc2 ,res 1)))

;; /* flags for null-terminated string */
(defconstant SQL_NTS -3)
;#define SQL_NTSL (-3L)

(defconstant SQL_MAX_MESSAGE_LENGTH 512)	; message buffer size

;; /* date/time length constants */
;; #if (ODBCVER >= 0x0300)
;#define SQL_DATE_LEN           10
;#define SQL_TIME_LEN            8  /* add P+1 if precision is nonzero */
;#define SQL_TIMESTAMP_LEN      19  /* add P+1 if precision is nonzero */
;; #endif

;; Whether an attribute is a pointer or not <= Moved here from {{sqlext.h}}
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_IS_POINTER	-4)
(defconstant SQL_IS_UINTEGER	-5)
(defconstant SQL_IS_INTEGER	-6)
(defconstant SQL_IS_USMALLINT	-7)
(defconstant SQL_IS_SMALLINT	-8)
;; #endif  /* ODBCVER >= 0x0300 */


;; handle type identifiers 
;#if (ODBCVER >= 0x0300)
(defconstant SQL_HANDLE_ENV 1)
(defconstant SQL_HANDLE_DBC 2)
(defconstant SQL_HANDLE_STMT 3)
(defconstant SQL_HANDLE_DESC 4)
;#endif

;; /* environment attribute */
;; #if (ODBCVER >= 0x0300)
;(defconstant SQL_ATTR_OUTPUT_NTS 10001)
;; #endif

;; /* connection attributes */
;; #if (ODBCVER >= 0x0300)
;#define SQL_ATTR_AUTO_IPD      10001
;#define SQL_ATTR_METADATA_ID   10014
;; #endif  /* ODBCVER >= 0x0300 */

;; /* statement attributes */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_ATTR_APP_ROW_DESC       10010)
(defconstant SQL_ATTR_APP_PARAM_DESC     10011)
(defconstant SQL_ATTR_IMP_ROW_DESC       10012)
(defconstant SQL_ATTR_IMP_PARAM_DESC     10013)
(defconstant SQL_ATTR_CURSOR_SCROLLABLE  -1)
(defconstant SQL_ATTR_CURSOR_SENSITIVITY -2)
;; #endif

;; /* SQL_ATTR_CURSOR_SCROLLABLE values */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_NONSCROLLABLE 0)
(defconstant SQL_SCROLLABLE 1)
;; #endif  /* ODBCVER >= 0x0300 */

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Descriptor field types used in SQLColAttribute
(eval-when (:compile-toplevel :load-toplevel :execute)
 (defvar *desc-attribute-result-types*     ; <0 for :long; >=0 for :string length
   (make-array 54 :element-type '(integer -1 32000) :initial-element 0))
  
 (defmacro %get-desc-attribute-type (attribute)
   `(aref *desc-attribute-result-types*
          (if (>= ,attribute 1000) (+ (mod ,attribute 1000) 40) ,attribute)))

 (defun %ensure-desc-attribute-type (attribute result-type)
   (let ((old (%get-desc-attribute-type attribute))
         (new (if (numberp result-type)
                  result-type
                  (ecase (first-or-self result-type)	 	; max length
                    (:string (if (consp result-type) (second result-type) 128))
                    ((:ptr :pointer)	SQL_IS_POINTER)
                    (:unsigned-long	SQL_IS_UINTEGER)
                    (:long		SQL_IS_INTEGER)
                    (:unsigned-short	SQL_IS_USMALLINT)
                    (:short		SQL_IS_SMALLINT)))))
     (unless (or (zerop old) (eql old new))
       (warn "Redefining SQLColAttribute constant: attribute ~s, new result type ~s=>~s~@[, old ~s~]."
             attribute result-type new old))
     (setf (%get-desc-attribute-type attribute) new)))

 (defmacro def-desc-attribute-type (name value result-type
                                    &key documentation X/Open deprecated)
   "Issue defconstant and register as an argument SQLColAttribute(). Arguments:
 NAME : constant name;
 RESULT-TYPE : one of :long, :string, or (:string max-length)."
    (declare (ignore X/Open deprecated))
    #+lispworks
    `(dspec:def (def-desc-attribute-type ,name)
       (defconstant ,name ,value ,@(when documentation `(,documentation)))
       (%ensure-desc-attribute-type ,name ,result-type))
    #-lispworks
    `(progn (defconstant ,name ,value ,@(when documentation `(,documentation)))
            (%ensure-desc-attribute-type ,name ,result-type)))
                           ;,(if (constantp result-type) result-type `',result-type))))
 #+lispworks
 (dspec:define-dspec-alias def-desc-attribute-type (name)
  `(defconstant ,name))
) ; eval-when

;; /* identifiers of fields in the SQL descriptor */
;; #if (ODBCVER >= 0x0300)
(def-desc-attribute-type SQL_DESC_COUNT                  1001 SQL_IS_SMALLINT)
(def-desc-attribute-type SQL_DESC_TYPE                   1002 SQL_IS_SMALLINT)
(def-desc-attribute-type SQL_DESC_LENGTH                 1003 SQL_IS_UINTEGER)
(def-desc-attribute-type SQL_DESC_OCTET_LENGTH_PTR       1004 SQL_IS_UINTEGER)
(def-desc-attribute-type SQL_DESC_PRECISION              1005 SQL_IS_SMALLINT)
(def-desc-attribute-type SQL_DESC_SCALE                  1006 SQL_IS_SMALLINT)
(def-desc-attribute-type SQL_DESC_DATETIME_INTERVAL_CODE 1007 SQL_IS_INTEGER)
(def-desc-attribute-type SQL_DESC_NULLABLE               1008 SQL_IS_SMALLINT)
(def-desc-attribute-type SQL_DESC_INDICATOR_PTR          1009 SQL_IS_POINTER)
(def-desc-attribute-type SQL_DESC_DATA_PTR               1010 SQL_IS_POINTER)
(def-desc-attribute-type SQL_DESC_NAME                   1011 '(:string 128))
(def-desc-attribute-type SQL_DESC_UNNAMED                1012 SQL_IS_SMALLINT)
(def-desc-attribute-type SQL_DESC_OCTET_LENGTH           1013 SQL_IS_INTEGER)
;#define SQL_DESC_ALLOC_TYPE             1099
;; #endif

;; identifiers of fields in the diagnostics area 
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_DIAG_RETURNCODE        1)
(defconstant SQL_DIAG_NUMBER            2)
(defconstant SQL_DIAG_ROW_COUNT         3)
(defconstant SQL_DIAG_SQLSTATE          4)
(defconstant SQL_DIAG_NATIVE            5)
(defconstant SQL_DIAG_MESSAGE_TEXT      6)
(defconstant SQL_DIAG_DYNAMIC_FUNCTION  7)
(defconstant SQL_DIAG_CLASS_ORIGIN      8)
(defconstant SQL_DIAG_SUBCLASS_ORIGIN   9)
(defconstant SQL_DIAG_CONNECTION_NAME  10)
(defconstant SQL_DIAG_SERVER_NAME      11)
(defconstant SQL_DIAG_DYNAMIC_FUNCTION_CODE 12)
;; #endif

#|/* dynamic function codes */
#if (ODBCVER >= 0x0300)
(defconstant SQL_DIAG_ALTER_DOMAIN		3)
(defconstant SQL_DIAG_ALTER_TABLE		4)
(defconstant SQL_DIAG_CALL			7)
(defconstant SQL_DIAG_CREATE_ASSERTION		6)
(defconstant SQL_DIAG_CREATE_CHARACTER_SET	8)
(defconstant SQL_DIAG_CREATE_COLLATION		10)
(defconstant SQL_DIAG_CREATE_DOMAIN		23)
(defconstant SQL_DIAG_CREATE_INDEX		-1)
(defconstant SQL_DIAG_CREATE_SCHEMA		64)
(defconstant SQL_DIAG_CREATE_TABLE		77)
(defconstant SQL_DIAG_CREATE_TRANSLATION	79)
(defconstant SQL_DIAG_CREATE_VIEW		84)
(defconstant SQL_DIAG_DELETE_WHERE		19)
(defconstant SQL_DIAG_DROP_ASSERTION		24)
(defconstant SQL_DIAG_DROP_CHARACTER_SET	25)
(defconstant SQL_DIAG_DROP_COLLATION		26)
(defconstant SQL_DIAG_DROP_DOMAIN		27)
(defconstant SQL_DIAG_DROP_INDEX		-2)
(defconstant SQL_DIAG_DROP_SCHEMA		31)
(defconstant SQL_DIAG_DROP_TABLE		32)
(defconstant SQL_DIAG_DROP_TRANSLATION		33)
(defconstant SQL_DIAG_DROP_VIEW			36)
(defconstant SQL_DIAG_DYNAMIC_DELETE_CURSOR	38)
(defconstant SQL_DIAG_DYNAMIC_UPDATE_CURSOR	81)
(defconstant SQL_DIAG_GRANT			48)
(defconstant SQL_DIAG_INSERT			50)
(defconstant SQL_DIAG_REVOKE			59)
(defconstant SQL_DIAG_SELECT_CURSOR		85)
(defconstant SQL_DIAG_UNKNOWN_STATEMENT		0)
(defconstant SQL_DIAG_UPDATE_WHERE		82)
#endif  /* ODBCVER >= 0x0300 */
 |#

;; Standard SQL datatypes, using ANSI type numbering
(defconstant SQL_UNKNOWN_TYPE 0)
(defconstant SQL_CHAR 1)
(defconstant SQL_NUMERIC 2)
(defconstant SQL_DECIMAL 3)
(defconstant SQL_INTEGER 4)
(defconstant SQL_SMALLINT 5)
(defconstant SQL_FLOAT 6)
(defconstant SQL_REAL 7)
(defconstant SQL_DOUBLE 8)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_DATETIME 9)
;; #endif
(defconstant SQL_VARCHAR 12)

;; {{ sqlucode.h
(defconstant SQL_WCHAR -8)
(defconstant SQL_WVARCHAR -9)
(defconstant SQL_WLONGVARCHAR -10)
(defconstant SQL_C_WCHAR SQL_WCHAR)
;; }}

;; One-parameter shortcuts for date/time data types
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_TYPE_DATE      91)
(defconstant SQL_TYPE_TIME      92)
(defconstant SQL_TYPE_TIMESTAMP 93)
;; #endif
;#define SQL_TYPE_TIME_WITH_TIMEZONE	94
;#define SQL_TYPE_TIMESTAMP_WITH_TIMEZONE 95

;; Statement attribute values for cursor sensitivity
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_UNSPECIFIED     0)
(defconstant SQL_INSENSITIVE     1)
(defconstant SQL_SENSITIVE       2)
;; #endif

(defconstant SQL_ALL_TYPES 0)		; GetTypeInfo() request for all data types 

;; /* Default conversion code for SQLBindCol(), SQLBindParam() and SQLGetData() */
;; #if (ODBCVER >= 0x0300)
;(defconstant SQL_DEFAULT 99)
;; #endif

;; /* SQLGetData() code indicating that the application row descriptor specifies the data type */
;; #if (ODBCVER >= 0x0300)
;(defconstant SQL_ARD_TYPE -99)
;;#endif

;; /* SQL date/time type subcodes */
;; #if (ODBCVER >= 0x0300)
;#define SQL_CODE_DATE       1
;#define SQL_CODE_TIME       2
;#define SQL_CODE_TIMESTAMP  3
;; #endif

;; /* CLI option values */
;#if (ODBCVER >= 0x0300)
(defconstant SQL_FALSE 0)
(defconstant SQL_TRUE 1)
;;#endif

;; NULL status constants.  These are used in SQLColumns, SQLColAttributes,
;; SQLDescribeCol, SQLDescribeParam, and SQLSpecialColumns to describe the
;; nullablity of a column in a table.

(defconstant SQL_NO_NULLS 0)
(defconstant SQL_NULLABLE 1)

;; Value returned by SQLGetTypeInfo() to denote that it is
;; not known whether or not a data type supports null values.
(defconstant SQL_NULLABLE_UNKNOWN 2)

;; Values returned by SQLGetTypeInfo() to show WHERE clause supported
;; #if (ODBCVER >= 0x0300)
;#define SQL_PRED_NONE     0
;#define SQL_PRED_CHAR     1
;#define SQL_PRED_BASIC    2
;; #endif

;; /* values of UNNAMED field in descriptor */
;; #if (ODBCVER >= 0x0300)
;#define SQL_NAMED           0
;#define SQL_UNNAMED         1
;; #endif

;; /* values of ALLOC_TYPE field in descriptor */
;; #if (ODBCVER >= 0x0300)
;#define SQL_DESC_ALLOC_AUTO 1
;#define SQL_DESC_ALLOC_USER 2
;; #endif


;; SQLFreeStmt defines
(defconstant SQL_CLOSE 0)
(defconstant SQL_DROP 1)	; deprecated
(defconstant SQL_UNBIND 2)
(defconstant SQL_RESET_PARAMS 3)

;; SQLExtendedFetch - deprecated
;; /* Codes used for FetchOrientation in SQLFetchScroll(), and in SQLDataSources() */
(defconstant SQL_FETCH_NEXT 1)
(defconstant SQL_FETCH_FIRST 2)
;; /* Other codes used for FetchOrientation in SQLFetchScroll() */
(defconstant SQL_FETCH_LAST 3)
(defconstant SQL_FETCH_PRIOR 4)
(defconstant SQL_FETCH_ABSOLUTE 5)
(defconstant SQL_FETCH_RELATIVE 6)

;; SQLTransact defines - deprecated
;; /* SQLEndTran() options */
(defconstant SQL_COMMIT 0)
(defconstant SQL_ROLLBACK 1)

;; /* null handles returned by SQLAllocHandle() */
(defconstant SQL_NULL_HENV 0)
(defconstant SQL_NULL_HDBC 0)
(defconstant SQL_NULL_HSTMT 0)
;; #if (ODBCVER >= 0x0300)
;#define SQL_NULL_HDESC      0
;; #endif

;; /* null handle used in place of parent handle when allocating HENV */
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_NULL_HANDLE 0)
;; #endif

;; Values that may appear in the result set of SQLSpecialColumns()
(defconstant SQL_SCOPE_CURROW	   0)
(defconstant SQL_SCOPE_TRANSACTION 1)
(defconstant SQL_SCOPE_SESSION	   2)

;; Defines for SQLSpecialColumns (returned in the result set
(defconstant SQL_PC_UNKNOWN	0)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_PC_NON_PSEUDO	1)
;; #endif
(defconstant SQL_PC_PSEUDO	2)

;; Reserved value for the IdentifierType argument of SQLSpecialColumns() 
;; #if (ODBCVER >= 0x0300)
;#define SQL_ROW_IDENTIFIER  1
;;#endif

;; Reserved values for UNIQUE argument of SQLStatistics()
(defconstant SQL_INDEX_UNIQUE	0)
(defconstant SQL_INDEX_ALL	1)

;; Values that may appear in the result set of SQLStatistics()
(defconstant SQL_TABLE_STAT	 0)
(defconstant SQL_INDEX_CLUSTERED 1)
(defconstant SQL_INDEX_HASHED	 2)
(defconstant SQL_INDEX_OTHER	 3)

;; Defines for SQLGetFunctions
;; Core Functions
;; SQLGetFunctions() values to identify ODBC APIs
(defconstant SQL_API_SQLALLOCCONNECT      1)
(defconstant SQL_API_SQLALLOCENV          2)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLALLOCHANDLE    1001)
;; #endif
(defconstant SQL_API_SQLALLOCSTMT         3)
(defconstant SQL_API_SQLBINDCOL           4)
;; #if (ODBCVER >= 0x0300)
;(defconstant SQL_API_SQLBINDPARAM      1002)
;; #endif
(defconstant SQL_API_SQLCANCEL            5)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLCLOSECURSOR    1003)
(defconstant SQL_API_SQLCOLATTRIBUTES	  6)
;; #endif
(defconstant SQL_API_SQLCONNECT           7)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLCOPYDESC       1004)
;; #endif
(defconstant SQL_API_SQLDATASOURCES      57)
(defconstant SQL_API_SQLDESCRIBECOL       8)
(defconstant SQL_API_SQLDISCONNECT        9)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLENDTRAN        1005)
;; #endif
(defconstant SQL_API_SQLERROR            10)
(defconstant SQL_API_SQLEXECDIRECT       11)
(defconstant SQL_API_SQLEXECUTE          12)
(defconstant SQL_API_SQLFETCH            13)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLFETCHSCROLL    1021)
;; #endif
(defconstant SQL_API_SQLFREECONNECT      14)
(defconstant SQL_API_SQLFREEENV          15)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLFREEHANDLE     1006)
;; #endif
(defconstant SQL_API_SQLFREESTMT         16)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLGETCONNECTATTR 1007)
;; #endif
(defconstant SQL_API_SQLGETCONNECTOPTION 42)
(defconstant SQL_API_SQLGETCURSORNAME    17)
(defconstant SQL_API_SQLGETDATA          43)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLGETDESCFIELD   1008)
(defconstant SQL_API_SQLGETDESCREC     1009)
(defconstant SQL_API_SQLGETDIAGFIELD   1010)
(defconstant SQL_API_SQLGETDIAGREC     1011)
(defconstant SQL_API_SQLGETENVATTR     1012)
;; #endif
(defconstant SQL_API_SQLGETFUNCTIONS     44)
(defconstant SQL_API_SQLGETINFO          45)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLGETSTMTATTR    1014)
;; #endif
(defconstant SQL_API_SQLGETSTMTOPTION    46)
(defconstant SQL_API_SQLGETTYPEINFO      47)
(defconstant SQL_API_SQLNUMRESULTCOLS    18)
(defconstant SQL_API_SQLPARAMDATA        48)
(defconstant SQL_API_SQLPREPARE          19)
(defconstant SQL_API_SQLPUTDATA          49)
(defconstant SQL_API_SQLROWCOUNT         20)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLSETCONNECTATTR 1016)
;; #endif
(defconstant SQL_API_SQLSETCONNECTOPTION 50)
(defconstant SQL_API_SQLSETCURSORNAME    21)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLSETDESCFIELD   1017)
(defconstant SQL_API_SQLSETDESCREC     1018)
(defconstant SQL_API_SQLSETENVATTR     1019)
;; #endif
(defconstant SQL_API_SQLSETPARAM         22)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_API_SQLSETSTMTATTR    1020)
 ;;#endif
(defconstant SQL_API_SQLSETSTMTOPTION    51)
(defconstant SQL_API_SQLSPECIALCOLUMNS   52)
(defconstant SQL_API_SQLSTATISTICS       53)
(defconstant SQL_API_SQLTABLES           54)
(defconstant SQL_API_SQLTRANSACT         23)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; SQLGetInfo types

(eval-when (:compile-toplevel :load-toplevel :execute)
 (defvar *get-info-result-types* (make-hash-table :size 50))
  
 (defmacro %get-info-result-type (info-type)
   `(gethash ,info-type *get-info-result-types*))

 (defun %ensure-get-info-type (info-type result-type)
   (let ((old (%get-info-result-type info-type)))
     (when (and old (not (eql old result-type)))
       (warn "Redefining SQLGetInfo constant: info-type ~s, new result type ~s~@[, old ~s~]."
             info-type result-type old)))
   (setf (gethash info-type *get-info-result-types*) result-type))

 (defmacro def-get-info-type (name value result-type
                              &key documentation X/Open deprecated)
   "Issue defconstant and register as an argument SQLGetInfo(). Arguments:
 NAME : constant name;
 RESULT-TYPE : one of :string, :boolean (Y or N chars to Lisp boolean),
   :long or :short."
    (declare (ignore X/Open deprecated))
    #+lispworks
    `(dspec:def (def-get-info-type ,name)
       (defconstant ,name ,value ,@(when documentation `(,documentation)))
       (%ensure-get-info-type ,name ,result-type))
    #-lispworks
    `(progn (defconstant ,name ,value ,@(when documentation `(,documentation)))
            (%ensure-get-info-type ,name ,result-type)))
 #+lispworks
 (dspec:define-dspec-alias def-get-info-type (name)
  `(defconstant ,name))
) ; eval-when

;; /* Information requested by SQLGetInfo() */
;; #if (ODBCVER >= 0x0300)
(def-get-info-type SQL_MAX_DRIVER_CONNECTIONS	 0 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_DRIVER_CONNECTIONS	SQL_MAX_DRIVER_CONNECTIONS)
(def-get-info-type SQL_MAX_CONCURRENT_ACTIVITIES 1 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_CONCURRENT_ACTIVITIES	SQL_MAX_CONCURRENT_ACTIVITIES)
;; #endif
(def-get-info-type SQL_DATA_SOURCE_NAME		 2 :string	:X/Open t)
(def-get-info-type SQL_FETCH_DIRECTION		 8 :long	:deprecated 3.0)
(def-get-info-type SQL_SERVER_NAME		13 :string	:X/Open t)
(def-get-info-type SQL_SEARCH_PATTERN_ESCAPE	14 :string	:X/Open t)

(def-get-info-type SQL_DBMS_NAME		17 :string	:X/Open t)
(def-get-info-type SQL_DBMS_VER			18 :string	:X/Open t)

(def-get-info-type SQL_ACCESSIBLE_TABLES	19 :boolean	:X/Open t)
(def-get-info-type SQL_ACCESSIBLE_PROCEDURES	20 :boolean	:X/Open t)
(def-get-info-type SQL_CURSOR_COMMIT_BEHAVIOR	23 :short	:X/Open t)
(def-get-info-type SQL_DATA_SOURCE_READ_ONLY	25 :boolean	:X/Open t)
(def-get-info-type SQL_DEFAULT_TXN_ISOLATION	26 :long	:X/Open t)
(def-get-info-type SQL_IDENTIFIER_CASE		28 :short	:X/Open t)
(def-get-info-type SQL_IDENTIFIER_QUOTE_CHAR	29 :string	:X/Open t)
(def-get-info-type SQL_MAX_COLUMN_NAME_LEN      30 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_COLUMN_NAME_LENGTH	SQL_MAX_COLUMN_NAME_LEN)
(def-get-info-type SQL_MAX_CURSOR_NAME_LEN      31 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_CURSOR_NAME_LENGTH	SQL_MAX_CURSOR_NAME_LEN)
(def-get-info-type SQL_MAX_SCHEMA_NAME_LEN      32 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_SCHEMA_NAME_LENGTH	SQL_MAX_SCHEMA_NAME_LEN)
(def-get-info-type SQL_MAX_CATALOG_NAME_LEN     34 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_CATALOG_NAME_LENGTH	SQL_MAX_CATALOG_NAME_LEN)
(def-get-info-type SQL_MAX_TABLE_NAME_LEN       35 :short	:X/Open t)
(def-get-info-type SQL_SCROLL_CONCURRENCY       43 :long	:deprecated 3.0)
(def-get-info-type SQL_TXN_CAPABLE              46 :short	:X/Open t)
;(defconstant SQL_TRANSACTION_CAPABLE		SQL_TXN_CAPABLE)
(def-get-info-type SQL_USER_NAME                47 :string	:X/Open t)
(def-get-info-type SQL_TXN_ISOLATION_OPTION     72 :short	:X/Open t)
;(defconstant SQL_TRANSACTION_ISOLATION_OPTION	SQL_TXN_ISOLATION_OPTION)
(def-get-info-type SQL_INTEGRITY                73 :boolean	:X/Open t)
(def-get-info-type SQL_GETDATA_EXTENSIONS	81 :long	:X/Open t) 
(def-get-info-type SQL_NULL_COLLATION		85 :short	:X/Open t)
(def-get-info-type SQL_ALTER_TABLE 		86 :long	:X/Open t)
(def-get-info-type SQL_ORDER_BY_COLUMNS_IN_SELECT 90 :boolean	:X/Open t)
(def-get-info-type SQL_SPECIAL_CHARACTERS	94 :string	:X/Open t)
;(def-get-info-type SQL_MAX_COLUMNS_IN_GROUP_BY	97)
;(defconstant SQL_MAXIMUM_COLUMNS_IN_GROUP_BY	SQL_MAX_COLUMNS_IN_GROUP_BY)
;(def-get-info-type SQL_MAX_COLUMNS_IN_INDEX	 98)
;(defconstant SQL_MAXIMUM_COLUMNS_IN_INDEX	SQL_MAX_COLUMNS_IN_INDEX)
;(def-get-info-type SQL_MAX_COLUMNS_IN_ORDER_BY	 99)
;(defconstant SQL_MAXIMUM_COLUMNS_IN_ORDER_BY	SQL_MAX_COLUMNS_IN_ORDER_BY)
;(def-get-info-type SQL_MAX_COLUMNS_IN_SELECT	100)
;(defconstant SQL_MAXIMUM_COLUMNS_IN_SELECT	SQL_MAX_COLUMNS_IN_SELECT)
(def-get-info-type SQL_MAX_COLUMNS_IN_TABLE	101 :short	:X/Open t)
(def-get-info-type SQL_MAX_INDEX_SIZE		102 :long	:X/Open t)
;(defconstant SQL_MAXIMUM_INDEX_SIZE		SQL_MAX_INDEX_SIZE)
(def-get-info-type SQL_MAX_ROW_SIZE		104 :long	:X/Open t)
;(defconstant SQL_MAXIMUM_ROW_SIZE		SQL_MAX_ROW_SIZE)
(def-get-info-type SQL_MAX_STATEMENT_LEN	105 :long	:X/Open t)
;(defconstant SQL_MAXIMUM_STATEMENT_LENGTH	SQL_MAX_STATEMENT_LEN)
(def-get-info-type SQL_MAX_TABLES_IN_SELECT	106 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_TABLES_IN_SELECT	SQL_MAX_TABLES_IN_SELECT
(def-get-info-type SQL_MAX_USER_NAME_LEN	107 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_USER_NAME_LENGTH	SQL_MAX_USER_NAME_LEN)

;; #if (ODBCVER >= 0x0300)
(def-get-info-type SQL_OJ_CAPABILITIES          115 :long	:X/Open t)
;(defconstant SQL_OUTER_JOIN_CAPABILITIES	115)
;; #elseif (0x0201 <= ODBCVER && ODBCVER < 0x0300)
;(def-get-info-type SQL_OJ_CAPABILITIES  	65003)	; Temp value until ODBC 3.0
;; #endif

;; #if (ODBCVER >= 0x0300)
(def-get-info-type SQL_XOPEN_CLI_YEAR		10000 :string	:X/Open t)
(def-get-info-type SQL_CURSOR_SENSITIVITY	10001 :long	:X/Open t)
(def-get-info-type SQL_DESCRIBE_PARAMETER	10002 :boolean	:X/Open t)
(def-get-info-type SQL_CATALOG_NAME		10003 :boolean	:X/Open t)
(def-get-info-type SQL_COLLATION_SEQ            10004 :string	:X/Open t)
(def-get-info-type SQL_MAX_IDENTIFIER_LEN       10005 :short	:X/Open t)
;(defconstant SQL_MAXIMUM_IDENTIFIER_LENGTH	 SQL_MAX_IDENTIFIER_LEN
;; #endif /* ODBCVER >= 0x0300 */

;; SQL_ALTER_TABLE values

;;/* SQL_ALTER_TABLE bitmasks */
(defconstant SQL_AT_ADD_COLUMN		#x00000001)
(defconstant SQL_AT_DROP_COLUMN		#x00000002)
;; #if (ODBCVER >= 0x0300)
(defconstant SQL_AT_ADD_CONSTRAINT     	#x00000008)
;; #endif

;; /* SQL_ASYNC_MODE values */
;; #if (ODBCVER >= 0x0300)
;#define SQL_AM_NONE                         0
;#define SQL_AM_CONNECTION                   1
;#define SQL_AM_STATEMENT                    2
;; #endif

;; SQL_CURSOR_COMMIT_BEHAVIOR and SQL_CURSOR_ROLLBACK_BEHAVIOR values
(defconstant SQL_CB_DELETE	#x0000)
(defconstant SQL_CB_CLOSE	#x0001)
(defconstant SQL_CB_PRESERVE	#x0002)

;; SQL_FETCH_DIRECTION bitmasks 
(defconstant SQL_FD_FETCH_NEXT		#x00000001)
(defconstant SQL_FD_FETCH_FIRST		#x00000002)
(defconstant SQL_FD_FETCH_LAST		#x00000004)
(defconstant SQL_FD_FETCH_PRIOR		#x00000008)
(defconstant SQL_FD_FETCH_ABSOLUTE	#x00000010)
(defconstant SQL_FD_FETCH_RELATIVE	#x00000020)

;; SQL_GETDATA_EXTENSIONS bitmasks 
;(defconstant SQL_GD_ANY_COLUMN		#x00000001)
(defconstant SQL_GD_ANY_ORDER		#x00000002)

;; SQL_IDENTIFIER_CASE values
(defconstant SQL_IC_UPPER	#x0001)
(defconstant SQL_IC_LOWER	#x0002)
(defconstant SQL_IC_SENSITIVE	#x0003)
(defconstant SQL_IC_MIXED	#x0004)

;; SQL_OJ_CAPABILITIES bitmasks 
;; #if (ODBCVER >= 0x0201)
(defconstant SQL_OJ_LEFT		#x00000001) 
(defconstant SQL_OJ_RIGHT		#x00000002)
(defconstant SQL_OJ_FULL		#x00000004)
(defconstant SQL_OJ_NESTED		#x00000008)
(defconstant SQL_OJ_NOT_ORDERED		#x00000010)
(defconstant SQL_OJ_INNER		#x00000020)
(defconstant SQL_OJ_ALL_COMPARISON_OPS	#x00000040)
;;#endif

;; SQL_SCROLL_CONCURRENCY bitmasks
(defconstant SQL_SCCO_READ_ONLY		#x00000001)
(defconstant SQL_SCCO_LOCK		#x00000002)
(defconstant SQL_SCCO_OPT_ROWVER	#x00000004)
(defconstant SQL_SCCO_OPT_VALUES	#x00000008)

;; SQL_TXN_CAPABLE values
(defconstant SQL_TC_NONE	0)
(defconstant SQL_TC_DML		1)
(defconstant SQL_TC_ALL		2)
(defconstant SQL_TC_DDL_COMMIT	3)
(defconstant SQL_TC_DDL_IGNORE	4)

;; SQL_TXN_ISOLATION_OPTION bitmasks 
(defconstant SQL_TXN_READ_UNCOMMITTED		#x00000001)
;(defconstant SQL_TRANSACTION_READ_UNCOMMITTED	#x00000001) ;SQL_TXN_READ_UNCOMMITTED
(defconstant SQL_TXN_READ_COMMITTED		#x00000002)
;(defconstant SQL_TRANSACTION_READ_COMMITTED	#x00000002) ;SQL_TXN_READ_COMMITTED
(defconstant SQL_TXN_REPEATABLE_READ 		#x00000004)
;(defconstant SQL_TRANSACTION_REPEATABLE_READ	#x00000004) ;SQL_TXN_REPEATABLE_READ)
(defconstant SQL_TXN_SERIALIZABLE		#x00000008)
;(defconstant SQL_TRANSACTION_SERIALIZABLE	#x00000008) ;SQL_TXN_SERIALIZABLE

;; SQL_NULL_COLLATION values - where NULLs are sorted in a result set wrt ASC/DESC keywords
;(defconstant SQL_NC_HIGH  #x0000)	; at the high end, depending on the ASC or DESC
;(defconstant SQL_NC_LOW   #x0001)	; at the low end, depending on the ASC or DESC
;(defconstant SQL_NC_START #x0002)	; at the start, regardless of the ASC or DESC
;(defconstant SQL_NC_END   #x0004)	; at the end, regardless of ASC or DESC
;; SQL/CLI (Subclause A.1, "C header file SQLCLI.H") differs from ODBC on NULL collation!
;#define SQL_NC_HIGH 1
;#define SQL_NC_LOW 2

