;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2005 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; ODBC foreign call and structure definitions.

(in-package :ystok.odbc)

#+odbc3.0
(define-foreign-function "SQLGetDiagField"
    ((HandleType SQL-SMALLINT)		; SQLSMALLINT   HandleType
     (Handle SQL-HANDLE)          	; SQLHANDLE     Handle
     (record-number SQL-SMALLINT)	; SQLSMALLINT	RecNumber
     (attribute SQL-SMALLINT)		; SQLSMALLINT	DiagIdentifier          
     (value-ptr SQL-POINTER)		; SQLPOINTER	DiagInfoPtr
     (buffer-length SQL-SMALLINT)	; SQLSMALLINT	BufferLength
     (string-length-ptr :ptr))		; SQLSMALLINT*	StringLengthPtr
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLGetDiagRec"
    ((HandleType SQL-SMALLINT)		; SQLSMALLINT   HandleType
     (Handle SQL-HANDLE)          	; SQLHANDLE     Handle
     (record-number SQL-SMALLINT)	; SQLSMALLINT	RecNumber
     (sql-state	string-ptr)		; SQLCHAR*	Sqlstate
     (secondary-error-id :ptr)		; SQLINTEGER*	NativeErrorPtr
     (message string-ptr)		; SQLCHAR*	MessageText
     (buffer-length SQL-SMALLINT)	; SQLSMALLINT	BufferLength
     (string-length-ptr :ptr))		; SQLSMALLINT*	TextLengthPtr
    SQL-RETURN)

(define-foreign-function "SQLError"	; Deprecated in ODBC 3.0
    ((henv SQL-HANDLE)          	; HENV        henv
     (hdbc SQL-HANDLE)          	; HDBC        hdbc
     (hstmt SQL-HANDLE)         	; HSTMT       hstmt
     (sql-state string-ptr)		; UCHAR  FAR *szSqlState
     (secondary-error-id :ptr)		; SDWORD FAR *pfNativeError
     (message string-ptr)		; UCHAR  FAR *szErrorMsg
     (buffer-length SQL-SMALLINT)	; SWORD       cbErrorMsgMax
     (string-length-ptr :ptr))		; SWORD  FAR *pcbErrorMsg
    :signed-short)              	; RETCODE_SQL_API

#+odbc3.0
(define-foreign-function  "SQLAllocHandle"
    ((HandleType SQL-SMALLINT)		; SQLSMALLINT   HandleType
     (InputHandle SQL-HANDLE)           ; SQLHANDLE     InputHandle
     (OutputHandlePtr SQL-HANDLE-PTR))	; SQLHANDLE *   OutputHandlePtr
    SQL-RETURN)

  #+odbc3.0
  (define-foreign-function "SQLFreeHandle"
    ((HandleType SQL-SMALLINT)		; SQLSMALLINT   HandleType
     (Handle SQL-HANDLE))          	; SQLHANDLE     Handle
    SQL-RETURN)

(define-foreign-function "SQLAllocEnv"	; Deprecated in ODBC 3.0
    ((*phenv sql-handle-ptr))   ; HENV   FAR *phenv
    SQL-RETURN)
  
(define-foreign-function "SQLFreeEnv"	; Deprecated in ODBC 3.0
    ((henv sql-handle))
    SQL-RETURN)

(define-foreign-function "SQLAllocConnect" ; Deprecated in ODBC 3.0
    ((henv sql-handle)          ; HENV        henv
     (*phdbc sql-handle-ptr))   ; HDBC   FAR *phdbc
    SQL-RETURN)

(define-foreign-function "SQLFreeConnect" ; Deprecated in ODBC 3.0
    ((ConnectionHandle SQL-HANDLE))	; SQLHDBC ConnectionHandle
    SQL-RETURN)

(define-foreign-function "SQLAllocStmt"	; Deprecated in ODBC 3.0
    ((hdbc sql-handle)          	; HDBC        hdbc
     (*phstmt sql-handle-ptr))   	; HSTMT  FAR *phstmt
    SQL-RETURN)

(define-foreign-function "SQLFreeStmt"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (Option SQL-USMALLINT))		; SQLUSMALLINT	Option
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLGetEnvAttr"
    ((EnvironmentHandle SQL-HANDLE)    	; SQLHENV      EnvironmentHandle
     (Attribute SQL-INTEGER)     	; SQLINTEGER   Attribute
     (value-ptr :ptr)			; SQLPOINTER   ValuePtr
     (buffer-length SQL-INTEGER)     	; SQLINTEGER   BufferLength,
     (string-length-ptr :ptr))		; SQLINTEGER * StringLengthPtr
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLSetEnvAttr"
    ((EnvironmentHandle SQL-HANDLE)    	; SQLHENV    EnvironmentHandle
     (Attribute SQL-INTEGER)		; SQLINTEGER Attribute
     (value-ptr SQL-POINTER)		; SQLPOINTER ValuePtr
     (string-length SQL-INTEGER))	; SQLINTEGER StringLength
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLGetConnectAttr"
    ((ConnectionHandle SQL-HANDLE)     	; SQLHDBC      ConnectionHandle
     (Attribute SQL-INTEGER)     	; SQLINTEGER   Attribute
     (value-ptr SQL-POINTER)		; SQLPOINTER   ValuePtr
     (buffer-length SQL-INTEGER)     	; SQLINTEGER   BufferLength,
     (string-length-ptr :ptr))		; SQLINTEGER * StringLengthPtr
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLSetConnectAttr"
    ((ConnectionHandle SQL-HANDLE)	; SQLHDBC    ConnectionHandle
     (Attribute SQL-INTEGER)		; SQLINTEGER Attribute
     (value-ptr SQL-POINTER)		; SQLPOINTER ValuePtr
     (string-length SQL-INTEGER))	; SQLINTEGER StringLength
    SQL-RETURN)

(define-foreign-function "SQLGetConnectOption"	; Deprecated in ODBC 3.0
    ((hdbc sql-handle)          ; HDBC        hdbc
     (Attribute SQL-SMALLINT)   ; UWORD       fOption
     (value-ptr :ptr))		; PTR         pvParam
    SQL-RETURN)
  
(define-foreign-function "SQLSetConnectOption"	; Deprecated in ODBC 3.0
    ((hdbc sql-handle)       	; HDBC        hdbc
     (Attribute SQL-SMALLINT)   ; UWORD       fOption
     (value-ptr SQL-UINTEGER))  ; UDWORD      vParam
    SQL-RETURN)

#+unused ;#+odbc3.0
(define-foreign-function "SQLGetDescField"
    ((hdesc SQL-HANDLE)			; SQLHDESC	DescriptorHandle
     (record-number SQL-SMALLINT)	; SQLSMALLINT	RecNumber
     (attribute     SQL-SMALLINT)	; SQLSMALLINT	FieldIdentifier
     (value-ptr	    SQL-POINTER)	; SQLPOINTER	ValuePtr
     (buffer-length SQL-INTEGER)	; SQLINTEGER	BufferLength
     (string-length-ptr :ptr))		; SQLINTEGER *	StringLengthPtr
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLSetDescField"
    ((hdesc SQL-HANDLE)			; SQLHDESC	DescriptorHandle
     (record-number SQL-SMALLINT)	; SQLSMALLINT	RecNumber
     (attribute     SQL-SMALLINT)	; SQLSMALLINT	FieldIdentifier
     (value-ptr	    SQL-POINTER)	; SQLPOINTER	ValuePtr
     (buffer-length SQL-INTEGER))	; SQLINTEGER	BufferLength
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLGetStmtAttr"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (Attribute SQL-INTEGER)     	; SQLINTEGER	Attribute
     (value-ptr SQL-POINTER)		; SQLPOINTER	ValuePtr
     (buffer-length SQL-INTEGER)     	; SQLINTEGER	BufferLength,
     (string-length-ptr :ptr))		; SQLINTEGER *	StringLengthPtr
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLSetStmtAttr"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (Attribute SQL-INTEGER)		; SQLINTEGER	Attribute
     (value-ptr SQL-POINTER)		; SQLPOINTER	ValuePtr
     (string-length SQL-INTEGER))	; SQLINTEGER	StringLength
    SQL-RETURN)
  
(define-foreign-function "SQLGetStmtOption" ; Deprecated in ODBC 3.0
    ((hstmt sql-handle)			; HDBC        hdbc
     (Attribute SQL-SMALLINT)		; UWORD       fOption
     (value-ptr SQL-POINTER))		; PTR         pvParam
    SQL-RETURN)
  
(define-foreign-function "SQLSetStmtOption" ; Deprecated in ODBC 3.0
    ((hstmt sql-handle)			; HDBC        hdbc
     (Attribute SQL-SMALLINT)		; UWORD       fOption
     (value-ptr SQL-POINTER))		; UDWORD      vParam     
    SQL-RETURN)

;;; Connecting

(define-foreign-function "SQLConnect"
    ((ConnectionHandle SQL-HANDLE)	; SQLHDBC	ConnectionHandle
     (server-name string-ptr)		; SQLCHAR *	ServerName
     (NameLength1  SQL-SMALLINT)	; SQLSMALLINT	NameLength1
     (user-name string-ptr)		; SQLCHAR *	UserName
     (NameLength2 SQL-SMALLINT)		; SQLSMALLINT	NameLength2
     (password string-ptr)		; SQLCHAR *	Authentication
     (NameLength3 SQL-SMALLINT))	; SQLSMALLINT	NameLength3
    SQL-RETURN)
  
(define-foreign-function "SQLDriverConnect"
    ((ConnectionHandle SQL-HANDLE)	; SQLHDBC	ConnectionHandle
     (hwnd SQL-HANDLE)			; SQLHWND	WindowHandle
     (in-connection-str string-ptr)	; SQLCHAR *	InConnectionString
     (string-length SQL-SMALLINT)	; SQLSMALLINT	StringLength1
     (out-connection-str-ptr string-ptr); SQLCHAR *	OutConnectionString
     (buffer-length SQL-SMALLINT)	; SQLSMALLINT	BufferLength
     (out-len-ind :ptr)			; SQLSMALLINT *	StringLength2Ptr
     (option SQL-SMALLINT))		; SQLUSMALLINT	DriverCompletion
    SQL-RETURN)
  
(define-foreign-function "SQLDisconnect"
    ((ConnectionHandle SQL-HANDLE))	; SQLHDBC	ConnectionHandle
    SQL-RETURN)
   
;;; Executing statements, fetching and depositing data

(define-foreign-function "SQLBindCol"
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (column-nr SQL-USMALLINT)		; SQLUSMALLINT	ColumnNumber
     (target-type SQL-SMALLINT)		; SQLSMALLINT	TargetType
     (data-ptr :ptr)			; SQLPOINTER	TargetValuePtr
     (octet-length SQL-INTEGER)		; SQLINTEGER	BufferLength 
     (octet-len-ind-ptr :ptr))		; SQLINTEGER *	StrLen_or_Ind
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLBulkOperations"
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (operation	SQL-SMALLINT))		; SQLSMALLINT	Operation
    SQL-RETURN)

(define-foreign-function "SQLCancel"
    ((hstmt SQL-HANDLE))		; SQLHSTMT	StatementHandle
    SQL-RETURN)
  
#+odbc3.0
(define-foreign-function "SQLCloseCursor"
    ((hstmt SQL-HANDLE))		; SQLHSTMT	StatementHandle
    SQL-RETURN)

#+odbc3.0
(define-foreign-function "SQLColAttribute"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (column-nr SQL-USMALLINT)		; SQLUSMALLINT	ColumnNumber
     (Attribute SQL-USMALLINT)		; SQLUSMALLINT	FieldIdentifier
     (CharacterValuePtr STRING-PTR)	; SQLPOINTER	CharacterAttributePtr
     (buffer-length SQL-SMALLINT)	; SQLSMALLINT	BufferLength
     (string-length-ptr :ptr)		; SQLSMALLINT *	StringLength
     (value-ptr :ptr))			; SQLPOINTER	NumericAttributePtr 32-bitsigned
    SQL-RETURN)

(define-foreign-function "SQLColAttributes"	; Deprecated in ODBC 3.0
    ((hstmt sql-handle)         	; HSTMT       hstmt
     (icol SQL-SMALLINT)		; UWORD       icol
     (fDescType SQL-SMALLINT)		; UWORD       fDescType
     (rgbDesc :ptr)             	; PTR         rgbDesc
     (cbDescMax SQL-SMALLINT)         	; SWORD       cbDescMax
     (*pcbDesc :ptr)            	; SWORD  FAR *pcbDesc
     (*pfDesc :ptr))            	; SDWORD FAR *pfDesc
    :signed-short)              	; RETCODE_SQL_API

(define-foreign-function "SQLDescribeCol"		; SQLDescribeColW
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (column-nr SQL-USMALLINT)		; SQLUSMALLINT	ColumnNumber
     (name-ptr STRING-PTR)		; SQLCHAR *	ColumnName
     (octet-length SQL-SMALLINT)	; SQLSMALLINT 	BufferLength
     (name-length-ptr :ptr)		; SQLSMALLINT *	NameLength
     (sql-type-ptr :ptr)		; SQLSMALLINT *	DataType
     (column-size-ptr :ptr)		; SQLUINTEGER *	ColumnSize
     (decimal-digits-ptr :ptr)		; SQLSMALLINT *	DecimalDigits
     (nullable-ptr :ptr))		; SQLSMALLINT *	Nullable
    SQL-RETURN)             
  
#+odbc3.0
(define-foreign-function "SQLEndTran"
    ((handle-type SQL-SMALLINT)		; SQLSMALLINT	HandleType
     (handle SQL-HANDLE)		; SQLHANDLE	Handle
     (completion-type SQL-SMALLINT))	; SQLSMALLINT	CompletionType
    SQL-RETURN)

(define-foreign-function "SQLExecDirect"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (text STRING-PTR)			; SQLCHAR *	StatementText
     (text-length SQL-INTEGER))		; SQLINTEGER	TextLength
    SQL-RETURN)
  
(define-foreign-function "SQLExtendedFetch" ; deprecated
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (orientation SQL-USMALLINT)	; SQLUSMALLINT	FetchOrientation
     (offset SQL-INTEGER)		; SQLINTEGER	FetchOffset
     (count-ptr :ptr)			; SQLUINTEGER * RowCountPtr
     (array :ptr))			; SQLUSMALLINT* RowStatusArray
    SQL-RETURN)            

(define-foreign-function "SQLFetch"
    ((hstmt SQL-HANDLE))	; SQLHSTMT StatementHandle
    SQL-RETURN)            
    
#+odbc3.0
(define-foreign-function "SQLFetchScroll"
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (orientation SQL-SMALLINT)		; SQLSMALLINT	FetchOrientation
     (offset SQL-INTEGER))		; SQLINTEGER	FetchOffset
    SQL-RETURN)            

(define-foreign-function "SQLGetData"
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (column-nr SQL-USMALLINT)		; SQLUSMALLINT	ColumnNumber
     (c-type SQL-SMALLINT)		; SQLSMALLINT	TargetType
     (data-ptr :ptr)			; SQLPOINTER	TargetValuePtr
     (buffer-length SQL-INTEGER)	; SQLINTEGER	BufferLength 
     (octet-len-ind-ptr :ptr))		; SQLINTEGER *	StrLen_or_Ind
    SQL-RETURN)

(define-foreign-function "SQLNumResultCols"
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (count-ptr :ptr))			; SWORD  FAR *pccol
    SQL-RETURN)
  
(define-foreign-function "SQLRowCount"
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (count-ptr :ptr))			; SDWORD FAR *pcrow
    SQL-RETURN)
  
(define-foreign-function "SQLPutData"
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (data-ptr :ptr)			; SQLPOINTER	Data
     (octet-len-ind SQL-INTEGER))	; SQLINTEGER	StrLen_or_Ind
    SQL-RETURN)
  
(define-foreign-function "SQLSetPos"
    ((hstmt SQL-HANDLE)			; SQLHSTMT 	StatementHandle
     (row-nr SQL-USMALLINT)		; SQLUSMALLINT	RowNumber
     (operation SQL-USMALLINT)		; SQLUSMALLINT	Operation
     (lock-type SQL-USMALLINT))		; SQLUSMALLINT	LockType
    SQL-RETURN)

(define-foreign-function "SQLTransact"	; deprecated in 3.0
    ((henv SQL-HANDLE)          	; HENV        henv
     (ConnectionHandle SQL-HANDLE)	; SQLHDBC	ConnectionHandle
     (completion-type SQL-SMALLINT))	; UWORD       fType (SQL_COMMIT or SQL_ROLLBACK)
    SQL-RETURN)

;;; Parameters, procedures, and prepared execution

(define-foreign-function "SQLDescribeParam"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (parameter-nr SQL-USMALLINT)	; SQLUSMALLINT	ParameterNumber
     (sql-type-ptr :ptr)		; SQLSMALLINT *	DataTypePtr
     (column-size-ptr :ptr)		; SQLUINTEGER * ParameterSizePtr,
     (decimal-digits-ptr :ptr)		; SQLSMALLINT * DecimalDigitsPtr
     (nullable-ptr :ptr))		; SQLSMALLINT * NullablePtr
    SQL-RETURN)
  
(define-foreign-function "SQLBindParameter"
    ((hstmt SQL-HANDLE)			; SQLHSTMT StatementHandle
     (parameter-nr SQL-USMALLINT)	; SQLUSMALLINT	ParameterNumber
     (io-type SQL-SMALLINT)		; SQLSMALLINT	InputOutputType
     (c-type SQL-SMALLINT)		; SQLSMALLINT	ValueType 
     (sql-type SQL-SMALLINT)		; SQLSMALLINT	ParameterType
     (column-size SQL-UINTEGER)		; SQLUINTEGER	ColumnSize
     (decimal-digits SQL-SMALLINT)	; SQLSMALLINT	DecimalDigits
     (data-ptr :ptr)			; SQLPOINTER	ParameterValuePtr
     (octet-length SQL-INTEGER)		; SQLINTEGER	BufferLength
     (octet-len-ind-ptr :ptr))		; SQLINTEGER *	StrLen_or_Ind
    SQL-RETURN)

(define-foreign-function "SQLExecute"
    ((hstmt sql-handle))
    SQL-RETURN)
  
(define-foreign-function "SQLNumParams"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (count-ptr :ptr))			; SQLSMALLINT * ParameterCountPtr
    SQL-RETURN)

(define-foreign-function "SQLParamData"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (value-ptr :ptr))			; SQLPOINTER *	Value
    SQL-RETURN)

(define-foreign-function "SQLPrepare"
    ((hstmt sql-handle)			; SQLHSTMT	StatementHandle
     (text-ptr string-ptr)		; SQLCHAR *	StatementText
     (text-length SQL-INTEGER))		; SQLINTEGER	TextLength
    SQL-RETURN)
  
;;; CATALOG Functions

(define-foreign-function "SQLColumns"
    ((hstmt SQL-HANDLE)			; SQLHSTMT StatementHandle
     (catalog-name string-ptr)		; UCHAR  FAR *szTableQualifier
     (catalog-length SQL-SMALLINT)	; SWORD       cbTableQualifier
     (schema-name string-ptr)		; UCHAR  FAR *szTableOwner
     (schema-length SQL-SMALLINT)	; SWORD       cbTableOwner
     (table-name string-ptr)		; UCHAR  FAR *szTableName
     (table-name-length SQL-SMALLINT)	; SWORD       cbTableName
     (column-name string-ptr)		; UCHAR  FAR *szColumnName
     (column-name-length SQL-SMALLINT))	; SWORD       cbColumnName
    SQL-RETURN)

#-pcl
(define-foreign-function "SQLDataSources"
    ((henv sql-handle)          	; SQLHENV	EnvironmentHandle
     (direction SQL-USMALLINT)		; SQLUSMALLINT	Direction
     (server-name string-ptr)		; SQLCHAR *     ServerName
     (namelength1 SQL-SMALLINT)		; SQLSMALLINT	BufferLength1
     (name-length-ptr :ptr)             ; SQLSMALLINT * NameLength1Ptr
     (description string-ptr)		; SQLCHAR *	Description
     (namelength2 SQL-SMALLINT)		; SQLSMALLINT	BufferLength2
     (description-length-ptr :ptr))	; SQLSMALLINT *	NameLength2Ptr
    SQL-RETURN)

(define-foreign-function "SQLDrivers"
    ((henv sql-handle)          	; SQLHENV	EnvironmentHandle
     (direction SQL-USMALLINT)		; SQLUSMALLINT	Direction
     (description string-ptr)		; SQLCHAR *	DriverDescription
     (namelength1 SQL-SMALLINT)		; SQLSMALLINT	BufferLength1
     (description-length-ptr :ptr)	; SQLSMALLINT *	DescriptionLengthPtr
     (attributes string-ptr)		; SQLCHAR *     DriverAttributes
     (namelength2 SQL-SMALLINT)		; SQLSMALLINT	BufferLength2
     (octet-length-ptr :ptr))		; SQLSMALLINT * AttributesLengthPtr
    SQL-RETURN)

(define-foreign-function "SQLForeignKeys"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (pk-catalog-name string-ptr)	; SQLCHAR *	PKCatalogName
     (pk-catalog-length SQL-SMALLINT)	; SQLSMALLINT	NameLength1
     (pk-schema-name string-ptr)	; SQLCHAR *	PKSchemaName
     (pk-schema-length SQL-SMALLINT)	; SQLSMALLINT	NameLength2
     (pk-table-name string-ptr)		; SQLCHAR *	PKTableName
     (pk-table-name-length SQL-SMALLINT); SQLSMALLINT	NameLength3
     (fk-catalog-name string-ptr)	; SQLCHAR *	FKCatalogName
     (fk-catalog-length SQL-SMALLINT)	; SQLSMALLINT	NameLength4
     (fk-schema-name string-ptr)	; SQLCHAR *	FKSchemaName
     (fk-schema-length SQL-SMALLINT)	; SQLSMALLINT	NameLength5
     (fk-table-name string-ptr)		; SQLCHAR *	FKTableName
     (fk-table-name-length SQL-SMALLINT)); SQLSMALLINT	NameLength6
    SQL-RETURN)

(define-foreign-function "SQLGetFunctions"
    ((ConnectionHandle SQL-HANDLE)	; SQLHDBC	ConnectionHandle
     (function-id SQL-USMALLINT)	; SQLUSMALLINT	FunctionId
     (value-ptr :ptr))			; SQLUSMALLINT *Supported
    SQL-RETURN)

(define-foreign-function "SQLGetInfo"
    ((ConnectionHandle SQL-HANDLE)	; SQLHDBC	ConnectionHandle
     (info-type SQL-USMALLINT)		; SQLUSMALLINT	InfoType
     (value-ptr SQL-POINTER)		; SQLPOINTER	InfoValue
     (buffer-length SQL-SMALLINT)	; SQLSMALLINT	BufferLength
     (string-length-ptr :ptr))		; SQLSMALLINT *	StringLength
    SQL-RETURN)             

#+pcl
(define-foreign-function ("SQLGetInfo" SQLGetInfo-Str)
    ((hdbc SQL-HANDLE)          	; HDBC        hdbc
     (fInfoType SQL-SMALLINT)         	; UWORD       fInfoType
     (rgbInfoValue string-ptr)  	; PTR         rgbInfoValue
     (cbInfoValueMax SQL-SMALLINT)    	; SWORD       cbInfoValueMax
     (*pcbInfoValue :ptr)       	; SWORD  FAR *pcbInfoValue
     )
    :signed-short)              	; RETCODE_SQL_API

(define-foreign-function "SQLGetTypeInfo"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (DataType SQL-SMALLINT))		; SQLSMALLINT	DataType
    SQL-RETURN)

(define-foreign-function "SQLPrimaryKeys"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (catalog-name string-ptr)		; SQLCHAR *	CatalogName
     (catalog-length SQL-SMALLINT)	; SQLSMALLINT	NameLength1
     (schema-name string-ptr)		; SQLCHAR *	SchemaName
     (schema-length SQL-SMALLINT)	; SQLSMALLINT	NameLength2
     (table-name string-ptr)		; SQLCHAR *	TableName
     (table-name-length SQL-SMALLINT))	; SQLSMALLINT	NameLength3
    SQL-RETURN)

(define-foreign-function "SQLSpecialColumns"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (info-type SQL-USMALLINT)		; SQLUSMALLINT	IdentifierType
     (catalog-name string-ptr)		; SQLCHAR *	CatalogName
     (catalog-length SQL-SMALLINT)	; SQLSMALLINT	NameLength1
     (schema-name string-ptr)		; SQLCHAR *	SchemaName
     (schema-length SQL-SMALLINT)	; SQLSMALLINT	NameLength2
     (table-name string-ptr)		; SQLCHAR *	TableName
     (table-name-length SQL-SMALLINT)	; SQLSMALLINT	NameLength3
     (scope SQL-USMALLINT)		; SQLUSMALLINT	Scope
     (nullable SQL-USMALLINT))		; SQLUSMALLINT	Nullable
    SQL-RETURN)

(define-foreign-function "SQLStatistics"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (catalog-name string-ptr)		; SQLCHAR *	CatalogName
     (catalog-length SQL-SMALLINT)	; SQLSMALLINT	NameLength1
     (schema-name string-ptr)		; SQLCHAR *	SchemaName
     (schema-length SQL-SMALLINT)	; SQLSMALLINT	NameLength2
     (table-name string-ptr)		; SQLCHAR *	TableName
     (table-name-length SQL-SMALLINT)	; SQLSMALLINT	NameLength3
     (Unique SQL-USMALLINT)		; SQLUSMALLINT	Unique
     (Reserved SQL-USMALLINT))		; SQLUSMALLINT	Reserved
    SQL-RETURN)

(define-foreign-function "SQLTables"
    ((hstmt SQL-HANDLE)			; SQLHSTMT	StatementHandle
     (catalog-name string-ptr)		; SQLCHAR *	CatalogName
     (catalog-length SQL-SMALLINT)	; SQLSMALLINT	NameLength1
     (schema-name string-ptr)		; SQLCHAR *	SchemaName
     (schema-length SQL-SMALLINT)	; SQLSMALLINT	NameLength2
     (table-name string-ptr)		; SQLCHAR *	TableName
     (table-name-length SQL-SMALLINT)	; SQLSMALLINT	NameLength3
     (table-type string-ptr)		; SQLCHAR *	TableType
     (table-type-length SQL-SMALLINT))	; SQLSMALLINT	NameLength4
  SQL-RETURN)

;) ; eval-when

;;; Foreign structure type definitions

;(eval-when (:load-toplevel :compile-toplevel :execute)

;typedef struct tagDATE_STRUCT {...} DATE_STRUCT;
;; #if (ODBCVER >= 0x0300)
;typedef DATE_STRUCT	SQL_DATE_STRUCT;
;; #endif  /* ODBCVER >= 0x0300 */
(define-foreign-struct sql-c-date
    (year  SQL-SMALLINT)	; SQLSMALLINT    year
    (month SQL-USMALLINT)	; SQLUSMALLINT   month
    (day   SQL-USMALLINT))	; SQLUSMALLINT   day
  
;typedef struct tagTIME_STRUCT {...} TIME_STRUCT;
;; #if (ODBCVER >= 0x0300)
;typedef TIME_STRUCT	SQL_TIME_STRUCT;
;; #endif /* ODBCVER >= 0x0300 */
(define-foreign-struct sql-c-time 
    (hour   SQL-USMALLINT)	; SQLUSMALLINT   hour
    (minute SQL-USMALLINT)	; SQLUSMALLINT   minute
    (second SQL-USMALLINT))	; SQLUSMALLINT   second
  
;typedef struct tagTIMESTAMP_STRUCT {...} TIMESTAMP_STRUCT;
;; #if (ODBCVER >= 0x0300)
;typedef TIMESTAMP_STRUCT	SQL_TIMESTAMP_STRUCT;
;; #endif  /* ODBCVER >= 0x0300 */
(define-foreign-struct sql-c-timestamp
    (year     SQL-SMALLINT)	; SQLSMALLINT	year
    (month    SQL-USMALLINT)	; SQLUSMALLINT	month
    (day      SQL-USMALLINT)	; SQLUSMALLINT	day
    (hour     SQL-USMALLINT)	; SQLUSMALLINT	hour
    (minute   SQL-USMALLINT)	; SQLUSMALLINT	minute
    (second   SQL-USMALLINT)	; SQLUSMALLINT	second
    (fraction SQL-UINTEGER))	; SQLUINTEGER	fraction

#|;; #if (ODBCVER >= 0x0300)
;; Enumerations for DATETIME_INTERVAL_SUBCODE values 
;; for interval data types these values are from SQL-92
typedef enum 
{	SQL_IS_YEAR		= 1,
	SQL_IS_MONTH		= 2,
	SQL_IS_DAY		= 3,
	SQL_IS_HOUR		= 4,
	SQL_IS_MINUTE		= 5,
	SQL_IS_SECOND		= 6,
	SQL_IS_YEAR_TO_MONTH	= 7,
	SQL_IS_DAY_TO_HOUR	= 8,
	SQL_IS_DAY_TO_MINUTE	= 9,
	SQL_IS_DAY_TO_SECOND	= 10,
	SQL_IS_HOUR_TO_MINUTE	= 11,
	SQL_IS_HOUR_TO_SECOND	= 12,
	SQL_IS_MINUTE_TO_SECOND	= 13
} SQLINTERVAL;

typedef struct tagSQL_YEAR_MONTH
{	SQLUINTEGER	year;
	SQLUINTEGER	month;
} SQL_YEAR_MONTH_STRUCT;

typedef struct tagSQL_DAY_SECOND
{	SQLUINTEGER	day;
	SQLUINTEGER	hour;
	SQLUINTEGER	minute;
	SQLUINTEGER	second;
	SQLUINTEGER	fraction;
} SQL_DAY_SECOND_STRUCT;

typedef struct tagSQL_INTERVAL_STRUCT
{	SQLINTERVAL	interval_type;
	SQLSMALLINT	interval_sign;
	union {	SQL_YEAR_MONTH_STRUCT	year_month;
		SQL_DAY_SECOND_STRUCT	day_second;
	      } intval;
} SQL_INTERVAL_STRUCT;
;;#endif  /* ODBCVER >= 0x0300 */
 |#

;;#if (ODBCVER >= 0x0300)
;; internal representation of numeric data type SQL_C_NUMERIC
;#define SQL_MAX_NUMERIC_LEN		16
(define-foreign-struct sql-c-numeric    ; SQL_NUMERIC_STRUCT
  (precision SQL-CHAR)			; SQLCHAR  precision;
  (scale     SQL-SCHAR)			; SQLSCHAR scale;
  (sign	     SQL-CHAR)			; SQLCHAR  sign; /* 1 if positive,0 if negative */
  (value (:c-array SQL-CHAR 16)))	; SQLCHAR  val[SQL_MAX_NUMERIC_LEN];

(define-foreign-struct sql-c-sbigint	; SQLBIGINT
  (low SQL-UINTEGER)			; SQLUINTEGER dwLowWord;
  (high SQL-INTEGER))			; SQLINTEGER sdwHighWord;
#|
;;; Do we really need the unsigned variant as we only get and store values to it?
(define-foreign-struct sql-c-ubigint	; SQLUBIGINT
  (low SQL-UINTEGER)			; SQLUINTEGER dwLowWord;
  (high SQL-UINTEGER))			; SQLUINTEGER dwHighWord;

  (define-foreign-struct sql-c-guid	; typedef struct tagSQLGUID { ... } SQLGUID;
    (data1 :unsigned-long)		;   DWORD Data1
    (data2 :unsigned-short)		;   WORD Data2
    (data3 :unsigned-short)		;   WORD Data3
    (data4 (:c-array :byte 8)))		;   BYTE Data4[ 8 ]
|#
;;#endif  /* ODBCVER >= 0x0300 */
