;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2005 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; ODBC foreign call wrappers

(in-package :ystok.odbc)

(defmacro %encode-handle-type (handle-type)
  `(ecase ,handle-type
     (:henv SQL_HANDLE_ENV)
     (:hdbc SQL_HANDLE_DBC)
     (:hstmt SQL_HANDLE_STMT)
     (:hdesc SQL_HANDLE_DESC)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Error handling and diagnostics

#|(defun find-database-from-handle (handle-type handle)
  (when-bind (environment (find-environment :odbc))
    (case handle-type
      (:hdbc  (find handle (connected-databases environment)
                    :key #'handle :test #'%ptr-eql))
      (:hstmt (find-if (lambda (database) (find handle (statements database)
                                                :key #'handle :test #'%ptr-eql))
                       (connected-databases environment))))))

(defun find-statement-from-handle (handle-type handle)
  (when (eq handle-type :hstmt)
    (when-bind (environment (find-environment :odbc))
      (dolist (database (connected-databases environment))
        (when-bind (statement (find handle (statements database)
                                    :key #'handle :test #'%ptr-eql))
          (return statement))))))|#

(defun find-database-statement-from-handle (handle-type handle)
 ;;; Values: database, statement - if found.
  (when-bind (environment (find-environment :odbc))
    (case handle-type
      (:hdbc
       (values (find handle (connected-databases environment)
                    :key #'handle :test #'%ptr-eql)
               nil))
      (:hstmt
       (loop for database in (connected-databases environment)
             for statement = (find handle (statements database)
                                   :key #'handle :test #'%ptr-eql)
             when statement
             return (values database statement))) )))

(defun %sql-get-diag-field (handle-type handle attribute &optional (record-number 1))
 ;;; Returns nil in case of error during error diagnostics!
  (let ((type (%encode-handle-type handle-type)))
    (case attribute
      ((#.SQL_DIAG_NUMBER		 ; header: The number of status records
        #.SQL_DIAG_ROW_COUNT		 ; header: The number of rows affected
        #.SQL_DIAG_DYNAMIC_FUNCTION_CODE ; header: Access=>0(SQL_DIAG_UNKNOWN_STATEMENT)
        #.SQL_DIAG_CURSOR_ROW_COUNT)	 ; header: depends on SQL_..._CURSOR_ATTRIBUTES2
       (when (>= $ODBCVER 0300)
         (with-temporary-allocation ((value-ptr sql-integer))
            (when (%sql-succeeded (SQLGetDiagField type handle 0 attribute
                                                   value-ptr SQL_IS_INTEGER
                                                   *null-pointer*))
              (%get-integer value-ptr)))))

      (#.SQL_DIAG_NATIVE
       (if (>= $ODBCVER 0300)
           (with-temporary-allocation ((value-ptr sql-integer))
             (when (%sql-succeeded (SQLGetDiagField type handle record-number attribute
                                                    value-ptr SQL_IS_INTEGER
                                                    *null-pointer*))
               (%get-integer value-ptr)))
           (nth-value 2 (%sql-get-diag-record handle-type handle record-number))))

      ((;; Row number in the rowset, or the parameter number in the paramset,
        ;; or SQL_NO_ROW_NUMBER(-1) if this status record is not associated with number,
        ;; or SQL_ROW_NUMBER_UNKNOWN(-2) if the driver cannot determine the number.
        #.SQL_DIAG_ROW_NUMBER
        ;; Column number in the result set or the parameter number in the paramset
        ;; if the SQL_DIAG_ROW_NUMBER field is a valid row number,
        ;; or SQL_NO_COLUMN_NUMBER(-1) if this status record is not associated,
        ;; or SQL_COLUMN_NUMBER_UNKNOWN(-2) if the driver cannot determine the number.
        #.SQL_DIAG_COLUMN_NUMBER)
       (when (>= $ODBCVER 0300)
         (with-temporary-allocation ((value-ptr sql-integer))
            (when (%sql-succeeded (SQLGetDiagField type handle record-number attribute
                                                   value-ptr SQL_IS_INTEGER
                                                   *null-pointer*))
            (%get-integer value-ptr)))))

      (#.SQL_DIAG_RETURNCODE		; header
       (when (>= $ODBCVER 0300)
         (with-temporary-allocation ((value-ptr sql-smallint))
           (when (%sql-succeeded (SQLGetDiagField type handle 0 attribute
                                                  value-ptr SQL_IS_SMALLINT
                                                  *null-pointer*))
             (%get-smallint value-ptr)))))

      (#.SQL_DIAG_SQLSTATE 
       (if (>= $ODBCVER 0300)
           (with-temporary-allocation ((value-ptr :string 6)
			               (string-length-ptr sql-smallint))
              (when (%sql-succeeded (SQLGetDiagField type handle record-number attribute
                                                     value-ptr 6 string-length-ptr))
                (%get-string value-ptr (%get-smallint string-length-ptr))))
                ;; Return a keyword for efficiency?
                ;(%cstring-to-keyword value-ptr (%get-smallint string-length-ptr))))
           (nth-value 1 (%sql-get-diag-record handle-type handle record-number))))

      (#.SQL_DIAG_MESSAGE_TEXT
       (if (>= $ODBCVER 0300)
           (with-temporary-allocation ((value-ptr :string #.SQL_MAX_MESSAGE_LENGTH)
			               (string-length-ptr sql-smallint))
              (when (%sql-succeeded (SQLGetDiagField type handle record-number attribute
                                                     value-ptr SQL_MAX_MESSAGE_LENGTH
                                                     string-length-ptr))
                (%get-string value-ptr (%get-smallint string-length-ptr))))
           (%sql-get-diag-record handle-type handle record-number)))
      (otherwise #|
       (;; Header: String that describes the SQL statement that the underlying function
        ;; executed; "" if unknown. ,  . 
        #.SQL_DIAG_DYNAMIC_FUNCTION
        #.SQL_DIAG_CONNECTION_NAME
        #.SQL_DIAG_SERVER_NAME
        #.SQL_DIAG_CLASS_ORIGIN
        #.SQL_DIAG_SUBCLASS_ORIGIN)|#
       (when (>= $ODBCVER 0300)     ; Unicode:  :ef-wc-string
         (with-temporary-allocation ((value-ptr :string #.SQL_MAX_MESSAGE_LENGTH)
                                     (string-length-ptr sql-smallint))
           (when (%sql-succeeded (SQLGetDiagField type handle
                                                  (case attribute
                                                    (#.SQL_DIAG_DYNAMIC_FUNCTION 0)
                                                    (otherwise record-number))
                                                  attribute
                                                  value-ptr SQL_MAX_MESSAGE_LENGTH
                                                  string-length-ptr))
             (%get-string value-ptr (%get-smallint string-length-ptr)))))))))

(defun %sql-get-diag-record (handle-type handle &optional (record-number 1))
 "Args:
 HANDLE-TYPE one of :henv, :hdbc, :hstmt, or :hdesc.
 INPUT-HANDLE for :henv, can be either a shared or an unshared environment handle.
 RECORD-NUMBER indicates the status record from which to seek (>=1)."
  (with-temporary-allocation ((sql-state :string 5)
			      (message :string #.SQL_MAX_MESSAGE_LENGTH)
			      (secondary-error-id sql-integer)
			      (string-length-ptr sql-smallint))
    (when (%sql-succeeded                         
           (if (>= $ODBCVER 0300)
               (SQLGetDiagRec (%encode-handle-type handle-type) handle record-number
                              sql-state secondary-error-id
                              message SQL_MAX_MESSAGE_LENGTH string-length-ptr)
               ;; elseif (eql record-number 1)
               (SQLError (if (eq handle-type :henv) handle +SQL-NULL-HANDLE+)
                         (if (eq handle-type :hdbc) handle +SQL-NULL-HANDLE+)
                         (if (eq handle-type :hstmt) handle +SQL-NULL-HANDLE+)
                         sql-state secondary-error-id
                         message SQL_MAX_MESSAGE_LENGTH string-length-ptr)))
      (values
       (%get-string message (%get-smallint string-length-ptr))
       (%get-cstring sql-state)
       (%get-integer secondary-error-id)))))

(defun %make-diag-records (handle-type handle &optional (start 2))
 ;;; NB1: Foresee null values returned from %sql-get-diag-field.
 ;;; NB2: row-number and column-number are meaningfull only when >0.
  (when (>= $ODBCVER 0300)
    (loop with records = ()
          for record-number fixnum upfrom start
                 upto (or (%sql-get-diag-field handle-type handle SQL_DIAG_NUMBER) 0)
          for row-number = (%sql-get-diag-field handle-type handle
                                                SQL_DIAG_ROW_NUMBER record-number)
          and column-number = (%sql-get-diag-field handle-type handle
                                                   SQL_DIAG_COLUMN_NUMBER record-number)
          do (multiple-value-bind (message sql-state secondary-error-id)
                 (%sql-get-diag-record handle-type handle record-number)
               (push (make-diag-record 
                      :sql-state sql-state
                      :secondary-error-id secondary-error-id
                      :message message
                      :row-number (and row-number (plusp row-number)	
                                       row-number)			
                      :column-number (and column-number (plusp column-number)
                                          column-number))
                     records))
          finally (return (nreverse records)))))

(declaim (ftype (function (fixnum t t &optional t t) fixnum) %analyse-return-code))
(defun %analyse-return-code (return-code handle-type handle &optional on-info args)
 ;;; Args: on-info  =T  Signal informational conditions.
  ;;       args	    Additional connection arguments.
  (declare (optimize (speed 3) (space 0) #-debug (debug 0) #-debug (hcl:fixnum-safety 0))
           (fixnum return-code))
  (case return-code
    (#.SQL_SUCCESS_WITH_INFO
     (when on-info
       (multiple-value-bind (message sql-state secondary-error-id)
           (%sql-get-diag-record handle-type handle)
        (let ((records (%make-diag-records handle-type handle)))
         (multiple-value-bind (database statement)
             (find-database-statement-from-handle handle-type handle)
           ;; Thread-safety NB: Release handle lock in here if any.
           (warn 'odbc-success-with-info-warning
                 :database database
                 :expression (if statement (expression statement))
                 :sql-state sql-state :secondary-error-id secondary-error-id
                 :message message
                 :records records))))))
    (#.SQL_INVALID_HANDLE
     ;; This indicates a programming error.
     ;; No additional information is available from SQLGetDiagRec or SQLGetDiagField. 
     (error 'odbc-invalid-handle-error :secondary-error-id return-code))
    (#.SQL_STILL_EXECUTING
     ;; This is not of much use - we should not execute asynchronously,
     ;; but run every statement in its own thread and wait on it instead.
     (when on-info
       (multiple-value-bind (message sql-state) ; error-code)
           (%sql-get-diag-record handle-type handle)
         (multiple-value-bind (database statement)
             (find-database-statement-from-handle handle-type handle)
           (error 'odbc-still-executing-error
                  :database database
                  :expression (if statement (expression statement))
                  :sql-state sql-state ;:secondary-error-id return-code
                  :message message)))))
    (#.SQL_ERROR
     (multiple-value-bind (database statement)
         (find-database-statement-from-handle handle-type handle)
       ;(let* ((statement (find-statement-from-handle handle-type handle))
       ;       (database (if statement (database statement)
       ;                     (find-database-from-handle handle-type handle))))
      (multiple-value-bind (message sql-state secondary-error-id)
           (%sql-get-diag-record handle-type handle)
       (let ((records (%make-diag-records handle-type handle))
             (expression (cond ((null database)
                                nil)
                               (statement
                                (expression statement))
                               ((eq handle-type :hstmt)
                                (%sql-get-diag-field handle-type handle
                                                     SQL_DIAG_DYNAMIC_FUNCTION)))))
         ;; Thread-safety NB: Release handle lock in here if any.
         (cond ((equal sql-state "HYT00")	; Timeout expired
                ;; The query timeout period expired before the data source returned
                ;; the result set or the connection to the data source completed.
                ;; The timeout period is set through
                ;;	SQLSetStmtAttr(hstmt, SQL_ATTR_QUERY_TIMEOUT)
                ;;  or	SQLSetConnectAttr(hdbc, SQL_ATTR_LOGIN_TIMEOUT)
                ;(record-sql-command "Timeout expired" database)
                (if statement
                    (error 'odbc-timeout-error :database database :expression expression
                           :timeout (%sql-get-attribute :hstmt handle
                                                        SQL_ATTR_QUERY_TIMEOUT)
                           :sql-state sql-state  :secondary-error-id secondary-error-id
                           :message message
                           :records records)
                    (error 'odbc-connection-timeout-error :connection-spec args
                           :timeout (unless database
                                      (%sql-get-attribute :hdbc handle
                                                          SQL_ATTR_LOGIN_TIMEOUT))
                           :sql-state sql-state  :secondary-error-id secondary-error-id
                           :message message
                           :records records)) )
               ((equal sql-state "HYT01")	; Connection timeout expired
                ;; The connection timeout period expired before the data source 
                ;; responded to the request. The period is set through
                ;;	SQLSetConnectAttr(hdbc, SQL_ATTR_CONNECTION_TIMEOUT)       
                ;; Does %sql-get-attribute can return anything in the case of this error
                ;; not causing the enter into this code recursively
                ;; (due to unsupported attribute SQL_ATTR_CONNECTION_TIMEOUT)?
                (if database
                    (error 'odbc-timeout-error :database database :expression expression
                           ;:timeout *default-timeout*
                                     ;(%sql-get-attribute :hdbc (handle database)
                                     ;              SQL_ATTR_CONNECTION_TIMEOUT))
                           :sql-state sql-state  :secondary-error-id secondary-error-id
                           :message message
                           :records records)
                    (error 'odbc-connection-timeout-error :connection-spec args
                           ;:timeout *default-timeout*
                           :sql-state sql-state  :secondary-error-id secondary-error-id
                           :message message
                           :records records)) )

               ;((equal sql-state "08S01")		       ; Communication link failure
                ;; The communication link between the driver and the data source
                ;; to which the driver was attempting to connect failed before
                ;; the function completed processing. 
               ((and (equal (subseq sql-state 0 2) "08")	; Connection Exception
                     ;(record-sql-command "Connection lost" database)
                     database
                     (imp (>= $ODBCVER 0350)
                          (eql (%sql-get-attribute :hdbc (handle database)
                                                   SQL_ATTR_CONNECTION_DEAD)
                                SQL_CD_TRUE)))
                (setf (connected database) nil)
                ; (when (reconnect :database database)
                ;   (when-bind (r (find-restart 'retry-database-operation))
                ;     (invoke-restart r)))))
                (error 'sql-fatal-error :database database :expression expression
                       :sql-state sql-state  :secondary-error-id secondary-error-id
                       :message message
                       :records records))

               (database
                (odbc-error database sql-state secondary-error-id
                            :expression expression :message message :records records))
               (t
                (error 'odbc-connection-error :connection-spec args
                       :sql-state sql-state  :secondary-error-id secondary-error-id
                       :message message
                       :records records)) )))))
    ;(#.SQL_SUCCESS #.SQL_NO_DATA #.SQL_NEED_DATA)
  ) ; case
  return-code)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Handles

(defun %sql-alloc-handle (handle-type &optional (input-handle +SQL-NULL-HANDLE+))
 "Args: HANDLE-TYPE one of :henv, :hdbc, :hstmt, or :hdesc.
   INPUT-HANDLE defaulted only when allocating environment."
  ;; Thread-safety Note:
  ;;	Usually, environment and connection handles can be allocated without
  ;;	synchronization. So there is no need to call SQLError or SQLGetDiagRec.
  ;;	Allocation of a hstmt is performed via obtain-statement, hence is synchronized.
  (with-temporary-allocation ((output-handle-ptr sql-handle))
    (with-error-signaling 
        ((case handle-type (:hdbc :henv) ((:hstmt :hdesc) :hdbc))
         input-handle)
      ;(if (>= $ODBCVER 0300)
      #+odbc3.0  
      (SQLAllocHandle (%encode-handle-type handle-type) input-handle output-handle-ptr)
      #-odbc3.0  
      (ecase handle-type
        (:henv  (SQLAllocEnv output-handle-ptr))
        (:hdbc  (SQLAllocConnect input-handle output-handle-ptr))
        (:hstmt (SQLAllocStmt input-handle output-handle-ptr))
        );) ;(:desc)        
      (%get-ptr output-handle-ptr))))

(defun %sql-free-handle (handle-type handle) ;&key henv hdbc hstmt hdesc
  (with-error-signaling (handle-type handle)
    (if (>= $ODBCVER 0300)
        (SQLFreeHandle (%encode-handle-type handle-type) handle)
        (ecase handle-type
           (:henv  (SQLFreeEnv handle))
           (:hdbc  (SQLFreeConnect handle))
           (:hstmt (SQLFreeStmt handle SQL_DROP))) )))

(defun %sql-free-statement (hstmt &rest options)
  (with-error-signaling (:hstmt hstmt)
      (dolist (option options)
        (SQLFreeStmt hstmt 
         (ecase option
           (:drop SQL_DROP) ; deprecated in ODBC 3.0 in favour of SQLFreeHandle
           (:close 	; Close the cursor associated with StatementHandle and 
            SQL_CLOSE)	; discard all pending results. If no cursor is open, no effect.
           (:unbind	; Release all column buffers bound by SQLBindCol; 
            SQL_UNBIND)	; does not unbind the bookmark column.
           (:reset	; Release all parameter buffers set by SQLBindParameter.
            SQL_RESET_PARAMS))))))

(defmacro with-statement-handle ((hstmt hdbc) &body body)
  `(let ((,hstmt (%sql-alloc-handle :hstmt ,hdbc))) ;  (%new-statement-handle ,hdbc)
     (unwind-protect
       (progn ,@body)
       (%sql-free-handle :hstmt ,hstmt)))) ; (%free-statement ,hstmt :drop)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Attributes and info inqueries

(defgeneric %sql-get-attribute (handle-type handle attribute)
 (:documentation "Args: HANDLE-TYPE one of :henv, :hdbc, or :hstmt.")

 (:method ((handle-type (eql :henv)) handle attribute)
  ;; Value: 32-bit SQLUINTEGER or SQLUINTEGER
  (when (>= $ODBCVER 0300)
    (with-temporary-allocation ((value-ptr sql-uinteger))
      (with-error-signaling (:henv handle)
          (SQLGetEnvAttr handle attribute value-ptr SQL_IS_UINTEGER *null-pointer*)
        (%get-uinteger value-ptr)))
   ;(warn 'odbc-warning :message "SQLGetEnvAttr is not supported in " SQL_SPEC_STRING))
  ))

 (:method ((handle-type (eql :hdbc)) handle attribute)
  ;(without-scheduling ;make attibute retrieval thread-safe?
  (case attribute
    ((#.SQL_ATTR_CURRENT_CATALOG
      #.SQL_ATTR_TRACEFILE	; does not require the Handle argument to be valid
      #.SQL_ATTR_TRANSLATE_LIB)	; the name of a library containing tr.functions
                   ; Unicode:  (value-ptr :ef-wc-string #.SQL_MAX_OPTION_STRING_LENGTH)
     (with-temporary-allocation ((value-ptr :string #.SQL_MAX_OPTION_STRING_LENGTH)
                                 (length-ptr sql-integer)) ; picks the number of bytes
       (with-error-signaling (:hdbc handle)
           (if (>= $ODBCVER 0300)
               (SQLGetConnectAttr handle attribute value-ptr
                                  SQL_MAX_OPTION_STRING_LENGTH	; Unicode *2
                                  length-ptr)
               (SQLGetConnectOption handle attribute value-ptr))
         ;(yl:debug-format :odbc "~&SQLGetConnectAttr << value-ptr ~S length ~S"
         ;                 value-ptr (%get-integer length-ptr))
         ;(unless (%ptr-eql value-ptr *null-pointer*)
         (if (>= $ODBCVER 0300)
             (%get-cstring value-ptr)
             (%get-string value-ptr (%get-integer length-ptr)))) ))
    (#.SQL_ATTR_QUIET_MODE		
     ;; A parent window handle of the application; does not apply to SQLDriverConnect.
     ;; If is a null pointer, the driver does not display any dialog boxes 
     (with-temporary-allocation ((value-ptr :ptr))
       (with-error-signaling (:hdbc handle)
           (if (>= $ODBCVER 0300)
               (SQLGetConnectAttr handle attribute value-ptr SQL_IS_POINTER *null-pointer*)
               (SQLGetConnectOption handle attribute value-ptr))
         (%get-ptr value-ptr))))    
    (otherwise                		; SQLUINTEGER or 32-bitmask
     (with-temporary-allocation ((value-ptr sql-uinteger))
       (with-error-signaling (:hdbc handle)
           (if (>= $ODBCVER 0300)
               (SQLGetConnectAttr handle attribute value-ptr
                                  SQL_IS_UINTEGER *null-pointer*)
               (SQLGetConnectOption handle attribute value-ptr))
         (%get-uinteger value-ptr)))) ))

 (:method ((handle-type (eql :hstmt)) handle attribute)
  (case attribute
    #+odbc3.0
    ((#.SQL_ATTR_APP_PARAM_DESC		; The handle to the APD  
      #.SQL_ATTR_APP_ROW_DESC		; The handle to the ARD for fetches
      #.SQL_ATTR_FETCH_BOOKMARK_PTR	; A pointer that points to a binary bookmark value.  
      #.SQL_ATTR_IMP_PARAM_DESC		; The handle to the IPD.  
      #.SQL_ATTR_IMP_ROW_DESC		; The handle to the IRD. 
      #.SQL_ATTR_PARAM_BIND_OFFSET_PTR	; An SQLUINTEGER* value that points to an offset
      #.SQL_ATTR_PARAM_OPERATION_PTR	; An SQLUSMALLINT* value that points to an array
      #.SQL_ATTR_PARAM_STATUS_PTR	; An SQLUSMALLINT* value that points to an array
      #.SQL_ATTR_PARAMS_PROCESSED_PTR	; An SQLUINTEGER* value that points to a buffer
      					; in which to return the num.of sets of params
      #.SQL_ATTR_ROW_BIND_OFFSET_PTR	; An SQLUINTEGER* value that points to an offset
					; added to ptrs to change binding of column data
      #.SQL_ATTR_ROW_OPERATION_PTR	; An SQLUSMALLINT* value that points to an array
					; used to ignore a row during a bulk operation 
      #.SQL_ATTR_ROW_STATUS_PTR		; An SQLUSMALLINT* value that points to an array
					; of values containing row status values
      #.SQL_ATTR_ROWS_FETCHED_PTR	; An SQLUINTEGER* value that points to a buffer
					; in which to return the number of rows fetched 
     )
     (with-temporary-allocation ((value-ptr :ptr))
        (with-error-signaling (:hstmt handle)
           (if (>= $ODBCVER 0300)
               (SQLGetStmtAttr handle attribute value-ptr SQL_IS_POINTER *null-pointer*)
               (SQLGetStmtOption handle attribute value-ptr))
          (%get-ptr value-ptr))))
    (otherwise                 		; SQLUINTEGER
     (with-temporary-allocation ((value-ptr sql-uinteger))
       (with-error-signaling (:hstmt handle)
           (if (>= $ODBCVER 0300)
               (SQLGetStmtAttr handle attribute value-ptr SQL_IS_UINTEGER *null-pointer*)
               (SQLGetStmtOption handle attribute value-ptr))
          (%get-uinteger value-ptr)))) ))
)

;;; An application can determine whether a statement or connection attribute
;;; is supported by calling SQLSetStmtAttr or SQLSetConnectAttr with that attribute.
;;; If the function returns SQL_SUCCESS or SQL_SUCCESS_WITH_INFO, the attribute
;;; is supported. If it returns SQL_ERROR and SQLSTATE "HYC00"
;;; (Optional feature not implemented), the attribute is not supported.

(defgeneric %sql-set-attribute (handle-type handle attribute value)
 (:documentation "Args: HANDLE-TYPE one of :henv, :hdbc, or :hstmt.
  VALUE : unsigned integer value or pointer to a null-terminated character string.")

 (:method ((handle-type (eql :henv)) handle attribute (value integer))
  ;; Args: handle can be nil if the shared environment Con.Pooling attribute is set.
  (when (>= $ODBCVER 0300)
    (let ((henv (or handle +SQL-NULL-HANDLE+)))
      (with-error-signaling (:henv henv)
          (SQLSetEnvAttr henv attribute value SQL_IS_UINTEGER)))))

 (:method ((handle-type (eql :hdbc)) handle attribute (value integer))
  ;(without-scheduling ;make attibute setting thread-safe?
  (with-error-signaling (:hdbc handle)
      (if (>= $ODBCVER 0300)
          (SQLSetConnectAttr handle attribute value SQL_IS_UINTEGER)
          (SQLSetConnectOption handle attribute value))))

 (:method ((handle-type (eql :hdbc)) handle attribute (value string))
  ;(without-scheduling ;make attibute setting thread-safe?
  (if (>= $ODBCVER 0300)
      (with-cstr (ptr value element-count)
        (with-error-signaling (:hdbc handle)
            (SQLSetConnectAttr handle attribute ptr element-count)))
      (with-cstr (ptr value)			; ensure NTS
        (with-error-signaling (:hdbc handle)
            (SQLSetConnectOption handle attribute ptr))) ))

 (:method ((handle-type (eql :hstmt)) handle attribute (value integer))
  (with-error-signaling (:hstmt handle)
      (if (>= $ODBCVER 0300)
          (SQLSetStmtAttr handle attribute value SQL_IS_UINTEGER)
          (SQLSetStmtOption handle attribute value))))

 (:method ((handle-type (eql :hstmt)) handle attribute (value null))
  (with-error-signaling (:hstmt handle)
      (if (>= $ODBCVER 0300)
          (SQLSetStmtAttr handle attribute *null-pointer* SQL_IS_POINTER)
          (SQLSetStmtOption handle attribute value))))

 #+lispworks
 (:method ((handle-type (eql :hstmt)) handle attribute (value FLI::POINTER))
  (with-error-signaling (:hstmt handle)
      (if (>= $ODBCVER 0300)
          (SQLSetStmtAttr handle attribute value SQL_IS_POINTER)
          (SQLSetStmtOption handle attribute value))))
)

#+unused ;#+odbc3.0
(defun %sql-get-desc-field (handle record-number attribute)
  (case attribute
    (#.SQL_DESC_BASE_TABLE_NAME
                   ; Unicode:  (value-ptr :ef-wc-string #.SQL_MAX_OPTION_STRING_LENGTH)
     (with-temporary-allocation ((value-ptr :string #.SQL_MAX_OPTION_STRING_LENGTH)
                                 (length-ptr sql-integer)) ; picks the number of bytes
       (with-error-signaling (:hdesc handle)
           (SQLGetDescField handle record-number attribute value-ptr
                            SQL_MAX_OPTION_STRING_LENGTH	; Unicode *2
                            length-ptr)
         (%get-cstring value-ptr))))
    ((#.SQL_DESC_TYPE #.SQL_DESC_CONCISE_TYPE
      #.SQL_DESC_PRECISION #.SQL_DESC_SCALE)
     (with-temporary-allocation ((value-ptr sql-smallint))
       (with-error-signaling (:hdbc handle)
           (SQLGetDescField handle record-number attribute value-ptr SQL_IS_SMALLINT
                            *null-pointer*)
         (%get-smallint value-ptr)))) ))

#+odbc3.0
(defgeneric %sql-set-desc-field (handle record-number attribute value)
 (:method (handle record-number attribute (value integer))
  (with-error-signaling (:hdesc handle)
      (SQLSetDescField handle record-number attribute value SQL_IS_SMALLINT))
))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Connecting and disconnecting

(defun %sql-connect (hdbc dsn uid pwd)
  (with-cstrings ((dsn-ptr dsn)
                  (uid-ptr uid)
                  (pwd-ptr pwd))
    (with-error-signaling (:hdbc hdbc :args (list dsn :uid uid :pwd pwd))
        (SQLConnect hdbc dsn-ptr SQL_NTS uid-ptr SQL_NTS pwd-ptr SQL_NTS))))

;;; SQLDriverConnect
;;; [991115 CStacy@Pilgrim.COM]
;;;
;;; The CONNX ODBC driver can bring up a nice GUI prompt for the user-name
;;; and password, so that applications don't have to supply their own.
;;;
;;; That is not desirable for non-interactive applications, such as
;;; web servers, so they should always supply complete login info
;;; to SQLConnect.   But the driver won't bring up a GUI anyway
;;; unless the SQL_QUIET_MODE is set to an HWND (nonzero).
;;; (CONNX version 6 did not have the GUI "Integrated Login" feature,
;;; and in version 7, it was broken such that the GUI always came up.)
;;;
;;; Connx version 8 respects to that connection option, so the first
;;; thing I tried was just setting it.  I hacked the DB-CONNECT ODBC
;;; method with this:
;;;        (without-error-handling
;;;         (SQLSetConnectOption hdbc SQL_QUIET_MODE hwnd))
;;; but that didn't work -- no GUI ever comes up from SQLConnect.
;;; That may be a bug in the CONNX driver.
;;;
;;; In the end, the luser tech support person at CONNX Integrated Solutions
;;; gave me the hint that if I were using VB, I should give it a string
;;; like "DSN=CONNX8SAMPLES32, prompt=2".  There's no ODBC API that wants
;;; a string like that, but SQLDriverConnect does take an attribute-value-list
;;; connection string (including driver-defined attributes).  Reading the SDK
;;; header files, I find that it also takes an argument that is 2 if you want
;;; the driver to use a GUI and prompt the user.  Eureka!
;;;
;;; If the user specified a DSN, we use SQL_DRIVER_COMPLETE and let the
;;; Driver Manager find the appropriate driver.  (Otherwise, aside from
;;; the gratuitous prompt about the driver, the CONNX driver would also
;;; prompting for the DSN and the Data Dictionary (CDD file).

(defun parse-driver-attributes (connection-string &optional (delimeter #\;) end)
  ;; Value: av-pairs list
  (loop for arg in (split-seq delimeter connection-string :test #'char= :end end)
        for (attribute value) = (split-seq #\= arg :test #'char= :keep-empty-subseqs t)
        ;when value 
        collect (list attribute (or (ignore-errors (parse-integer value))
                                    value))))

(defun merge-av-pairs (av-pairs &rest args &key &allow-other-keys)
 ;;; Merge the attributes specified as keyword arguments to the ones in av-pairs
  ;; Value: av-pairs list
  ;; NB: string-equal comparison guarantees coincidence of keywords (e.g. :uid) 
  ;;	 and uppercased strings (e.g. "UID")
  (loop for (key value) on args by 'cddr
        when value
         collect (list key value) into list
         and do (setf av-pairs (remove key av-pairs :key #'first :test #'string-equal))
        finally (return (nconc list av-pairs))))

(defun make-driver-attributes-string (av-pairs &optional (delimeter #\;))
  (format nil (format nil "~~:{~~A=~~A~C~~}" delimeter) av-pairs))

(defun %sql-driver-connect (hdbc hwnd in-connection-string option)
 "Return NIL if user cancels a dialog box presented by the Driver Manager or the driver"
 ;; Args: hwnd if *null-pointer*, no window
 (with-cstr (in-connection-str-ptr in-connection-string)
   (with-temporary-allocation ((out-connection-str-ptr :string 1024)
                               (out-len-ind sql-smallint))
     (case (with-error-signaling (:hdbc hdbc :args in-connection-string)
               (SQLDriverConnect hdbc hwnd
                                 in-connection-str-ptr SQL_NTS
                                 out-connection-str-ptr 1024 out-len-ind
                                 (ecase option
                                   (:complete SQL_DRIVER_COMPLETE)
                                   (:required SQL_DRIVER_COMPLETE_REQUIRED)
                                   (:prompt SQL_DRIVER_PROMPT)
                                   (:noprompt SQL_DRIVER_NOPROMPT)))) ; error if incompl
       (#.SQL_NO_DATA)	; the user cancels a dialog box 
       (otherwise ; (print
        (%get-string out-connection-str-ptr (%get-smallint out-len-ind)))))))


(defun %sql-disconnect (hdbc)
  (with-error-signaling (:hdbc hdbc)
      (SQLDisconnect hdbc)))

(defun %sql-end-transaction (handle-type handle completion-type)
  (let ((c-type (ecase completion-type (:commit SQL_COMMIT)
                                       (:rollback SQL_ROLLBACK))))
    (with-error-signaling (handle-type handle)
    ;(with-error-signaling (:henv (when (eq handle-type :henv) handle)
    ;                       :hdbc (when (eq handle-type :hdbc) handle))
        (ecase handle-type
          (:henv
           (if (>= $ODBCVER 0300)
               (SQLEndTran SQL_HANDLE_ENV handle c-type)     
               (SQLTransact handle +SQL-NULL-HANDLE+ c-type)))
          (:hdbc
           (if (>= $ODBCVER 0300)
               (SQLEndTran SQL_HANDLE_DBC handle c-type)     
               (SQLTransact +SQL-NULL-HANDLE+ handle c-type)))) )))

#+ignore
(defmacro %with-transaction ((handle-type handle) &body body)
  (with-gensyms (successp)
    `(let ((,successp nil))
       (unwind-protect
         (prog1
           (progn ,@body)
           (setf ,successp t))
         (%sql-end-transaction handle-type handle
                               (if ,successp :commit :rollback))))))
         ;(with-error-signaling (:henv ,henv :hdbc ,hdbc)
         ;    (SQLTransact ,henv ,hdbc 
         ;                 (if ,successp SQL_COMMIT SQL_ROLLBACK)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Statements
  
(defun %sql-bind-column (hstmt column-nr target-type data-ptr
                         octet-length		; length in bytes of one element
                         octet-len-ind-ptr)
 ;;; Args: column-nr If =0, retrieves a bookmark.
  (with-error-signaling (:hstmt hstmt)
      (SQLBindCol hstmt column-nr
                  target-type data-ptr octet-length octet-len-ind-ptr)))

(defun %sql-unbind-column (hstmt column-nr)
  (with-error-signaling (:hstmt hstmt)
      (SQLBindCol hstmt column-nr
                  SQL_C_DEFAULT *null-pointer* 0 *null-pointer*)))

(defun %sql-bulk-operations (hstmt operation)
 ;;; Adds a new rows or perform operation where each row is identified by a bookmark.
  ;; NB1: After a call to SQLBulkOperations, the block cursor position is undefined.
  ;; NB2: No reason to set the SQL_ATTR_ROW_OPERATION_PTR statement attribute beforehand
  ;;      because rows cannot be ignored when performing bulk operations via this call.
  ;; NB3: SQLBulkOperations with an Operation of SQL_ADD can be called in state
  ;;	  S5 (before any fetch) or S6 (in ODBC 2.x after SQLFetch()?),
  ;;	  while SQLSetPos with an Operation of SQL_ADD cannot.
  ;; Signaling:
  ;;  [ODBC error -1045 state HY000].
  ;; 	[Microsoft][ODBC Microsoft Access Driver] Not a valid bookmark.
  ;;  [ODBC error -1017 state HY109] using database Sample.
  ;;	[Microsoft][ODBC Microsoft Access Driver] Record is deleted.
  (with-error-signaling (:hstmt hstmt)
      (SQLBulkOperations hstmt (case operation
                                 (:add    SQL_ADD)
                                 (:update SQL_UPDATE_BY_BOOKMARK)
                                 (:delete SQL_DELETE_BY_BOOKMARK)
                                 (:fetch  SQL_FETCH_BY_BOOKMARK)))))

(defun %sql-cancel (hstmt)
  (with-error-signaling (:hstmt hstmt)
      (SQLCancel hstmt)))

(defun %sql-close-cursor (hstmt)
 ;; Discard all pending results.
  (with-error-signaling (:hstmt hstmt)
      (if (>= $ODBCVER 0300)	     ; If no cursor is open,
          (SQLCloseCursor hstmt)     ; - returns SQLSTATE 24000 (Invalid cursor state)
          (SQLFreeStmt hstmt SQL_CLOSE)))) ; - has no effect

(defun %sql-column-attribute (hstmt column-nr attribute)
  (let ((result-type (%get-desc-attribute-type attribute)))
    (cond ((plusp result-type)  				; string length
                      ; Unicode:  (value-ptr :ef-wc-string (incf result-type))
           (with-temporary-allocation ((value-ptr :string (incf result-type))
                                       ; picks the number of bytes without null-term.
                                       (string-length-ptr sql-integer))
             (with-error-signaling (:hstmt hstmt)
                 (if (>= $ODBCVER 0300)
                     (SQLColAttribute hstmt column-nr attribute value-ptr
                                      result-type 	; buffer-length: Unicode *2
                                      string-length-ptr *null-pointer*)
                     (SQLColAttributes hstmt column-nr
                                       (case attribute
                                         (#.SQL_DESC_NAME SQL_COLUMN_NAME)
                                         (otherwise 	  attribute))
                                       value-ptr result-type string-length-ptr
                                       *null-pointer*))
                 (%get-string value-ptr (%get-integer string-length-ptr)))))
          ((minusp result-type)  			;  32-bit signed
           (with-temporary-allocation ((value-ptr sql-integer))
             (with-error-signaling (:hstmt hstmt)
                 (if (>= $ODBCVER 0300)
                     (SQLColAttribute hstmt column-nr attribute
                                      *null-pointer* result-type *null-pointer* value-ptr)
                     (SQLColAttributes hstmt column-nr
                                       (case attribute
                                         (#.SQL_DESC_LENGTH	SQL_COLUMN_LENGTH)
                                         (#.SQL_DESC_PRECISION	SQL_COLUMN_PRECISION)
                                         (#.SQL_DESC_SCALE	SQL_COLUMN_SCALE)
                                         (#.SQL_DESC_NULLABLE	SQL_COLUMN_NULLABLE)
                                         (#.SQL_DESC_COUNT	SQL_COLUMN_COUNT)
                                         (otherwise 		attribute))
                                       *null-pointer* result-type
                                       *null-pointer* value-ptr))
               (%get-integer value-ptr))))
          (t (error "Wrong column attribute ~S" attribute)))))

#| Deprecated in 3.0
(defun %column-attributes (hstmt column-nr descriptor-type)
  (with-temporary-allocation ((descriptor-info-ptr :string 256)
                               (descriptor-length-ptr :short)
                               (numeric-descriptor-ptr :long))
    (with-error-signaling
      (:hstmt hstmt) 
      (SQLColAttributes hstmt column-nr descriptor-type descriptor-info-ptr 256
                        descriptor-length-ptr
			numeric-descriptor-ptr)
      (values
       (%get-cstring descriptor-info-ptr)
       (%get-integer numeric-descriptor-ptr)))))
 |#

(defun %sql-columns-count (hstmt)
 ;;; Return the number of columns in the result set; it does not include a bound 
  ;; bookmark column and is the same value as the SQL_DESC_COUNT field of the IRD.
  (with-temporary-allocation ((count-ptr sql-smallint))
    (with-error-signaling (:hstmt hstmt)
        (SQLNumResultCols hstmt count-ptr)
      (%get-smallint count-ptr))))

(defun %sql-describe-column (hstmt column-nr &optional (max-name-length 128))
 ;;; Args: column-nr is 1-based; =0 for a bookmark
  ;; Maximum length of a column name in characters could be obtained by:
  ;;	%sql-get-info(SQL_MAX_COLUMN_NAME_LEN)
  ;; An FIPS Intermediate levelconformant driver will return at least 128.
  ;; Unicode NB: Is the number of bytes or chars returned into name-length-ptr?
  (with-temporary-allocation
      ((name-ptr :string (incf max-name-length))
       (name-length-ptr    sql-smallint) ; ?? total nr of bytes (excl.null-termination)
       (sql-type-ptr       sql-smallint) ; = DESC_CONCISE_TYPE or driver-specific SQL
       (column-size-ptr    sql-uinteger) ; = DESC_PRECISION for all numeric types 
 					 ; = DESC_LENGTH for other types
       (decimal-digits-ptr sql-smallint) ; = DESC_SCALE for all exact numeric types
					 ; = DESC_PRECISION for all datetime types
       (nullable-ptr       sql-smallint))
    (with-error-signaling (:hstmt hstmt)
        (SQLDescribeCol hstmt column-nr name-ptr max-name-length name-length-ptr
                        sql-type-ptr column-size-ptr decimal-digits-ptr nullable-ptr)
      (values (%get-string name-ptr (%get-smallint name-length-ptr))
              (%get-smallint sql-type-ptr)
              (%get-uinteger column-size-ptr)
              (%get-smallint decimal-digits-ptr)
              (%get-smallint nullable-ptr)))))

(defun %sql-exec-direct (hstmt text)
 ;;; Executes a preparable statement, using the current values of the parameter marker
  ;; variables if any parameters exist in the statement.
  ;; SQLExecDirect is the fastest way to submit an SQL statement for one-time execution.
  (with-cstr (text-ptr text)
    (with-error-signaling (:hstmt hstmt)
        (SQLExecDirect hstmt text-ptr SQL_NTS))))

(defun %sql-fetch (hstmt &optional)
 ;;; Returns the next rowset in the result set. It is equivalent to calling
  ;; SQLFetchScroll with FetchOrientation set to SQL_FETCH_NEXT. 
  ;; The SQL_ATTR_ROW_ARRAY_SIZE statement attribute specifies the number of rows
  ;; in the rowset. If the rowset being fetched by SQLFetch overlaps the end of
  ;; the result set, SQLFetch returns a partial rowset. That is, if
  ;;	 S + R  1 > L, 
  ;; where S is the starting row of the rowset being fetched,
  ;; 	R is the rowset size, 
  ;;	L is the last row in the result set,
  ;; then only the first L  S + 1 rows of the rowset are valid. The remaining rows
  ;; are empty and have a status of SQL_ROW_NOROW, and cursor is positioning after end
  ;; after the call.
  ;;  After SQLFetch returns, the current row is the first row of the rowset.
  (with-error-signaling (:hstmt hstmt)
      (SQLFetch hstmt)))

(defun %sql-fetch-scroll (hstmt row-descriptor orientation &key offset (retrieve-data t))
 ;;; 
  ;; Args:
  ;;  offset 	If negative in an absolute fetch, rows are counted from the end
  ;; 	  	of the result set. Thus, an absolute fetch of row 1 means
  ;;		to fetch the rowset that starts with the last row in the result set.
  ;;		If 0 in an relative fetch, refetch the same rowset.
  ;;  retrieve-data
  ;;		By setting SQL_RETRIEVE_DATA to SQL_RD_OFF, an application can verify
  ;;		that a row exists or retrieve a bookmark for the row without incurring
  ;;		the overhead of retrieving rows. 
  ;; Value: SQL_SUCCESS
  ;;	    SQL_SUCCESS_WITH_INFO  ,    
  ;;		 ,    ;  rowset  
  ;;		  SQL_ROW_ERROR.
  ;;		  , .. array-size=1,   
  ;;		-    - SQL_SUCCESS.
  ;;            -  (    ,   ) 
  ;;		   [ODBC error -1017 state HY109] [] Record is deleted..
  ;;		   state HY109  MSDN    !
  ;;        SQL_NO_DATA  End of result set.
  ;; After SQLFetchScroll returns, the current row is the first row of the rowset
  ;; When SQLFetchScroll returns a new rowset that has a position relative to
  ;; the current rowset  that is, FetchOrientation is SQL_FETCH_NEXT, SQL_FETCH_PRIOR,
  ;; or SQL_FETCH_RELATIVE  it does not include changes to the current rowset when
  ;; calculating the starting position of the new rowset.
  ;; However, it does include changes outside the current rowset if it is capable
  ;; of detecting them. 
  ;; Furthermore, when SQLFetchScroll returns a new rowset that has a position
  ;; independent of the current rowset  that is, FetchOrientation is SQL_FETCH_FIRST,
  ;; SQL_FETCH_LAST, SQL_FETCH_ABSOLUTE, or SQL_FETCH_BOOKMARK  it includes all
  ;; changes it is capable of detecting, even if they are in the current rowset.
  ;; Driver Manager or Access BUG:
  ;;	 SQL_RD_OFF       
  ;;        Access,   Driver Manager.    , 
  ;;	,   ,  e.g. SQLColumns  SQLDescribeCol
  ;;	     !
  (when (and row-descriptor (xor (retrieve-data row-descriptor) retrieve-data))
    (%sql-set-attribute :hstmt hstmt SQL_ATTR_RETRIEVE_DATA
                        (if retrieve-data SQL_RD_ON SQL_RD_OFF))
    (setf (retrieve-data row-descriptor) retrieve-data))
  (with-error-signaling (:hstmt hstmt)
      (prog1
          (if (>= $ODBCVER 0300)         
              (SQLFetchScroll hstmt
                #1=(ecase orientation
                     (:next SQL_FETCH_NEXT)
                     (:prior SQL_FETCH_PRIOR)
                     (:first SQL_FETCH_FIRST)
                     (:last SQL_FETCH_LAST)
                     (:absolute SQL_FETCH_ABSOLUTE)  ; requires offset
                     (:relative SQL_FETCH_RELATIVE)  ; -''-
                     (:bookmark SQL_FETCH_BOOKMARK)) ; return offset rows 
                (or offset 0))
              (SQLExtendedFetch hstmt #1# (or offset 0)
                (if row-descriptor (desc-rows-processed-ptr row-descriptor) *null-pointer*)
                (if row-descriptor (desc-array-status-ptr row-descriptor) *null-pointer*)))
        (when row-descriptor
          (setf (last-operation row-descriptor) orientation))
          ;(setf (last-operation row-descriptor) (if retrieve-data orientation :probe)))
) )   )

(defun %sql-get-data (hstmt column-nr c-type data-ptr octet-length octet-len-ind-ptr)
   (with-error-signaling (:hstmt hstmt :on-info nil)   ; column-nr is zero-based
      (SQLGetData hstmt column-nr	;(if (eq column-nr :bookmark) 0 (1+ column-nr))
                  c-type data-ptr octet-length octet-len-ind-ptr)))

(defun %sql-put-data (hstmt data-ptr octet-len-ind)
 ;;; SQLPutData never returns SQL_NEED_DATA itself!
  (with-error-signaling (:hstmt hstmt :on-info t) ; nil
      (SQLPutData hstmt data-ptr octet-len-ind)))

(defun %sql-rows-count (hstmt)
  (with-temporary-allocation ((count-ptr sql-integer))
    (with-error-signaling (:hstmt hstmt)
        (SQLRowCount hstmt count-ptr)
      (%get-integer count-ptr))))

(defun %sql-set-pos (hstmt row-nr &optional (operation :position)
                                            (lock-type :no-change))
 ;;;  Set the cursor position in a rowset and allows an application to refresh data
  ;;  in the rowset or to update or delete data in the result set.
  ;; Args: row-nr Position of the row in the rowset on which to perform the operation.
  ;;	          If eql SQL_ENTIRE_ROWSET (0), applies to every row in the rowset.
  ;; Value: SQL_SUCCESS_WITH_INFO    :refresh 
  ;;             ,   rowset  ,   
  ;;		 ;      ROW_ERROR.
  ;;		  , .. array-size=1,   
  ;;		-    - SQL_SUCCESS.
  ;; NB: Call with :position can precede the following operations, which require
  ;;     a cursor position: positioned update and delete statements, call to SQLGetData.
  (with-error-signaling (:hstmt hstmt)
      (SQLSetPos hstmt row-nr
                 (ecase operation
                   (:position SQL_POSITION)	; position the cursor on the row-nr
                   (:refresh  SQL_REFRESH)
                   (:update   SQL_UPDATE)
                   (:delete   SQL_DELETE)
                   (:add      SQL_ADD))		; deprecated in ODBC 3.0
                 (ecase lock-type
                   (:no-change SQL_LOCK_NO_CHANGE) ; the row will be in the same lock.
                   (:exclusive SQL_LOCK_EXCLUSIVE) ; different connection or application
                                                   ; cannot acquire any locks on the row
                   (:unlock    SQL_LOCK_UNLOCK))))) ; unlocks the row
                   
;;; Macros for SQLSetPos
(defmacro %sql-position-to (hstmt row-nr)
  `(%sql-set-pos ,hstmt ,row-nr))
(defmacro %sql-lock-record (hstmt row-nr lock-type)
  `(%sql-set-pos ,hstmt ,row-nr :position ,lock-type))
(defmacro %sql-refresh-record (hstmt row-nr &optional lock-type)
  `(%sql-set-pos ,hstmt ,row-nr :refresh ,@(when lock-type `(,lock-type))))
(defmacro %sql-update-record (hstmt row-nr)
  `(%sql-set-pos ,hstmt ,row-nr :update))
(defmacro %sql-delete-record(hstmt row-nr)
  `(%sql-set-pos ,hstmt ,row-nr :delete))
(defmacro %sql-add-record (hstmt row-nr)
  `(%sql-set-pos ,hstmt ,row-nr :add))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Procedures and parameters
;;; Parameter counting is 1-based in ODBC

(defun %sql-execute (hstmt)
 ;;; Executes a prepared statement using the current values in parameter descriptor
  (with-error-signaling (:hstmt hstmt)
      (SQLExecute hstmt)))

(defun %sql-describe-param (hstmt parameter-nr)
 ;;; Drivers rarely support this function
  ;; Oracle: always return SQL_VARCHAR as sql-type
 "Values: <1> sql-type, <2> column-size, <3> decimal-digits, <4> nullable."
  (with-temporary-allocation
      ((sql-type-ptr       sql-smallint) ; = DESC_CONCISE_TYPE or driver-specific SQL
       (column-size-ptr	   sql-uinteger) ; = DESC_PRECISION for all numeric types 
 					 ; = DESC_LENGTH for other types
       (decimal-digits-ptr sql-smallint) ; = DESC_SCALE for all exact numeric types
					 ; = DESC_PRECISION for all datetime types
       (nullable-ptr       sql-smallint))
    (with-error-signaling (:hstmt hstmt)
        (SQLDescribeParam hstmt parameter-nr sql-type-ptr 
                          column-size-ptr decimal-digits-ptr nullable-ptr)
      (values (%get-smallint sql-type-ptr)
              (%get-uinteger column-size-ptr)
              (%get-smallint decimal-digits-ptr)
              (%get-smallint nullable-ptr)))))

(defun %sql-bind-parameter (hstmt parameter-nr io-type c-type sql-type
                            column-size decimal-digits
                            data-ptr octet-length octet-len-ind-ptr)
  (with-error-signaling (:hstmt hstmt)
      (SQLBindParameter hstmt parameter-nr io-type c-type sql-type
                        column-size decimal-digits
                        data-ptr octet-length octet-len-ind-ptr)))

(defun %sql-params-count (hstmt)
  (with-temporary-allocation ((count-ptr sql-smallint))
    (with-error-signaling (:hstmt hstmt)
        (SQLNumParams hstmt count-ptr)
      (%get-smallint count-ptr))))

(defun %sql-param-data (hstmt value-ptr)
  (with-error-signaling (:hstmt hstmt :on-info t) ;; nil
      (SQLParamData hstmt value-ptr)))

(defun %sql-prepare (hstmt sql-text)
  (with-cstr (text-ptr sql-text text-length)
    (with-error-signaling (:hstmt hstmt)
        (SQLPrepare hstmt text-ptr text-length))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Catalog functions

(defun %sql-columns (hstmt catalog-name schema-name table-name column-name)
 ;;; Allow nil for schema, as Access driver doesn't support "" but only null ptr
  (with-cstrings ((catalog-ptr catalog-name catalog-length)
                  (schema-ptr (when schema-name (string schema-name)) schema-length) 
                  (table-name-ptr (string table-name) table-name-length)
                  (column-name-ptr (string column-name) column-name-length))
    (with-error-signaling (:hstmt hstmt) 
        (SQLColumns hstmt
                    catalog-ptr catalog-length
                    schema-ptr schema-length
                    table-name-ptr table-name-length
                    column-name-ptr column-name-length))))
#| Value: result set
1 TABLE_CAT Varchar.
  Catalog name; NULL if not applicable to the data source. 
  Access =>    ( )  
2 TABLE_SCHEM (TABLE_OWNER Jet 3.5) Varchar.
  Schema name; NULL if not applicable to the data source.
3 TABLE_NAME Varchar not NULL. Table name. 
4 COLUMN_NAME Varchar not NULL. Column name.
5 DATA_TYPE Smallint not NULL. SQL concise data type
  (Access-> SQL_TIMESTAMP = 11).
6 TYPE_NAME  Varchar not NULL. Data sourcedependent data type name.
7 COLUMN_SIZE (PRECISION Jet 3.5) Integer.
  Maximum length in characters, the total number of digits or the total number of
  bits allowed in the column, according to the NUM_PREC_RADIX column.
8 BUFFER_LENGTH (LENGTH Jet 3.5) Integer. The length in bytes of data transferred.
9 DECIMAL_DIGITS (SCALE Jet 3.5) Smallint. 
  The total number of significant digits to the right of the decimal point.
10 NUM_PREC_RADIX (RADIX Jet 3.5) Smallint. 
  For numeric data types, either 10 or 2. NULL if not applicable.
11 NULLABLE Smallint not NULL. 
  SQL_NO_NULLS(0), SQL_NULLABLE(1), or SQL_NULLABLE_UNKNOWN(2)
12 REMARKS Varchar. A description of the column. 
13 COLUMN_DEF Varchar. The default value of the column;
  or 13 ORDINAL (Jet 3.5)
14 SQL_DATA_TYPE Smallint not NULL. Nonconcise SQL data type (Access=>SQL_DATE = 9)
15 SQL_DATETIME_SUB Smallint. 
  The subtype code for datetime and interval data types (Access=>3). 
16 CHAR_OCTET_LENGTH Integer. 
  The maximum length in bytes of a character or binary column.
17 ORDINAL_POSITION Integer not NULL.
18 IS_NULLABLE Varchar. "YES"/"NO" if the column does (not) include NULLs. 
;; ------- Non-standard -------
;; ------- Jet 4
19 ORDINAL
;; ------- PosgreSQL
19 DISPLAY_SIZE
20 FIELD_TYPE
21 AUTO_INCREMENT ::= SQL_TRUE or SQL_FALSE
22 PHYSICAL NUMBER
22 TABLE OID

;; Result set is ordered by TABLE_CAT, TABLE_SCHEM, TABLE_NAME, and ORDINAL_POSITION. 
 |#

(defun %sql-data-sources (henv &key (direction :first))
 ;;; Args: direction
  ;;		SQL_FETCH_NEXT (to fetch the next data source name in the list),
  ;;		SQL_FETCH_FIRST (to fetch from the beginning of the list),
  ;;		SQL_FETCH_FIRST_USER (to fetch the first user DSN),
  ;;		SQL_FETCH_FIRST_SYSTEM (to fetch the first system DSN).
  (with-temporary-allocation ((name-ptr :string (1+ SQL_MAX_DSN_LENGTH))
                              (name-length-ptr sql-smallint)
                              (description-ptr :string 1024)
                              (description-length-ptr sql-smallint))
    (let ((res (with-error-signaling (:henv henv)
                 (SQLDataSources henv
                                 (ecase direction
				   ((:all :first)	SQL_FETCH_FIRST)
				   (:next   		SQL_FETCH_NEXT)
				   #+odbc3.0 (:user	SQL_FETCH_FIRST_USER)
				   #+odbc3.0 (:system	SQL_FETCH_FIRST_SYSTEM))
                                 name-ptr (1+ SQL_MAX_DSN_LENGTH) name-length-ptr
                                 description-ptr 1024 description-length-ptr))))
      (unless (= res SQL_NO_DATA)
        (values (%get-string name-ptr (%get-smallint name-length-ptr))
                (%get-string description-ptr (%get-smallint description-length-ptr)))))))

(defun %sql-drivers (henv &key (direction :first))
  (with-temporary-allocation ((description-ptr :string 1024)
                              (description-length-ptr sql-smallint)
                              (data-ptr :string 1024)
                              (octet-length-ptr sql-smallint))
    (let ((res (with-error-signaling (:henv henv)
                 (SQLDrivers henv
                             (ecase direction
                               (:first	SQL_FETCH_FIRST)
                               (:next  	SQL_FETCH_NEXT))
                             description-ptr 1024 description-length-ptr
                             data-ptr 1024 octet-length-ptr))))
      (unless (= res SQL_NO_DATA)
        (values (%get-string description-ptr (%get-smallint description-length-ptr))
                (%get-string data-ptr (%get-smallint octet-length-ptr)))))))

(defun %sql-foreign-keys (hstmt pk-catalog-name pk-schema-name pk-table-name
                                fk-catalog-name fk-schema-name fk-table-name)
 ;;; Args:
  ;; Value: 
  ;;  - If the foreign keys associated with a primary key are requested, the result set
  ;;    is ordered by FKTABLE_CAT, FKTABLE_SCHEM, FKTABLE_NAME, and KEY_SEQ.
  ;;  - If the primary keys associated with a foreign key are requested, the result set
  ;;    is ordered by PKTABLE_CAT, PKTABLE_SCHEM, PKTABLE_NAME, and KEY_SEQ
  (with-cstrings
   ((pk-catalog-ptr pk-catalog-name pk-catalog-length)  
    (pk-schema-ptr (when pk-schema-name (string pk-schema-name)) pk-schema-length) 
    (pk-table-name-ptr (when pk-table-name (string pk-table-name)) pk-table-name-length)
    (fk-catalog-ptr fk-catalog-name fk-catalog-length)  
    (fk-schema-ptr (when fk-schema-name (string fk-schema-name)) fk-schema-length) 
    (fk-table-name-ptr (when fk-table-name (string fk-table-name)) fk-table-name-length)
   )
   (with-error-signaling (:hstmt hstmt) 
         (SQLForeignKeys hstmt
                         pk-catalog-ptr pk-catalog-length
                         pk-schema-ptr pk-schema-length
                         pk-table-name-ptr pk-table-name-length
                         fk-catalog-ptr fk-catalog-length
                         fk-schema-ptr fk-schema-length
                         fk-table-name-ptr fk-table-name-length))))
#| Value: result set
1 PKTABLE_CAT Varchar. Primary key table catalog name;  NULL if not applicable.
2 PKTABLE_SCHEM Varchar.  Primary key table schema name; NULL if not applicable.
3 PKTABLE_NAME Varchar not NULL. Primary key table name. 
4 PKCOLUMN_NAME Varchar not NULL. Primary key column name. 
  The driver returns an empty string for a column that does not have a name. 
5 FKTABLE_CAT Varchar. Foreign key table catalog name; NULL if not applicable.
6 FKTABLE_SCHEM Varchar.  Foreign key table schema name; NULL if not applicable.
7 FKTABLE_NAME Varchar not NULL. Foreign key table name. 
8 FKCOLUMN_NAME Varchar not NULL. Foreign key column name. 
  The driver returns an empty string for a column that does not have a name. 
9 KEY_SEQ Smallint not NULL. Column sequence number in key (starting with 1). 
10 UPDATE_RULE Smallint.
  Action to be applied to the foreign key when the SQL operation is UPDATE.
  Can have one of the following values.:
  SQL_CASCADE:   The foreign key of the referencing table is also updated. 
  SQL_NO_ACTION: If an update of a row in the referenced table would cause 
		 a "dangling reference", the update is rejected
  SQL_SET_NULL:	 Each component of the foreign key of referencing table is set to NULL.
  SQL_SET_DEFAULT: The components of the foreign key in the referencing table are set
		   to the applicable default values.
  NULL		 If not applicable to the data source.
11 DELETE_RULE Smallint. Action to be applied to the foreign key when the SQL operation
   is DELETE. Can have one of the following values:
  SQL_CASCADE:   All the matching rows in the referencing tables are also deleted
  SQL_NO_ACTION: If a delete of a row in the referenced table would cause 
		 a "dangling reference", the delete is rejected.
  SQL_SET_NULL:  Each component of the foreign key of referencing table is set to NULL.
  SQL_SET_DEFAULT:
  NULL		 If not applicable to the data source.
12 FK_NAME Varchar. Foreign key name. NULL if not applicable to the data source. 
13 PK_NAME Varchar. Primary key name. NULL if not applicable to the data source. 
14 DEFERRABILITY (ODBC 3.0) Smallint.
   SQL_INITIALLY_DEFERRED, SQL_INITIALLY_IMMEDIATE, SQL_NOT_DEFERRABLE. 
 |#

(defun %sql-func-exists (database function-id)
  (if-bind (functions-vector (slot-value database 'functions-vector))
    (not (zerop (sbit functions-vector function-id)))
    (let* ((length 1024)  ; Observable maximum of my own (SQL_API_SQLSETSTMTATTR=1020)
           (vector (make-array length :element-type 'bit))
           (ptr (%new-binary SQL_API_ODBC3_ALL_FUNCTIONS_SIZE 'sql-usmallint)))
      (declare (type fixnum length) (type simple-bit-vector vector))
      (unwind-protect
          ;; Get all function information into a bitmap - word array.
          ;;        .
          ;; #define SQL_FUNC_EXISTS(pfExists, uwAPI) 
          ;;   ((*(((UWORD*) (pfExists)) + ((uwAPI) >> 4)) & (1 << ((uwAPI) & 0x000F)))
          ;;	 ? SQL_TRUE : SQL_FALSE )
          (loop initially (%sql-get-functions (handle database) ptr)
                for index fixnum upfrom 0 below length
                do (setf (sbit vector index)
                         (if (zerop (logand (%get-usmallint ptr (ash index -4))
                                            (ash 1 (logand index #x000F))))
                             0 1))
                finally (setf (slot-value database 'functions-vector) vector)
                        (return (not (zerop (sbit vector function-id)))))
        (%dispose-ptr ptr)))))

(defun %sql-get-functions (hdbc function-id)
  (if (integerp function-id)
      (with-temporary-allocation ((value-ptr sql-usmallint))
        (with-error-signaling (:hdbc hdbc) 
            (SQLGetFunctions hdbc function-id value-ptr)
          (eql (%get-usmallint value-ptr) SQL_TRUE)))
      ;; Otherwise function-id must be a pointer to a bitmap of 250 words.
      (with-error-signaling (:hdbc hdbc) 
          (SQLGetFunctions hdbc (if (>= $ODBCVER 0300) #.SQL_API_ODBC3_ALL_FUNCTIONS
                                                       #.SQL_API_ALL_FUNCTIONS)
                           function-id))))

(defun %sql-get-info (hdbc info-type)
  (ecase (%get-info-result-type info-type)
    (:string					; -> A null-terminated character string
     (with-temporary-allocation ((value-ptr :string 1024)
                                 (string-length-ptr sql-smallint))
       (with-error-signaling (:hdbc hdbc)
	   #-pcl (SQLGetInfo hdbc info-type value-ptr 1024 string-length-ptr)
	   #+pcl (SQLGetInfo-Str hdbc info-type value-ptr 1024 string-length-ptr)
         (%get-string value-ptr (%get-smallint string-length-ptr)))))
    (:boolean					; -> A null-terminated string "Y" or "N"
     (with-temporary-allocation ((value-ptr sql-integer) ; Must be wide enough for "Y" or "N"
                                 (string-length-ptr sql-smallint))
       (member (with-error-signaling (:hdbc hdbc)
                   (SQLGetInfo hdbc info-type value-ptr 4 string-length-ptr)
                 (%get-char value-ptr))
               '(#\Y #\T))))			; As OpenLink driver (NT) gives "T"
    (:short					; -> SQLUSMALLINT 
     (with-temporary-allocation ((value-ptr sql-usmallint))
       (with-error-signaling (:hdbc hdbc)
           (SQLGetInfo hdbc info-type value-ptr 2 *null-pointer*)
         (%get-usmallint value-ptr))))
    (:long					; -> SQLUINTEGER
     (with-temporary-allocation ((value-ptr sql-uinteger))
       (with-error-signaling (:hdbc hdbc)
           (SQLGetInfo hdbc info-type value-ptr 4 *null-pointer*)
         (%get-uinteger value-ptr))))))

(defun %sql-get-type-info (hstmt sql-type)
 ;;; Args: sql-type ::= SQL_ALL_TYPES specifies that information about all data
  ;;             types should be returned.
  (with-error-signaling (:hstmt hstmt) 
      (SQLGetTypeInfo hstmt sql-type)))
#| Value: result set
1 TYPE_NAME Varchar not NULL.
  Data sourcedependent data-type name; for example, "CHAR()", "VARCHAR()", "MONEY",
  "LONG VARBINARY", or "CHAR ( ) FOR BIT DATA". Applications must use this name in
  CREATE TABLE and ALTER TABLE statements. 

2 DATA_TYPE Smallint not NULL.
  SQL data type. This can be an ODBC SQL data type or a driver-specific SQL data type.
  For datetime or interval data types, this column returns the concise data type 
  (such as SQL_TYPE_TIME or SQL_INTERVAL_YEAR_TO_MONTH).

3 COLUMN_SIZE Integer (PRECISION in ODBC 2.0).
  The maximum column size that the server supports for this data type.
  For numeric data, this is the maximum precision. 
  For string data, this is the length in characters. 
  For datetime data types, this is the length in characters of the string
  representation (assuming the maximum allowed precision of the fractional seconds
  component). 
  NULL is returned for data types where column size is not applicable. 
  For interval data types, this is the number of characters in the character 
  representation of the interval literal.

4 LITERAL_PREFIX Varchar.
  Character or characters used to prefix a literal; for example, a single quotation
  mark (') for character data types or 0x for binary data types; 
  NULL is returned for data types where a literal prefix is not applicable. 

5 LITERAL_SUFFIX Varchar.
  Character or characters used to terminate a literal; for example, a single quotation
  mark (') for character data types; 
  NULL is returned for data types where a literal suffix is not applicable. 

6 CREATE_PARAMS Varchar.
  A list of keywords, separated by commas, corresponding to each parameter that the
  application may specify in parentheses when using the name that is returned in
  the TYPE_NAME field. The keywords in the list can be any of the following: 
  length, precision, or scale. They appear in the order that the syntax requires them
  to be used. 
  For example, CREATE_PARAMS for DECIMAL would be "precision,scale"; 
  CREATE_PARAMS for VARCHAR would equal "length." 
  NULL is returned if there are no parameters for the data type definition; 
  for example, INTEGER. 
  The driver supplies the CREATE_PARAMS text in the language of the country where 
  it is used.

7 NULLABLE Smallint not NULL.
  Whether the data type accepts a NULL value: 
  SQL_NO_NULLS if the data type does not accept NULL values.
  SQL_NULLABLE if the data type accepts NULL values.
  SQL_NULLABLE_UNKNOWN if it is not known whether the column accepts NULL values.
 
8 CASE_SENSITIVE Smallint not NULL.
  Whether a character data type is case-sensitive in collations and comparisons: 
  SQL_TRUE if the data type is a character data type and is case-sensitive.
  SQL_FALSE if the data type is not a character data type or is not case-sensitive.
 
9 SEARCHABLE Smallint not NULL.
  How the data type is used in a WHERE clause: 
  SQL_PRED_NONE if the column cannot be used in a WHERE clause. 
    (This is the same as the SQL_UNSEARCHABLE value in ODBC 2.x.)
  SQL_PRED_CHAR if the column can be used in a WHERE clause, but only with the 
    LIKE predicate. (This is the same as the SQL_LIKE_ONLY value in ODBC 2.x.)
  SQL_PRED_BASIC if the column can be used in a WHERE clause with all the comparison
    operators except LIKE (comparison, quantified comparison, BETWEEN, DISTINCT, IN,
    MATCH, and UNIQUE). (This is the same as the SQL_ALL_EXCEPT_LIKE value in ODBC 2.x.)
  SQL_SEARCHABLE if the column can be used in WHERE clause with any comparison operator.

10 UNSIGNED_ATTRIBUTE Smallint.
  Whether the data type is unsigned: 
  SQL_TRUE if the data type is unsigned.
  SQL_FALSE if the data type is signed.
  NULL is returned if the attribute is not applicable to the data type or the data type
       is not numeric.

11 FIXED_PREC_SCALE Smallint not NULL. (MONEY in ODBC 2.0)
  Whether the data type has predefined fixed precision and scale (which are data 
  sourcespecific), such as a money data type: 
  SQL_TRUE if it has predefined fixed precision and scale.
  SQL_FALSE if it does not have predefined fixed precision and scale.
 
12 AUTO_UNIQUE_VALUE Smallint. (AUTO_INCREMENT in ODBC 2.0)
  Whether the data type is autoincrementing: 
  SQL_TRUE if the data type is autoincrementing.
  SQL_FALSE if the data type is not autoincrementing.
  NULL is returned if the attribute is not applicable to the data type or the data type
      is not numeric.
  An application can insert values into a column having this attribute,
  but typically cannot update the values in the column. 
  When an insert is made into an auto-increment column, a unique value is inserted
  into the column at insert time. The increment is not defined, but is data 
  sourcespecific. An application should not assume that an auto-increment column
  starts at any particular point or increments by any particular value.
 
13 LOCAL_TYPE_NAME Varchar.
  Localized version of the data sourcedependent name of the data type. 
  NULL is returned if a localized name is not supported by the data source.
  This name is intended for display only, such as in dialog boxes. 

14 MINIMUM_SCALE Smallint.
  The minimum scale of the data type on the data source. If a data type has a fixed
  scale, the MINIMUM_SCALE and MAXIMUM_SCALE columns both contain this value. For 
  example, an SQL_TYPE_TIMESTAMP column might have a fixed scale for fractional seconds.
  NULL is returned where scale is not applicable.

15 MAXIMUM_SCALE Smallint. 
  The maximum scale of the data type on the data source. NULL is returned where scale
  is not applicable. If the maximum scale is not defined separately on the data source,
  but is instead defined to be the same as the maximum precision, this column contains
  the same value as the COLUMN_SIZE column.

16 SQL_DATA_TYPE (ODBC 3.0) Smallint NOT NULL.
  The value of the SQL data type as it appears in the SQL_DESC_TYPE field of the
  descriptor. This column is the same as the DATA_TYPE column, except for interval
  and datetime data types. 
 
17 SQL_DATETIME_SUB (ODBC 3.0) Smallint.
  When the value of SQL_DATA_TYPE is SQL_DATETIME or SQL_INTERVAL, this column contains
  the datetime/interval subcode. 
  For data types other than datetime and interval, this field is NULL.
 
18 NUM_PREC_RADIX (ODBC 3.0) Integer.
  If the data type is an approximate numeric type, this column contains the value 2
  to indicate that COLUMN_SIZE specifies a number of bits. 
  For exact numeric types, this column contains the value 10 to indicate that
  COLUMN_SIZE specifies a number of decimal digits. Otherwise, this column is NULL. 

19 INTERVAL_PRECISION (ODBC 3.0) Smallint.
  If the data type is an interval data type, then this column contains the value of 
  the interval leading precision. Otherwise, this column is NULL.
 |#

(defun %sql-primary-keys (hstmt catalog-name schema-name table-name)
  ;; Values: (TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME KEY_SEQ PK_NAME)
  (with-cstrings ((catalog-ptr catalog-name catalog-length)  
                  (schema-ptr (when schema-name (string schema-name)) schema-length) 
                  (table-name-ptr (string table-name) table-name-length))
     (with-error-signaling (:hstmt hstmt) 
         (SQLPrimaryKeys hstmt
                         catalog-ptr catalog-length
                         schema-ptr schema-length
                         table-name-ptr table-name-length))))

(defun %sql-special-columns (hstmt info-type catalog-name schema-name table-name
                             scope nullable)
  ;; Args:
  ;;  info-type  Type of column to return. Must be one of the following values:
  ;;	 SQL_BEST_ROWID Returns the optimal column or set of columns that allows any
  ;;                    row in the specified table to be uniquely identified.
  ;;	 SQL_ROWVER	Returns the column or columns in the specified table, if any,
  ;;			that are automatically updated by the data source when any
  ;;			value in the row is updated by any transaction.
  ;;  Scope  Minimum required scope of the rowid. The returned rowid may be of
  ;;         greater scope. Must be one of the following:
  ;;	SQL_SCOPE_CURROW  The rowid is guaranteed to be valid only while positioned 
  ;;			  on that row. A later reselect using rowid may not return a row
  ;;			  if the row was updated or deleted by another transaction.
  ;;	SQL_SCOPE_TRANSACTION   The rowid is guaranteed to be valid for the duration
  ;;				of the current transaction.
  ;;	SQL_SCOPE_SESSION  	The rowid is guaranteed to be valid for the duration of
  ;;				the session (across transaction boundaries).
  ;;  Nullable  Determines whether to return special columns that can have a NULL value.
  ;;    SQL_NO_NULLS	Exclude special columns that can have NULL values.
  ;;	SQL_NULLABLE	Return special columns even if they can have NULL values.
  (with-cstrings ((catalog-ptr catalog-name catalog-length)  
                  (schema-ptr (when schema-name (string schema-name)) schema-length)
                  (table-name-ptr (string table-name) table-name-length))
     (with-error-signaling (:hstmt hstmt) 
         (SQLSpecialColumns hstmt info-type
                            catalog-ptr catalog-length
                            schema-ptr schema-length
                            table-name-ptr table-name-length
                            scope nullable))))
#| Value: result set ordered by SCOPE
1 SCOPE Smallint
  Actual scope of the rowid. NULL is returned when IdentifierType is SQL_ROWVER.
2 COLUMN_NAME Varchar not NULL
  The driver returns an empty string for a column that does not have a name. 
3 DATA_TYPE Smallint not NULL
  SQL data type. This can be an ODBC SQL data type or a driver-specific SQL data type.
4 TYPE_NAME Varchar not NULL
  Data sourcedependent data type name
5 COLUMN_SIZE Integer
  The size of the column on the data source.
6 BUFFER_LENGTH Integer
  The length in bytes of data transferred on an SQLGetData or SQLFetch operation
  if SQL_C_DEFAULT is specified. For numeric data, this size may be different
  than the size of the data stored on the data source. This value is the same as
  the COLUMN_SIZE column for character or binary data.
7 DECIMAL_DIGITS Smallint
  The decimal digits of the column on the data source. NULL is returned for data types
  where decimal digits are not applicable.
8 PSEUDO_COLUMN Smallint
  Indicates whether the column is a pseudo-column, such as Oracle ROWID: 
  SQL_PC_UNKNOWN
  SQL_PC_NON_PSEUDO
  SQL_PC_PSEUDO
 |#

(defun %sql-statistics (hstmt catalog-name schema-name table-name unique reserved)
 ;;; Args: unique   Type of index: SQL_INDEX_UNIQUE or SQL_INDEX_ALL.
  ;;       reserved Indicates the importance of the CARDINALITY and PAGES columns.
  ;;           SQL_ENSURE the driver unconditionally retrieve the statistics.
  ;;           SQL_QUICK the driver retrieve the CARDINALITY and PAGES only if
  ;;                     they are readily available from the server.
  (with-cstrings ((catalog-ptr catalog-name catalog-length)  
                  (schema-ptr (when schema-name (string schema-name)) schema-length) 
                  (table-name-ptr (string table-name) table-name-length))
     (with-error-signaling (:hstmt hstmt) 
         (SQLStatistics hstmt
                        catalog-ptr catalog-length
                        schema-ptr schema-length
                        table-name-ptr table-name-length
                        unique reserved))))
#| Value: result set, ordered by
    NON_UNIQUE, TYPE, INDEX_QUALIFIER, INDEX_NAME, and ORDINAL_POSITION
 where TYPE ::=  SQL_TABLE_STAT (0) indicates a statistic for the table
1 TABLE_CAT Varchar. Catalog name
2 TABLE_SCHEM Varchar. Schema name
3 TABLE_NAME Varchar not NULL.
 Table name of the table to which the statistic or index applies. 
4 NON_UNIQUE Smallint. Indicates whether the index prohibits duplicate values: 
  SQL_TRUE if the index values can be nonunique.
  SQL_FALSE if the index values must be unique.
  NULL is returned if TYPE is SQL_TABLE_STAT.
5 INDEX_QUALIFIER Varchar.
  The identifier that is used to qualify the index name doing a DROP INDEX;
  NULL is returned if an index qualifier is not supported by the data source or
  if TYPE is SQL_TABLE_STAT. If a non-null value is returned in this column,
  it must be used to qualify the index name on a DROP INDEX statement;
  otherwise, the TABLE_SCHEM should be used to qualify the index name. 
6 INDEX_NAME Varchar. Index name; NULL is returned if TYPE is SQL_TABLE_STAT. 
7 TYPE Smallint not NULL. Type of information being returned: 
  SQL_TABLE_STAT indicates a statistic for the table
   (in the CARDINALITY or PAGES column). 
  SQL_INDEX_BTREE indicates a B-Tree index.
  SQL_INDEX_CLUSTERED indicates a clustered index. 
  SQL_INDEX_CONTENT indicates a content index.
  SQL_INDEX_HASHED indicates a hashed index.
  SQL_INDEX_OTHER indicates another type of index.
8 ORDINAL_POSITION Smallint. Column sequence number in index (starting with 1);
  NULL is returned if TYPE is SQL_TABLE_STAT. 
9 COLUMN_NAME Varchar. Column name.
  If the column is based on an expression, such as SALARY + BENEFITS, the expression
  is returned; if the expression cannot be determined, an empty string is returned. 
  NULL is returned if TYPE is SQL_TABLE_STAT. 
10 ASC_OR_DESC Char(1). Sort sequence for the column:
  "A" for ascending; "D" for descending;
  NULL is returned if column sort sequence is not supported by the data source
   or if TYPE is SQL_TABLE_STAT. 
11 CARDINALITY Integer. Cardinality of table or index;
  number of rows in table if TYPE is SQL_TABLE_STAT;
  number of unique values in the index if TYPE is not SQL_TABLE_STAT;
  NULL is returned if the value is not available from the data source. 
12 PAGES Integer. Number of pages used to store the index or table;
  number of pages for the table if TYPE is SQL_TABLE_STAT;
  number of pages for the index if TYPE is not SQL_TABLE_STAT;
  NULL is returned if value is not available from the data source or if not applicable.
13 FILTER_CONDITION Varchar.
  If the index is a filtered index, this is the filter condition, such as
  SALARY > 30000; if the filter condition cannot be determined, this is an empty string.
  NULL if the index is not a filtered index, it cannot be determined whether
  the index is a filtered index, or TYPE is SQL_TABLE_STAT.
 |# 

(defun %sql-tables (hstmt catalog-name schema-name table-name table-type)
  (with-cstrings ((catalog-ptr catalog-name catalog-length)  
                  (schema-ptr (when schema-name (string schema-name)) schema-length)
                  (table-name-ptr (string table-name) table-name-length)
                  (table-type-ptr table-type table-type-length))
     (with-error-signaling (:hstmt hstmt) 
         (SQLTables hstmt
                    catalog-ptr catalog-length
                    schema-ptr schema-length
                    table-name-ptr table-name-length
                    table-type-ptr table-type-length))))
#| Value: result set
1 TABLE_CAT Varchar Catalog name.
  NULL if not applicable to the data source. If a driver supports catalogs for some
  tables but not for others, such as when the driver retrieves data from different
  DBMSs, it returns an empty string ("") for those tables that do not have catalogs. 
2 TABLE_SCHEM Varchar Schema name.
  NULL if not applicable to the data source. If a driver supports schemas for some
  tables but not for others, such as when the driver retrieves data from different
  DBMSs, it returns an empty string ("") for those tables that do not have schemas. 
3 TABLE_NAME Varchar Table name. 
4 TABLE_TYPE Varchar Table type name.
  One of the following: "TABLE", "VIEW", "SYSTEM TABLE", "GLOBAL TEMPORARY",
  "LOCAL TEMPORARY", "ALIAS", "SYNONYM", or a data sourcespecific type name.
  The meanings of "ALIAS" and "SYNONYM" are driver-specific.
5 REMARKS Varchar A description of the table.
;; Result set is ordered by TABLE_TYPE, TABLE_CAT, TABLE_SCHEM, and TABLE_NAME.
 |#


