;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2005 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; ODBC Installer DLL constants and API.

(in-package :ystok.odbc)

#-win32 (error "This module is only for use on Windows")

(eval-when (:load-toplevel :compile-toplevel :execute)
  (defparameter *installer-module* "odbccp32.dll"))

;;; Constants from ODBCInst.h -------------------------------------------------
;; SQLConfigDataSource request flags
(defconstant ODBC_ADD_DSN	1)	; Add a new user data source.
(defconstant ODBC_CONFIG_DSN	2)	; Modify an existing user data source
(defconstant ODBC_REMOVE_DSN	3)	; Remove an existing user data source
(defconstant ODBC_ADD_SYS_DSN	4)	; Add a new system DSN
(defconstant ODBC_CONFIG_SYS_DSN 5)	; Modify an existing system data source
(defconstant ODBC_REMOVE_SYS_DSN 6)	; Remove an existing system data source
;; #if (ODBCVER >= 0x0300)		; Remove the default data source specification
(defconstant ODBC_REMOVE_DEFAULT_DSN 7)	; section from the system information. (It also
;; #endif				; removes the default driver specification
					; section from the Odbcinst.ini entry.
#|
// install request flags
#define	 ODBC_INSTALL_INQUIRY	1		
#define  ODBC_INSTALL_COMPLETE	2

// config driver flags
#define  ODBC_INSTALL_DRIVER	1
#define  ODBC_REMOVE_DRIVER		2
#define  ODBC_CONFIG_DRIVER		3
#define  ODBC_CONFIG_DRIVER_MAX 100
#endif

// SQLGetConfigMode and SQLSetConfigMode flags
#if (ODBCVER >= 0x0300)
#define ODBC_BOTH_DSN		0
#define ODBC_USER_DSN		1
#define ODBC_SYSTEM_DSN		2
#endif  /* ODBCVER >= 0x0300 */

// SQLInstallerError code
#if (ODBCVER >= 0x0300)
#define ODBC_ERROR_GENERAL_ERR                   1
#define ODBC_ERROR_INVALID_BUFF_LEN              2
#define ODBC_ERROR_INVALID_HWND                  3
#define ODBC_ERROR_INVALID_STR                   4
#define ODBC_ERROR_INVALID_REQUEST_TYPE          5
#define ODBC_ERROR_COMPONENT_NOT_FOUND           6
#define ODBC_ERROR_INVALID_NAME                  7
#define ODBC_ERROR_INVALID_KEYWORD_VALUE         8
#define ODBC_ERROR_INVALID_DSN                   9
#define ODBC_ERROR_INVALID_INF                  10
#define ODBC_ERROR_REQUEST_FAILED               11
#define ODBC_ERROR_INVALID_PATH                 12
#define ODBC_ERROR_LOAD_LIB_FAILED              13
#define ODBC_ERROR_INVALID_PARAM_SEQUENCE       14
#define ODBC_ERROR_INVALID_LOG_FILE             15
#define ODBC_ERROR_USER_CANCELED                16
#define ODBC_ERROR_USAGE_UPDATE_FAILED          17
#define ODBC_ERROR_CREATE_DSN_FAILED            18
#define ODBC_ERROR_WRITING_SYSINFO_FAILED       19
#define ODBC_ERROR_REMOVE_DSN_FAILED            20
#define ODBC_ERROR_OUT_OF_MEM                   21
#define ODBC_ERROR_OUTPUT_STRING_TRUNCATED      22
#endif /* ODBCVER >= 0x0300 */

;#define INSTAPI __stdcall

;;; High level APIs
BOOL INSTAPI SQLInstallODBC (HWND hwndParent, LPCSTR     lpszInfFile,
	                  LPCSTR     lpszSrcPath, LPCSTR     lpszDrivers);
 |#
(define-foreign-function "SQLManageDataSources"	; BOOL INSTAPI
    ((hwnd SQL-POINTER))			; HWND     hwndParent
    :boolean
    :module #.*installer-module*)
#|
BOOL INSTAPI SQLCreateDataSource (HWND       hwndParent, LPCSTR     lpszDSN);
BOOL INSTAPI SQLGetTranslator    (HWND       hwnd, LPSTR      lpszName,
	WORD  cbNameMax, WORD  	*pcbNameOut, LPSTR      lpszPath,
        WORD  cbPathMax,  WORD 	*pcbPathOut, DWORD 	*pvOption);

;;; Low level APIs
// NOTE: The high-level APIs should always be used. These APIs
//       have been left for compatibility.
BOOL INSTAPI SQLInstallDriver        (LPCSTR     lpszInfFile,
                                      LPCSTR     lpszDriver,
                                      LPSTR      lpszPath,
                                      WORD       cbPathMax,
                                      WORD 		* pcbPathOut);
BOOL INSTAPI SQLInstallDriverManager (LPSTR      lpszPath,
                                      WORD       cbPathMax,
                                      WORD 		* pcbPathOut);

;BOOL INSTAPI SQLGetInstalledDrivers  (LPSTR lpszBuf, WORD cbBufMax, WORD * pcbBufOut);
(define-foreign-function "SQLGetInstalledDrivers"
    ((data-ptr string-ptr)		; LPSTR lpszBuf
     (buffer-length SQL-SMALLINT)	; WORD	cbBufMax
     (text-length :ptr))		; WORD* pcbBufOut
    :boolean
    :module #.*installer-module*)

BOOL INSTAPI SQLGetAvailableDrivers  (LPCSTR     lpszInfFile, LPSTR      lpszBuf,
                                      WORD       cbBufMax, WORD * pcbBufOut);
 |#
(define-foreign-function "SQLConfigDataSource"	; BOOL INSTAPI
    ((hwnd SQL-POINTER)				; HWND	 hwndParent
     (operation SQL-SMALLINT)			; WORD	 fRequest
     (description string-ptr)			; LPCSTR lpszDriver
     (attributes string-ptr))			; LPCSTR lpszAttributes
    :boolean
    :module #.*installer-module*)
#|
BOOL INSTAPI SQLRemoveDefaultDataSource(void);
BOOL INSTAPI SQLWriteDSNToIni        (LPCSTR     lpszDSN,
                                      LPCSTR     lpszDriver);
BOOL INSTAPI SQLRemoveDSNFromIni     (LPCSTR     lpszDSN);
BOOL INSTAPI SQLValidDSN             (LPCSTR     lpszDSN);

BOOL INSTAPI SQLWritePrivateProfileString(LPCSTR lpszSection,
					 LPCSTR lpszEntry,
					 LPCSTR lpszString,
					 LPCSTR lpszFilename);
int  INSTAPI SQLGetPrivateProfileString( LPCSTR lpszSection,
					LPCSTR lpszEntry,
					LPCSTR lpszDefault,
					LPSTR  lpszRetBuffer,
					int    cbRetBuffer,
					LPCSTR lpszFilename);
BOOL INSTAPI SQLRemoveDriverManager(LPDWORD lpdwUsageCount);
BOOL INSTAPI SQLInstallTranslator(LPCSTR lpszInfFile,
				  LPCSTR lpszTranslator,
				  LPCSTR lpszPathIn,
				  LPSTR  lpszPathOut,
				  WORD   cbPathOutMax,
				  WORD 	*pcbPathOut,
				  WORD	 fRequest,
				  LPDWORD	lpdwUsageCount);
BOOL INSTAPI SQLRemoveTranslator(LPCSTR lpszTranslator, LPDWORD lpdwUsageCount);
BOOL INSTAPI SQLRemoveDriver(LPCSTR lpszDriver, BOOL fRemoveDSN, LPDWORD lpdwUsageCount);
BOOL INSTAPI SQLConfigDriver(HWND hwndParent, 
			 WORD fRequest, 
			 LPCSTR lpszDriver,
			 LPCSTR lpszArgs,
			 LPSTR  lpszMsg,
			 WORD   cbMsgMax,
                         WORD 	*pcbMsgOut);
 |#
;;#if (ODBCVER >=  0x0300)
(define-foreign-function "SQLInstallerError"
    ((record-number SQL-SMALLINT)	; WORD iError
     (error-id :ptr)			; DWORD *pfErrorCode
     (message string-ptr)		; LPSTR	lpszErrorMsg   
     (buffer-length SQL-SMALLINT)	; WORD	cbErrorMsgMax
     (text-length :ptr))		; WORD	*pcbErrorMsg
    SQL-RETURN
    :module #.*installer-module*)
#|
SQLRETURN INSTAPI SQLPostInstallerError(DWORD dwErrorCode, LPCSTR lpszErrMsg);

BOOL INSTAPI SQLWriteFileDSN(LPCSTR  lpszFileName, LPCSTR  lpszAppName,
                             LPCSTR  lpszKeyName, LPCSTR  lpszString);

BOOL INSTAPI  SQLReadFileDSN(LPCSTR  lpszFileName, LPCSTR  lpszAppName,
                             LPCSTR  lpszKeyName, LPSTR   lpszString,
                             WORD    cbString, WORD   *pcbString);
BOOL INSTAPI SQLInstallDriverEx(LPCSTR lpszDriver, LPCSTR	   lpszPathIn,
				 LPSTR	   lpszPathOut, WORD	   cbPathOutMax,
				 WORD	  *pcbPathOut, WORD		fRequest,
				 LPDWORD	lpdwUsageCount);
BOOL INSTAPI SQLInstallTranslatorEx(LPCSTR lpszTranslator, LPCSTR lpszPathIn,
				  LPSTR  lpszPathOut, WORD   cbPathOutMax,
				  WORD 	*pcbPathOut, WORD	 fRequest,
				  LPDWORD	lpdwUsageCount);
BOOL INSTAPI SQLGetConfigMode(UWORD *pwConfigMode);
BOOL INSTAPI SQLSetConfigMode(UWORD wConfigMode);
;;#endif /* ODBCVER >= 0x0300 */

;;; Driver specific Setup APIs called by installer
BOOL INSTAPI ConfigDSN (HWND	hwndParent, WORD	fRequest,
			LPCSTR	lpszDriver, LPCSTR	lpszAttributes);
BOOL INSTAPI ConfigTranslator (	HWND hwndParent, DWORD	*pvOption);
BOOL INSTAPI ConfigDriver(HWND hwndParent, WORD fRequest, 
                          LPCSTR lpszDriver, LPCSTR lpszArgs, 
                          LPSTR  lpszMsg, WORD   cbMsgMax, WORD 	*pcbMsgOut);
 |#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Lisp interface 

(defun null-terminate (string)
 ;;; Concat #\Null char at the end of the string if it is not null-terminated yet.
  (let ((length (length string)))
    (if (or (zerop length) (char/= (char string (1- length)) #\Null))
        (string-append string #.(make-string 1 :initial-element #\Null))
        string)))

(defun %sql-installer-error (&optional (record-number 1))
  (with-temporary-allocation ((message :string #.SQL_MAX_MESSAGE_LENGTH)
			      (error-id sql-integer)
			      (text-length sql-usmallint))
    (if (and (>= $ODBCVER 0300)
             (%sql-succeeded (SQLInstallerError record-number error-id message
                                                SQL_MAX_MESSAGE_LENGTH text-length)))
        (values
         (%get-string message (%get-usmallint text-length))
         (%get-integer error-id))
        "Installer error.")))

(defun %sql-config-data-source (operation driver-name av-pairs
                                &rest args &key hwnd &allow-other-keys)
 ;;; Args:
  ;;  operation   Either of: ODBC_ADD_DSN, ODBC_CONFIG_DSN, ODBC_REMOVE_DSN,
  ;;		             ODBC_ADD_SYS_DSN, ODBC_CONFIG_SYS_DSN, ODBC_REMOVE_SYS_DSN,
  ;;			     ODBC_REMOVE_DEFAULT_DSN
  ;;  driver	  Driver description (usually the name of the associated DBMS), e.g.
  ;;              "Microsoft Access driver (*.mdb)" or "Oracle ODBC Driver".
  ;;		  Invoke SQLDrives() for an exact description list.
  ;;               ,    ODBC_ERROR_COMPONENT_NOT_FOUND (6)
  ;;  av-pairs    Keyword-value pairs. Either of:
  ;;     - string like  "key=value\0..."  where pair is terminated with a null byte, 
  ;;       and the entire list is terminated with a null byte.
  ;;       (That is, two null bytes mark the end of the list.)
  ;;     - av-pairs list like  ((key value)...)
  ;;     - nil  means that args must specified
  ;;  args	Additional keyword arguments that take precedence over attributes.
  ;; NB: Spaces are not allowed around the equal sign in the keyword-value pair.
  ;;     Keywords and their values should not contain the []{}(),;?*=!@\ characters, 
  ;;     and the value of the DSN keyword cannot consist only of blanks.

  ;; First, convert attributes to string of Null-separated key=value pairs if needed.
  (without-properties (args args :hwnd)
    (cond (args
           (setq av-pairs (make-driver-attributes-string
                           (apply #'merge-av-pairs
                                  (if (stringp av-pairs)
                                      (parse-driver-attributes av-pairs #\Null)
                                      av-pairs)
                                  args)
                           #\Null)))
          ((not (stringp av-pairs))
           (setq av-pairs (make-driver-attributes-string av-pairs #\Null)))))
  ;(print attributes t)
  (or (with-cstrings ((name-ptr driver-name)
                      (data-ptr (null-terminate av-pairs)))
        (SQLConfigDataSource (%ensure-hwnd hwnd) operation name-ptr data-ptr))
      (multiple-value-bind (message error-id) (%sql-installer-error)
        (error 'odbc-error :message message :error-id error-id))))
      
(defun %sql-manage-data-sources (&optional hwnd)
 ;;; Pop up the ODBC Administrator applet.
  (or (SQLManageDataSources (%ensure-hwnd hwnd))
      (multiple-value-bind (message error-id) (%sql-installer-error)
        (error 'odbc-error :message message :error-id error-id))))
