;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2005 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; SQL object-oriented to ODBC interface

(in-package :ystok.odbc)

(defmethod db-fetch-into ((cursor odbc-cursor) (instance standard-db-object)
                          &key current (start-column 1) end-column columns
                               (slots t) refresh)
 ;;; Store one row into the.
  ;; Args: instance     If prototype, make new or ensure-db-object provided refresh is T;
  ;;			otherwise the instance to be filled in.
  ;;	   slots	List of slot-definitions or T meaning all slots.
  ;;	   columns	List of names (strings or symbols, :bookmark is allowed?)
  ;;			or numbers of the same lenght as slots;
  ;;			if null, start-column specifies descriptor records.
  ;;       start-column >0 - bookmarks can only be fetched via the columns argument!
  ;; Value: Newly created or extant instance on success, nil if end of result set;
  ;;	    or some error raised.
  ;; Called by: fetch-objects, update-records-from-instance
  (when (active cursor)
   (let ((handle (handle cursor))
         (row-descriptor (row-descriptor cursor))
         (class (class-of instance))
         (changed nil))
    (when-bind (row-nr (if current
                           (current-row-nr row-descriptor)
                           (%fetch-row handle row-descriptor)))
     (if (eq instance (clos:class-prototype class))
         (if refresh
             (multiple-value-setq (instance changed) (ensure-db-object
                                                      cursor instance
                                                      :start-column start-column
                                                      :slots slots :columns columns
                                                      :refresh refresh))
             (setq instance (allocate-db-object (db-class-to-allocate instance
                                                                      cursor start-column)
                                                (database cursor)))))
         ;(setq changed nil))
     (do* ((sd)
           (sds (if (eq slots t) (clos:class-slots class) slots))
           (end-nr (or end-column (desc-count row-descriptor)))
           (records (desc-records row-descriptor))
           (record)
           (array-size (desc-array-size row-descriptor))
           (index (1- row-nr))
           (rest sds (rest rest))
           (columns columns (rest columns))
           (column-nr start-column (1+ column-nr)))
          ((or (null rest) (> column-nr end-nr))
           (when changed
             ;; TODO: For an extant instance, must we make unbound
             ;;	      - all join slots, if refresh is :join,
             ;;       - deferred join slots, if refresh is true,
             ;;       corresponding to updated home keys?
             ;(dolist (join-slot (join-slots class (if (eq refresh :join) t :deferred)))
             ;  (let ((slot-name (clos:slot-definition-name join-slot)))
             ;    (when (assoc slot-name changed)
             ;      (slot-makunbound instance slot-name))))
             (instance-updated instance :changed (if (listp changed) changed ())
                               :slots sds ;(mapcar #'clos:slot-definition-name sds)
                               :source refresh))
             ;(instance-refreshed instance (if (consp refresh) refresh)))
           instance)
       (declare (fixnum array-size index end-nr column-nr))
       (when (base-slot-p (setq sd (first rest)))
         (setq record (if columns
                          (find-descriptor-record row-descriptor (first columns))
                          (aref records (1- column-nr))))
         (set-slot-value-from-datum
           (if (desc-long-p record)
               (progn (when (> array-size 1)		; change current pos in rowset
                        (%sql-position-to handle row-nr))
                 (%read-datum-in-chunks handle record))
               (%read-datum record index))
           instance (clos:slot-definition-name sd) (clos:slot-definition-type sd) sd))
) ))))
