;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2005 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Unicode-related funcitons

(in-package :ystok.odbc)

;;; The program must invoke the following connection functions
;;; in order to expose itself as a Unicode application.

(define-foreign-function "SQLConnectW"
    ((hdbc sql-handle)			; SQLHDBC	ConnectionHandle
     (server-name string-ptr)		; SQLCHAR *     ServerName
     (NameLength1 SQL-SMALLINT)		; SQLSMALLINT	NameLength1
     (user-name string-ptr)		; SQLCHAR *     UserName
     (NameLength2 SQL-SMALLINT)		; SQLSMALLINT	NameLength2
     (password string-ptr)		; SQLCHAR *     Authentication
     (NameLength3 SQL-SMALLINT))	; SQLSMALLINT	NameLength3
    SQL-RETURN)

(define-foreign-function "SQLDriverConnectW"
    ((hdbc sql-handle)			; SQLHDBC	ConnectionHandle
     (hwnd sql-handle)			; SQLHWND	WindowHandle
     (in-connection-str-ptr string-ptr)	; SQLCHAR *	InConnectionString
     (StringLength1 SQL-SMALLINT)	; SQLSMALLINT	StringLength1
     (out-connection-str-ptr string-ptr); SQLCHAR *	OutConnectionString
     (buffer-length SQL-SMALLINT)	; SQLSMALLINT	BufferLength
     (string-length-ptr :ptr)		; SQLSMALLINT * StringLength2Ptr
     (option SQL-USMALLINT))		; SQLUSMALLINT	DriverCompletion
    SQL-RETURN)

(defun %sql-connectw (hdbc dsn uid pwd)
  (with-cstrings ((dsn-ptr dsn nil :Unicode)
                  (uid-ptr uid nil :Unicode)
                  (pwd-ptr pwd nil :Unicode))
    (with-error-signaling (:hdbc hdbc :args (list dsn :uid uid :pwd pwd))
        (SQLConnectW hdbc dsn-ptr SQL_NTS uid-ptr SQL_NTS pwd-ptr SQL_NTS))))

(defun %sql-driver-connectw (hdbc hwnd in-connection-string option)
 "Return NIL if user cancels a dialog box presented by the Driver Manager or the driver"
 ;; Args: hwnd if *null-pointer*, no window
  (with-cstr (in-connection-str-ptr in-connection-string nil :Unicode)
    (with-temporary-allocation ((out-connection-str-ptr :ef-wc-string 1024)
                                (out-len-ind sql-smallint))
      (case (with-error-signaling (:hdbc hdbc :args in-connection-string)
                (SQLDriverConnectW hdbc hwnd
                                   in-connection-str-ptr SQL_NTS
                                   out-connection-str-ptr 2048 out-len-ind
                                   (ecase option
                                     (:complete SQL_DRIVER_COMPLETE)
                                     (:required SQL_DRIVER_COMPLETE_REQUIRED)
                                     (:prompt SQL_DRIVER_PROMPT)
                                     (:noprompt SQL_DRIVER_NOPROMPT)))) ;error if incompl
        (#.SQL_NO_DATA)	; the user cancels a dialog box 
        (otherwise ;(print
         (%get-wstring out-connection-str-ptr (%get-smallint out-len-ind)))))))

#|
(fli:with-dynamic-foreign-objects ()
  (let ((dsn-ptr (fli:convert-to-dynamic-foreign-string ,string-gensym
                                :external-format *foreign-ef*
                                :null-terminated-p t)))
            into setqs

(fli:with-foreign-string (ptr nil nil :allow-null nil) "abdc"
      ptr)
 |#