;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface 
;;; Copyright (c) 2002-2007 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; SQL and SQL.SYS package definitions
;;; LW5 NB: We have to remove off-the-shelf SQL packages at all.

(in-package :cl-user)

#+lispworks
(eval-when (:compile-toplevel :load-toplevel :execute)
  (setf *packages-for-warn-on-redefinition*
        (remove "SQL" *packages-for-warn-on-redefinition* :test #'equal)))

(eval-when (:compile-toplevel :load-toplevel :execute)
(defpackage :ystok.sql.sys
  (:documentation "Internal developer's SQL-interface package")
  (:nicknames :sql.sys)
  (:use :common-lisp :ystok.library
        #+:mcl "CCL")
  #+local-time
  (:import-from :local-time
   #:current-date #:current-timestamp #:localtimestamp)
  (:export
   ;; "Private" exports that can be referred by DBMS interface packages, test suite etc.
   ;; by means of using either the sql.sys package or specifying it explicitly.
   ;; Generally, those DBMS modules do not need the entire sql package.
   #+local-time #:*local-time-decode-function*  #-local-time #:*time-encode-function*
   #:*sql-stream*
   #:active
   #:alias
   #:args
   #:bit-vector-to-hex-string
   #:boolean-true-p
   #:byte-vector-to-hex-string
   #:check-connection-spec
   #:collect-parameters
   #:collect-table-refs
   #:collect-types
   #:column
   #:components
   #:connected
   #:cursor-type
   #:db-alter-sequence
   #:db-column-type
   #:db-cancel
   #:db-close
   #:db-columns-count
   #:db-connect
   #:db-create
   #:db-create-sequence
   #:db-describe-tables
   #:db-delete-row
   #:db-destroy
   #:db-disconnect			;database-disconnect
   #:db-list				; was db-data-sources
   #:db-list-indexes			; 		 yc
   #:db-drop-index			;		 yc
   #:db-drop-sequence
   #:db-drop-table
   #:db-drop-view
   #:db-end-transaction
   #:db-execute-command
   #:db-fetch-into
   #:db-fetch-item
   #:db-fetch-rows
   #:db-identifier-case
   #:db-identifier-quote-char
   #:db-insert-row
   #:db-list-columns
   #:db-list-sequences
   #:db-list-tables
   #:db-list-views
   #:db-name-from-spec
   #:db-map-parameters
   #:db-object
   #:db-open-cursor
   #:db-password-from-spec
   #:db-prepare-statement
   #:db-query
   #:db-sequence-last
   #:db-sequence-next
   #:db-sql-type-specifier
   #:db-special-character-p
   #:db-start-transaction		;database-start-transaction
   ;#:db-table-exists-p			;database-tablep
   #:db-type
   #:db-type-initialize			;database-initialize-database-type
   #:db-type-load-foreign		;database-type-load-foreign
   #:db-update-row
   #:db-user-name-from-spec		;datbase-name-from-spec
   #:limit				; functional	xyu
   #:modifier
   #:operator
   #:output-column-constraints
   #:output-sql
   #:output-sql-identifier
   #:output-table-constraint
   #:prepared
   #:qualifier
   #:sql-call-procedure-exp
   #:sql-cast-exp
   #:sql-concat-exp
   #:sql-condition
   #:sql-function-exp
   #:sql-ident
   #:sql-ident-table
   #:sql-join-exp
   #:statements
   #:table
   #:write-drop-exp-to-string
   #:write-sql-identifier
   #:write-sql-identifier-to-string
   #:write-sql-to-string		;database-output-sql

   ;; Mixins for specific databases - subclasses of db-mixin
   #:access-mixin
   #:desktop-mixin			; MS Desktop Drivers
   ;#:excel-mixin
   #:mysql-mixin
   #:oracle-mixin
   #:postgresql-mixin
   #:sqlite3-mixin

   ;; Security
   ;#:db-alter-role
   #:db-create-role
   #:db-create-user
   #:db-drop-role
   #:db-drop-user
   #:db-grant
   #:db-grant-role
   #:db-revoke
   #:db-revoke-role

   ;; Object-oriented
   #:allocate-db-object
   #:base-slots				; metaobjects
   #:base-slot-p			; metaobjects
   #:base-slot-position
   #:db-class-to-allocate
   #:db-create-class-view
   #:db-delete-instance-records
   #:db-update-record-from-slot
   #:db-update-records-from-instance
   #:db-datum-value
   #:db-drop-class-view
   #:db-info				; metaobjects
   #:db-sql-value			; x: read-sql-value
   #:home-slot-names			; metaobjects
   #:join-slots				; metaobjects
   #:key-slots				; metaobjects
   #:standard-db-class

   ;; SQL-ODBC stuff
   ; #:*active-transactions* #:*commit-nested-transactions-p*
   ;; streams
   ;#+(and :mcl (not :ccl-4.3)) #:read-sequence
   ;#+(and :mcl (not :ccl-4.3)) #:write-sequence
   ;#:make-sql-stream
   ;#:open-sql-stream
   ;#:sql-string-stream

   .
  ;; Shared exports for re-exporting by SQL as public interface. Implemented in:
  ;;  X = Xanalys CommonSQL, Y = YstokSQL, U = UncommonSQL, C = CLSQL, D = Documented
  ;;  SYMBOL				  FILE		XYUC D
  #1=
  (#:*connect-if-exists*		; functional	xyu
   #:*default-environment*		; generics	 y
   #:*default-database*			; generics	xyu
   #:*default-database-type*		; generics	xyu
   #:*default-read-binary-type*		; generics	 y
   #:*default-read-numeric-type*	; generics	 y
   #:*initialized-database-types*	; functional	xyu
   #:*sql-readtable*			; syntax	 yu
   #:*sql-identable*			; syntax	 y
   #:add-sql-stream			; recording     xyu
   #:alter-sequence			; functional	 yuC    ; = set-sequence-position
   #:alter-table			; functional	 y
   #:attribute-type			; functional	xyu	; better name: column-type
   #:cancel-statement			; functional	 y
   #:close-statement			; functional	 y
   #:columns				;		 y
   #:commit				; transactions	xyuc
   #:connect				; functional	xyu
   #:connected-databases		; functional	xyu
   ;#:connection-database		; pools		  u
   #:constraints			; expressions	 y
   #:create-database               	; functional     yu
   #:create-index			; functional    xyu
   #:create-sequence			; functional	 yuC
   #:create-table			; functional	xyu
   #:create-view			; functional	xyu
   #:current-row-number			; odbc-cursor	 y
   #:current-row-status			; odbc-cursor	 y
   #:cursor				; generics	 y
   #:database				; generics	xyu
   #:database-name			; generics	xyu
   #:database-type			; generics	xyu
   #:define-commit-action		; transactions	 y C
   #:define-rollback-action		; transactions	 y C
   #:defsql-function			; syntax   	 yu
   #:delete-records			; functional	xyu
   #:delete-sql-stream			; recording	xyu
   #:destroy-database			; functional     yu
   ;#:destroy-connection-pool		; pools
   #:disable-sql-reader-syntax		; syntax	xyu
   #:disconnect				; functional	xyu
   #:do-query				; functional	xyu
   #:drop-index				; functional	xyu
   #:drop-sequence			; functional	 yuC
   #:drop-table				; functional	xyu
   #:drop-view				; functional	xyu
   ;#:sequence-from-class		; objects    	x
   #:sequence-exists-p			; sql            yuC
   #:enable-sql-reader-syntax		; syntax	x
   #:environment
   #:execute-command			; functional	xyu
   #:expression				; expressions
   #:fetch-into				; functional	 y
   #:fetch-item				; functional	 y
   #:fetch-rows				; functional	 y
   #:find-database			; functional	xyu
   #:find-environment			; functional	 y
   #:in-identable			; syntax	 y
   #:index-exists-p			; functional     yu
   #:initialize-database-type		; functional	xyu
   #:insert-records			; functional	xyu
   #:list-attributes			; functional	xyu   ; better name: list-columns
   #:list-databases			; functional     yu
   #:list-indexes			; functional     yu
   #:list-sequences			; functional     yuC
   #:list-sql-streams			; recording    	xyu
   #:list-tables			; functional	xyu
   #:list-views				; functional	 yu
   #:locally-disable-sql-reader-syntax	; syntax	xyu
   #:locally-enable-sql-reader-syntax	; syntax	xyu
   ;#:make-connection-pool		; pools		  u
   #:map-query				; functional	xyu
   #:name
   #:on
   #:open-cursor
   #:param-cursor
   #:param-statement
   #:password
   #:plist
   #:prepare-statement			; functional
   #:print-query			; sql		xy
   #:query				; sql		xyu
   #:restore-sql-reader-syntax-state	; syntax    	xyu
   #:rollback				; transactions	xyuc
   #:select				; functional	xyu
   #:sequence-exists-p			; functional	 yuC
   #:sequence-last			; functional	 yuC
   #:sequence-next			; functional	 yuC
   #:set-cursor-position		; functional	 y
   #:sql				; expressions	xyu
   #:sql-column
   #:sql-create-index-exp		; expressions	 yu
   #:sql-create-table-exp		; expressions	 yu
   #:sql-expression			; expressions	xyu
   #:sql-operation			; syntax   	xyu
   #:sql-operator			; syntax   	 yu
   #:sql-query-exp
   ;#:sql-recording-p			; recording     xyu
   ;#:sql-stream			; recording    	xyu
   ;#:start-sql-recording		; recording    	x u
   #:start-transaction			; transactions	 yuc
   #:statement				; generics	 y
   #:status				; functional	xyc
   ;#:stop-sql-recording		; recording    	x u
   #:table-exists-p			; functional	xy
   #:timeout
   #:timestring
   #:user-name
   #:update-records			; functional	xyu
   #:view-exists-p			; functional     yu
   #:with-cursor			; functional	 y
   #:with-database           		; functional    xyu
   ;#:with-database-connection		; pools
   #:with-transaction			; transactions	 yuc
   #:with-connection
   #:with-prepared-statement
        
   ;; SQL functions that are not in the common-lisp package
   #:avg #:sum
   #:coalesce
   #:current-date #:current-time #:current-timestamp #:localtimestamp
    
   ;; Conditions and slot accessors
   #:sql-error
   #:sql-check-error			; y
   #:sql-connection-error
   #:sql-database-condition
   #:sql-database-data-error
   #:sql-database-error
   #:sql-fatal-error
   #:sql-foreign-key-error		; y
   #:sql-not-nullable-error		; y
   #:sql-reader-error
   #:sql-simple-error
   #:sql-temporary-error
   #:sql-timeout-error
   #:sql-type-check-error
   #:sql-type-coercion-error
   #:sql-undefined-function-warning
   #:sql-unique-error			; y
   #:sql-user-error		; is not simple in YSQL!
   #:sql-warning
   #:connection-spec
   #:sql-state			; SQLSTATE compliant with SQL standard and ODBC
   #:error-id			; synonym of sql-state, errno in MaiSQL
   #:secondary-error-id
   #:message

   ;; Security
   ;#:alter-role
   #:create-role
   #:create-user
   #:drop-role
   #:drop-user
   #:grant
   #:grant-role
   #:revoke
   #:revoke-role

   ;; Object-oriented
   ;#:*db-auto-sync*
   #:*default-update-objects-max-len*
   ;#:create-sequence-from-class
   #:create-view-from-class
   #:def-view-class
   #:delete-instance-records
   ;#:drop-sequence-from-class
   #:drop-view-from-class
   #:ensure-db-object
   #:find-join-object
   #+deprecated #:instance-refreshed	; in favor of instance-updated/deleted
   #:instance-deleted  
   #:instance-updated  
   #:list-classes
   #:standard-db-object
   #:set-slot-value-from-datum		; CLSQL/USQL: UPDATE-INSTANCE-FROM-DB
   #:slot-value-to-datum
   #:slot-value-to-sql
   #:universal-time			; metaobjects	xyc
   #:update-instance-from-records
   #:update-objects-joins
   #:update-records-from-instance
   #:update-record-from-slot
   #+deprecated #:update-record-from-slots	; in favor of update-records-from-instance
   #:update-slot-from-record

   ;; Additional stuff a la SQL-ODBC
   ;#:*trace-sql*
   ;#:bind-execute
   ;#:bind-execute-query
   ;#:bind-execute-with-stream
   ;#:create-user
   #+deprecated #:data-sources
  ))
  ;; CMUCL PCL MOP, which is kinda whacky
  #+(or cmu sbcl) (:shadowing-import-from #+cmu :pcl #+sbcl :sb-pcl
                   :built-in-class 
                   :class-direct-slots
                   :class-name
                   :class-of
                   :class-slots
                   :compute-effective-slot-definition
                   :direct-slot-definition-class
                   :effective-slot-definition-class
                   :find-class
                   :slot-boundp
                   :slot-definition-name
                   :slot-definition-type
                   :slot-value-using-class
                   :standard-direct-slot-definition
                   :standard-effective-slot-definition
                   :validate-superclass)
  #+(or sbcl cmu) (:shadowing-import-from #+cmu :pcl #+sbcl sb-pcl
                   :class-direct-superclasses
                   :name
                   :standard-class))
  #| #-(or mcl cormanlisp pcl)			; No SQL stream support at the moment
  (:import-from #+lispworks :stream
		#+allegro :excl
		#+pcl :pcl
                "FUNDAMENTAL-STREAM" "FUNDAMENTAL-CHARACTER-STREAM"
                "FUNDAMENTAL-CHARACTER-INPUT-STREAM"
                "FUNDAMENTAL-STREAM"
                "FUNDAMENTAL-INPUT-STREAM"
                "FUNDAMENTAL-CHARACTER-OUTPUT-STREAM"
                "FUNDAMENTAL-OUTPUT-STREAM"
                "FUNDAMENTAL-BINARY-STREAM"
                "FUNDAMENTAL-BINARY-INPUT-STREAM"
                "FUNDAMENTAL-BINARY-OUTPUT-STREAM"
                "STREAM-READ-CHAR"
                "STREAM-PEEK-CHAR"
                "STREAM-READ-SEQUENCE"
                "STREAM-WRITE-SEQUENCE")|#


(defpackage :ystok.sql
  (:documentation "User's SQL-interface package")
  (:nicknames :sql)
  (:use :common-lisp :ystok.library)
  (:import-from :ystok.sql.sys . #1#)
  (:export . #1#))
) ; eval-when

(pushnew :ysql *features*)
