;;;-*- Mode: Lisp; Encoding: (win32:code-page :id 1251) -*-
;;; YstokSQL - Common Lisp ODBC interface - Locale-specific comparing and sorting
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; The database must be created with Window-1251 encoding. (Not supported by SQLite3.)

(in-package :sql.test)

(locally-enable-sql-reader-syntax)

(defmethod test-collate (env-type database)
  (setf (first-name employee1)  ""	(last-name employee1)  ""
        (first-name employee2)  ""		(last-name employee2)  ""
        (first-name employee3)  ""		(last-name employee3)  ""
        (first-name employee4)  ""	(last-name employee4)  ""
        (first-name employee5)  ""	(last-name employee5)  ""
        (first-name employee6)  ""		(last-name employee6)  ""
        (first-name employee7)  ""	(last-name employee7)  ""
        (first-name employee8)  ""	(last-name employee8)  ""
        (first-name employee9)  ""		(last-name employee9)  ""
        (first-name employee10) ""	(last-name employee10) "")

  (update-records-from-instance employee1)
  (update-records-from-instance employee2)
  (update-records-from-instance employee3)
  (update-records-from-instance employee4)
  (update-records-from-instance employee5)
  (update-records-from-instance employee6)
  (update-records-from-instance employee7)
  (update-records-from-instance employee8)
  (update-records-from-instance employee9)
  (update-records-from-instance employee10)

  (test '("" "" "" "" "" "" ""
          "" "" "")
    (select [last-name] :from [employee] :order-by [last-name] :flatp t)
    :multiple-values :ignore
    :reported-form :fdml/collate/order-by)

  ;; For PostgreSQL, locale settings do not affect WHERE clause.
  (test '(("" "") ("" ""))
    (select [e1 last-name] [e2 last-name]
            :from [inner-join [employee :alias e1] [employee :alias e2]
                              :on [= [e1 first-name] [e2 first-name]]]
            :where [/= [e1 emplid] [e2 emplid]]
            :order-by [e1 last-name])
    :multiple-values :ignore
    :known-failure (typep database 'sql.sys:postgresql-mixin)
    :reported-form :fdml/collate/join)

  (test '((4 ""))
    (select [emplid] [last-name] :from [employee] :where [= [last-name] ""])
    :multiple-values :ignore
    :known-failure (typep database 'sql.sys:postgresql-mixin)
    :reported-form :fdml/collate/=)

  (test '((2 "") (3 "") (4 "") (5 "") (6 "") (7 "")
          (8 "") (9 ""))
    (select [emplid] [first-name] :from [employee] :where [<> [first-name] ""]
            :order-by [emplid])
    :multiple-values :ignore
    :known-failure (typep database 'sql.sys:postgresql-mixin)
    :reported-form :fdml/collate/<>)

  (test (typecase database
          (sql.sys:access-mixin		; by default, case-insensitive
           '((1 "") (7 "")))
          (t
           '((1 ""))))
    (select [emplid] [last-name] :from [employee]
            :where [like [last-name] "%%"]
            :order-by [emplid])
    :multiple-values :ignore
    :reported-form :fdml/collate/like)

  (test '((1 "") (7 ""))
    (select [emplid] [last-name] :from [employee]
            :where [like [last-name] "%%" :case :ignore]
            :order-by [emplid])
    :multiple-values :ignore
    :reported-form :fdml/collate/like/case-ignore)
)

(defmethod test-collate :before (env-type (database sql.sys:oracle-mixin))
 ;;; The database character set must be CL8MSWIN1251
  ;; NB: NLS_SORT='RUSSIAN_CI' does not help to :fdml/collate/like
  ;;     behaving similar to Access.
  (execute-command "ALTER SESSION SET NLS_COMP=ANSI;")
  (execute-command "ALTER SESSION SET NLS_SORT='RUSSIAN_CI';"))

(defmethod test-collate :after (env-type (database sql.sys:postgresql-mixin))
  ;; For PostgreSQL, locale settings do not affect WHERE clause, so we have to 
  ;; convert everything to upper case explicitly.
  (test '(("" "") ("" ""))
    (select [e1 last-name] [e2 last-name]
            :from [inner-join [employee :alias e1] [employee :alias e2]
                              :on [= [upper [e1 first-name]] [upper [e2 first-name]]]]
            :where [/= [e1 emplid] [e2 emplid]]
            :order-by [e1 last-name])
    :multiple-values :ignore
    :reported-form :fdml/collate/postgresql/join)

  (test '((4 ""))
    (select [emplid] [last-name] :from [employee]
            :where [= [upper [last-name]] (locale-string-upcase "")])
    :multiple-values :ignore
    :reported-form :fdml/collate/postgresql/=)

  (test '((2 "") (3 "") (4 "") (5 "") (6 "") (7 "")
          (8 "") (9 ""))
    (select [emplid] [first-name] :from [employee]
            :where [<> [upper [first-name]] (locale-string-upcase "")]
            :order-by [emplid])
    :multiple-values :ignore
    :reported-form :fdml/collate/postgresql/<>))

(defmethod test-collate (env-type (database sql.sys:sqlite3-mixin))
 ;;; No locale-specific collation supported in SQLite3.
  (push :fdml/collate/order-by *test-skips*)
  (push :fdml/collate/join *test-skips*)
  (push :fdml/collate/= *test-skips*)
  (push :fdml/collate/<> *test-skips*)
  (push :fdml/collate/like *test-skips*)
  (push :fdml/collate/like/case-ignore *test-skips*))

(restore-sql-reader-syntax-state)

#|
(delete-records :from [employee])
|#