;;;-*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface - Establish connection and post-connection tests


(in-package :sql.test)

(defmethod test-connect (env-type be-type spec &rest args
                                               &key create-destroy &allow-other-keys)
 ;;; Destroy and create if indicated, set *test-user-name*, and connect
  (without-properties (args args :create-destroy)
    (when create-destroy
      (ignore-errors (destroy-database spec :database-type (or be-type env-type)))
      (apply 'create-database spec :database-type (or be-type env-type)
             (if (listp create-destroy) create-destroy ())))
  
    ;(setf *test-database-type* db-type)
    (setq *test-user-name* (sql.sys:db-user-name-from-spec env-type spec))
    ;  (cond ((eq :oracle env-type) (second spec))
    ;        ((>= (length spec) 3) (third spec))))
  
    (apply 'connect spec :database-type (or be-type env-type)
           :make-default t :if-exists :old
           args)
    ;(truncate-database :database *default-database*)    ; ensure database is empty
  )
  *default-database*)

(defmethod test-connect :after ((env-type (eql :odbc)) (be-type (eql :oracle)) spec
                                &rest args)
  ;; Oracle catalog functions prefer schema names uppercased.
  (setq *test-user-name* (string-upcase *test-user-name*))
  ;; When using without local-time
  (execute-command "ALTER SESSION SET NLS_TIMESTAMP_FORMAT='YYYY-MM-DD HH24:MI:SS';"))

;;; For Access without local-time, you should have Windows regional settings
;;; corresponded to ISO8601 time format.
;(defmethod test-connect :after ((env-type (eql :odbc)) (be-type (eql :access)) spec &rest args))

;;; Tests to pass after the connection is established

(defmethod test-connection (type database)
 ;;; Args: type ::= (or be-type env-type)
  (test database
        (find-database (database-name database) :database-type (sql.sys:db-type database))
        :reported-form :connection/1)

  (test '("localhost" "dbname" "user" "passwd")
        (let ((conn '("dbname" :user "user" :password "passwd")))
          (list (sql.sys:db-name-from-spec type conn)
                (sql.sys:db-user-name-from-spec type conn)))
        :known-failure t
        :reported-form :connection/2)

  (test '("dbname" "user")
        (let ((conn "dbname/user/passwd"))
          (list (sql.sys:db-name-from-spec type conn)
                (sql.sys:db-user-name-from-spec type conn)))
        :reported-form :connection/3)
)
