;;;-*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface - Tests for functional interface

(in-package :sql.test)

;(enable-sql-reader-syntax)
(locally-enable-sql-reader-syntax)

(defmethod test-functional (env-type database)
  (test-functional-ddl env-type database)
  (test-functional-dml env-type database))

(defun test-functional-ddl (env-type database)
 ;;; Data Definition Language (DDL)
  (declare (ignore env-type))
  ;; List current tables 
  (test t
    (subsetp `("type_table"
               ,@(when (db-has-bigint? '("type_bigint")))
                   ;"addr" "big" "company" "ea_join" "employee"
               )
             (list-tables :owner *test-user-name*)
             :test #'string-equal)
                 ;(sort  (list-tables :owner *test-user-name*)
                 ;      #'string< :key #'string-downcase)
    :reported-form :fddl/table/1)

  (drop-table [foo] :if-does-not-exist :ignore)

  ;; Create a table, test for its existence, drop it and test again 
  ;; longchar works with Access, SQLite3, but not with Oracle
  (test '(t nil)
        (progn #1=(create-table [foo]
                                '(([id] integer)
                                  ([height] float)
                                  ([name] (string 24))
                                  ([comments] :clob)))
          (list
            (table-exists-p [foo] :owner *test-user-name*)
            (progn
              (drop-table [foo] :if-does-not-exist :ignore)
              (table-exists-p [foo] :owner *test-user-name*))))
        :reported-form :fddl/table/2)

  (test '(nil t nil)
        (list (table-exists-p [foo] :owner *test-user-name*)
              (progn
                (execute-command "create table foo (bar integer)")
                (table-exists-p [foo] :owner *test-user-name*))
              (progn
                (execute-command "drop table foo")
                (table-exists-p [foo] :owner *test-user-name*)))
        :reported-form :fddl/execute-command/1)


  ;; Create a table, list its attributes and drop it 
  (test '("COMMENTS" "HEIGHT" "ID" "NAME")
    (prog2 #1#
        (sort (list-attributes [foo]) #'string<)
      (drop-table [foo]))	;:if-does-not-exist :ignore
    :test #'equalp
    :reported-form :fddl/table/3)

  ;;; In create/drop-table and relatives, a symbol or a string as a value of the table
  ;;; argument is treated specially.
  (drop-table "MyMixedCase" :if-does-not-exist :ignore)
  (test '(nil t nil)
        (list (table-exists-p "MyMixedCase")
              (progn
                (create-table "MyMixedCase" '(([a] integer)))
                (table-exists-p "MyMixedCase"))
              (progn
                (drop-table "MyMixedCase")
                (table-exists-p "MyMixedCase")))
        :reported-form :fddl/table/4)

  (test '((5) (6))
        (prog1 (progn
                 (create-table [|MyMixedCase|] '(([a] integer)))
                 (insert-records :into '|MyMixedCase| :values '(5))
                 (insert-records :into "MyMixedCase" :values '(6))
                 (select [a] :from [|MyMixedCase|] :order-by '(a :asc)))
          (drop-table '|MyMixedCase|))
        :reported-form :fddl/table/5)

  (test '(nil t nil)
    (list (table-exists-p [foo])
          (progn
            (typecase database ;env-type
              ((or odbc::mssql-database sql.sys:oracle-mixin)
               (create-table [foo] 
                             '(([bar] integer :not-null :primary-key)
                               ([baz] string :not-null :unique))))
              ;; ORA-02259: duplicate UNIQUE/PRIMARY KEY specifications 
              (t (create-table [foo] 
                               '(([bar] integer :not-null :unique :primary-key)
                                 ([baz] string :not-null :unique)))))
            (table-exists-p [foo]))
          (progn (drop-table [foo])
            (table-exists-p [foo])))
    ;; PosgreSQL issues the warning: NOTICE:
    ;;  CREATE TABLE / UNIQUE will create implicit index "foo_baz_key" for table "foo"
    :errorset (if (typep database 'sql.sys:postgresql-mixin)
                  :warning)
    :reported-form :fddl/table/6)

  (test '(nil t nil)
        (list (table-exists-p [foo])
              (let ();(*backend-warning-behavior*
                ;(if (member *test-database-type* '(:postgresql :postgresql-socket))
                ;  :ignore :warn)))
                (create-table [foo]
                              '(([bar] integer :not-null) 
                                ([baz] string :not-null))
                              :constraints '((:unique ([bar] [baz])) 
                                             (:primary-key ([bar]))))
                (table-exists-p [foo]))
              (progn (drop-table [foo])
                (table-exists-p [foo])))
    ;; PosgreSQL issues the warning: NOTICE:
    ;;  CREATE TABLE / UNIQUE will create implicit index "foo_bar_key" for table "foo"
    :errorset (if (typep database 'sql.sys:postgresql-mixin)
                  :warning)
    :reported-form :fddl/table/7)    


  (test '("BIRTH_DATE" "BIRTH_TS" "ECOMPANYID" "EMAIL" "EMPLID" "FIRST_NAME" "GROUPID"
          "HEIGHT" "LAST_NAME" "MANAGERID" "MARRIED")
    (sort (list-attributes [employee] :owner *test-user-name*) #'string-lessp)
    :test #'equalp
    :reported-form :fddl/attributes/1)
  
  (test '("BIRTH_DATE" "BIRTH_TS" "ECOMPANYID" "EMAIL" "EMPLID" "FIRST_NAME" "GROUPID"
          "HEIGHT" "LAST_NAME" "MANAGERID" "MARRIED")
    (sort (mapcar #'fourth
                  (odbc:db-describe-columns database [employee] :schema *test-user-name*))
          #'string-lessp)
    :test #'equalp
    :reported-form :fddl/attributes/2)

  ;; Attribute types are vendor specific so need to test a range
  (test-true (member (first-or-self (attribute-type [emplid] [employee]))
                     '(:int :integer :int4 :number :decimal))
    :reported-form :fddl/attributes/integer)

  (test '(t 30 1)
        (multiple-value-bind (type size scale nullable)
            (attribute-type [first-name] [employee])
          (declare (ignore scale))
          (list (and (member (first-or-self type) '(:varchar :varchar2)) t)
                size nullable))
        :reported-form :fddl/attributes/varchar)

  (test-true (member (attribute-type [height] [employee]) 
                     '(:double			; Access
                       :double\ precision	; Oracle
                       :float :float8 :number))
             :reported-form :fddl/attributes/double)

  (test-true (member (first-or-self (attribute-type [birth-date] [employee]))
                     '(:datetime :timestamp :date))
             :reported-form :fddl/attributes/date)

  (test-true (member (first-or-self (attribute-type [birth-ts] [employee]))
                     ;'(:bigint :int8 :char)	; - CLSQL speciality
                     '(:datetime :timestamp
                       :timestamp\ without\ time\ zone)) ; SQLite
             :reported-form :fddl/attributes/timestamp)


  (test-no-error (drop-view [lenins-group] :if-does-not-exist :ignore))

  ;; create a view, test for existence, drop it and test again
  (test '(t nil)
        (progn #2=(create-view [lenins-group]
                               :as [select [first-name] [last-name] [email]
                                           :from [employee]
                                           :where [= [managerid] 1]])
	   (list (view-exists-p [lenins-group] :owner *test-user-name*)
                 (progn (drop-view [lenins-group])
                   (view-exists-p [lenins-group] :owner *test-user-name*))))
        :reported-form :fddl/view/1)
  
  ;; create a view, list its attributes and drop it 
  ;(when (db-has-views? database)
  (test '("EMAIL" "FIRST_NAME" "LAST_NAME")
    (prog2 #2#
        (sort (list-attributes [lenins-group]) #'string-lessp)
      (drop-view [lenins-group]))
    :test #'equalp
    :reported-form :fddl/view/2)
  
  ;; create a view, select stuff from it and drop it 
  (test '(nil ("Josef" "Stalin" "stalin@soviet.org"))
        (prog2 #2#
            (list ;; Shouldn't exist 
		  (select [first-name] [last-name] [email]
                          :from [lenins-group]
                          :where [= [last-name] "Lenin"])
                  ;; Should exist 
                  (first (select [first-name] [last-name] [email]
                                 :from [lenins-group]
                                 :where [= [last-name] "Stalin"])))
          (drop-view [lenins-group]))
        :reported-form :fddl/view/3)
  
  (test '(nil ("Josef" "Stalin" "stalin@soviet.org"))
        (prog2 (create-view [lenins-group]
                            :columns '([forename] [surname] [email])
                            :as [select [first-name] [last-name] [email]
                                        :from [employee]
                                        :where [= [managerid] 1]])
            (list ;; Shouldn't exist 
                  (select [forename] [surname] [email]
                          :from [lenins-group]
                          :where [= [surname] "Lenin"])
                  ;; Should exist 
                  (first (select [forename] [surname] [email]
                                 :from [lenins-group]
                                 :where [= [surname] "Stalin"])))
          (drop-view [lenins-group]))
        :reported-form :fddl/view/4)

  ;; create an index, test for existence, drop it and test again 
  (test '(t nil)
        (progn (create-index [bar] :on [employee]
                             :attributes '([first-name] [last-name] [email])
                             :unique t)
           (list (index-exists-p [bar] :owner *test-user-name*)
                 (progn (drop-index [bar] :on [employee])
                   (index-exists-p [bar] :owner *test-user-name*))))
        :reported-form :fddl/index/1)

  ;; create indexes with names as strings, symbols and in square brackets 
  (test '(t t t)
        (do ((names '("foo" foo [foo]) (rest names))
             (acc '())
             name)
            ((null names) (nreverse acc))
          (create-index (setq name (first names)) :on [employee]
                        :attributes '([last-name]))
          (push (index-exists-p name :owner *test-user-name*) acc)
          (drop-index name :on [employee] :if-does-not-exist :ignore))
        :reported-form :fddl/index/2)

  ;; list-indexes with keyword :ON 
  (test-no-error (drop-table [i3test] :if-does-not-exist :ignore))

  (test '(t t t ("BAR" "FOO") t)
        (progn
          (create-table [i3test] '(([a] (string 10))
                                   ([b] integer)))
          (create-index [foo] :on [i3test] :attributes '([b]) :unique nil)
          (create-index [bar] :on [i3test] :attributes '([a]) :unique t)
          (list (table-exists-p [i3test])
                (index-exists-p [foo])
                (index-exists-p [bar])
                (sort (list-indexes :on [i3test] :owner *test-user-name*)
                      #'string-lessp)
                (progn
                  (drop-index [bar] :on [i3test])
                  (drop-index [foo] :on [i3test])
                  (drop-table [i3test])
                  t)))
        :test #'equalp
        :reported-form :fddl/index/3)

  ;; Expression-based index
  (typecase database
    ((or sql.sys:oracle-mixin sql.sys:postgresql-mixin)
     (test '(t nil)
       (progn (create-index [idx-employee-name] :on [employee]
                            :columns [upper [|| [last-name] #\Space [first-name]]]
                            :unique t)
         (list (index-exists-p [idx-employee-name] :owner *test-user-name*)
               (progn (drop-index [idx-employee-name] :on [employee])
                 (index-exists-p [idx-employee-name] :owner *test-user-name*))))
       ;:known-failure (typep database 'odbc:postgresql-odbc-database)
       ;:fail-info "With PostgreSQL driver before 08.02.0105, SQLStatistics does not mention indices based on expression."
       :reported-form :fddl/index/expression))
    (t
     (push :fddl/index/expression *test-skips*)))

#|
  ;; create an sequence, test for existence, drop it and test again 
  (test '(t nil)
        (progn (create-sequence [foo])
          (values (sequence-exists-p [foo] :owner *test-database-user*)
                  (progn (drop-sequence [foo] :if-does-not-exist :ignore)
                    (sequence-exists-p [foo] :owner *test-database-user*))))
        :reported-form :fddl/sequence/1)

  ;; create and increment a sequence
  (test t
        (let ((val1 nil))
          (create-sequence [foo])
          (setf val1 (sequence-next [foo]))
          (prog1
              (< val1 (sequence-next [foo]))
            (drop-sequence [foo] :if-does-not-exist :ignore)))
        :reported-form :fddl/sequence/2)

  ;; explicitly set the value of a sequence
  (test 6
        (progn
          (create-sequence [foo])
          (set-sequence-position [foo] 5)
          (prog1 (sequence-next [foo])
            (drop-sequence [foo] :if-does-not-exist :ignore)))
        :reported-form :fddl/sequence/3)
 |#
  (if (db-has-bigint? database)
      (test '(555 t)
            (let ((rows (select [*] :from [big])))
              (list (length rows)
                    (do ((i 0 (1+ i))
                         (max (expt 2 60))
                         (rest rows (rest rest)))
                        ((= i (length rows)) t)
                      (let ((index (1+ i))
                            (int (first (car rest)))
                            (bigint (second (car rest))))
                        (when ;(and (or (eq env-type :oracle)
                              ;         (and (eq env-type :odbc)
                              ;          (eq *test-database-underlying-type* :postgresql)))
                              (stringp bigint)
                          (setf bigint (parse-integer bigint)))
                        (unless (and (eql int index)
                                     (eql bigint (truncate max index)))
                          (return nil))))))
            :reported-form :bigint/2)
      (push :bigint/2 *test-skips*))

  (test-true
    (and 
     ;; user tables are an improper subset of all tables 
     (= (length (intersection (list-tables :owner nil) (list-tables :owner :all)
                              :test #'string=))
        (length (list-tables :owner nil)))
     (>= (length (list-tables :owner :all))
         (length (list-tables :owner nil)))) 
    :reported-form :fddl/owner/1) 
#|
  (deftest :fddl/cache-table-queries/1 
 (list 
  (gethash "EMPLOYEE" (clsql-sys::attribute-cache clsql:*default-database*))
  (progn 
    (clsql:cache-table-queries "EMPLOYEE" :action t)
    (gethash "EMPLOYEE" (clsql-sys::attribute-cache clsql:*default-database*)))
  (progn 
    (clsql:list-attribute-types "EMPLOYEE")
    (not 
     (null 
      (cadr 
       (gethash "EMPLOYEE" 
                (clsql-sys::attribute-cache clsql:*default-database*))))))
  (progn 
    (clsql:cache-table-queries "EMPLOYEE" :action :flush)
    (gethash "EMPLOYEE" (clsql-sys::attribute-cache clsql:*default-database*))))
 (nil (t nil) t (t nil)))
 |#
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun test-functional-dml (env-type database)
 ;;; Data Manipualton/Query Language
  (declare (ignore env-type))
  ;; inserts a record using all values only and then deletes it 
  ;; YstokSQL specific: CREATE TABLE collect columns from superclasses to subclasses
  ;; (this is different from CLSQL).
  ;; ADVICE: Be sure to specify explicit column names in insert-records!
  (test #3='((("Yuri" "Gagarin" "gagarin@soviet.org")) nil)
        (let ((now (get-universal-time)))
          (insert-records :into [employee] 
                          :values `(1.85 :true ; t
                                    #+local-time ,(lt:local-time :universal now)
                                    #-local-time ,(timestring now)
                                    #+local-time ,(lt:local-time :universal now)
                                    #-local-time ,(timestring now)
                                    11 1 "Yuri" "Gagarin" "gagarin@soviet.org" 1 1))
          (list (select [first-name] [last-name] [email]
                        :from [employee] :where [= [emplid] 11])
                (progn (delete-records :from [employee] :where [= [emplid] 11])
                  (select [*] :from [employee] :where [= [emplid] 11]))))
        :reported-form :fdml/insert/1)

  ;; inserts a record using attributes and values and then deletes it
  (test #3#
        (progn (insert-records :into [employee] 
                               :attributes '(emplid groupid first_name last_name
                                             email ecompanyid managerid)
                               :values '(11 1 "Yuri" "Gagarin" "gagarin@soviet.org" 1 1))
          (list (select [first-name] [last-name] [email] :from [employee]
                        :where [= [emplid] 11])
                (progn (delete-records :from [employee] :where [= [emplid] 11])
                  (select [*] :from [employee] :where [= [emplid] 11]))))
        :reported-form :fdml/insert/2)

  ;; inserts a record using av-pairs and then deletes it
  (test #3#
        (progn (insert-records :into [employee] 
                               :av-pairs '((emplid 11) (groupid 1)
                                           (first_name "Yuri")
                                           (last_name "Gagarin")
                                           (email "gagarin@soviet.org")
                                           (ecompanyid 1) (managerid 1)))
          (list (select [first-name] [last-name] [email] :from [employee]
                        :where [= [emplid] 11])
                (progn (delete-records :from [employee] :where [= [emplid] 11])
                  (select [first-name] [last-name] [email] :from [employee]
                          :where [= [emplid] 11]))))
        :reported-form :fdml/insert/3)

  ;; YstokSQL specific: inserts a record using cv-alist and then deletes it
  (test #3#
        (progn (insert-records :into [employee] 
                               :cv-alist '((emplid . 11) (groupid . 1)
                                          (first_name . "Yuri")
                                          (last_name . "Gagarin")
                                          (email . "gagarin@soviet.org")
                                          (ecompanyid . 1) (managerid . 1)))
          (list (select [first-name] [last-name] [email] :from [employee]
                        :where [= [emplid] 11])
                (progn (delete-records :from [employee] :where [= [emplid] 11])
                  (select [first-name] [last-name] [email] :from [employee]
                          :where [= [emplid] 11]))))
        :reported-form :fdml/insert/cv-alist)

  (test-no-error (drop-table [employee2] :if-does-not-exist :ignore))

  ;; inserts a records using a query from another table 
  (test t
        (progn
          (create-table [employee2] '(([forename] string)
                                      ([surname] string)
                                      ([email] string)))
          (insert-records :into [employee2] 
                          :query [select [first-name] [last-name] [email] 
                                         :from [employee]]
                          :attributes '(forename surname email))
          (equal (select [*] :from [employee2])
                 (select [first-name] [last-name] [email] :from [employee])))
            ;(drop-table [employee2])))
        :reported-form :fdml/insert/4)


  ;; updates a record using attributes and values and then deletes it
  (test #4='((("Yuri" "Gagarin" "gagarin@soviet.org"))
             (("Vladimir" "Lenin" "lenin@soviet.org")))
        (progn
          (update-records [employee] 
                          :attributes '([employee first_name] last_name email)
                          :values '("Yuri" "Gagarin" "gagarin@soviet.org")
                          :where [= [emplid] 1])
          (list (select [first-name] [last-name] [email] :from [employee]
                        :where [= [emplid] 1])
                (progn
                  (update-records [employee] 
                                  :av-pairs '((first_name "Vladimir")
                                              (last_name "Lenin")
                                              (email "lenin@soviet.org"))
                                  :where [= [emplid] 1])
                  (select [first-name] [last-name] [email] :from [employee]
                          :where [= [emplid] 1]))))
        :reported-form :fdml/update/1)

  ;; updates a record using av-pairs and then deletes it
  (test #4#
        (progn
          (update-records [employee] 
                           :av-pairs '((first_name "Yuri")
                                       (last_name "Gagarin")
                                       (email "gagarin@soviet.org"))
                           :where [= [emplid] 1])
          (list (select [first-name] [last-name] [email] :from [employee]
                        :where [= [emplid] 1])
                (progn
                  (update-records [employee]
                                  :av-pairs' ((first_name "Vladimir")
                                              (last_name "Lenin")
                                              (email "lenin@soviet.org"))
                                  :where [= [emplid] 1])
                  (select [first-name] [last-name] [email]
                          :from [employee] :where [= [emplid] 1]))))
        :reported-form :fdml/update/2)

  ;; Computed values are not always classified as numeric by psqlodbc
  (test 10
        (let ((count (caar (query "SELECT COUNT(*) FROM EMPLOYEE WHERE (EMAIL LIKE '%org')"))))
          (if (stringp count)
              (nth-value 0 (parse-integer count))
              (nth-value 0 (truncate count))))
        :reported-form :fdml/query/count)

  (test '((("Leonid" "Brezhnev") ("Nikita" "Kruschev") ("Vladimir" "Lenin")
           ("Josef" "Stalin") ("Leon" "Trotsky"))
          ("FIRST_NAME" "LAST_NAME"))
        (multiple-value-bind (rows field-names)
            (query "SELECT FIRST_NAME,LAST_NAME FROM EMPLOYEE WHERE (EMPLID <= 5) ORDER BY LAST_NAME")
          (values rows (mapcar 'string-upcase field-names)))
        :multiple-values t
        :reported-form :fdml/query/2)

  (test 6
        (caar (query "SELECT EMPLID FROM EMPLOYEE WHERE LAST_NAME = 'Andropov'"))
        :reported-form :fdml/query/3)
  
  (test t
        (typep (caar (query "SELECT HEIGHT FROM EMPLOYEE WHERE LAST_NAME = 'Andropov'"))
               'float)
        :reported-form :fdml/query/4)
  
  (test '(("Josef" 2) ("Leon" 3) ("Nikita" 4) ("Leonid" 5) ("Yuri" 6)
          ("Konstantin" 7) ("Mikhail" 8) ("Boris" 9) ("Vladimir" 11))
        (query (sql [select [first-name] [sum [emplid]] :from [employee]
                    :group-by [first-name] :order-by [sum [emplid]]]))
        :test #'equalp				; as SUM returns double float in Access
        :multiple-values :ignore
        :reported-form :fdml/query/5)

  ;; Oracle returns double floats for columns resulted from UNION, INTERSECT, and MINUS
  (test '(t (1 2 3 4 5 6 7 8 9 10))
        (let ((res (query [select (if (typep database 'sql.sys:oracle-mixin)
                                      [cast [emplid] :as :integer]
                                      [emplid])             
                                  :from [employee]
                                  :set-operation [union [select [companyid]
                                                                :from [company]]]]
                             :flatp t)))
          ;(values (every #'stringp res)
          ;        (mapcar (lambda (f) (truncate (read-from-string f))) res))))
          (list (every #'integerp res) res))
        :reported-form :fdml/query/union)

  (typecase database
    (sql.sys:access-mixin
     (push :fdml/query/intersect *test-skips*)
     (push :fdml/query/except *test-skips*))
    (t
     (test '(t 1)
        (let ((res (first (query [select [emplid :type :integer] :from [employee]
                                   :set-operation [intersect [select [companyid]
                                                                     :from [company]]]]
                                 :flatp t))))
          ;(values (stringp res) (nth-value 0 (truncate (read-from-string res))))
          (list (integerp res) res))
        :reported-form :fdml/query/intersect)

     (test '(t (3 4 5 6 7 8 9 10))
        (let ((res (query [select [emplid] :from [employee]
                                  :set-operation [except [select [companyid] 
                                                                 :from [company]]]]
                          :result-types (if (typep database 'sql.sys:oracle-mixin)
                                            '(:integer)
                                            ())
                          :flatp t)))
          ;(values (every #'stringp res)
          ;        (mapcar (lambda (f) (truncate (read-from-string f))) res)))
          (list (every #'integerp res) res))
        :reported-form :fdml/query/except) ))

  ;; compare min, max and average heights in inches (they're quite short these guys!) 
  (test t
        (let ((max (select [sql-function 'round [/ [* [max [height]] 100] 2.54]]
                           :from [employee]
                           :flatp t))
              (min (select [sql-function 'round [/ [* [min [height]] 100] 2.54]]
                           :from [employee]
                           :flatp t))
              (avg (select [sql-function 'round [avg [/ [* [height] 100] 2.54]]]
                           :from [employee]
                           :flatp t)))
          (apply #'< ;(mapcar #'(lambda (s) (parse-integer s :junk-allowed t))
                 (append min avg max)))
        :reported-form :fdml/select/min-avg-max)

  (test '("Boris" "Josef" "Konstantin" "Leon" "Leonid" "Mikhail" "Nikita" "Vladimir" "Yuri")
        (select [first-name] :from [employee] :order-by [first-name] :distinct t
                :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/distinct)

  (test '(("Boris" 1) ("Josef" 1) ("Konstantin" 1) ("Leon" 1) ("Leonid" 1)
          ("Mikhail" 1) ("Nikita" 1) ("Vladimir" 2) ("Yuri" 1))
        (select [first-name] [count [*]] :from [employee]
                :group-by [first-name]
                :order-by [first-name])
        :test #'equalp				; COUNT returns double floats in Oracle
        :multiple-values :ignore
        :reported-form :fdml/select/group-by)

  (test '("Andropov" "Brezhnev" "Chernenko" "Gorbachev" "Kruschev" "Lenin" "Putin"
                     "Stalin" "Trotsky" "Yeltsin")
        (select [last-name] :from [employee]
                :where [like [email] "%org"]
                :order-by [last-name]
                :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/like)

  (test '("lenin@soviet.org")
        (select [email] :from [employee]
                :where [in [employee emplid] [select [managerid] :from [employee]]]
                :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/in-select)

  ;; Testing predicate is equalp due to Access returning floats.
  ;(if (clsql-sys:db-type-has-fancy-math? *test-database-underlying-type*)
  (test '(2 2 2 2 2 2 2 2 2 2)
        (select [ceiling [height]] :from [employee] :flatp t)
        :test #'equalp
        :multiple-values :ignore
        :known-failure (typep database 'sql.sys:sqlite3-mixin)
        :reported-form :fdml/select/ceiling)

  (test '(1 1 1 1 1 1 1 1 1 1)
        (select [floor [height]] :from [employee] :flatp t)
        :test #'equalp
        :multiple-values :ignore
        :known-failure (typep database 'sql.sys:sqlite3-mixin)
        :reported-form :fdml/select/floor)

  (test '(1 1 1 1 1 1 1 1 1 1)
        (select [truncate [height]] :from [employee] :flatp t
                :result-types (make-list 10 :initial-element :integer))	; compared by equal
        :multiple-values :ignore
        :known-failure (typep database 'sql.sys:sqlite3-mixin)
        :reported-form :fdml/select/truncate)

  ;; ROUND
  (test-true (let ((result (first (select [min [round [height]]] [max [round [height]]]
                                          :from [employee]))))
               (<= 1 (first result) (second result) 2))
        :reported-form :fdml/select/round)

  (test 2
    (progn (sql:update-records [employee] :columns [height] :values 2.5
                               :where [= [emplid] 7])
      (first (sql:select [round [height]]
                         :from [employee] :where [= [emplid] 7] :flatp t)))
    ;(first (sql:select (round-sql [height]) :from [employee] :where [= [emplid] 7]))
    :test #'=
    :reported-form :fdml/select/round/2.5)

  (test 2.14
    (progn (sql:update-records [employee] :columns [height] :values 2.145
                               :where [= [emplid] 7])
      (first (sql:select [round [height] 2]
                         :from [employee] :where [= [emplid] 7] :flatp t)))
    :test #'=
    :reported-form :fdml/select/round/2.145)

  #+ys-product
  (test 3.0
    (progn (sql:update-records [employee] :columns [height] :values 2.5
                               :where [= [emplid] 7])
      (first (sql:select (math-round-sql [height])
                         :from [employee] :where [= [emplid] 7] :flatp t)))
    :test #'=
    :known-failure (typep database 'sql.sys:sqlite3-mixin)
    :reported-form :fdml/select/round-sql/math-2.5)

  #+ys-product
  (test 2.15
    (progn (sql:update-records [employee] :columns [height] :values 2.145
                               :where [= [emplid] 7])
      (first (sql:select (math-round-sql [height] 2)
                         :from [employee] :where [= [emplid] 7] :flatp t)))
    :test #'=
    :known-failure (typep database 'sql.sys:sqlite3-mixin)
    :reported-form :fdml/select/round/math-2.145)


  (test '(t 10)
        (let ((result (first (select [max [emplid]] :from [employee] :flatp t))))
          (list (if (typep database 'sql.sys:oracle-mixin)
                    (floatp result)
                    (integerp result))
                result))
        :test #'equalp
        :reported-form :fdml/select/max)

  (test '(t 1)
        (let ((result (first (select [min [emplid]] :from [employee] :flatp t))))
          (list (if (typep database 'sql.sys:oracle-mixin)
                    (floatp result)
                    (integerp result))
                result))
        :test #'equalp
        :reported-form :fdml/select/min)

  (test 5.5						; 11/2 in PostgreSQL
        (first (select [avg [emplid]] :from [employee] :flatp t))
        :test #'=
        :reported-form :fdml/select/avg)

  (test '("Andropov" "Brezhnev" "Chernenko" "Gorbachev" "Kruschev" "Putin" "Stalin"
          "Trotsky")
        (select [last-name] :from [employee]
                :where [not [in [emplid] [select [presidentid] :from [company]]]]
                :order-by [last-name]
                :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/10)

  (test '("Lenin" "Stalin" "Trotsky")
        (select [last-name] :from [employee]
                :where (typecase database
                         (sql.sys:oracle-mixin [= [married] :true])
                         (t [married]))
                :order-by [emplid] :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/11)

  (test '(("Lenin"))
        (let ((v 1))
          (select [last-name] :from [employee] :where [= [emplid] v]))
        :multiple-values :ignore
        :reported-form :fdml/select/12)

  ;; select is not obliged to return column name list - use query!
  (test '(((1 "Lenin"))
          ("emplid" "last_name"))
        (multiple-value-bind (results field-names)
            (query [select [emplid] [last-name] :from [employee] :where [= [emplid] 1]])
          (list results (mapcar #'string-downcase field-names)))
        :reported-form :fdml/select/13)

  (test t
        (floatp (first (select [height] :from [employee] :where [= [emplid] 1] :flatp t)))
        :reported-form :fdml/select/14)

  (test '(((1 10 "Park Place" "Leningrad" 123))
          ("addressid" "street_number" "street_name" "city_field" "zip"))
        (multiple-value-bind (rows field-names)
            (query [select [addressid] [street-number] [street-name] [|city-field|] [zip] 
                           :from [addr]
                           :where [= 1 [addressid]]])
          (list rows (mapcar #'string-downcase field-names)))
        :reported-form :fdml/select/15)

  (test '((1))
        (select [emplid] :from [employee] :where [= 1 [emplid]])
        :multiple-values :ignore
        :reported-form :fdml/select/16)

  (test '((1 "Lenin"))
        (select [emplid] [last-name] :from [employee] :where [= 1 [emplid]])
        :multiple-values :ignore
        :reported-form :fdml/select/17)

  (test '(("1" "Lenin"))
        (select [emplid :type string] [last-name] :from [employee] :where [= 1 [emplid]])
        :multiple-values :ignore
        :reported-form :fdml/select/18)

  (test '("5" "6" "7" "8" "9" "10")
        (select [emplid] :from [employee] :order-by [emplid] 
                :where [between [* [emplid] 10] [* 5 10] [* 10 10]]
                :result-types '(string)
                :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/between)

  (test '(1 2 3 4)
        (select [emplid] :from [employee] :order-by [emplid] 
                :where [not [between [* [emplid] 10] [* 5 10] [* 10 10]]]
                :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/not-between)

  (test '("Vlad" "Jose" "Leon" "Niki" "Leon" "Yuri" "Kons" "Mikh" "Bori" "Vlad") 
        (select [substring [first-name] 1 4] :from [employee] :order-by [emplid]
                :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/substring)

  (test '("Vladimir Lenin" "Josef Stalin" "Leon Trotsky" "Nikita Kruschev"
          "Leonid Brezhnev" "Yuri Andropov" "Konstantin Chernenko" "Mikhail Gorbachev"
          "Boris Yeltsin" "Vladimir Putin")
        ;(case *test-database-underlying-type* (:mssql
        (select [concat [first-name] " " [last-name]] :from [employee] :order-by [emplid]
                :flatp t)
        ;'(t (clsql:select [|| [first-name] " " [last-name]] :from [employee]
        ;              :flatp t :order-by [emplid] :field-names nil)))
        :multiple-values :ignore
        :reported-form :fdml/select/concat)

   (test '(1 2 3 4)
         (select [emplid] :from [employee] :where [in [emplid] '(1 2 3 4)]
                 :order-by [emplid]
                 :flatp t)
        :multiple-values :ignore
         :reported-form :fdml/select/in)

  #|(test :fdml/select/25
  (clsql:select [first-name]
               :from (clsql-sys:convert-to-db-default-case "employee" *default-database*)
  :flatp t :distinct t
  :field-names nil 
  :result-types nil 
  :order-by [first-name])
  ("Boris" "Josef" "Konstantin" "Leon" "Leonid" "Mikhail" "Nikita" "Vladimir" "Yuri"))|#

  (test '(("Vladimir" "Lenin") ("Vladimir" "Putin"))
        (select ["table" first-name] ["table" last-name] 
                :from '([employee "table"] [employee "join"])
                :where [and [= ["table" first-name] ["join" first-name]]
                            [not [= ["table" emplid] ["join" emplid]]]]
                :order-by '(["table" last-name]))
        :multiple-values :ignore
        :reported-form :fdml/select/alias)

  (test '(10 1 1 1 1 1 1 1 1 1) 
        ;(mapcar (lambda (f) (truncate (read-from-string f)))
        (select [coalesce [managerid] 10] :from [employee] :order-by [emplid]
                :flatp t)
        :test #'equalp
        :multiple-values :ignore
        :reported-form :fdml/select/coalesce)
  
  (test '(10 10) 
        ;(mapcar (lambda (f) (truncate (read-from-string (car f))))
        (loop for column in (list [*] [emplid]) collect
              (truncate (first (select [count column] :from [employee] :flatp t))))
        :reported-form :fdml/select/28)

  (test '(("Boris" "Yeltsin") ("Josef" "Stalin") ("Konstantin" "Chernenko")
          ("Leon" "Trotsky") ("Leonid" "Brezhnev") ("Mikhail" "Gorbachev")
          ("Nikita" "Kruschev") ("Vladimir" "Putin") ("Vladimir" "Lenin")
          ("Yuri" "Andropov")) 
        (select [first-name] [last-name] :from [employee] 
                :order-by '([first-name :asc] [last-name :desc]))
        :multiple-values :ignore
        :reported-form :fdml/select/asc-desc)


  (test '("Andropov" "Boris" "Brezhnev" "Chernenko" "Gorbachev" "Josef" "Konstantin"
          "Kruschev" "Lenin" "Leon" "Leonid" "Mikhail" "Nikita" "Putin" "Stalin"
          "Trotsky" "Vladimir" "Yeltsin" "Yuri")
        (select [last-name] :from [employee]                   
              :set-operation [union [select [first-name] :from [employee]
                                            :order-by [last-name]]]
              :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/31)

  (test '(1 2)
        (select [emplid] :from [employee]
                :where [= [emplid] [any [select [companyid] :from [company]]]]
                :order-by [emplid]
                :flatp t)
        :multiple-values :ignore
        :known-failure (typep database 'sql.sys:sqlite3-mixin)
	:fail-info "ANY is not supported by SQLite."
        :reported-form :fdml/select/any)

  (test '("Andropov" "Brezhnev" "Chernenko" "Gorbachev" "Kruschev" "Putin" "Stalin"
          "Trotsky" "Yeltsin")
        (select [last-name] :from [employee] 
              :where [> [emplid] [all [select [groupid] :from [employee]]]]
              :order-by [last-name] 
              :flatp t)
        :multiple-values :ignore
        :known-failure (typep database 'sql.sys:sqlite3-mixin)
	:fail-info "ALL is not supported by SQLite."
        :reported-form :fdml/select/all)

  (test '("Lenin" "Yeltsin")
        (select [last-name] :from [employee]
                :where [exists [select [companyid] :from [company]
                                       :where [= [company presidentid] [employee emplid]]]]
                :order-by [last-name]
                :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/exists)


  (test '("Lenin" "Stalin" "Trotsky" "Kruschev")
        (loop for x from 1 below 5
              collect (first (select [last-name] :from [employee] 
                                     :where [= [emplid] x]
                                     :flatp t)))
        :reported-form :fdml/select/34)

  ;; test escaping of single quotes 
  (test '("What's up doc?" "What's up doc?" "What's up doc?" "What's up doc?"
          "What's up doc?" "What's up doc?" "What's up doc?" "What's up doc?"
          "What's up doc?" "What's up doc?") 
        (select "What's up doc?" :from [employee] :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/35)

  ;; test proper treatment of backslash (depending on backend) 
  (test '("foo\\bar\\baz" "foo\\bar\\baz" "foo\\bar\\baz" "foo\\bar\\baz" 
          "foo\\bar\\baz" "foo\\bar\\baz" "foo\\bar\\baz" "foo\\bar\\baz" 
          "foo\\bar\\baz" "foo\\bar\\baz")
        (select "foo\\bar\\baz" :from [employee] :flatp t)
        :multiple-values :ignore
        :reported-form :fdml/select/36)

  ;; Joins
  (test '(("Andropov" "Soviet Inc.") ("Brezhnev" "Soviet Inc.") ("Chernenko" "Soviet Inc.")
          ("Kruschev" "Soviet Inc.") ("Lenin" "Soviet Inc.")
          ("Putin" "Russian Inc.") ("Stalin" "Soviet Inc.") ("Trotsky" "Soviet Inc.")
          ("Yeltsin" "Russian Inc."))
    (select [employee last-name] [company name]
            :from [inner-join [employee] [company]
                              :on [= [employee ecompanyid] [company companyid]]]
            :order-by [last-name])
    :multiple-values :ignore
    :reported-form :fdml/join/inner)

  (test '(("Andropov" "Soviet Inc.") ("Brezhnev" "Soviet Inc.") ("Chernenko" "Soviet Inc.")
          ("Gorbachev" nil) ("Kruschev" "Soviet Inc.") ("Lenin" "Soviet Inc.")
          ("Putin" "Russian Inc.") ("Stalin" "Soviet Inc.") ("Trotsky" "Soviet Inc.")
          ("Yeltsin" "Russian Inc."))
    (select [employee last-name] [company name]
            :from [left-join [employee] [company]
                              :on [= [employee ecompanyid] [company companyid]]]
            :order-by [last-name])
    :multiple-values :ignore
    :reported-form :fdml/join/left)

  (test '("Yeltsin" "Trotsky" "Stalin" "Putin" "Lenin" "Kruschev" "Gorbachev"
          "Chernenko" "Brezhnev" "Andropov")
        (let ((result '()))
          (do-query ((name) [select [last-name] :from [employee] :order-by [last-name]])
            (push name result))
          result)
        :reported-form :fdml/do-query/1)

  ;; map-query: CommonSQL (and CLSQL) non-conformance when flatp is nil.
  ;; In this case, the function argument is invoked on each tuple via apply!
  (test '("Andropov" "Brezhnev" "Chernenko" "Gorbachev" "Kruschev" "Lenin" "Putin"
          "Stalin" "Trotsky" "Yeltsin")
        (map-query 'list #'identity
                    [select [last-name] :from [employee] :order-by [last-name] :flatp t])
        :reported-form :fdml/map-query/1)

  (test #("Andropov" "Brezhnev" "Chernenko" "Gorbachev" "Kruschev" "Lenin" "Putin"
          "Stalin" "Trotsky" "Yeltsin")
        (map-query 'vector #'identity
                    [select [last-name] :from [employee] :order-by [last-name] :flatp t])
        :test #'equalp
        :reported-form :fdml/map-query/2)

  (test '(("Andropov") ("Brezhnev") ("Chernenko") ("Gorbachev") ("Kruschev") ("Lenin")
          ("Putin") ("Stalin") ("Trotsky") ("Yeltsin")) 
        (map-query 'list #'list		; #'identity
                   [select [last-name] :from [employee] :order-by [last-name]])
        :reported-form :fdml/map-query/3)

  (test '(("Yuri" "Andropov") ("Leonid" "Brezhnev") ("Konstantin" "Chernenko")
          ("Mikhail" "Gorbachev") ("Nikita" "Kruschev") ("Vladimir" "Lenin")
          ("Vladimir" "Putin") ("Josef" "Stalin") ("Leon" "Trotsky") 
          ("Boris" "Yeltsin")) 
        (map-query 'list #'list		;#'identity
                  [select [first-name] [last-name] :from [employee] :order-by [last-name]])
        :reported-form :fdml/map-query/4)

  
  (test '("Yuri Andropov" "Leonid Brezhnev" "Konstantin Chernenko" "Mikhail Gorbachev"
          "Nikita Kruschev" "Vladimir Lenin" "Vladimir Putin"
          "Josef Stalin" "Leon Trotsky" "Boris Yeltsin")
        (loop for (forename surname) being each record in
                  [select [first-name] [last-name] :from [employee] :order-by [last-name]]
              collect (concatenate 'string forename " " surname))
        :reported-form :fdml/loop/1)

  (test '(1 2)
        (loop for (addressid) being each record in
                 [select [addressid] :from [addr] :order-by [addressid]]
              collect addressid)
        :reported-form :fdml/loop/2)

  ;; YstokSQL speciality: Loop allocates a single tuple object used during all iterations,
  ;; therefore, a copy should be made to collect results carefully
  ;; (especially for non-destructuring form).
  (test '((1) (2))				; CLSQL: '(1 2) - CLSQL bug?
        (loop for x being each tuple in
                 [select [addressid] :from [addr] :order-by [addressid]]
              collect (copy-list x))
        :reported-form :fdml/loop/3)

  ;; Starts a transaction deletes a record and then rolls back the deletion 
  (test '(nil ("putin@soviet.org")) ;'(nil t nil nil ("putin@soviet.org"))
        (let ((results '()))
          ;(push (clsql:in-transaction-p) results)
          (start-transaction)
          ;(push (clsql:in-transaction-p) results)
          ;; Putin has got to go
          (delete-records :from [employee] :where [= [last-name] "Putin"])
          ;; Should be nil 
          (push (select [*] :from [employee] :where [= [last-name] "Putin"]) results)
          ;; Oh no, he's still there
          (rollback :autocommit t)
          ;(push (clsql:in-transaction-p) results)
          ;; Check that we got him back alright 
          (push (select [email] :from [employee] :where [= [last-name] "Putin"]
                        :flatp t)
                results)
          (nreverse results))
        :reported-form :fdml/transaction/1)

  ;; Starts a transaction, updates a record and then rolls back the update
  ;(test '(nil t ("putin-nospam@soviet.org") nil ("putin@soviet.org"))
  (test '(("putin-nospam@soviet.org") ("putin@soviet.org"))
        (let ((results '()))
          ;(push (clsql:in-transaction-p) results)
          (start-transaction)
          ;(push (clsql:in-transaction-p) results)
          ;; Putin has got to go
          (update-records [employee] :av-pairs '((email "putin-nospam@soviet.org"))
                          :where [= [last-name] "Putin"])
          ;; Should be new value  
          (push (select [email] :from [employee] :where [= [last-name] "Putin"]
                        :flatp t)
                results)
          ;; Oh no, he's still there
          (rollback :autocommit t)
          ;(push (clsql:in-transaction-p) results)
          ;; Check that we got him back alright 
          (push (select [email] :from [employee] :where [= [last-name] "Putin"]
                         :flatp t)
                results)
          (nreverse results))
        :reported-form :fdml/transaction/2) 

  ;; Runs an update within a transaction and checks it is committed
  ;(test '(nil nil ("lenin-nospam@soviet.org") nil ("lenin@soviet.org"))
  (test '(("lenin-nospam@soviet.org") ("lenin@soviet.org"))
        (let ((results '()))
          ;(push (clsql:in-transaction-p) results)
          (with-transaction () 
            (update-records [employee] :av-pairs '((email "lenin-nospam@soviet.org"))
                            :where [= [emplid] 1]))
          ;(push (clsql:in-transaction-p) results)
          ;; check that was committed 
          (push (select [email] :from [employee] :where [= [emplid] 1] :flatp t)
                results)
          ;; undo the changes 
          (with-transaction () 
            (update-records [employee] :av-pairs '((email "lenin@soviet.org"))
                            :where [= [emplid] 1]))
          ;(push (clsql:in-transaction-p) results)
          ;; check that was committed 
          (push (select [email] :from [employee] :where [= [emplid] 1] :flatp t)
                results)
          (nreverse results))
        :reported-form :fdml/transaction/3)

  (test-error (update-records [employee] 
                              :av-pairs '((emale "lenin-nospam@soviet.org"))
                              :where [= [emplid] 1])
              :announce t
              :condition-type 'sql-database-error :include-subtypes t
              :reported-form :fdml/error/misspelled-where)

  (test-error (select [email] :from [employee] #| presidentid |#
                      :where [in [emplid] [select [managerid] :from [company]]])
	:announce t
	:condition-type 'sql-database-error :include-subtypes t
        :known-failure (typep database
                              '(or sql.sys:access-mixin
                                   sql.sys:oracle-mixin
                                   sql.sys:postgresql-mixin
                                   sql.sys:sqlite3-mixin))
        :fail-info "Misspelled column in subquery is not signaled by many DBMSs."
        :reported-form :fdml/error/misspelled-in)

  ;; Run a valid update and an invalid one within a transaction and checks
  ;; that the valid update is rolled back when the invalid one fails. 
  ;(test '(nil nil ("lenin@soviet.org"))
  (test '(("lenin@soviet.org"))
        (let ((results '()))
          ;(push (clsql:in-transaction-p) results)
          (handler-case 
              (with-transaction () 
                ;; valid update
                (update-records [employee] 
                                :av-pairs '((email "lenin-nospam@soviet.org"))
                                :where [= [emplid] 1])
                ;; invalid update which generates an error 
                (update-records [employee] 
                                :av-pairs '((emale "lenin-nospam@soviet.org"))
                                :where [= [emplid] 1]))
            (sql-database-error ()
              (progn
                ;(push (clsql:in-transaction-p) results)
                ;; check nothing done 
                (push (select [email] :from [employee] :where [= [emplid] 1] :flatp t)
                      results)
            (nreverse results)))))
        :reported-form :fdml/transaction/4)
)

(restore-sql-reader-syntax-state)
