;;;-*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface - Initializing schema and populating database

(in-package :sql.test)

(defmethod test-initialize (env-type database)
  ;; Dropping extant schema objects
  ;(test-basic-initialize)
  (drop-view-from-class 'employee)   ;(drop-table 'employee :cascade t)
  (drop-view-from-class 'company)
  (drop-view-from-class 'address)
  (drop-view-from-class 'employee-address)
  (drop-view-from-class 'big)

  ;; Creating schema objects actually
  (create-view-from-class 'employee)
  (create-view-from-class 'company)
  (create-view-from-class 'address)
  (create-view-from-class 'employee-address)

  (when (db-has-bigint? database)
    (create-view-from-class 'big))

  ;; Creating test instances
  (multiple-value-bind (sec min hour day month year)
      (decode-universal-time *test-start-utime*)
   (declare (ignore sec min hour))
   (let* (;(sql.sys::*db-auto-sync* t))
          (now-date #+local-time (lt:encode-date year month day)
                    #-local-time (encode-universal-time 0 0 0 day month year))
          (now-time #+local-time (lt:local-time :universal *test-start-utime*)
                    #-local-time *test-start-utime*))
    (setf company1 (make-instance 'company
				  :companyid 1
				  :presidentid 1
				  ;:groupid 1
				  :name "Soviet Inc.")
          company2 (make-instance 'company
				  :companyid 2
				  :presidentid 9
				  :name "Russian Inc.")
	  employee1 (make-instance 'employee
                       :emplid 1
                       :groupid 1
                       :married t 
                       :height (1+ (random 1.00))
                       :birth-date #+local-time (lt:encode-date 1870 04 22)
                                   #-local-time (encode-universal-time 0 0 0 22 4 1870)
                       :birth-ts now-time ;*test-start-utime*
                       :first-name "Vladimir"
                       :last-name "Lenin"
                       :email "lenin@soviet.org"
                       :companyid 1)
	  employee2 (make-instance 'employee
				   :emplid 2
				   :groupid 1
				   :height (1+ (random 1.00))
				   :married t 
				   :birth-date now-date
				   :birth-ts now-time
				   :first-name "Josef"
				   :last-name "Stalin"
				   :email "stalin@soviet.org"
				   :managerid 1
				   :companyid 1)
	  employee3 (make-instance 'employee
				   :emplid 3
				   :groupid 1
				   :height (1+ (random 1.00))
				   :married t 
				   :birth-date now-date
				   :birth-ts now-time
				   :first-name "Leon"
				   :last-name "Trotsky"
				   :email "trotsky@soviet.org"
				   :managerid 1
				   :companyid 1)
	  employee4 (make-instance 'employee
				   :emplid 4
				   :groupid 1
				   :height (1+ (random 1.00))
				   :married nil
				   :birth-date now-date
				   :birth-ts now-time
				   :first-name "Nikita"
				   :last-name "Kruschev"
				   :email "kruschev@soviet.org"
				   :managerid 1
				   :companyid 1)
	  employee5 (make-instance 'employee
		      :emplid 5
                      :groupid 1
                      :married nil
                      :height (1+ (random 1.00))
                      :birth-date #+local-time (lt:encode-date 1906 12 19)
                                  #-local-time (encode-universal-time 0 0 0 19 12 1906)
                      :birth-ts now-time
                      :first-name "Leonid"
                      :last-name "Brezhnev"
                      :email "brezhnev@soviet.org"
                      :managerid 1
                      :companyid 1)
	  employee6 (make-instance 'employee
				   :emplid 6
				   :groupid 1
				   :married nil
				   :height (1+ (random 1.00))
				   :birth-date now-date
				   :birth-ts now-time
				   :first-name "Yuri"
				   :last-name "Andropov"
				   :email "andropov@soviet.org"
				   :managerid 1
				   :companyid 1)
	  employee7 (make-instance 'employee
				   :emplid 7
				   :groupid 1
				   :height (1+ (random 1.00))
				   :married nil
				   :birth-date now-date
				   :birth-ts now-time
				   :first-name "Konstantin"
				   :last-name "Chernenko"
				   :email "chernenko@soviet.org"
				   :managerid 1
				   :companyid 1)
	  employee8 (make-instance 'employee
				   :emplid 8
				   :groupid 1
				   :height (1+ (random 1.00))
				   :married nil
				   :birth-date now-date
				   :birth-ts now-time
				   :first-name "Mikhail"
				   :last-name "Gorbachev"
				   :email "gorbachev@soviet.org"
				   :managerid 1)
				   ;:companyid 1)
	  employee9 (make-instance 'employee
				   :emplid 9
				   :groupid 1 
				   :married nil
				   :height (1+ (random 1.00))
				   :birth-date now-date
				   :birth-ts now-time
				   :first-name "Boris"
				   :last-name "Yeltsin"
				   :email "yeltsin@soviet.org"
				   :managerid 1
				   :companyid 2)
	  employee10 (make-instance 'employee
				    :emplid 10
				    :groupid 1
				    :married nil
				    :height (1+ (random 1.00))
				    :birth-date now-date
				    :birth-ts now-time
				    :first-name "Vladimir"
				    :last-name "Putin"
				    :email "putin@soviet.org"
				    :managerid 1
				    :companyid 2)
	  address1 (make-instance 'address
				  :addressid 1
				  :street-number 10
				  :street-name "Park Place"
				  :city "Leningrad"
				  :postal-code 123)
	  address2 (make-instance 'address
				  :addressid 2)
	  employee-address1 (make-instance 'employee-address
					   :emplid 1
					   :addressid 1
					   :verified t)
	  employee-address2 (make-instance 'employee-address
					   :emplid 2
					   :addressid 2
					   :verified t)
	  employee-address3 (make-instance 'employee-address
					   :emplid 3
					   :addressid 1
					   :verified nil)
	  employee-address4 (make-instance 'employee-address
					   :emplid 1
					   :addressid 2
					   :verified nil)
	  employee-address5 (make-instance 'employee-address
					   :emplid 3
					   :addressid 2))))

  ;; sleep to ensure birthdays are no longer at current time
  (sleep 1) 
  
  #||
  ;; Lenin manages everyone
  (clsql:add-to-relation employee2 'manager employee1)
  (clsql:add-to-relation employee3 'manager employee1)
  (clsql:add-to-relation employee4 'manager employee1)
  (clsql:add-to-relation employee5 'manager employee1)
  (clsql:add-to-relation employee6 'manager employee1)
  (clsql:add-to-relation employee7 'manager employee1)
  (clsql:add-to-relation employee8 'manager employee1)
  (clsql:add-to-relation employee9 'manager employee1)
  (clsql:add-to-relation employee10 'manager employee1)
  ;; Everyone works for Widgets Inc.
  (clsql:add-to-relation company1 'employees employee1)
  (clsql:add-to-relation company1 'employees employee2)
  (clsql:add-to-relation company1 'employees employee3)
  (clsql:add-to-relation company1 'employees employee4)
  (clsql:add-to-relation company1 'employees employee5)
  (clsql:add-to-relation company1 'employees employee6)
  (clsql:add-to-relation company1 'employees employee7)
  (clsql:add-to-relation company1 'employees employee8)
  (clsql:add-to-relation company1 'employees employee9)
  (clsql:add-to-relation company1 'employees employee10)
  ;; Lenin is president of Widgets Inc.
  (clsql:add-to-relation company1 'president employee1)
  ||#
  
  ;; store these instances
  (with-transaction ()
    (update-records-from-instance employee1)
    (update-records-from-instance employee2)
    (update-records-from-instance employee3)
    (update-records-from-instance employee4)
    (update-records-from-instance employee5)
    (update-records-from-instance employee6)
    (update-records-from-instance employee7)
    (update-records-from-instance employee8)
    (update-records-from-instance employee9)
    (update-records-from-instance employee10)
    (update-records-from-instance company1)
    (update-records-from-instance company2)
    (update-records-from-instance address1)
    (update-records-from-instance address2)
    
    (when (db-has-bigint? database)
      (let ((max (expt 2 60)))
        (dotimes (i 555)
          (update-records-from-instance 
           (make-instance 'big :i (1+ i) :bi (truncate max (1+ i)))))))
) )

#|
(db-has-bigint? sql:*default-database*)

(test-initialize :access sql:*default-database*)
|#

