;;;-*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface - Tests for object-oriented interface

(in-package :sql.test)

(locally-enable-sql-reader-syntax)

(defmethod test-object (env-type database)
  ;; Ensure slots inherited from standard-classes are :standard
  (test '(:standard :standard)			; CLSQL: '(:virtual :virtual)
        (list (sql.sys::db-kind (clos:find-slot-definition 'extraterrestrial
                                                           (find-class 'person) t))
              (sql.sys::db-kind (clos:find-slot-definition 'hobby (find-class 'person) t)))
        :reported-form :oo/metaclass/1)

  ;; Ensure all slots in view-class are of db-class-effective-slot-definition
  (test '(t t t)
        (flet ((predicate (sd) (typep sd 'sql.sys::db-class-esd)))
          (list (every #'predicate (clos:class-slots (find-class 'person)))
                (every #'predicate (clos:class-slots (find-class 'employee)))
                (every #'predicate (clos:class-slots (find-class 'company)))))
        :reported-form :oo/metaclass/2)

  (test '(1 1 1 1 1 1 1)
        (mapcar (lambda (e) (slot-value e 'ecompanyid))
                (company-employees company1))
        :reported-form :oo/join/1)

  (test "Lenin" (slot-value (president company1) 'last-name)
        :reported-form :oo/join/2)

  (test "Lenin" (slot-value (employee-manager employee2) 'last-name)
        :reported-form :oo/join/3)

  #+local-time
  (test '("Lenin" t)
    (let ((ts (lt:localtimestamp (typecase database
                                   ((or sql.sys:access-mixin sql.sys:sqlite3-mixin)
                                    0)))))	; driver returns 0 millisec
      (update-records [employee] :av-pairs `((birth-ts ,ts)) :where [= [emplid] 1])
      (when-bind (dbobj (car (select 'employee
                                :where [= [birth-ts] ts]
                                :flatp t)))
        (list (slot-value dbobj 'last-name)
              (lt:local-time= (slot-value dbobj 'birth-ts) ts))))
    :reported-form :oo/time/1)
  #-local-time
  (test '("Lenin" t)
    (let ((ts (get-universal-time)))
      (update-records [employee] :av-pairs `((birth-ts ,(timestring ts)))
                      :where [= [emplid] 1])
      ;; Access accepts timestrings in INSERT and UPDATE but not in WHERE!
      ;; So we have to use the explicit conversion by means of CDate.
      (when-bind (dbobj (car (select 'employee
                                :where [= [birth-ts]
                                          (if (typep database 'sql.sys:access-mixin)
                                              [sql-function 'cdate (timestring ts)]
                                              (timestring ts))]
                                :flatp t)))
        (list (slot-value dbobj 'last-name)
              (= (slot-value dbobj 'birth-ts) ts))))
    :errorset :warning
    :reported-form :oo/time/1)

  #+local-time
  (test '("Lenin" t)
        (let* ((date (lt:local-time+ (lt:current-date) 10)))
          ;(typecase database
          ;  (sql.sys:postgresql-mixin (execute-command "set datestyle to 'iso'")))
          (update-records [employee] :av-pairs `(([birth-date] ,date))
                          :where [= [emplid] 1])
          (when-bind (dbobj (car (select 'employee :where [= [birth-date] date] :flatp t)))
            (list (slot-value dbobj 'last-name)
                  (lt:local-time= (slot-value dbobj 'birth-date) date))))
        :reported-form :oo/date/1)
  #-local-time
  (push :oo/date/1 *test-skips*)

  #+local-time
  (test -1
        (let* ((date (lt:current-date))
               (fail-index -1))
          ;(typecase database
          ;  (sql.sys:postgresql-mixin (execute-command "set datestyle to 'iso'")))
          (dotimes (x 40)
            (update-records [employee] :av-pairs `((birth_date ,date))
                            :where [= [emplid] 1])
            (when-bind (dbobj (car (select 'employee :where [= [birth-date] date]
                                           :flatp t)))
              (unless (lt:local-time= (slot-value dbobj 'birth-date) date)
                (setf fail-index x))
              (setf date (lt:local-time+ date (* 10 x)))))
          fail-index)
        :reported-form :oo/date/2)
  #-local-time
  (push :oo/date/2 *test-skips*)

  #+ignore
  (test '(t #+local-time t)
        (progn
          (typecase database
            (sql.sys:postgresql-mixin			;:postgresql :postgresql-socket
             (execute-command "set datestyle to 'iso'")))
          (when-bind (dbobj (car (select 'employee :where [= [emplid] 10] :flatp t)))
            (list (eql *test-start-utime* (slot-value dbobj 'birth-ts))
                  #+local-time (lt:local-time= (slot-value dbobj 'birth-ts)
                                               (lt:local-time
                                               :universal (slot-value dbobj 'birth-ts))))))
        :reported-form :oo/time/3)
)

(restore-sql-reader-syntax-state)

#|
(setq ts (lt:current-timestamp))
(lt:format-timestring nil now nil nil 4 4 #\- #\: #\Space)

(update-records [employee] :av-pairs `((birth-date ,now)) :where [= [emplid] 1])
(setq dbobj (car (select 'employee :where [= [birth-date] now] :flatp t)))
  
(slot-value company1 'president)
(slot-makunbound company1 'president)
(setf (database company1) sql:*default-database*)

(employee-manager employee2)
(slot-makunbound employee2 'manager)
(setf (database employee2) sql:*default-database*)
|#