;;;-*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface - Object-oriented schema of database

(in-package :sql.test)

;;; Lispworks does not allow inheritance among classes of different metaclasses
#-lispworks
(defclass thing ()
  ((extraterrestrial :initform nil :initarg :extraterrestrial)))
#+lispworks
(def-view-class thing ()
  ((extraterrestrial :initform nil :initarg :extraterrestrial
   :db-kind :standard)))

(def-view-class person (thing)
  ((height :db-kind :base :accessor height :type float
           :initarg :height)
   (married :db-kind :base :accessor married :type boolean
            :initarg :married)
   (birth-date :type #+local-time lt:date #-local-time sql:universal-time
               :initarg :birth-date)
   (birth-ts :type #+local-time lt:local-time #-local-time sql:universal-time
             :initarg :birth-ts)
   (hobby :db-kind :virtual :initarg :hobby :initform nil)))
  
(def-view-class employee (person)
  ((emplid
    :db-kind :key
    :db-info (:constraints (:not-null))		; :unique - excessive and signals in Oracle
    :type integer
    :initarg :emplid)
   (groupid					; department ?
    ;:db-kind :key ??
    :db-info (:constraints :not-null)
    :type integer
    :initarg :groupid)
   (first-name
    :accessor first-name
    :type (string 30)
    :initarg :first-name)
   (last-name
    :accessor last-name
    :type (string 30)
    :initarg :last-name)
   (email
    :accessor employee-email
    :type (string 100)
    :initarg :email)
   (ecompanyid
    :type integer
    :initarg :companyid)
   (company
    :accessor employee-company
    :db-kind :join
    :db-info (:join-class company  :home-key ecompanyid  :foreign-key companyid  :set nil))
   (managerid
    :type integer
    :initarg :managerid)
   (manager
    :accessor employee-manager
    :db-kind :join
    :db-info (:join-class employee  :home-key managerid  :foreign-key emplid  :set nil))
   (addresses
    :accessor employee-addresses
    :db-kind :join
    :db-info (:join-class employee-address :home-key emplid :foreign-key aemplid ;:join-key
              :target-slot address  :set t)))
  (:base-table employee))

(def-view-class company ()
  ((companyid
    :db-kind :key
    :db-info (:constraints :not-null)
    :type integer
    :initarg :companyid)
   #|(groupid
    :db-kind :key
    :db-info (:constraints :not-null)
    :type integer
    :initarg :groupid)|#
   (name
    :type (string 100)
    :initarg :name)
   (presidentid
    :type integer
    :initarg :presidentid)
   (president
    :reader president
    :db-kind :join
    :db-info (:join-class employee  :home-key presidentid  :foreign-key emplid  :set nil))
   (employees
    :reader company-employees
    :db-kind :join  					; groupid ??
    :db-info (:join-class employee :home-key (companyid) :foreign-key (ecompanyid)
              :set t))))

(def-view-class address ()
  ((addressid
    :db-kind :key
    :db-info (:constraints :not-null)
    :type integer
    :initarg :addressid)
   (street-number
    :type integer
    :initarg :street-number)
   (street-name
    :type (string 30)
    ;:void-value ""
    :initarg :street-name)
   (city
    :column "city_field"
    ;:void-value "no city"
    :type (string 30)
    :initarg :city)
   (postal-code
    :column zip
    :type integer
    ;:void-value 0
    :initarg :postal-code))
  (:base-table addr))

;; many employees can reside at many addressess
(def-view-class employee-address ()
  ((aemplid :type integer :initarg :emplid)
   (aaddressid :type integer :initarg :addressid)
   (verified :type boolean :initarg :verified)
   (address :db-kind :join
	    :db-info (:join-class address :home-key aaddressid :foreign-key addressid
                      :retrieval :immediate))
   (employee :db-kind :join
	     :db-info (:join-class employee :home-key aemplid :foreign-key emplid
                       :retrieval :immediate)))
  (:base-table "ea_join"))

(def-view-class deferred-employee-address ()
  ((aemplid :type integer :initarg :emplid)
   (aaddressid :type integer :initarg :addressid)
   (verified :type boolean :initarg :verified)
   (address :db-kind :join
	    :db-info (:join-class address :home-key aaddressid :foreign-key addressid
                      :retrieval :deferred :set nil)))
  (:base-table "ea_join"))

(defvar company1 nil)
(defvar company2 nil)
(defvar employee1 nil)
(defvar employee2 nil)
(defvar employee3 nil)
(defvar employee4 nil)
(defvar employee5 nil)
(defvar employee6 nil)
(defvar employee7 nil)
(defvar employee8 nil)
(defvar employee9 nil)
(defvar employee10 nil)
(defvar address1 nil)
(defvar address2 nil)
(defvar employee-address1 nil)
(defvar employee-address2 nil)
(defvar employee-address3 nil)
(defvar employee-address4 nil)
(defvar employee-address5 nil)

;;; Define the class but may not create actual view from it.
(def-view-class big ()
  ((i :type integer :initarg :i)
   (bi :type :bigint :initarg :bi)))

#|
(setq c (find-class 'employee))
(setq c (find-class 'address))
(setq c (find-class 'employee-address))
(setq c (find-class 'big))

|#

