;;;-*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface - Tests for bracket SQL syntax

(in-package :sql.test)

;(enable-sql-reader-syntax)
(locally-enable-sql-reader-syntax)

;; no need to always put [identifier] under evaluation
#+ys-product (setq ys::*compile-for-lazyload* t)

(eval-when (:compile-toplevel :load-toplevel :execute)
  (defsql-function () TEST-FUN))

(defmethod test-syntax (env-type database)
  ;; sql is implemented by means of write-sql-to-string over *default-database*,
  ;; which is eq to the database argument.
  (test "'foo'" (sql "foo")
        :reported-form :syntax/generic/1)

  (test "23" (sql 23)
        :reported-form :syntax/generic/2)
    
  (test "BAR" (sql 'bar)
        :reported-form :syntax/generic/3)

  (test "('ten',10,TEN)" (sql '("ten" 10 ten))
        :reported-form :syntax/generic/4)

  (test "'What''s up Doc?'" (sql "What's up Doc?")
        :reported-form :syntax/generic/5)

  (test "SELECT FOO,BAR FROM BAZ" (sql ["SELECT FOO,BAR FROM BAZ"])
        :reported-form :syntax/generic/6)


  (test "FOO" (sql [foo])
        :reported-form :syntax/ident/1)

  (test "FOO.BAR" (sql [foo bar])
        :reported-form :syntax/ident/2)

  (test "FOO" (sql [foo :integer])
        :reported-form :syntax/ident/3)

  (test "FOO.BAR" (sql [foo bar :integer])
        :reported-form :syntax/ident/4)

  (let ((ch (sql.sys:db-identifier-quote-char database)))
    (test "FOO"
          (let ((sql.sys::*in-subselect* nil))		; aliases are disabled
            (sql [foo "bar"]))
          :reported-form :syntax/ident/5-0)
    (test (format nil "FOO ~Cbar~C" ch ch)
          (let ((sql.sys::*in-subselect* t))		; aliases are enabled
            (sql [foo "bar"]))
          :reported-form :syntax/ident/5-1)

    (test (format nil "~Cfoo~C.BAR" ch ch) (sql ["foo" bar])
          :reported-form :syntax/ident/6)

    (test (format nil "~Cfoo~C.BAR" ch ch) (sql ["foo" bar :integer])
          :reported-form :syntax/ident/7)
  )
  (test "ALL (FOO,BAR,BAZ)" (sql [all '(foo bar baz)])
        :reported-form :syntax/subquery/all)

  (test "ANY (3,4)" (sql [any '(3 4)])
        :reported-form :syntax/subquery/any)

  (test "ANY (FOO,BAR,BAZ)" (sql [some '(foo bar baz)])	; SOME is a synonym of ANY
        :reported-form :syntax/subquery/some)

  (test "(EXISTS (FOO,BAR,BAZ))" (sql [exists '(foo bar baz)])
        :reported-form :syntax/subquery/exists)

  (test "(FOO IN (FOO,BAR,BAZ))" (sql [in [foo] '(foo bar baz)])
        :reported-form :syntax/subquery/in)


  (test "AVG((FOO + (1000 * BAR)))" (sql [avg [+ [foo] [* 1000 [bar]]]])
        :reported-form :syntax/aggregate/avg)

  (test "MAX((FOO + (1000 * BAR)))" (sql [max [+ [foo] [* 1000 [bar]]]])
        :reported-form :syntax/aggregate/max)

  (test "MIN((FOO + (1000 * BAR)))" (sql [min [+ [foo] [* 1000 [bar]]]])
        :reported-form :syntax/aggregate/min)

  (test "SUM(FOO,BAR)" (sql [sum [foo] [bar]])
        :reported-form :syntax/aggregate/sum)

  (test "COUNT(FOO)" (sql [count [foo]])
        :reported-form :syntax/aggregate/count)


  (test "(FOO AND BAR)" (sql [and [foo] [bar]])
        :reported-form :syntax/logical/1)

  (test "(FOO OR BAR)" (sql [or [foo] [bar]])
        :reported-form :syntax/logical/2)

  (test "(NOT FOO)" (sql [not [foo]])
        :reported-form :syntax/logical/3)


  (test "(FOO IS NULL)" (sql [null [foo]])
        :known-failure t :fail-info "Back-end optimization possible"
        :reported-form :syntax/null/1)

  (test "(NOT (FOO IS NULL))" (sql [not [null [foo]]])
        :known-failure t :fail-info "Back-end optimization possible"
        :reported-form :syntax/null/2)

  (test "(FOO IS NOT NULL)" (sql [is [foo] [not-null]])
        :reported-form :syntax/null/not-null)

  (test "NULL" (sql [null])
        :reported-form :syntax/null/3)

  (test "(NOT NULL)" (sql [not [null]])
        :reported-form :syntax/null/4)


  (test "(BAZ > BEEP)" (sql [> [baz] [beep]])
        :reported-form :syntax/relational/1)

  (test "(FOO > 10)"  (let ((x 10)) (sql [> [foo] x]))
        :reported-form :syntax/relational/2)

  (test "(BAZ >= BEEP)" (sql [>= [baz] [beep]])
        :reported-form :syntax/relational/3)

  (test "(BAZ < BEEP)" (sql [< [baz] [beep]])
        :reported-form :syntax/relational/4)

  (test "(BAZ = BEEP)" (sql [= [baz] [beep]])
        :reported-form :syntax/relational/5)

  (test "(BAZ <> BEEP)" (sql [<> [baz] [beep]])
        :reported-form :syntax/relational/6)


  (test "((FOO - 1) BETWEEN (BAR * 5) AND (BAZ / 9))"
        (sql [between [- [foo] 1] [* [bar] 5] [/ [baz] 9]])
        :reported-form :syntax/between/1)

  (test "(NOT ((FOO - 1) BETWEEN (BAR * 5) AND (BAZ / 9)))"
        (sql [not [between [- [foo] 1] [* [bar] 5] [/ [baz] 9]]])
        :reported-form :syntax/between/2)


  (test "(FOO.BAR + BAZ)" (sql [+ [foo bar] [baz]])
        :reported-form :syntax/arithmetic/1)

  (test "(FOO.BAR - BAZ)" (sql [- [foo bar] [baz]])
        :reported-form :syntax/arithmetic/2)

  (test "(FOO.BAR / BAZ)" (sql [/ [foo bar] [baz]])
        :reported-form :syntax/arithmetic/3)

  (test "(FOO.BAR * BAZ)" (sql [* [foo bar] [baz]])
        :reported-form :syntax/arithmetic/4)

  (test "(- FOO.BAR)" (sql [- [foo bar]])
        :reported-form :syntax/arithmetic/5)

  (test "(2 * 3)" (sql [* 2 3])
        :reported-form :syntax/arithmetic/6)

  (test "COALESCE(FOO,BAR,'not specified')"
        (sql [coalesce [foo] [bar] "not specified"])
        :known-failure t :fail-info "Back-end specific replacement is possible."
        :reported-form :syntax/coalesce/1)

  (test "CAST(HELLO AS INTEGER)" (sql [cast [hello] :as :integer])
        :known-failure (typep database 'sql.sys:access-mixin)
        :fail-info "With ODBC may be replaced by {fn CONVERT()}"
        :reported-form :syntax/cast)

  ;; Non-conformance to CommonSQL: As function names, we accept only symbols.
  ;; String is not a lispy way!
  (test "COS(AGE)" (sql [sql-function 'COS [age]])
        :known-failure t :fail-info "With ODBC as scalar function"
        :reported-form :syntax/function/1)

  ;; test-warning does not detect during read or macroexpansion
  ;(test-warning (sql [sql-function "TO_DATE" "02/06/99" "mm/DD/RR"]))
  (test-error (sql [sql-function "TO_DATE" "02/06/99" "mm/DD/RR"])
	:announce t  :condition-type 'sql-undefined-function-warning)

  (test "TEST_FUN(AGE)" (sql [funcall 'TEST-FUN [age]])	; YstokSQL extension
        :reported-form :syntax/function/2)

  (test "TEST_FUN(AGE)" (sql [test-fun [age]])		; YstokSQL extension
        :reported-form :syntax/function/3)

  ;(defsql-function (:procedure t) TEST-PROC)
  ;(test "TEST_PROC(AGE)" (sql [funcall 'TEST-PROC [age]])	; YstokSQL extension
  ;      :reported-form :syntax/procedure/1)

  ;(test "SUBSTR(HELLO,1,4)" (sql [substr [hello] 1 4])
  ;      :reported-form :syntax/substr/1)

  (test "(FOO || BAR || BAZ)" (sql [\|\| [foo] [bar] [baz]])
        :known-failure t :fail-info "Back-end specific, e.g. scalar function with ODBC."
        :reported-form :syntax/concat/1)

  (test "(FOO || BAR || BAZ)" (sql [|| [foo] [bar] [baz]])
        :known-failure t :fail-info "Back-end specific, e.g. scalar function with ODBC."
        :reported-form :syntax/concat/2)

  (test "(FOO || BAR)" (sql [concat [foo] [bar]])
        :known-failure t :fail-info "Back-end specific, e.g. scalar function with ODBC."
        :reported-form :syntax/concat/3)

  (test (if (typep database 'sql.sys:sqlite3-mixin)
            "SUBSTR(HELLO,1,4)"
            "SUBSTRING(HELLO,1,4)")
        (sql [substring [hello] 1 4])
        :known-failure t  :fail-info "With ODBC as scalar function"
        :reported-form :syntax/substring/1)

  (test "(FOO LIKE '%v')" (sql [like [foo] "%v"])
        :reported-form :syntax/like)

  (test "(NOT (FOO LIKE '%v'))" (sql [not [like [foo] "%v"]])
        :reported-form :syntax/not-like)

  (test "(FOO LIKE '%v~_' ESCAPE '~')" (sql [like [foo] "%v~_" :escape #\~])
        :reported-form :syntax/like/escape)

  ;; Leads to sql-reader-error: Illegal SQL column name
  ;(test-error (sql [distinct [foo bar :string]])) :reported-form :syntax/distinct/error)
  ;(test-error (sql [order-by [foo]])) 		   :reported-form :syntax/order-by/error)
  ;(test-error (sql [group-by [foo]]))             :reported-form :syntax/group-by/error)

  (test "COUNT(DISTINCT FOO.BAR)" (sql [count [foo bar :string] :distinct])
        :reported-form :syntax/distinct/count)

  (test "SELECT DISTINCT FOO,BAR FROM BAZ"
        (sql [select [foo :string] [bar :integer] :from [baz] :distinct t])
        :reported-form :syntax/distinct/select)


  (test "SELECT FOO,BAR,COUNT(FOO) FROM TABLE GROUP BY FOO,BAR ORDER BY FOO,BAR"
        (sql (sql-query-exp [foo] [bar] [count [foo]]
                            :from [table]
                            :group-by '([foo] [bar])
                            :order-by '([foo] [bar])))
        :reported-form :syntax/group-by/select)


  ;(test "SELECT FOO FROM BAR UNION SELECT BAZ FROM BAR"	; from CLSQL
  ;      (sql [union [select [foo] :from [bar]] [select [baz] :from [bar]]])
  ;      :reported-form :syntax/sets/1)

  (test "SELECT FOO FROM BAR UNION SELECT BAZ FROM BAR"
        (sql [select [foo] :from [bar]
                     :set-operation [union [select [baz] :from [bar]]]])
        :reported-form :syntax/sets/1)

  (test "SELECT FOO FROM BAR INTERSECT ALL SELECT BAZ FROM BAR INTERSECT ALL SELECT BAZ FROM FOO"
        (sql [select [foo] :from [bar]
                     :set-operation [intersect :all
                                               [select [baz] :from [bar]]
                                               [select [baz] :from [foo]]]])
        :reported-form :syntax/sets/2)

  (test (typecase database
          (sql.sys:oracle-mixin
           "SELECT FOO FROM BAR MINUS DISTINCT SELECT BAZ FROM BAR")
          (t
           "SELECT FOO FROM BAR EXCEPT DISTINCT SELECT BAZ FROM BAR"))
        (sql [select [foo] :from [bar]
                     :set-operation [except :distinct [select [baz] :from [bar]]]])
        :reported-form :syntax/sets/3)

  ;; Leads to sql-reader-error: Illegal SQL column name
  ;(test-error (sql [minus [select [foo] :from [bar]] [select [baz] :from [bar]]]))
        ;:reported-form :syntax/sets/minus)


  (test "SELECT PERSON_ID,SURNAME FROM PERSON"
        (sql [select [person_id] [surname] :from [person]])
        :reported-form :syntax/query/1)

  (test "SELECT FOO,BAR.* FROM BAZ,BAR WHERE ((FOO = 3) OR (BAZ.QUUX > 10))"
        (sql [select [foo] [bar *]
                     :from '([baz] [bar])
                     :where [or [= [foo] 3] [> [baz.quux] 10]]])
        :known-failure t
        :fail-info "As dot '.' may not be allowed within an identifier, quote it"
        :reported-form :syntax/query/2)

  (test "SELECT FOO.BAR,BAZ FROM FOO,QUUX WHERE ((BAZ > 3) OR (FOO.BAR LIKE 'SU%'))"
        (sql [select [foo bar] [baz]
                     :from '([foo] [quux])
                     :where [or [> [baz] 3] [like [foo bar] "SU%"]]])
        :reported-form :syntax/query/3)

  (test "SELECT COUNT(*) FROM EMP"
        (sql [select [count [*]] :from [emp]])
        :reported-form :syntax/query/4)

  ;; Due to possible optimizations and coallesced operators,
  ;; the following two forms are not always equivalent:
  ;;   (sql-operation op args) vs. (apply (sql-operator op) args).
  ;; To be on the safe side, use the former one.
  (test "SELECT FOO.BAR,BAZ FROM FOO,QUUX WHERE ((BAZ > 3) OR (FOO.BAR LIKE 'SU%'))"
        (sql (sql-operation 'select
                            (sql-expression :table 'foo :attribute 'bar)
                            (sql-expression :attribute 'baz)
                            :from (list (sql-expression :table 'foo)
                                        (sql-expression :table 'quux))
                            :where (sql-operation 'or
                                     (sql-operation '> (sql-expression :attribute 'baz) 3)
                                     (sql-operation 'like
                                                    (sql-expression :table 'foo
                                                                    :attribute 'bar)
                                                    "SU%"))))
        :reported-form :syntax/expression1)

  (test "((THISTIME.BAR BETWEEN (HIP * HOP) AND 42) AND (THISTIME.BAZ LIKE 'THISTIME') AND (NEXTTIME.BAR BETWEEN (HIP * HOP) AND 43) AND (NEXTTIME.BAZ LIKE 'NEXTTIME') AND (SOMETIME.BAR BETWEEN (HIP * HOP) AND 44) AND (SOMETIME.BAZ LIKE 'SOMETIME') AND (NEVER.BAR BETWEEN (HIP * HOP) AND 45) AND (NEVER.BAZ LIKE 'NEVER'))"
        (sql (apply (sql-operator 'and)
                    (loop for table in '(thistime nexttime sometime never)
                          for count from 42
                          collect [between
                                   (sql-expression :table table :attribute 'bar)
                                   (sql-operation '* [hip] [hop])
                                   count]
                          collect [like (sql-expression :table table :attribute 'baz)
                                        (sql table)])))
        :reported-form :syntax/expression/2)

  (test "((FOO.BAR = THISTIME) AND (FOO.BAR = NEXTTIME) AND (FOO.BAR = SOMETIME) AND (FOO.BAR = NEVER))"
    (sql (apply 'sql-operation 'and
                (loop for column in '(thistime nexttime sometime never)
                      collect (sql-operation '=
                                             (sql-expression :table 'foo :attribute 'bar)
                                             (sql-expression :column column)))))
    :reported-form :syntax/expression/3)

  ;; CASE variants
  (test "COALESCE(FOO.BAR,3)"
    (sql [coalesce [foo bar] 3])
    :known-failure (typep database 'sql.sys:desktop-mixin)
    :reported-form :syntax/coalesce)

  (test "CASE FOO.BAR WHEN 3 THEN FOO.X WHEN 4 THEN FOO.Y END"
    (sql [case [foo bar]
           3 [foo x]
           4 [foo y]])
    :known-failure (typep database 'sql.sys:desktop-mixin)
    :reported-form :syntax/case)

  (test "CASE FOO.BAR WHEN 3 THEN FOO.X WHEN 4 THEN FOO.Y ELSE FOO.BAZ END"
    (sql [case [foo bar]
           3 [foo x]
           4 [foo y]
           t [foo baz]])
    :known-failure (typep database 'sql.sys:desktop-mixin)
    :reported-form :syntax/case/else)

  (test "CASE WHEN (FOO.BAR < 0) THEN FOO.X WHEN (FOO.BAR > 0) THEN FOO.Y END"
    (sql [cond [< [foo bar] 0] [foo x]
               [> [foo bar] 0] [foo y]])
    :known-failure (typep database 'sql.sys:desktop-mixin)
    :reported-form :syntax/cond)

  (test "CASE WHEN (FOO.BAR < 0) THEN FOO.X WHEN (FOO.BAR > 0) THEN FOO.Y ELSE FOO.BAZ END"
    (sql [cond [< [foo bar] 0] [foo x]
               [> [foo bar] 0] [foo y]
               t [foo baz]])
    :known-failure (typep database 'sql.sys:desktop-mixin)
    :reported-form :syntax/cond/else)

  ;; Joins
  (test (if (typep database 'sql.sys:sqlite3-mixin)
            "SELECT FOO.X,BAR.Y FROM FOO CROSS JOIN BAR"
            "SELECT FOO.X,BAR.Y FROM (FOO CROSS JOIN BAR)")
    (sql [select [foo x] [bar y] :from [cross-join [foo] [bar]]])
    :reported-form :syntax/join/cross)

  (test (if (typep database 'sql.sys:sqlite3-mixin)
            "SELECT FOO.X,BAR.Y FROM FOO INNER JOIN BAR ON (FOO.BAZ = BAR.BAZ)"
            "SELECT FOO.X,BAR.Y FROM (FOO INNER JOIN BAR ON (FOO.BAZ = BAR.BAZ))")
    (sql [select [foo x] [bar y]
                 :from [inner-join [foo] [bar] :on [= [foo baz] [bar baz]]]])
    :reported-form :syntax/join/inner)

  (test (if (typep database 'sql.sys:sqlite3-mixin)
            "SELECT FOO.X,BAR.Y FROM FOO LEFT OUTER JOIN BAR ON (FOO.BAZ = BAR.BAZ)"
            "SELECT FOO.X,BAR.Y FROM (FOO LEFT OUTER JOIN BAR ON (FOO.BAZ = BAR.BAZ))")
    (sql [select [foo x] [bar y]
                 :from [left-join [foo] [bar] :on [= [foo baz] [bar baz]]]])
    :reported-form :syntax/join/left)

  (cond ((typep database 'sql.sys:desktop-mixin)
         (push :syntax/join/using *test-skips*)
         (push :syntax/join/natural *test-skips*))
        (t
         (test (if (typep database 'sql.sys:sqlite3-mixin)
                   "SELECT FOO.X,BAR.Y FROM FOO RIGHT OUTER JOIN BAR USING (BAZ)"
                   "SELECT FOO.X,BAR.Y FROM (FOO RIGHT OUTER JOIN BAR USING (BAZ))")
           (sql [select [foo x] [bar y]
                        :from [right-join [foo] [bar] :using [baz]]])
           :reported-form :syntax/join/using)

         (test (if (typep database 'sql.sys:sqlite3-mixin)
                   "SELECT FOO.X,BAR.Y FROM FOO NATURAL INNER JOIN BAR"
                   "SELECT FOO.X,BAR.Y FROM (FOO NATURAL INNER JOIN BAR)")
           (sql [select [foo x] [bar y]
                        :from [inner-join [foo] [bar] :using :natural]])
           :reported-form :syntax/join/natural) ))
)

;;; ODBC generic

(defmethod test-syntax :after (env-type (database odbc:odbc-database))
								; SQL_FN_NUM_COS
  (test (if (logtest (odbc:db-get-info database odbc::SQL_NUMERIC_FUNCTIONS) #x00000040)
            "{fn COS(AGE)}"
            "COS(AGE)")
    (sql [sql-function 'COS [age]])
    :reported-form :syntax/odbc/cos)
						 		; SQL_FN_NUM_TRUNCATE
  (when (logtest (odbc:db-get-info database odbc::SQL_NUMERIC_FUNCTIONS) #x00800000)
    (test "{fn TRUNCATE(FOO)}" (sql [truncate [foo]])
        :known-failure (typep database '(or sql.sys:access-mixin sql.sys:postgresql-mixin))
        :fail-info "Replaced by Fix in Access or by trunc in PostgreSQL"
        :reported-form :syntax/odbc/truncate))
								 ; SQL_FN_STR_CONCAT
  (when (logtest (odbc:db-get-info database odbc::SQL_STRING_FUNCTIONS) #x00000001)
    (test "{fn CONCAT(FOO,{fn CONCAT(BAR,BAZ)})}" (sql [|| [foo] [bar] [baz]])
        :known-failure t
        :fail-info "Back-end can use standard operator ||."
        :reported-form :syntax/odbc/concat/2))

								; SQL_FN_STR_SUBSTRING
  (test (if (logtest (odbc:db-get-info database odbc::SQL_STRING_FUNCTIONS) #x00000800)
            "{fn SUBSTRING(HELLO,1,4)}"
            (if (typep database 'sql.sys:sqlite3-mixin)
                "SUBSTR(HELLO,1,4)"
                "SUBSTRING(HELLO,1,4)"))
    (sql [substring [hello] 1 4])
    :reported-form :syntax/odbc/substring)

  ;(test "SELECT FOO.X,BAR.Y FROM {oj FOO INNER JOIN BAR ON (FOO.BAZ = BAR.BAZ)}"
  ;      (sql [select [foo x] [bar y]
  ;                   :from [inner-join [foo] [bar] :on [= [foo baz] [bar baz]]]])
  ;      :reported-form :syntax/odbc/join/inner)
)

;;; Access

#+access
(defmethod test-syntax :after (env-type (database sql.sys:desktop-mixin))
  (test "IsNull(FOO)" (sql [null [foo]])
        :reported-form :syntax/desktop/null/1)

  (test "(NOT IsNull(FOO))" (sql [not [null [foo]]])
        :reported-form :syntax/desktop/null/2)

  (test "Iif(IsNull(FOO),Iif(IsNull(BAR),'not specified',BAR),FOO)"
        (sql [coalesce [foo] [bar] "not specified"])
        :reported-form :syntax/desktop/coalesce)

  (test "{fn CONVERT(HELLO,SQL_INTEGER)}" (sql [cast [hello] :as :integer])
        :reported-form :syntax/desktop/cast)

  (test "FIX(FOO)" (sql [truncate [foo]])		; Truncate is undefined in Access
        :reported-form :syntax/desktop/truncate)	; - replaced by fix

  (test "(FOO & BAR & BAZ)" (sql [|| [foo] [bar] [baz]])
        :reported-form :syntax/desktop/concat/2)

  ;; CASE variants
  (test "Iif(IsNull(FOO.BAR),3,FOO.BAR)"
    (sql [coalesce [foo bar] 3])
    :reported-form :syntax/desktop/coalesce)

  (test "Iif(FOO.BAR=3,FOO.X,Iif(FOO.BAR=4,FOO.Y,NULL))"
    (sql [case [foo bar]
           3 [foo x]
           4 [foo y]])
    :reported-form :syntax/desktop/case)

  (test "Iif(FOO.BAR=3,FOO.X,Iif(FOO.BAR=4,FOO.Y,FOO.BAZ))"
    (sql [case [foo bar]
           3 [foo x]
           4 [foo y]
           t [foo baz]])
    :reported-form :syntax/desktop/case/else)

  (test "Iif((FOO.BAR < 0),FOO.X,Iif((FOO.BAR > 0),FOO.Y,NULL))"
    (sql [cond [< [foo bar] 0] [foo x]
               [> [foo bar] 0] [foo y]])
    :reported-form :syntax/desktop/cond)

  (test "Iif((FOO.BAR < 0),FOO.X,Iif((FOO.BAR > 0),FOO.Y,FOO.BAZ))"
    (sql [cond [< [foo bar] 0] [foo x]
               [> [foo bar] 0] [foo y]
               t [foo baz]])
    :reported-form :syntax/desktop/cond/else)
)

;;; Oracle

#+oracle
(defmethod test-syntax :after (env-type (database sql.sys:oracle-mixin))
  (test "NVL(FOO,'not specified')"
        (sql [nvl [foo] "not specified"])
        :reported-form :syntax/oracle/coalesce/nvl)
)

;;; PostgreSQL

#+postgresql
(defmethod test-syntax :after (env-type (database sql.sys:postgresql-mixin))
  (test "TRUNC(FOO)" (sql [truncate [foo]])		; Truncate is undefined in postgres
    :reported-form :syntax/postgresql/truncate))	; - replaced by trunc


(restore-sql-reader-syntax-state)

#|
(sql [select ["a1" aa] ["a2" bb] :from '([a "a1"] [a "a2"])
             :where [= ["a1" id] ["a2" id]]])

(setq s [union :all [select [foo] :from [bar]] [select [baz] :from [bar]]])

|#
