;;;-*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface - Tests for basic DB types

(in-package :sql.test)

(defmethod db-has-bigint? (database) nil)
(defmethod db-has-bigint? ((database odbc:odbc-database))
  (odbc::%get-type-descriptor database odbc::SQL_BIGINT))
  ;(odbc:db-describe-type database odbc::SQL_BIGINT))

(defparameter *type-table-row-count* 11)

(defmethod test-types :before (env-type database)
 ;;; Initialize type tables
  (drop-table 'TYPE_TABLE :if-does-not-exist :ignore)
  (drop-table 'TYPE_BIGINT :if-does-not-exist :ignore)

  (test-no-error 
   (execute-command 
    (format nil "CREATE TABLE TYPE_TABLE (t_int INTEGER, t_float ~A, t_str VARCHAR(30))"
            (sql.sys:db-sql-type-specifier database 'double-float))))

  (if (db-has-bigint? database)
      (test-no-error 
       (execute-command "CREATE TABLE TYPE_BIGINT (t_int integer, t_bigint BIGINT)"))
      (push :types/bigint/create *test-skips*))

  (dotimes (i *type-table-row-count*)
    (let* ((test-int (- i 5))
	   (test-flt (transform-float-1 test-int)))
      (execute-command
       (format nil "INSERT INTO TYPE_TABLE VALUES (~a,~a,'~a')"
	       test-int
	       (sql.sys::float-to-sql-string test-flt)
	       (sql.sys::float-to-sql-string test-flt) ))
      (when (db-has-bigint? database)
	(execute-command
	 (format nil "INSERT INTO TYPE_BIGINT VALUES (~a,~a)"
		 test-int (transform-bigint-1 test-int) ))))))

(defmethod test-types (env-type database)
  (test '(11 3)
        (multiple-value-bind (rows columns) (query "SELECT * FROM TYPE_TABLE")
          (values (length rows) (length columns)))
        :multiple-values t
        :reported-form :basic/select/1)

    
  (test (make-list *type-table-row-count* :initial-element '(t t t))
        (let ((results '()))
	  (dolist (row (query "SELECT * FROM TYPE_TABLE") results)
	    (destructuring-bind (int float str) row
	      (push (list (integerp int) (typep float 'double-float) (stringp str))
		    results))))
        :reported-form :types/basic/1)

  (test (make-list *type-table-row-count* :initial-element '(t t t))
	(let ((results '()))
	  (dolist (row
                   (query "SELECT * FROM TYPE_TABLE" :result-types '(string string string))
                   results)
	    (destructuring-bind (int float str) row
	      (push (list (stringp int) (stringp float) (stringp str))
		    results))))
        :test 'equal
        :reported-form :types/basic/1/strings)

  (test (make-list *type-table-row-count* :initial-element '(t t))
        (let ((results '()))
          (dolist (row (query "SELECT * FROM TYPE_TABLE") results)
            (destructuring-bind (int float str) row
              (push (list (double-float-equal (transform-float-1 int) float)
                          (double-float-equal (parse-double str) float))
                    results))))
        :reported-form :types/basic/2)

   (test (make-list *type-table-row-count* :initial-element '(t t))
         (let ((results '()))
           (dolist (row
                    (query "SELECT * FROM TYPE_TABLE" :result-types'(string string string))
                    results)
             (destructuring-bind (int float str) row
               (push (list (double-float-equal (transform-float-1 (parse-integer int))
                                               (parse-double float))
                           (double-float-equal (parse-double str)
                                               (parse-double float)))
                     results))))
         :reported-form :types/basic/2/strings)


  (test nil
        (map-query nil
                   (lambda (int float str) (list (+ int float) str))
                   "SELECT * FROM TYPE_TABLE")
        :reported-form :types/map/nil)

  (test (make-list *type-table-row-count* :initial-element '(t t))
        (let ((i 0))
          (map-query 'list
                     (lambda (int float &optional str)
                       (declare (ignore str))
                       (prog1 (list (eql (- i 5) int)
                                    (double-float-equal (transform-float-1 int) float))
                         (incf i)))
                     "SELECT * FROM TYPE_TABLE"))
        :reported-form :types/map/list)

  (test (make-list *type-table-row-count* :initial-element '(t 3 t t))
	(let ((results '())
	      (rows (map-query 'list #'list
                               "SELECT * FROM TYPE_TABLE" 
			       :result-types '(string string string))))
	  (dotimes (i (length rows) results)
	    (push (list (listp (nth i rows))
                        (length (nth i rows))
                        (eql (- i 5) (parse-integer (first (nth i rows)) 
                                                    :junk-allowed nil))
                        (double-float-equal (transform-float-1 (parse-integer
                                                                (first (nth i rows))))
                                            (parse-double (second (nth i rows)))))
                  results)))
        :reported-form :types/map/list/strings)
    
  (test (make-list *type-table-row-count* :initial-element '(t 3 t t))
        (let ((results '())
	      (rows (map-query 'vector #'vector
                               "SELECT * FROM TYPE_TABLE" 
			       :result-types '(string string string))))
	  (declare (type (simple-array vector (*)) rows))
	  (dotimes (i (length rows) results)
	    (push (list (simple-vector-p (aref rows i))
                        (length (aref rows i))
                        (eql (- i 5) (parse-integer (svref (aref rows i) 0)
                                                    :junk-allowed nil))
                        (double-float-equal (transform-float-1 (parse-integer
                                                                (svref (aref rows i) 0)))
                                            (parse-double (svref (aref rows i) 1))))
                  results)))
        :reported-form :types/map/vector/strings)
    
  ;; Query on a single column returns a list of one element or element itself if flatp t
  (test '(t 1)
	(let ((rows (map-query 'list #'list "SELECT t_int FROM TYPE_TABLE")))
	  (values (consp (first rows)) (length (first rows))))
        :multiple-values t
        :reported-form :types/map/singleton)

  (test t
	(integerp (first (map-query 'list #'identity
                                    (sql-query-exp 't_int :from 'TYPE_TABLE :flatp t))))
        :reported-form :types/map/flatp)

    
  (test (make-list *type-table-row-count* :initial-element '(t t))
	(let ((results '()))
	  (do-query ((int float str) "SELECT * FROM TYPE_TABLE")
	    (push (list (double-float-equal (transform-float-1 int) float)
                        (double-float-equal (parse-double str) float))
                  results))
	  results)
        :reported-form :types/do)

  (test (make-list *type-table-row-count* :initial-element '(t t))
	(let ((results '()))
	  (do-query ((int float str) "SELECT * FROM TYPE_TABLE"
                     :result-types '(string string string))
            (push (list (double-float-equal (transform-float-1 (parse-integer int))
                                            (parse-double float))
                        (double-float-equal (parse-double str) (parse-double float)))
                  results))
	  results)
        :reported-form :types/do/strings)

  (if (db-has-bigint? database)
      ;; PostgreSQL may return returns bigints as strings depending on platform.
      ;; This is unlikely possible with ODBC.
      (test (make-list *type-table-row-count* :initial-element '(t t))
            (let ((results '()))
              (dolist (row (query "SELECT * FROM TYPE_BIGINT") ;:result-types :auto)
                           results)
                (destructuring-bind (int bigint) row
                  (push (list (integerp int) (integerp bigint))
                              ;(if (typep database 'sql.sys:postgresql-mixin)
                              ;    t (integerp bigint)))
                        results))))
            :reported-form :types/bigint/select)
      (push :types/bigint/select *test-skips*))
)

;;; Helpers

(defun transform-float-1 (i)
  (coerce (* i (abs (/ i 2)) (expt 10 (* 2 i))) 'double-float))

(defun transform-bigint-1 (i)
  (* i (expt 10 (* 3 (abs i)))))

(defun parse-double (num-str)
  (let ((*read-default-float-format* 'double-float))
    (coerce (read-from-string num-str) 'double-float)))

(defun double-float-equal (a b)
  (if (zerop a)
      (zerop b)
      (<= (abs (/ (- a b) a)) (* 10 double-float-epsilon))))
