;;; -*- Mode: Lisp; -*-
;;; YstokSQL - Common Lisp ODBC interface - ASDF system definition

(in-package :cl-user)

(asdf:defsystem ysql
  ;:name "YstokSQL"
  :author "Dmitriy Ivanov <divanov_nosp@m_aha.ru (replace _nosp@m_ by @)>"
  :maintainer "Dmitriy Ivanov <divanov_nosp@m_aha.ru (replace _nosp@m_ by @)>"
  :licence "Lessor Lisp General Public License, see file COPYING.txt"
  :description "Common Lisp ODBC interface"
  :depends-on (ystok-library #+local-time local-time)
  :components 
  ((:module ysql.sql
    :pathname "sql/"
    :components
    ((:file "packages")
     (:file "conditions"	:depends-on ("packages"))
     (:file "generics"		:depends-on ("packages"))
     (:file "helpers"		:depends-on ("conditions" "generics"))
     (:file "syntax"		:depends-on ("helpers"))
     (:file "expressions"	:depends-on ("helpers"))
     (:file "functional"	:depends-on ("syntax" "expressions"))
     (:file "role-privileges"	:depends-on ("functional"))
     #+lispworks (:file "metaobjects" :depends-on ("syntax" "expressions"))
     #+lispworks (:file "object-oriented" :depends-on ("functional" "metaobjects"))
     (:file "loop"		:depends-on ("functional" #+lispworks "object-oriented"))
     (:file "transactions"	:depends-on ("functional" #+lispworks "object-oriented"))))

   (:module ysql.ffc
    :pathname #+lispworks "ffc/lispworks/" #-lispworks "?"
    :components
    ((:file "ffc-package")
     (:file "ffc-lispworks" :depends-on ("ffc-package")) ))

   (:module ysql.odbc
    :pathname "odbc/"
    :depends-on (ysql.ffc ysql.sql)
    :components
    ((:file "odbc-package")
     (:file "odbc-loader")
     (:file "odbc-basics"	:depends-on ("odbc-package"))
     (:file "odbc-constants"	:depends-on ("odbc-basics"))
     (:file "odbc-constants-ext" :depends-on ("odbc-constants"))
     (:file "odbc-ff-interface"	 :depends-on ("odbc-package" "odbc-loader"))
     #+Unicode (:file "odbc-unicode" :depends-on ("odbc-loader" "odbc-constants-ext"))
     (:file "odbc-functions"	:depends-on ("odbc-constants-ext" "odbc-ff-interface"))
     #+local-time (:file "odbc-local-time" 
				:depends-on ("odbc-constants-ext" "odbc-ff-interface"))
     (:file "odbc-descriptors"	:depends-on ("odbc-functions"
                                             #+local-time "odbc-local-time"))
     (:file "odbc-cursor"	:depends-on ("odbc-descriptors"))
     (:file "odbc-sql"		:depends-on ("odbc-cursor" #+Unicode "odbc-unicode"))
     (:file "odbc-prepared"	:depends-on ("odbc-cursor" #+Unicode "odbc-unicode"))
     #+lispworks (:file "odbc-object-oriented"
                  		:depends-on ("odbc-cursor" #+Unicode "odbc-unicode"))
     #+win32 (:file "odbc-installer"  :depends-on ("odbc-constants")) ))

   (:module ysql.dbms
    :pathname "dbms/"
    :depends-on (ysql.odbc)
    :components
    (#+access (:file "access-odbc")
     #+access (:file "access-sql" :depends-on ("access-odbc"))

     #+oracle (:file "oracle-odbc")
     #+oracle (:file "oracle-sql" :depends-on ("oracle-odbc"))

     #+postgresql (:file "postgresql-odbc")
     #+postgresql (:file "postgresql-sql" :depends-on ("postgresql-odbc"))

     #+sqlite3 (:file "sqlite3-odbc")
     #+sqlite3 (:file "sqlite3-sql" :depends-on ("sqlite3-odbc")) ))

   ;; SQL utils
   #+ys-product (:file "localization" :depends-on (ysql.sql)
                 :pathname "utils/localization.lisp")
   #+ys-product (:file "sql-utils" :depends-on (ysql.sql "localization" #+access ysql.dbms)
                 :pathname "utils/sql-utils.lisp")
   #+(and ys-product ywidgets)
		(:file "sql-widgets" :depends-on (ysql.sql "localization")
                 :pathname "utils/sql-widgets.lisp")
   #+ys-product (:file "schema-systema" :depends-on (ysql.sql #+ywidgets "sql-widgets")
                 :pathname "utils/schema-systema.lisp")
) )

;(asdf:operate 'asdf:load-op 'ysql)
